﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;
using Syncfusion.Data;
using Syncfusion.Data.Extensions;
using System.Collections.Specialized;
using Syncfusion.SfDataGrid.XForms;
using System.Runtime.CompilerServices;

namespace SupportXForms
{
   public class ViewModel : INotifyPropertyChanged, INotifyCollectionChanged
   {
      ///------------------------------------------------
      private OrderInfoRepository orderRepository;
      private ObservableCollection<OrderInfo> ordersInfo;
      internal const int SortIconWidth = 15;


      public ObservableCollection<string> CustomerNames { get; set; }

      internal string[] Customers = new string[] { "Adams", "Crowley", "Ellis", "Gable", "Irvine", "Keefe", "Mendoza", "Owens", "Rooney", "Wadded", };

      private bool showGroupedColumn = true;

      public bool ShowGroupedColumn
      {
         get
         {
            return this.showGroupedColumn;
         }
         set
         {
            this.showGroupedColumn = value;
            this.RaisePropertyChanged("ShowGroupedColumn");
         }
      }


      public ViewModel()
      {
         this.SelectedIndex = new GridSelectedRowsCollection();
         this.CustomerNames = Customers.ToObservableCollection();
         orderRepository = new OrderInfoRepository();
      }

      public ObservableCollection<OrderInfo> OrdersInfo
      {
         get { return ordersInfo; }
         set
         {
            ordersInfo = value;
            RaisePropertyChanged("OrdersInfo");
         }
      }


      private Label _MeasureLabel;
      public Label MeasureLabel
      {
         get
         {
            return _MeasureLabel;
         }
         set
         {
            _MeasureLabel = value;
            RaisePropertyChanged("MeasureLabel");


         }
      }

      private SfDataGrid _DataGrid;
      /// <summary>
      /// Das DataGrid der Seite
      /// </summary>
      public SfDataGrid DataGrid
      {
         get => _DataGrid;
         set
         {
            _DataGrid = value;
            //_DataGrid.CellRenderers.Remove("HeaderView");
            //_DataGrid.CellRenderers.Add("HeaderView", new HeaderCellRenderer() { Model = this });
            //_DataGrid.GridColumnSizer = new CustomColumnSizer() { Model = this };
         }
      }


      private bool isVisible = false;

      public bool IsVisible
      {
         get
         {
            return this.isVisible;
         }
         set
         {
            this.isVisible = value;
            RaisePropertyChanged("IsVisible");
         }
      }

      private double _HeaderRowHeight = 180;

      public double HeaderRowHeight
      {
         get
         {
            return _HeaderRowHeight;
         }
         set
         {
            _HeaderRowHeight = value;
            RaisePropertyChanged("HeaderRowHeight");
         }
      }

      private double _HeaderRowWidth = 60;

      public double HeaderRowWidth
      {
         get
         {
            return _HeaderRowWidth;
         }
         set
         {
            _HeaderRowWidth = value;
            RaisePropertyChanged("HeaderRowWidth");
         }
      }


      private GridSelectedRowsCollection _selectedIndex;

      public GridSelectedRowsCollection SelectedIndex
      {
         get
         {
            return this._selectedIndex;
         }
         set
         {
            this._selectedIndex = value;
            RaisePropertyChanged("SelectedIndex");
         }
      }
      #region INotifyPropertyChanged

      public event PropertyChangedEventHandler PropertyChanged;
      public event NotifyCollectionChangedEventHandler CollectionChanged;

      private void RaisePropertyChanged(string name)
      {
         if (PropertyChanged != null)
            this.PropertyChanged(this, new PropertyChangedEventArgs(name));
      }

      public void OnCollectionChange(NotifyCollectionChangedEventArgs e)
      {
         if (CollectionChanged != null)
            CollectionChanged(this, e);
      }
      #endregion

      public void Refresh()
      {
         Dictionary<string, string> columns = new Dictionary<string, string> {{ "CustomerID", "Customer" },
                                                                           { "OrderID", "Order ID" },
                                                                           { "Icon1", "Icon1 no" + Environment.NewLine + "two line row" },
                                                                           { "Icon2", "Icon2" },
                                                                           { "Icon3", "Icon3" },
                                                                           { "Icon4", "Icon4 with text" + Environment.NewLine + "two line row" },
                                                                           { "Icon5", "Icon5 and text" + Environment.NewLine + "two line row" }};

         try
         {

            this.DataGrid.Columns.Suspend();

            int count = 0;
            columns.ForEach(x =>
            {
               count++;

               if (count > 7)
                  return;

               GridColumn gridColumn = new GridTextColumn();

               if (count > 2)
               {
                  Label label = new Label()
                  {
                     BackgroundColor = Color.Azure,
                     TextColor = Color.DarkGray,
                     HorizontalOptions = LayoutOptions.End,
                     VerticalOptions = LayoutOptions.Center,
                     Padding = 0,
                     Margin = 0,
                     LineBreakMode = LineBreakMode.NoWrap,
                     Rotation = -90,
                     MaxLines = 2,
                     FontAttributes = FontAttributes.Bold,
                     FontSize = 14,
                     Text = x.Value,
                  };

                  gridColumn.HeaderTemplate = new DataTemplate(() =>
                  {
                     return label;
                  });
                  gridColumn.ColumnSizer = ColumnSizer.Auto;
                  gridColumn.Padding = new Thickness(0, 0);
                  gridColumn.LoadUIView = true;
                  gridColumn.RecordFont = FontAwesome6.Fonts.FontFamilySolid;
                  gridColumn.CellTextSize = 20;
                  gridColumn.MaximumWidth = 100;

                  this.MeasureLabel.FontAttributes = label.FontAttributes;
                  this.MeasureLabel.FontSize = label.FontSize;
                  this.MeasureLabel.Text = label.Text;
                  this.MeasureLabel.LineBreakMode = label.LineBreakMode;
                  this.MeasureLabel.MaxLines = label.MaxLines;

                  SizeRequest sizeRequest = this.MeasureLabel.Measure(Double.PositiveInfinity, Double.PositiveInfinity, MeasureFlags.None);

                  label.WidthRequest = sizeRequest.Request.Width + 10;
                  label.MinimumWidthRequest = sizeRequest.Request.Width + 10;
                  //label.Rotation = -90;

                  this.HeaderRowHeight = Math.Max(this.HeaderRowHeight, sizeRequest.Request.Width + 10);
                  this.HeaderRowWidth = Math.Max(this.HeaderRowWidth, sizeRequest.Request.Height + 10);

               }
               else
               {
                  gridColumn.HeaderText = x.Key;
                  gridColumn.HeaderCellTextSize = (Device.RuntimePlatform == Device.iOS) ? 12 : 14;
                  gridColumn.HeaderTextAlignment = TextAlignment.Start;
                  gridColumn.HeaderFontAttribute = FontAttributes.Bold;
                  gridColumn.TextAlignment = TextAlignment.Start;
                  gridColumn.MinimumWidth = 30;
                  gridColumn.MaximumWidth = 200;
                  gridColumn.LineBreakMode = Xamarin.Forms.LineBreakMode.WordWrap;
                  gridColumn.CellTextSize = (Device.RuntimePlatform == Device.iOS) ? 12 : 14;
                  gridColumn.Padding = new Thickness(5, 0);
               }

               //gridColumn.HeaderText = x.Key;

               gridColumn.MappingName = x.Key;

               this.DataGrid.Columns.Add(gridColumn);
            });

         }
         catch (Exception ex)
         {
            System.Diagnostics.Debugger.Break();
         }
         finally
         {
            this.DataGrid.Columns.Resume();
            this.DataGrid.RefreshColumns();
         }

         try
         {
            this.DataGrid.BatchBegin();

            this.OrdersInfo = orderRepository.GetOrderDetails(10);
         }
         catch (Exception ex)
         {
            System.Diagnostics.Debugger.Break();
         }
         finally
         {
            this.DataGrid.BatchCommit();
         }

         this.DataGrid.ColumnSizer = ColumnSizer.Auto;
         this.DataGrid.FrozenColumnsCount = 1;

      }
   }

   /// <summary>
   /// Eigener Renderer zum korrigieren der SortIndicator-Breite
   /// </summary>
   public class HeaderCellRenderer : GridHeaderCellRenderer
   {
      /// <inheritdoc/>
      public ViewModel Model { get; set; }

      /// <inheritdoc/>
      public override void OnInitializeDisplayView(DataColumnBase dataColumn, ContentView view)
      {
         base.OnInitializeDisplayView(dataColumn, view);

         Grid grid = ((view as BorderView).Content as Grid);

         grid.ColumnDefinitions[1].Width = new GridLength(ViewModel.SortIconWidth, GridUnitType.Absolute);
         if (grid.RowDefinitions.Count == 0)
            grid.RowDefinitions.Add(new RowDefinition() { Height = new GridLength(1, GridUnitType.Star) });

      }
   }

   /// <summary>
   /// Eigener Resizer, da das Original beim SortIndicator die Breite zu schmal rechnet.
   /// Die Breite wird in <see cref="HeaderCellRenderer"/> gesetzt.
   /// </summary>
   public class CustomColumnSizer : GridColumnSizer
   {
      public ViewModel Model { get; set; }


      /// <inheritdoc/>
      protected override double GetHeaderCellWidth(GridColumn column)
      {
         double a;
         if (column is GridTextColumn c && c.HeaderTemplate != null)
            a = this.Model.HeaderRowWidth;
         else
            a = base.GetHeaderCellWidth(column);

         double b = (column.AllowSorting ? ViewModel.SortIconWidth : 0);
         return a + b;

      }
   }

   /// <inheritdoc />
   public class CustomDataGridStyle : DataGridStyle
   {
      public override float GetBorderWidth()
      {
         return 2;
      }

      /// <inheritdoc />
      public override ImageSource GetHeaderSortIndicatorUp()
      {
         return new FontAwesome6.FontImageSource(FontAwesome6.Icons.ArrowUp, FontAwesome6.Fonts.FontType.Solid, 100, Color.FromHex("#6C6C6C"));
      }

      /// <inheritdoc />
      public override ImageSource GetHeaderSortIndicatorDown()
      {
         return null;
      }

      /// <inheritdoc />
      public override Color GetHeaderBackgroundColor()
      {
         return Color.FromHex("#F2F2F2");
      }

      /// <inheritdoc />
      public override Color GetHeaderForegroundColor()
      {
         return Color.FromHex("#6C6C6C");
      }

      /// <inheritdoc />
      public override Color GetFrozenIndicatorColor()
      {
         return Color.FromHex("#03A9F4");
      }

      /// <inheritdoc />
      public override Color GetDataGridBackgroundColor()
      {
         return Color.FromHex("#FFFFFF");
      }

   }

}
