﻿using System.ComponentModel;
using Xamarin.Forms;

namespace SupportXForms.FontAwesome6
{

   /// <summary>
   /// Ableitung des FontImageSource, um aus einem FontAwesome Character/Icon ein Bild zu machen.
   /// </summary>
   public class FontImageSource : Xamarin.Forms.FontImageSource
   {
      private const double @defaultSize = 25;

      /// <summary>
      /// Konstruktor
      /// </summary>
      public FontImageSource()
      {
         FontFamily = Fonts.FontFamily(Fonts.FontType.Solid);
         Size = @defaultSize;
         Color = Color.White;
      }

      /// <summary>
      /// Konstruktor
      /// </summary>
      /// <param name="glyph"></param>
      /// <param name="type"></param>
      /// <param name="size"></param>
      /// <param name="color"></param>
      public FontImageSource(string glyph, Fonts.FontType type = Fonts.FontType.Solid, double size = @defaultSize, Color color = default)
      {
         Glyph = glyph;
         FontFamily = Fonts.FontFamily(type);
         Size = size;
         Color = (color == default) ?  Color.White : color;
      }

      /// <summary>
      /// 
      /// </summary>
      public Fonts.FontType FontType { set => this.FontFamily = Fonts.FontFamily(value); }
   }

   /// <summary>
   /// Statische Klasse zum globalen Zugriff
   /// </summary>
   public class Fonts
   {
      /// <summary>
      /// Liste der Fontarten
      /// </summary>
      public enum FontType
      {
         /// <summary>Solid-Variant von FontAwesome</summary>
         Solid = 2,
      }

      /// <summary>
      /// Ruft den plattformabhängigen Font-Namen ab.
      /// </summary>
      public static string FontFamily(FontType type)
      {
         return "FontAwesome6" + type.ToString();
      }

      /// <summary>Name des Fonts</summary>
      public static string FontFamilySolid => "FontAwesome6" + FontType.Solid.ToString();

   }


   /// <summary>
   /// Die Unicode-Werte für alle FontAwesome Icons 6.4.0 (05.06.2023).
   /// <para/>
   /// Siehe https://fontawesome.com/cheatsheet
   /// <para/>
   /// Der Code wurde automatisch generiert von FA2CS (https://github.com/matthewrdev/fa2cs).
   /// </summary>
   public static class Icons
   {
      /// <summary>
      /// fa-0 unicode value ("\u0030").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/0
      /// </summary>
      [Description("0")]
      public const string Zero = "\u0030";

      /// <summary>
      /// fa-1 unicode value ("\u0031").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/1
      /// </summary>
      [Description("1")]
      public const string One = "\u0031";

      /// <summary>
      /// fa-2 unicode value ("\u0032").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/2
      /// </summary>
      [Description("2")]
      public const string Two = "\u0032";

      /// <summary>
      /// fa-3 unicode value ("\u0033").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/3
      /// </summary>
      [Description("3")]
      public const string Three = "\u0033";

      /// <summary>
      /// fa-4 unicode value ("\u0034").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/4
      /// </summary>
      [Description("4")]
      public const string Four = "\u0034";

      /// <summary>
      /// fa-5 unicode value ("\u0035").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/5
      /// </summary>
      [Description("5")]
      public const string Five = "\u0035";

      /// <summary>
      /// fa-6 unicode value ("\u0036").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/6
      /// </summary>
      [Description("6")]
      public const string Six = "\u0036";

      /// <summary>
      /// fa-7 unicode value ("\u0037").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/7
      /// </summary>
      [Description("7")]
      public const string Seven = "\u0037";

      /// <summary>
      /// fa-8 unicode value ("\u0038").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/8
      /// </summary>
      [Description("8")]
      public const string Eight = "\u0038";

      /// <summary>
      /// fa-9 unicode value ("\u0039").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/9
      /// </summary>
      [Description("9")]
      public const string Nine = "\u0039";

      /// <summary>
      /// fa-00 unicode value ("\ue467").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/00
      /// </summary>
      [Description("00")]
      public const string ZeroZero = "\ue467";

      /// <summary>
      /// fa-360-degrees unicode value ("\ue2dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/360-degrees
      /// </summary>
      [Description("360-degrees")]
      public const string ThreeHundred_And_SixtyDegrees = "\ue2dc";

      /// <summary>
      /// fa-42-group unicode value ("\ue080").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/42-group
      /// </summary>
      [Description("42-group")]
      public const string FortyTwoGroup = "\ue080";

      /// <summary>
      /// fa-500px unicode value ("\uf26e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/500px
      /// </summary>
      [Description("500px")]
      public const string FiveHundredPX = "\uf26e";

      /// <summary>
      /// fa-a unicode value ("\u0041").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/a
      /// </summary>
      [Description("a")]
      public const string A = "\u0041";

      /// <summary>
      /// fa-abacus unicode value ("\uf640").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/abacus
      /// </summary>
      [Description("abacus")]
      public const string Abacus = "\uf640";

      /// <summary>
      /// fa-accent-grave unicode value ("\u0060").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/accent-grave
      /// </summary>
      [Description("accent-grave")]
      public const string AccentGrave = "\u0060";

      /// <summary>
      /// fa-accessible-icon unicode value ("\uf368").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/accessible-icon
      /// </summary>
      [Description("accessible-icon")]
      public const string AccessibleIcon = "\uf368";

      /// <summary>
      /// fa-accusoft unicode value ("\uf369").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/accusoft
      /// </summary>
      [Description("accusoft")]
      public const string Accusoft = "\uf369";

      /// <summary>
      /// fa-acorn unicode value ("\uf6ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/acorn
      /// </summary>
      [Description("acorn")]
      public const string Acorn = "\uf6ae";

      /// <summary>
      /// fa-address-book unicode value ("\uf2b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/address-book
      /// </summary>
      [Description("address-book")]
      public const string AddressBook = "\uf2b9";

      /// <summary>
      /// fa-address-card unicode value ("\uf2bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/address-card
      /// </summary>
      [Description("address-card")]
      public const string AddressCard = "\uf2bb";

      /// <summary>
      /// fa-adn unicode value ("\uf170").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/adn
      /// </summary>
      [Description("adn")]
      public const string Adn = "\uf170";

      /// <summary>
      /// fa-adversal unicode value ("\uf36a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/adversal
      /// </summary>
      [Description("adversal")]
      public const string Adversal = "\uf36a";

      /// <summary>
      /// fa-affiliatetheme unicode value ("\uf36b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/affiliatetheme
      /// </summary>
      [Description("affiliatetheme")]
      public const string Affiliatetheme = "\uf36b";

      /// <summary>
      /// fa-air-conditioner unicode value ("\uf8f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/air-conditioner
      /// </summary>
      [Description("air-conditioner")]
      public const string AirConditioner = "\uf8f4";

      /// <summary>
      /// fa-airbnb unicode value ("\uf834").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/airbnb
      /// </summary>
      [Description("airbnb")]
      public const string Airbnb = "\uf834";

      /// <summary>
      /// fa-airplay unicode value ("\ue089").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/airplay
      /// </summary>
      [Description("airplay")]
      public const string Airplay = "\ue089";

      /// <summary>
      /// fa-alarm-clock unicode value ("\uf34e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alarm-clock
      /// </summary>
      [Description("alarm-clock")]
      public const string AlarmClock = "\uf34e";

      /// <summary>
      /// fa-alarm-exclamation unicode value ("\uf843").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alarm-exclamation
      /// </summary>
      [Description("alarm-exclamation")]
      public const string AlarmExclamation = "\uf843";

      /// <summary>
      /// fa-alarm-plus unicode value ("\uf844").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alarm-plus
      /// </summary>
      [Description("alarm-plus")]
      public const string AlarmPlus = "\uf844";

      /// <summary>
      /// fa-alarm-snooze unicode value ("\uf845").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alarm-snooze
      /// </summary>
      [Description("alarm-snooze")]
      public const string AlarmSnooze = "\uf845";

      /// <summary>
      /// fa-album unicode value ("\uf89f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album
      /// </summary>
      [Description("album")]
      public const string Album = "\uf89f";

      /// <summary>
      /// fa-album-circle-plus unicode value ("\ue48c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album-circle-plus
      /// </summary>
      [Description("album-circle-plus")]
      public const string AlbumCirclePlus = "\ue48c";

      /// <summary>
      /// fa-album-circle-user unicode value ("\ue48d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album-circle-user
      /// </summary>
      [Description("album-circle-user")]
      public const string AlbumCircleUser = "\ue48d";

      /// <summary>
      /// fa-album-collection unicode value ("\uf8a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album-collection
      /// </summary>
      [Description("album-collection")]
      public const string AlbumCollection = "\uf8a0";

      /// <summary>
      /// fa-album-collection-circle-plus unicode value ("\ue48e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album-collection-circle-plus
      /// </summary>
      [Description("album-collection-circle-plus")]
      public const string AlbumCollectionCirclePlus = "\ue48e";

      /// <summary>
      /// fa-album-collection-circle-user unicode value ("\ue48f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/album-collection-circle-user
      /// </summary>
      [Description("album-collection-circle-user")]
      public const string AlbumCollectionCircleUser = "\ue48f";

      /// <summary>
      /// fa-algolia unicode value ("\uf36c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.3.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/algolia
      /// </summary>
      [Description("algolia")]
      public const string Algolia = "\uf36c";

      /// <summary>
      /// fa-alicorn unicode value ("\uf6b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alicorn
      /// </summary>
      [Description("alicorn")]
      public const string Alicorn = "\uf6b0";

      /// <summary>
      /// fa-alien unicode value ("\uf8f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alien
      /// </summary>
      [Description("alien")]
      public const string Alien = "\uf8f5";

      /// <summary>
      /// fa-alien-8bit unicode value ("\uf8f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alien-8bit
      /// </summary>
      [Description("alien-8bit")]
      public const string Alien8bit = "\uf8f6";

      /// <summary>
      /// fa-align-center unicode value ("\uf037").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/align-center
      /// </summary>
      [Description("align-center")]
      public const string AlignCenter = "\uf037";

      /// <summary>
      /// fa-align-justify unicode value ("\uf039").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/align-justify
      /// </summary>
      [Description("align-justify")]
      public const string AlignJustify = "\uf039";

      /// <summary>
      /// fa-align-left unicode value ("\uf036").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/align-left
      /// </summary>
      [Description("align-left")]
      public const string AlignLeft = "\uf036";

      /// <summary>
      /// fa-align-right unicode value ("\uf038").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/align-right
      /// </summary>
      [Description("align-right")]
      public const string AlignRight = "\uf038";

      /// <summary>
      /// fa-align-slash unicode value ("\uf846").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/align-slash
      /// </summary>
      [Description("align-slash")]
      public const string AlignSlash = "\uf846";

      /// <summary>
      /// fa-alipay unicode value ("\uf642").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '5.3.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alipay
      /// </summary>
      [Description("alipay")]
      public const string Alipay = "\uf642";

      /// <summary>
      /// fa-alt unicode value ("\ue08a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/alt
      /// </summary>
      [Description("alt")]
      public const string Alt = "\ue08a";

      /// <summary>
      /// fa-amazon unicode value ("\uf270").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/amazon
      /// </summary>
      [Description("amazon")]
      public const string Amazon = "\uf270";

      /// <summary>
      /// fa-amazon-pay unicode value ("\uf42c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.2', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/amazon-pay
      /// </summary>
      [Description("amazon-pay")]
      public const string AmazonPay = "\uf42c";

      /// <summary>
      /// fa-amilia unicode value ("\uf36d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/amilia
      /// </summary>
      [Description("amilia")]
      public const string Amilia = "\uf36d";

      /// <summary>
      /// fa-amp-guitar unicode value ("\uf8a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/amp-guitar
      /// </summary>
      [Description("amp-guitar")]
      public const string AmpGuitar = "\uf8a1";

      /// <summary>
      /// fa-ampersand unicode value ("\u0026").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ampersand
      /// </summary>
      [Description("ampersand")]
      public const string Ampersand = "\u0026";

      /// <summary>
      /// fa-anchor unicode value ("\uf13d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/anchor
      /// </summary>
      [Description("anchor")]
      public const string Anchor = "\uf13d";

      /// <summary>
      /// fa-anchor-circle-check unicode value ("\ue4aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/anchor-circle-check
      /// </summary>
      [Description("anchor-circle-check")]
      public const string AnchorCircleCheck = "\ue4aa";

      /// <summary>
      /// fa-anchor-circle-exclamation unicode value ("\ue4ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/anchor-circle-exclamation
      /// </summary>
      [Description("anchor-circle-exclamation")]
      public const string AnchorCircleExclamation = "\ue4ab";

      /// <summary>
      /// fa-anchor-circle-xmark unicode value ("\ue4ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/anchor-circle-xmark
      /// </summary>
      [Description("anchor-circle-xmark")]
      public const string AnchorCircleXmark = "\ue4ac";

      /// <summary>
      /// fa-anchor-lock unicode value ("\ue4ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/anchor-lock
      /// </summary>
      [Description("anchor-lock")]
      public const string AnchorLock = "\ue4ad";

      /// <summary>
      /// fa-android unicode value ("\uf17b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/android
      /// </summary>
      [Description("android")]
      public const string Android = "\uf17b";

      /// <summary>
      /// fa-angel unicode value ("\uf779").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angel
      /// </summary>
      [Description("angel")]
      public const string Angel = "\uf779";

      /// <summary>
      /// fa-angellist unicode value ("\uf209").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angellist
      /// </summary>
      [Description("angellist")]
      public const string Angellist = "\uf209";

      /// <summary>
      /// fa-angle unicode value ("\ue08c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle
      /// </summary>
      [Description("angle")]
      public const string Angle = "\ue08c";

      /// <summary>
      /// fa-angle-90 unicode value ("\ue08d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle-90
      /// </summary>
      [Description("angle-90")]
      public const string Angle90 = "\ue08d";

      /// <summary>
      /// fa-angle-down unicode value ("\uf107").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle-down
      /// </summary>
      [Description("angle-down")]
      public const string AngleDown = "\uf107";

      /// <summary>
      /// fa-angle-left unicode value ("\uf104").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle-left
      /// </summary>
      [Description("angle-left")]
      public const string AngleLeft = "\uf104";

      /// <summary>
      /// fa-angle-right unicode value ("\uf105").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle-right
      /// </summary>
      [Description("angle-right")]
      public const string AngleRight = "\uf105";

      /// <summary>
      /// fa-angle-up unicode value ("\uf106").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angle-up
      /// </summary>
      [Description("angle-up")]
      public const string AngleUp = "\uf106";

      /// <summary>
      /// fa-angles-down unicode value ("\uf103").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angles-down
      /// </summary>
      [Description("angles-down")]
      public const string AnglesDown = "\uf103";

      /// <summary>
      /// fa-angles-left unicode value ("\uf100").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angles-left
      /// </summary>
      [Description("angles-left")]
      public const string AnglesLeft = "\uf100";

      /// <summary>
      /// fa-angles-right unicode value ("\uf101").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angles-right
      /// </summary>
      [Description("angles-right")]
      public const string AnglesRight = "\uf101";

      /// <summary>
      /// fa-angles-up unicode value ("\uf102").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angles-up
      /// </summary>
      [Description("angles-up")]
      public const string AnglesUp = "\uf102";

      /// <summary>
      /// fa-angrycreative unicode value ("\uf36e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angrycreative
      /// </summary>
      [Description("angrycreative")]
      public const string Angrycreative = "\uf36e";

      /// <summary>
      /// fa-angular unicode value ("\uf420").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/angular
      /// </summary>
      [Description("angular")]
      public const string Angular = "\uf420";

      /// <summary>
      /// fa-ankh unicode value ("\uf644").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ankh
      /// </summary>
      [Description("ankh")]
      public const string Ankh = "\uf644";

      /// <summary>
      /// fa-apartment unicode value ("\ue468").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apartment
      /// </summary>
      [Description("apartment")]
      public const string Apartment = "\ue468";

      /// <summary>
      /// fa-aperture unicode value ("\ue2df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/aperture
      /// </summary>
      [Description("aperture")]
      public const string Aperture = "\ue2df";

      /// <summary>
      /// fa-apostrophe unicode value ("\u0027").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apostrophe
      /// </summary>
      [Description("apostrophe")]
      public const string Apostrophe = "\u0027";

      /// <summary>
      /// fa-app-store unicode value ("\uf36f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/app-store
      /// </summary>
      [Description("app-store")]
      public const string AppStore = "\uf36f";

      /// <summary>
      /// fa-app-store-ios unicode value ("\uf370").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/app-store-ios
      /// </summary>
      [Description("app-store-ios")]
      public const string AppStoreIos = "\uf370";

      /// <summary>
      /// fa-apper unicode value ("\uf371").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apper
      /// </summary>
      [Description("apper")]
      public const string Apper = "\uf371";

      /// <summary>
      /// fa-apple unicode value ("\uf179").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apple
      /// </summary>
      [Description("apple")]
      public const string Apple = "\uf179";

      /// <summary>
      /// fa-apple-core unicode value ("\ue08f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apple-core
      /// </summary>
      [Description("apple-core")]
      public const string AppleCore = "\ue08f";

      /// <summary>
      /// fa-apple-pay unicode value ("\uf415").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apple-pay
      /// </summary>
      [Description("apple-pay")]
      public const string ApplePay = "\uf415";

      /// <summary>
      /// fa-apple-whole unicode value ("\uf5d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/apple-whole
      /// </summary>
      [Description("apple-whole")]
      public const string AppleWhole = "\uf5d1";

      /// <summary>
      /// fa-archway unicode value ("\uf557").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/archway
      /// </summary>
      [Description("archway")]
      public const string Archway = "\uf557";

      /// <summary>
      /// fa-arrow-down unicode value ("\uf063").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down
      /// </summary>
      [Description("arrow-down")]
      public const string ArrowDown = "\uf063";

      /// <summary>
      /// fa-arrow-down-1-9 unicode value ("\uf162").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-1-9
      /// </summary>
      [Description("arrow-down-1-9")]
      public const string ArrowDown19 = "\uf162";

      /// <summary>
      /// fa-arrow-down-9-1 unicode value ("\uf886").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-9-1
      /// </summary>
      [Description("arrow-down-9-1")]
      public const string ArrowDown91 = "\uf886";

      /// <summary>
      /// fa-arrow-down-a-z unicode value ("\uf15d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-a-z
      /// </summary>
      [Description("arrow-down-a-z")]
      public const string ArrowDownAZ = "\uf15d";

      /// <summary>
      /// fa-arrow-down-arrow-up unicode value ("\uf883").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-arrow-up
      /// </summary>
      [Description("arrow-down-arrow-up")]
      public const string ArrowDownArrowUp = "\uf883";

      /// <summary>
      /// fa-arrow-down-big-small unicode value ("\uf88c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-big-small
      /// </summary>
      [Description("arrow-down-big-small")]
      public const string ArrowDownBigSmall = "\uf88c";

      /// <summary>
      /// fa-arrow-down-from-dotted-line unicode value ("\ue090").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-from-dotted-line
      /// </summary>
      [Description("arrow-down-from-dotted-line")]
      public const string ArrowDownFromDottedLine = "\ue090";

      /// <summary>
      /// fa-arrow-down-from-line unicode value ("\uf345").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-from-line
      /// </summary>
      [Description("arrow-down-from-line")]
      public const string ArrowDownFromLine = "\uf345";

      /// <summary>
      /// fa-arrow-down-left unicode value ("\ue091").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-left
      /// </summary>
      [Description("arrow-down-left")]
      public const string ArrowDownLeft = "\ue091";

      /// <summary>
      /// fa-arrow-down-left-and-arrow-up-right-to-center unicode value ("\ue092").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-left-and-arrow-up-right-to-center
      /// </summary>
      [Description("arrow-down-left-and-arrow-up-right-to-center")]
      public const string ArrowDownLeftAndArrowUpRightToCenter = "\ue092";

      /// <summary>
      /// fa-arrow-down-long unicode value ("\uf175").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-long
      /// </summary>
      [Description("arrow-down-long")]
      public const string ArrowDownLong = "\uf175";

      /// <summary>
      /// fa-arrow-down-right unicode value ("\ue093").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-right
      /// </summary>
      [Description("arrow-down-right")]
      public const string ArrowDownRight = "\ue093";

      /// <summary>
      /// fa-arrow-down-short-wide unicode value ("\uf884").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-short-wide
      /// </summary>
      [Description("arrow-down-short-wide")]
      public const string ArrowDownShortWide = "\uf884";

      /// <summary>
      /// fa-arrow-down-small-big unicode value ("\uf88d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-small-big
      /// </summary>
      [Description("arrow-down-small-big")]
      public const string ArrowDownSmallBig = "\uf88d";

      /// <summary>
      /// fa-arrow-down-square-triangle unicode value ("\uf889").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-square-triangle
      /// </summary>
      [Description("arrow-down-square-triangle")]
      public const string ArrowDownSquareTriangle = "\uf889";

      /// <summary>
      /// fa-arrow-down-to-arc unicode value ("\ue4ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-to-arc
      /// </summary>
      [Description("arrow-down-to-arc")]
      public const string ArrowDownToArc = "\ue4ae";

      /// <summary>
      /// fa-arrow-down-to-bracket unicode value ("\ue094").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-to-bracket
      /// </summary>
      [Description("arrow-down-to-bracket")]
      public const string ArrowDownToBracket = "\ue094";

      /// <summary>
      /// fa-arrow-down-to-dotted-line unicode value ("\ue095").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-to-dotted-line
      /// </summary>
      [Description("arrow-down-to-dotted-line")]
      public const string ArrowDownToDottedLine = "\ue095";

      /// <summary>
      /// fa-arrow-down-to-line unicode value ("\uf33d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-to-line
      /// </summary>
      [Description("arrow-down-to-line")]
      public const string ArrowDownToLine = "\uf33d";

      /// <summary>
      /// fa-arrow-down-to-square unicode value ("\ue096").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-to-square
      /// </summary>
      [Description("arrow-down-to-square")]
      public const string ArrowDownToSquare = "\ue096";

      /// <summary>
      /// fa-arrow-down-triangle-square unicode value ("\uf888").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-triangle-square
      /// </summary>
      [Description("arrow-down-triangle-square")]
      public const string ArrowDownTriangleSquare = "\uf888";

      /// <summary>
      /// fa-arrow-down-up-across-line unicode value ("\ue4af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-up-across-line
      /// </summary>
      [Description("arrow-down-up-across-line")]
      public const string ArrowDownUpAcrossLine = "\ue4af";

      /// <summary>
      /// fa-arrow-down-up-lock unicode value ("\ue4b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-up-lock
      /// </summary>
      [Description("arrow-down-up-lock")]
      public const string ArrowDownUpLock = "\ue4b0";

      /// <summary>
      /// fa-arrow-down-wide-short unicode value ("\uf160").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-wide-short
      /// </summary>
      [Description("arrow-down-wide-short")]
      public const string ArrowDownWideShort = "\uf160";

      /// <summary>
      /// fa-arrow-down-z-a unicode value ("\uf881").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-down-z-a
      /// </summary>
      [Description("arrow-down-z-a")]
      public const string ArrowDownZA = "\uf881";

      /// <summary>
      /// fa-arrow-left unicode value ("\uf060").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-left
      /// </summary>
      [Description("arrow-left")]
      public const string ArrowLeft = "\uf060";

      /// <summary>
      /// fa-arrow-left-from-line unicode value ("\uf344").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-left-from-line
      /// </summary>
      [Description("arrow-left-from-line")]
      public const string ArrowLeftFromLine = "\uf344";

      /// <summary>
      /// fa-arrow-left-long unicode value ("\uf177").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-left-long
      /// </summary>
      [Description("arrow-left-long")]
      public const string ArrowLeftLong = "\uf177";

      /// <summary>
      /// fa-arrow-left-long-to-line unicode value ("\ue3d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-left-long-to-line
      /// </summary>
      [Description("arrow-left-long-to-line")]
      public const string ArrowLeftLongToLine = "\ue3d4";

      /// <summary>
      /// fa-arrow-left-to-line unicode value ("\uf33e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-left-to-line
      /// </summary>
      [Description("arrow-left-to-line")]
      public const string ArrowLeftToLine = "\uf33e";

      /// <summary>
      /// fa-arrow-pointer unicode value ("\uf245").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-pointer
      /// </summary>
      [Description("arrow-pointer")]
      public const string ArrowPointer = "\uf245";

      /// <summary>
      /// fa-arrow-progress unicode value ("\ue5df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-progress
      /// </summary>
      [Description("arrow-progress")]
      public const string ArrowProgress = "\ue5df";

      /// <summary>
      /// fa-arrow-right unicode value ("\uf061").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right
      /// </summary>
      [Description("arrow-right")]
      public const string ArrowRight = "\uf061";

      /// <summary>
      /// fa-arrow-right-arrow-left unicode value ("\uf0ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-arrow-left
      /// </summary>
      [Description("arrow-right-arrow-left")]
      public const string ArrowRightArrowLeft = "\uf0ec";

      /// <summary>
      /// fa-arrow-right-from-arc unicode value ("\ue4b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-from-arc
      /// </summary>
      [Description("arrow-right-from-arc")]
      public const string ArrowRightFromArc = "\ue4b1";

      /// <summary>
      /// fa-arrow-right-from-bracket unicode value ("\uf08b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-from-bracket
      /// </summary>
      [Description("arrow-right-from-bracket")]
      public const string ArrowRightFromBracket = "\uf08b";

      /// <summary>
      /// fa-arrow-right-from-line unicode value ("\uf343").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-from-line
      /// </summary>
      [Description("arrow-right-from-line")]
      public const string ArrowRightFromLine = "\uf343";

      /// <summary>
      /// fa-arrow-right-long unicode value ("\uf178").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-long
      /// </summary>
      [Description("arrow-right-long")]
      public const string ArrowRightLong = "\uf178";

      /// <summary>
      /// fa-arrow-right-long-to-line unicode value ("\ue3d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-long-to-line
      /// </summary>
      [Description("arrow-right-long-to-line")]
      public const string ArrowRightLongToLine = "\ue3d5";

      /// <summary>
      /// fa-arrow-right-to-arc unicode value ("\ue4b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-to-arc
      /// </summary>
      [Description("arrow-right-to-arc")]
      public const string ArrowRightToArc = "\ue4b2";

      /// <summary>
      /// fa-arrow-right-to-bracket unicode value ("\uf090").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-to-bracket
      /// </summary>
      [Description("arrow-right-to-bracket")]
      public const string ArrowRightToBracket = "\uf090";

      /// <summary>
      /// fa-arrow-right-to-city unicode value ("\ue4b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-to-city
      /// </summary>
      [Description("arrow-right-to-city")]
      public const string ArrowRightToCity = "\ue4b3";

      /// <summary>
      /// fa-arrow-right-to-line unicode value ("\uf340").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-right-to-line
      /// </summary>
      [Description("arrow-right-to-line")]
      public const string ArrowRightToLine = "\uf340";

      /// <summary>
      /// fa-arrow-rotate-left unicode value ("\uf0e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-rotate-left
      /// </summary>
      [Description("arrow-rotate-left")]
      public const string ArrowRotateLeft = "\uf0e2";

      /// <summary>
      /// fa-arrow-rotate-right unicode value ("\uf01e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-rotate-right
      /// </summary>
      [Description("arrow-rotate-right")]
      public const string ArrowRotateRight = "\uf01e";

      /// <summary>
      /// fa-arrow-trend-down unicode value ("\ue097").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-trend-down
      /// </summary>
      [Description("arrow-trend-down")]
      public const string ArrowTrendDown = "\ue097";

      /// <summary>
      /// fa-arrow-trend-up unicode value ("\ue098").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-trend-up
      /// </summary>
      [Description("arrow-trend-up")]
      public const string ArrowTrendUp = "\ue098";

      /// <summary>
      /// fa-arrow-turn-down unicode value ("\uf149").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-turn-down
      /// </summary>
      [Description("arrow-turn-down")]
      public const string ArrowTurnDown = "\uf149";

      /// <summary>
      /// fa-arrow-turn-down-left unicode value ("\ue2e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-turn-down-left
      /// </summary>
      [Description("arrow-turn-down-left")]
      public const string ArrowTurnDownLeft = "\ue2e1";

      /// <summary>
      /// fa-arrow-turn-down-right unicode value ("\ue3d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-turn-down-right
      /// </summary>
      [Description("arrow-turn-down-right")]
      public const string ArrowTurnDownRight = "\ue3d6";

      /// <summary>
      /// fa-arrow-turn-up unicode value ("\uf148").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-turn-up
      /// </summary>
      [Description("arrow-turn-up")]
      public const string ArrowTurnUp = "\uf148";

      /// <summary>
      /// fa-arrow-up unicode value ("\uf062").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up
      /// </summary>
      [Description("arrow-up")]
      public const string ArrowUp = "\uf062";

      /// <summary>
      /// fa-arrow-up-1-9 unicode value ("\uf163").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-1-9
      /// </summary>
      [Description("arrow-up-1-9")]
      public const string ArrowUp19 = "\uf163";

      /// <summary>
      /// fa-arrow-up-9-1 unicode value ("\uf887").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-9-1
      /// </summary>
      [Description("arrow-up-9-1")]
      public const string ArrowUp91 = "\uf887";

      /// <summary>
      /// fa-arrow-up-a-z unicode value ("\uf15e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-a-z
      /// </summary>
      [Description("arrow-up-a-z")]
      public const string ArrowUpAZ = "\uf15e";

      /// <summary>
      /// fa-arrow-up-arrow-down unicode value ("\ue099").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-arrow-down
      /// </summary>
      [Description("arrow-up-arrow-down")]
      public const string ArrowUpArrowDown = "\ue099";

      /// <summary>
      /// fa-arrow-up-big-small unicode value ("\uf88e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-big-small
      /// </summary>
      [Description("arrow-up-big-small")]
      public const string ArrowUpBigSmall = "\uf88e";

      /// <summary>
      /// fa-arrow-up-from-arc unicode value ("\ue4b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-arc
      /// </summary>
      [Description("arrow-up-from-arc")]
      public const string ArrowUpFromArc = "\ue4b4";

      /// <summary>
      /// fa-arrow-up-from-bracket unicode value ("\ue09a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-bracket
      /// </summary>
      [Description("arrow-up-from-bracket")]
      public const string ArrowUpFromBracket = "\ue09a";

      /// <summary>
      /// fa-arrow-up-from-dotted-line unicode value ("\ue09b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-dotted-line
      /// </summary>
      [Description("arrow-up-from-dotted-line")]
      public const string ArrowUpFromDottedLine = "\ue09b";

      /// <summary>
      /// fa-arrow-up-from-ground-water unicode value ("\ue4b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-ground-water
      /// </summary>
      [Description("arrow-up-from-ground-water")]
      public const string ArrowUpFromGroundWater = "\ue4b5";

      /// <summary>
      /// fa-arrow-up-from-line unicode value ("\uf342").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-line
      /// </summary>
      [Description("arrow-up-from-line")]
      public const string ArrowUpFromLine = "\uf342";

      /// <summary>
      /// fa-arrow-up-from-square unicode value ("\ue09c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-square
      /// </summary>
      [Description("arrow-up-from-square")]
      public const string ArrowUpFromSquare = "\ue09c";

      /// <summary>
      /// fa-arrow-up-from-water-pump unicode value ("\ue4b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-from-water-pump
      /// </summary>
      [Description("arrow-up-from-water-pump")]
      public const string ArrowUpFromWaterPump = "\ue4b6";

      /// <summary>
      /// fa-arrow-up-left unicode value ("\ue09d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-left
      /// </summary>
      [Description("arrow-up-left")]
      public const string ArrowUpLeft = "\ue09d";

      /// <summary>
      /// fa-arrow-up-left-from-circle unicode value ("\ue09e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-left-from-circle
      /// </summary>
      [Description("arrow-up-left-from-circle")]
      public const string ArrowUpLeftFromCircle = "\ue09e";

      /// <summary>
      /// fa-arrow-up-long unicode value ("\uf176").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-long
      /// </summary>
      [Description("arrow-up-long")]
      public const string ArrowUpLong = "\uf176";

      /// <summary>
      /// fa-arrow-up-right unicode value ("\ue09f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-right
      /// </summary>
      [Description("arrow-up-right")]
      public const string ArrowUpRight = "\ue09f";

      /// <summary>
      /// fa-arrow-up-right-and-arrow-down-left-from-center unicode value ("\ue0a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-right-and-arrow-down-left-from-center
      /// </summary>
      [Description("arrow-up-right-and-arrow-down-left-from-center")]
      public const string ArrowUpRightAndArrowDownLeftFromCenter = "\ue0a0";

      /// <summary>
      /// fa-arrow-up-right-dots unicode value ("\ue4b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-right-dots
      /// </summary>
      [Description("arrow-up-right-dots")]
      public const string ArrowUpRightDots = "\ue4b7";

      /// <summary>
      /// fa-arrow-up-right-from-square unicode value ("\uf08e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-right-from-square
      /// </summary>
      [Description("arrow-up-right-from-square")]
      public const string ArrowUpRightFromSquare = "\uf08e";

      /// <summary>
      /// fa-arrow-up-short-wide unicode value ("\uf885").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-short-wide
      /// </summary>
      [Description("arrow-up-short-wide")]
      public const string ArrowUpShortWide = "\uf885";

      /// <summary>
      /// fa-arrow-up-small-big unicode value ("\uf88f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-small-big
      /// </summary>
      [Description("arrow-up-small-big")]
      public const string ArrowUpSmallBig = "\uf88f";

      /// <summary>
      /// fa-arrow-up-square-triangle unicode value ("\uf88b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-square-triangle
      /// </summary>
      [Description("arrow-up-square-triangle")]
      public const string ArrowUpSquareTriangle = "\uf88b";

      /// <summary>
      /// fa-arrow-up-to-dotted-line unicode value ("\ue0a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-to-dotted-line
      /// </summary>
      [Description("arrow-up-to-dotted-line")]
      public const string ArrowUpToDottedLine = "\ue0a1";

      /// <summary>
      /// fa-arrow-up-to-line unicode value ("\uf341").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-to-line
      /// </summary>
      [Description("arrow-up-to-line")]
      public const string ArrowUpToLine = "\uf341";

      /// <summary>
      /// fa-arrow-up-triangle-square unicode value ("\uf88a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-triangle-square
      /// </summary>
      [Description("arrow-up-triangle-square")]
      public const string ArrowUpTriangleSquare = "\uf88a";

      /// <summary>
      /// fa-arrow-up-wide-short unicode value ("\uf161").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-wide-short
      /// </summary>
      [Description("arrow-up-wide-short")]
      public const string ArrowUpWideShort = "\uf161";

      /// <summary>
      /// fa-arrow-up-z-a unicode value ("\uf882").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrow-up-z-a
      /// </summary>
      [Description("arrow-up-z-a")]
      public const string ArrowUpZA = "\uf882";

      /// <summary>
      /// fa-arrows-cross unicode value ("\ue0a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-cross
      /// </summary>
      [Description("arrows-cross")]
      public const string ArrowsCross = "\ue0a2";

      /// <summary>
      /// fa-arrows-down-to-line unicode value ("\ue4b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-down-to-line
      /// </summary>
      [Description("arrows-down-to-line")]
      public const string ArrowsDownToLine = "\ue4b8";

      /// <summary>
      /// fa-arrows-down-to-people unicode value ("\ue4b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-down-to-people
      /// </summary>
      [Description("arrows-down-to-people")]
      public const string ArrowsDownToPeople = "\ue4b9";

      /// <summary>
      /// fa-arrows-from-dotted-line unicode value ("\ue0a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-from-dotted-line
      /// </summary>
      [Description("arrows-from-dotted-line")]
      public const string ArrowsFromDottedLine = "\ue0a3";

      /// <summary>
      /// fa-arrows-from-line unicode value ("\ue0a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-from-line
      /// </summary>
      [Description("arrows-from-line")]
      public const string ArrowsFromLine = "\ue0a4";

      /// <summary>
      /// fa-arrows-left-right unicode value ("\uf07e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-left-right
      /// </summary>
      [Description("arrows-left-right")]
      public const string ArrowsLeftRight = "\uf07e";

      /// <summary>
      /// fa-arrows-left-right-to-line unicode value ("\ue4ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-left-right-to-line
      /// </summary>
      [Description("arrows-left-right-to-line")]
      public const string ArrowsLeftRightToLine = "\ue4ba";

      /// <summary>
      /// fa-arrows-maximize unicode value ("\uf31d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-maximize
      /// </summary>
      [Description("arrows-maximize")]
      public const string ArrowsMaximize = "\uf31d";

      /// <summary>
      /// fa-arrows-minimize unicode value ("\ue0a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-minimize
      /// </summary>
      [Description("arrows-minimize")]
      public const string ArrowsMinimize = "\ue0a5";

      /// <summary>
      /// fa-arrows-repeat unicode value ("\uf364").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-repeat
      /// </summary>
      [Description("arrows-repeat")]
      public const string ArrowsRepeat = "\uf364";

      /// <summary>
      /// fa-arrows-repeat-1 unicode value ("\uf366").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-repeat-1
      /// </summary>
      [Description("arrows-repeat-1")]
      public const string ArrowsRepeat1 = "\uf366";

      /// <summary>
      /// fa-arrows-retweet unicode value ("\uf361").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-retweet
      /// </summary>
      [Description("arrows-retweet")]
      public const string ArrowsRetweet = "\uf361";

      /// <summary>
      /// fa-arrows-rotate unicode value ("\uf021").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-rotate
      /// </summary>
      [Description("arrows-rotate")]
      public const string ArrowsRotate = "\uf021";

      /// <summary>
      /// fa-arrows-spin unicode value ("\ue4bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-spin
      /// </summary>
      [Description("arrows-spin")]
      public const string ArrowsSpin = "\ue4bb";

      /// <summary>
      /// fa-arrows-split-up-and-left unicode value ("\ue4bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-split-up-and-left
      /// </summary>
      [Description("arrows-split-up-and-left")]
      public const string ArrowsSplitUpAndLeft = "\ue4bc";

      /// <summary>
      /// fa-arrows-to-circle unicode value ("\ue4bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-to-circle
      /// </summary>
      [Description("arrows-to-circle")]
      public const string ArrowsToCircle = "\ue4bd";

      /// <summary>
      /// fa-arrows-to-dot unicode value ("\ue4be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-to-dot
      /// </summary>
      [Description("arrows-to-dot")]
      public const string ArrowsToDot = "\ue4be";

      /// <summary>
      /// fa-arrows-to-dotted-line unicode value ("\ue0a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-to-dotted-line
      /// </summary>
      [Description("arrows-to-dotted-line")]
      public const string ArrowsToDottedLine = "\ue0a6";

      /// <summary>
      /// fa-arrows-to-eye unicode value ("\ue4bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-to-eye
      /// </summary>
      [Description("arrows-to-eye")]
      public const string ArrowsToEye = "\ue4bf";

      /// <summary>
      /// fa-arrows-to-line unicode value ("\ue0a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-to-line
      /// </summary>
      [Description("arrows-to-line")]
      public const string ArrowsToLine = "\ue0a7";

      /// <summary>
      /// fa-arrows-turn-right unicode value ("\ue4c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-turn-right
      /// </summary>
      [Description("arrows-turn-right")]
      public const string ArrowsTurnRight = "\ue4c0";

      /// <summary>
      /// fa-arrows-turn-to-dots unicode value ("\ue4c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-turn-to-dots
      /// </summary>
      [Description("arrows-turn-to-dots")]
      public const string ArrowsTurnToDots = "\ue4c1";

      /// <summary>
      /// fa-arrows-up-down unicode value ("\uf07d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-up-down
      /// </summary>
      [Description("arrows-up-down")]
      public const string ArrowsUpDown = "\uf07d";

      /// <summary>
      /// fa-arrows-up-down-left-right unicode value ("\uf047").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-up-down-left-right
      /// </summary>
      [Description("arrows-up-down-left-right")]
      public const string ArrowsUpDownLeftRight = "\uf047";

      /// <summary>
      /// fa-arrows-up-to-line unicode value ("\ue4c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/arrows-up-to-line
      /// </summary>
      [Description("arrows-up-to-line")]
      public const string ArrowsUpToLine = "\ue4c2";

      /// <summary>
      /// fa-artstation unicode value ("\uf77a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/artstation
      /// </summary>
      [Description("artstation")]
      public const string Artstation = "\uf77a";

      /// <summary>
      /// fa-asterisk unicode value ("\u002a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/asterisk
      /// </summary>
      [Description("asterisk")]
      public const string Asterisk = "\u002a";

      /// <summary>
      /// fa-asymmetrik unicode value ("\uf372").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/asymmetrik
      /// </summary>
      [Description("asymmetrik")]
      public const string Asymmetrik = "\uf372";

      /// <summary>
      /// fa-at unicode value ("\u0040").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/at
      /// </summary>
      [Description("at")]
      public const string At = "\u0040";

      /// <summary>
      /// fa-atlassian unicode value ("\uf77b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/atlassian
      /// </summary>
      [Description("atlassian")]
      public const string Atlassian = "\uf77b";

      /// <summary>
      /// fa-atom unicode value ("\uf5d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/atom
      /// </summary>
      [Description("atom")]
      public const string Atom = "\uf5d2";

      /// <summary>
      /// fa-atom-simple unicode value ("\uf5d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/atom-simple
      /// </summary>
      [Description("atom-simple")]
      public const string AtomSimple = "\uf5d3";

      /// <summary>
      /// fa-audible unicode value ("\uf373").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/audible
      /// </summary>
      [Description("audible")]
      public const string Audible = "\uf373";

      /// <summary>
      /// fa-audio-description unicode value ("\uf29e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/audio-description
      /// </summary>
      [Description("audio-description")]
      public const string AudioDescription = "\uf29e";

      /// <summary>
      /// fa-audio-description-slash unicode value ("\ue0a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/audio-description-slash
      /// </summary>
      [Description("audio-description-slash")]
      public const string AudioDescriptionSlash = "\ue0a8";

      /// <summary>
      /// fa-austral-sign unicode value ("\ue0a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/austral-sign
      /// </summary>
      [Description("austral-sign")]
      public const string AustralSign = "\ue0a9";

      /// <summary>
      /// fa-autoprefixer unicode value ("\uf41c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/autoprefixer
      /// </summary>
      [Description("autoprefixer")]
      public const string Autoprefixer = "\uf41c";

      /// <summary>
      /// fa-avianex unicode value ("\uf374").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/avianex
      /// </summary>
      [Description("avianex")]
      public const string Avianex = "\uf374";

      /// <summary>
      /// fa-aviato unicode value ("\uf421").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/aviato
      /// </summary>
      [Description("aviato")]
      public const string Aviato = "\uf421";

      /// <summary>
      /// fa-avocado unicode value ("\ue0aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/avocado
      /// </summary>
      [Description("avocado")]
      public const string Avocado = "\ue0aa";

      /// <summary>
      /// fa-award unicode value ("\uf559").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/award
      /// </summary>
      [Description("award")]
      public const string Award = "\uf559";

      /// <summary>
      /// fa-award-simple unicode value ("\ue0ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/award-simple
      /// </summary>
      [Description("award-simple")]
      public const string AwardSimple = "\ue0ab";

      /// <summary>
      /// fa-aws unicode value ("\uf375").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/aws
      /// </summary>
      [Description("aws")]
      public const string Aws = "\uf375";

      /// <summary>
      /// fa-axe unicode value ("\uf6b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/axe
      /// </summary>
      [Description("axe")]
      public const string Axe = "\uf6b2";

      /// <summary>
      /// fa-axe-battle unicode value ("\uf6b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/axe-battle
      /// </summary>
      [Description("axe-battle")]
      public const string AxeBattle = "\uf6b3";

      /// <summary>
      /// fa-b unicode value ("\u0042").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/b
      /// </summary>
      [Description("b")]
      public const string B = "\u0042";

      /// <summary>
      /// fa-baby unicode value ("\uf77c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baby
      /// </summary>
      [Description("baby")]
      public const string Baby = "\uf77c";

      /// <summary>
      /// fa-baby-carriage unicode value ("\uf77d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baby-carriage
      /// </summary>
      [Description("baby-carriage")]
      public const string BabyCarriage = "\uf77d";

      /// <summary>
      /// fa-backpack unicode value ("\uf5d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/backpack
      /// </summary>
      [Description("backpack")]
      public const string Backpack = "\uf5d4";

      /// <summary>
      /// fa-backward unicode value ("\uf04a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/backward
      /// </summary>
      [Description("backward")]
      public const string Backward = "\uf04a";

      /// <summary>
      /// fa-backward-fast unicode value ("\uf049").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/backward-fast
      /// </summary>
      [Description("backward-fast")]
      public const string BackwardFast = "\uf049";

      /// <summary>
      /// fa-backward-step unicode value ("\uf048").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/backward-step
      /// </summary>
      [Description("backward-step")]
      public const string BackwardStep = "\uf048";

      /// <summary>
      /// fa-bacon unicode value ("\uf7e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bacon
      /// </summary>
      [Description("bacon")]
      public const string Bacon = "\uf7e5";

      /// <summary>
      /// fa-bacteria unicode value ("\ue059").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bacteria
      /// </summary>
      [Description("bacteria")]
      public const string Bacteria = "\ue059";

      /// <summary>
      /// fa-bacterium unicode value ("\ue05a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bacterium
      /// </summary>
      [Description("bacterium")]
      public const string Bacterium = "\ue05a";

      /// <summary>
      /// fa-badge unicode value ("\uf335").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badge
      /// </summary>
      [Description("badge")]
      public const string Badge = "\uf335";

      /// <summary>
      /// fa-badge-check unicode value ("\uf336").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badge-check
      /// </summary>
      [Description("badge-check")]
      public const string BadgeCheck = "\uf336";

      /// <summary>
      /// fa-badge-dollar unicode value ("\uf645").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badge-dollar
      /// </summary>
      [Description("badge-dollar")]
      public const string BadgeDollar = "\uf645";

      /// <summary>
      /// fa-badge-percent unicode value ("\uf646").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badge-percent
      /// </summary>
      [Description("badge-percent")]
      public const string BadgePercent = "\uf646";

      /// <summary>
      /// fa-badge-sheriff unicode value ("\uf8a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badge-sheriff
      /// </summary>
      [Description("badge-sheriff")]
      public const string BadgeSheriff = "\uf8a2";

      /// <summary>
      /// fa-badger-honey unicode value ("\uf6b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badger-honey
      /// </summary>
      [Description("badger-honey")]
      public const string BadgerHoney = "\uf6b4";

      /// <summary>
      /// fa-badminton unicode value ("\ue33a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/badminton
      /// </summary>
      [Description("badminton")]
      public const string Badminton = "\ue33a";

      /// <summary>
      /// fa-bag-seedling unicode value ("\ue5f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bag-seedling
      /// </summary>
      [Description("bag-seedling")]
      public const string BagSeedling = "\ue5f2";

      /// <summary>
      /// fa-bag-shopping unicode value ("\uf290").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bag-shopping
      /// </summary>
      [Description("bag-shopping")]
      public const string BagShopping = "\uf290";

      /// <summary>
      /// fa-bagel unicode value ("\ue3d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bagel
      /// </summary>
      [Description("bagel")]
      public const string Bagel = "\ue3d7";

      /// <summary>
      /// fa-bags-shopping unicode value ("\uf847").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bags-shopping
      /// </summary>
      [Description("bags-shopping")]
      public const string BagsShopping = "\uf847";

      /// <summary>
      /// fa-baguette unicode value ("\ue3d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baguette
      /// </summary>
      [Description("baguette")]
      public const string Baguette = "\ue3d8";

      /// <summary>
      /// fa-bahai unicode value ("\uf666").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bahai
      /// </summary>
      [Description("bahai")]
      public const string Bahai = "\uf666";

      /// <summary>
      /// fa-baht-sign unicode value ("\ue0ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baht-sign
      /// </summary>
      [Description("baht-sign")]
      public const string BahtSign = "\ue0ac";

      /// <summary>
      /// fa-ball-pile unicode value ("\uf77e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ball-pile
      /// </summary>
      [Description("ball-pile")]
      public const string BallPile = "\uf77e";

      /// <summary>
      /// fa-balloon unicode value ("\ue2e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/balloon
      /// </summary>
      [Description("balloon")]
      public const string Balloon = "\ue2e3";

      /// <summary>
      /// fa-balloons unicode value ("\ue2e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/balloons
      /// </summary>
      [Description("balloons")]
      public const string Balloons = "\ue2e4";

      /// <summary>
      /// fa-ballot unicode value ("\uf732").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ballot
      /// </summary>
      [Description("ballot")]
      public const string Ballot = "\uf732";

      /// <summary>
      /// fa-ballot-check unicode value ("\uf733").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ballot-check
      /// </summary>
      [Description("ballot-check")]
      public const string BallotCheck = "\uf733";

      /// <summary>
      /// fa-ban unicode value ("\uf05e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ban
      /// </summary>
      [Description("ban")]
      public const string Ban = "\uf05e";

      /// <summary>
      /// fa-ban-bug unicode value ("\uf7f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ban-bug
      /// </summary>
      [Description("ban-bug")]
      public const string BanBug = "\uf7f9";

      /// <summary>
      /// fa-ban-parking unicode value ("\uf616").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ban-parking
      /// </summary>
      [Description("ban-parking")]
      public const string BanParking = "\uf616";

      /// <summary>
      /// fa-ban-smoking unicode value ("\uf54d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ban-smoking
      /// </summary>
      [Description("ban-smoking")]
      public const string BanSmoking = "\uf54d";

      /// <summary>
      /// fa-banana unicode value ("\ue2e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/banana
      /// </summary>
      [Description("banana")]
      public const string Banana = "\ue2e5";

      /// <summary>
      /// fa-bandage unicode value ("\uf462").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bandage
      /// </summary>
      [Description("bandage")]
      public const string Bandage = "\uf462";

      /// <summary>
      /// fa-bandcamp unicode value ("\uf2d5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.13.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/bandcamp
      /// </summary>
      [Description("bandcamp")]
      public const string Bandcamp = "\uf2d5";

      /// <summary>
      /// fa-bangladeshi-taka-sign unicode value ("\ue2e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bangladeshi-taka-sign
      /// </summary>
      [Description("bangladeshi-taka-sign")]
      public const string BangladeshiTakaSign = "\ue2e6";

      /// <summary>
      /// fa-banjo unicode value ("\uf8a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/banjo
      /// </summary>
      [Description("banjo")]
      public const string Banjo = "\uf8a3";

      /// <summary>
      /// fa-barcode unicode value ("\uf02a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/barcode
      /// </summary>
      [Description("barcode")]
      public const string Barcode = "\uf02a";

      /// <summary>
      /// fa-barcode-read unicode value ("\uf464").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/barcode-read
      /// </summary>
      [Description("barcode-read")]
      public const string BarcodeRead = "\uf464";

      /// <summary>
      /// fa-barcode-scan unicode value ("\uf465").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/barcode-scan
      /// </summary>
      [Description("barcode-scan")]
      public const string BarcodeScan = "\uf465";

      /// <summary>
      /// fa-bars unicode value ("\uf0c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bars
      /// </summary>
      [Description("bars")]
      public const string Bars = "\uf0c9";

      /// <summary>
      /// fa-bars-filter unicode value ("\ue0ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bars-filter
      /// </summary>
      [Description("bars-filter")]
      public const string BarsFilter = "\ue0ad";

      /// <summary>
      /// fa-bars-progress unicode value ("\uf828").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bars-progress
      /// </summary>
      [Description("bars-progress")]
      public const string BarsProgress = "\uf828";

      /// <summary>
      /// fa-bars-sort unicode value ("\ue0ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bars-sort
      /// </summary>
      [Description("bars-sort")]
      public const string BarsSort = "\ue0ae";

      /// <summary>
      /// fa-bars-staggered unicode value ("\uf550").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bars-staggered
      /// </summary>
      [Description("bars-staggered")]
      public const string BarsStaggered = "\uf550";

      /// <summary>
      /// fa-baseball unicode value ("\uf433").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baseball
      /// </summary>
      [Description("baseball")]
      public const string Baseball = "\uf433";

      /// <summary>
      /// fa-baseball-bat-ball unicode value ("\uf432").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/baseball-bat-ball
      /// </summary>
      [Description("baseball-bat-ball")]
      public const string BaseballBatBall = "\uf432";

      /// <summary>
      /// fa-basket-shopping unicode value ("\uf291").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/basket-shopping
      /// </summary>
      [Description("basket-shopping")]
      public const string BasketShopping = "\uf291";

      /// <summary>
      /// fa-basket-shopping-simple unicode value ("\ue0af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/basket-shopping-simple
      /// </summary>
      [Description("basket-shopping-simple")]
      public const string BasketShoppingSimple = "\ue0af";

      /// <summary>
      /// fa-basketball unicode value ("\uf434").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/basketball
      /// </summary>
      [Description("basketball")]
      public const string Basketball = "\uf434";

      /// <summary>
      /// fa-basketball-hoop unicode value ("\uf435").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/basketball-hoop
      /// </summary>
      [Description("basketball-hoop")]
      public const string BasketballHoop = "\uf435";

      /// <summary>
      /// fa-bat unicode value ("\uf6b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bat
      /// </summary>
      [Description("bat")]
      public const string Bat = "\uf6b5";

      /// <summary>
      /// fa-bath unicode value ("\uf2cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bath
      /// </summary>
      [Description("bath")]
      public const string Bath = "\uf2cd";

      /// <summary>
      /// fa-battery-bolt unicode value ("\uf376").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-bolt
      /// </summary>
      [Description("battery-bolt")]
      public const string BatteryBolt = "\uf376";

      /// <summary>
      /// fa-battery-empty unicode value ("\uf244").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-empty
      /// </summary>
      [Description("battery-empty")]
      public const string BatteryEmpty = "\uf244";

      /// <summary>
      /// fa-battery-exclamation unicode value ("\ue0b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-exclamation
      /// </summary>
      [Description("battery-exclamation")]
      public const string BatteryExclamation = "\ue0b0";

      /// <summary>
      /// fa-battery-full unicode value ("\uf240").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-full
      /// </summary>
      [Description("battery-full")]
      public const string BatteryFull = "\uf240";

      /// <summary>
      /// fa-battery-half unicode value ("\uf242").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-half
      /// </summary>
      [Description("battery-half")]
      public const string BatteryHalf = "\uf242";

      /// <summary>
      /// fa-battery-low unicode value ("\ue0b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-low
      /// </summary>
      [Description("battery-low")]
      public const string BatteryLow = "\ue0b1";

      /// <summary>
      /// fa-battery-quarter unicode value ("\uf243").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-quarter
      /// </summary>
      [Description("battery-quarter")]
      public const string BatteryQuarter = "\uf243";

      /// <summary>
      /// fa-battery-slash unicode value ("\uf377").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-slash
      /// </summary>
      [Description("battery-slash")]
      public const string BatterySlash = "\uf377";

      /// <summary>
      /// fa-battery-three-quarters unicode value ("\uf241").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battery-three-quarters
      /// </summary>
      [Description("battery-three-quarters")]
      public const string BatteryThreeQuarters = "\uf241";

      /// <summary>
      /// fa-battle-net unicode value ("\uf835").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/battle-net
      /// </summary>
      [Description("battle-net")]
      public const string BattleNet = "\uf835";

      /// <summary>
      /// fa-bed unicode value ("\uf236").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bed
      /// </summary>
      [Description("bed")]
      public const string Bed = "\uf236";

      /// <summary>
      /// fa-bed-bunk unicode value ("\uf8f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bed-bunk
      /// </summary>
      [Description("bed-bunk")]
      public const string BedBunk = "\uf8f8";

      /// <summary>
      /// fa-bed-empty unicode value ("\uf8f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bed-empty
      /// </summary>
      [Description("bed-empty")]
      public const string BedEmpty = "\uf8f9";

      /// <summary>
      /// fa-bed-front unicode value ("\uf8f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bed-front
      /// </summary>
      [Description("bed-front")]
      public const string BedFront = "\uf8f7";

      /// <summary>
      /// fa-bed-pulse unicode value ("\uf487").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bed-pulse
      /// </summary>
      [Description("bed-pulse")]
      public const string BedPulse = "\uf487";

      /// <summary>
      /// fa-bee unicode value ("\ue0b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bee
      /// </summary>
      [Description("bee")]
      public const string Bee = "\ue0b2";

      /// <summary>
      /// fa-beer-mug unicode value ("\ue0b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/beer-mug
      /// </summary>
      [Description("beer-mug")]
      public const string BeerMug = "\ue0b3";

      /// <summary>
      /// fa-beer-mug-empty unicode value ("\uf0fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/beer-mug-empty
      /// </summary>
      [Description("beer-mug-empty")]
      public const string BeerMugEmpty = "\uf0fc";

      /// <summary>
      /// fa-behance unicode value ("\uf1b4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/behance
      /// </summary>
      [Description("behance")]
      public const string Behance = "\uf1b4";

      /// <summary>
      /// fa-bell unicode value ("\uf0f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell
      /// </summary>
      [Description("bell")]
      public const string Bell = "\uf0f3";

      /// <summary>
      /// fa-bell-concierge unicode value ("\uf562").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-concierge
      /// </summary>
      [Description("bell-concierge")]
      public const string BellConcierge = "\uf562";

      /// <summary>
      /// fa-bell-exclamation unicode value ("\uf848").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-exclamation
      /// </summary>
      [Description("bell-exclamation")]
      public const string BellExclamation = "\uf848";

      /// <summary>
      /// fa-bell-on unicode value ("\uf8fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-on
      /// </summary>
      [Description("bell-on")]
      public const string BellOn = "\uf8fa";

      /// <summary>
      /// fa-bell-plus unicode value ("\uf849").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-plus
      /// </summary>
      [Description("bell-plus")]
      public const string BellPlus = "\uf849";

      /// <summary>
      /// fa-bell-school unicode value ("\uf5d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-school
      /// </summary>
      [Description("bell-school")]
      public const string BellSchool = "\uf5d5";

      /// <summary>
      /// fa-bell-school-slash unicode value ("\uf5d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-school-slash
      /// </summary>
      [Description("bell-school-slash")]
      public const string BellSchoolSlash = "\uf5d6";

      /// <summary>
      /// fa-bell-slash unicode value ("\uf1f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bell-slash
      /// </summary>
      [Description("bell-slash")]
      public const string BellSlash = "\uf1f6";

      /// <summary>
      /// fa-bells unicode value ("\uf77f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bells
      /// </summary>
      [Description("bells")]
      public const string Bells = "\uf77f";

      /// <summary>
      /// fa-bench-tree unicode value ("\ue2e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bench-tree
      /// </summary>
      [Description("bench-tree")]
      public const string BenchTree = "\ue2e7";

      /// <summary>
      /// fa-bezier-curve unicode value ("\uf55b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bezier-curve
      /// </summary>
      [Description("bezier-curve")]
      public const string BezierCurve = "\uf55b";

      /// <summary>
      /// fa-bicycle unicode value ("\uf206").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bicycle
      /// </summary>
      [Description("bicycle")]
      public const string Bicycle = "\uf206";

      /// <summary>
      /// fa-bilibili unicode value ("\ue3d9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/bilibili
      /// </summary>
      [Description("bilibili")]
      public const string Bilibili = "\ue3d9";

      /// <summary>
      /// fa-billboard unicode value ("\ue5cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/billboard
      /// </summary>
      [Description("billboard")]
      public const string Billboard = "\ue5cd";

      /// <summary>
      /// fa-bimobject unicode value ("\uf378").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bimobject
      /// </summary>
      [Description("bimobject")]
      public const string Bimobject = "\uf378";

      /// <summary>
      /// fa-bin-bottles unicode value ("\ue5f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bin-bottles
      /// </summary>
      [Description("bin-bottles")]
      public const string BinBottles = "\ue5f5";

      /// <summary>
      /// fa-bin-bottles-recycle unicode value ("\ue5f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bin-bottles-recycle
      /// </summary>
      [Description("bin-bottles-recycle")]
      public const string BinBottlesRecycle = "\ue5f6";

      /// <summary>
      /// fa-bin-recycle unicode value ("\ue5f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bin-recycle
      /// </summary>
      [Description("bin-recycle")]
      public const string BinRecycle = "\ue5f7";

      /// <summary>
      /// fa-binary unicode value ("\ue33b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/binary
      /// </summary>
      [Description("binary")]
      public const string Binary = "\ue33b";

      /// <summary>
      /// fa-binary-circle-check unicode value ("\ue33c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/binary-circle-check
      /// </summary>
      [Description("binary-circle-check")]
      public const string BinaryCircleCheck = "\ue33c";

      /// <summary>
      /// fa-binary-lock unicode value ("\ue33d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/binary-lock
      /// </summary>
      [Description("binary-lock")]
      public const string BinaryLock = "\ue33d";

      /// <summary>
      /// fa-binary-slash unicode value ("\ue33e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/binary-slash
      /// </summary>
      [Description("binary-slash")]
      public const string BinarySlash = "\ue33e";

      /// <summary>
      /// fa-binoculars unicode value ("\uf1e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/binoculars
      /// </summary>
      [Description("binoculars")]
      public const string Binoculars = "\uf1e5";

      /// <summary>
      /// fa-biohazard unicode value ("\uf780").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/biohazard
      /// </summary>
      [Description("biohazard")]
      public const string Biohazard = "\uf780";

      /// <summary>
      /// fa-bird unicode value ("\ue469").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bird
      /// </summary>
      [Description("bird")]
      public const string Bird = "\ue469";

      /// <summary>
      /// fa-bitbucket unicode value ("\uf171").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bitbucket
      /// </summary>
      [Description("bitbucket")]
      public const string Bitbucket = "\uf171";

      /// <summary>
      /// fa-bitcoin unicode value ("\uf379").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bitcoin
      /// </summary>
      [Description("bitcoin")]
      public const string Bitcoin = "\uf379";

      /// <summary>
      /// fa-bitcoin-sign unicode value ("\ue0b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bitcoin-sign
      /// </summary>
      [Description("bitcoin-sign")]
      public const string BitcoinSign = "\ue0b4";

      /// <summary>
      /// fa-bity unicode value ("\uf37a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bity
      /// </summary>
      [Description("bity")]
      public const string Bity = "\uf37a";

      /// <summary>
      /// fa-black-tie unicode value ("\uf27e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/black-tie
      /// </summary>
      [Description("black-tie")]
      public const string BlackTie = "\uf27e";

      /// <summary>
      /// fa-blackberry unicode value ("\uf37b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blackberry
      /// </summary>
      [Description("blackberry")]
      public const string Blackberry = "\uf37b";

      /// <summary>
      /// fa-blanket unicode value ("\uf498").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blanket
      /// </summary>
      [Description("blanket")]
      public const string Blanket = "\uf498";

      /// <summary>
      /// fa-blanket-fire unicode value ("\ue3da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blanket-fire
      /// </summary>
      [Description("blanket-fire")]
      public const string BlanketFire = "\ue3da";

      /// <summary>
      /// fa-blender unicode value ("\uf517").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blender
      /// </summary>
      [Description("blender")]
      public const string Blender = "\uf517";

      /// <summary>
      /// fa-blender-phone unicode value ("\uf6b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blender-phone
      /// </summary>
      [Description("blender-phone")]
      public const string BlenderPhone = "\uf6b6";

      /// <summary>
      /// fa-blinds unicode value ("\uf8fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blinds
      /// </summary>
      [Description("blinds")]
      public const string Blinds = "\uf8fb";

      /// <summary>
      /// fa-blinds-open unicode value ("\uf8fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blinds-open
      /// </summary>
      [Description("blinds-open")]
      public const string BlindsOpen = "\uf8fc";

      /// <summary>
      /// fa-blinds-raised unicode value ("\uf8fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blinds-raised
      /// </summary>
      [Description("blinds-raised")]
      public const string BlindsRaised = "\uf8fd";

      /// <summary>
      /// fa-block unicode value ("\ue46a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/block
      /// </summary>
      [Description("block")]
      public const string Block = "\ue46a";

      /// <summary>
      /// fa-block-brick unicode value ("\ue3db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/block-brick
      /// </summary>
      [Description("block-brick")]
      public const string BlockBrick = "\ue3db";

      /// <summary>
      /// fa-block-brick-fire unicode value ("\ue3dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/block-brick-fire
      /// </summary>
      [Description("block-brick-fire")]
      public const string BlockBrickFire = "\ue3dc";

      /// <summary>
      /// fa-block-question unicode value ("\ue3dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/block-question
      /// </summary>
      [Description("block-question")]
      public const string BlockQuestion = "\ue3dd";

      /// <summary>
      /// fa-block-quote unicode value ("\ue0b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/block-quote
      /// </summary>
      [Description("block-quote")]
      public const string BlockQuote = "\ue0b5";

      /// <summary>
      /// fa-blog unicode value ("\uf781").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blog
      /// </summary>
      [Description("blog")]
      public const string Blog = "\uf781";

      /// <summary>
      /// fa-blogger unicode value ("\uf37c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blogger
      /// </summary>
      [Description("blogger")]
      public const string Blogger = "\uf37c";

      /// <summary>
      /// fa-blogger-b unicode value ("\uf37d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blogger-b
      /// </summary>
      [Description("blogger-b")]
      public const string BloggerB = "\uf37d";

      /// <summary>
      /// fa-blueberries unicode value ("\ue2e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/blueberries
      /// </summary>
      [Description("blueberries")]
      public const string Blueberries = "\ue2e8";

      /// <summary>
      /// fa-bluetooth unicode value ("\uf293").
      /// <para/>
      /// This icon supports the following styles: Brands, Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bluetooth
      /// </summary>
      [Description("bluetooth")]
      public const string Bluetooth = "\uf293";

      /// <summary>
      /// fa-bluetooth-b unicode value ("\uf294").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bluetooth-b
      /// </summary>
      [Description("bluetooth-b")]
      public const string BluetoothB = "\uf294";

      /// <summary>
      /// fa-bold unicode value ("\uf032").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bold
      /// </summary>
      [Description("bold")]
      public const string Bold = "\uf032";

      /// <summary>
      /// fa-bolt unicode value ("\uf0e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bolt
      /// </summary>
      [Description("bolt")]
      public const string Bolt = "\uf0e7";

      /// <summary>
      /// fa-bolt-auto unicode value ("\ue0b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bolt-auto
      /// </summary>
      [Description("bolt-auto")]
      public const string BoltAuto = "\ue0b6";

      /// <summary>
      /// fa-bolt-lightning unicode value ("\ue0b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bolt-lightning
      /// </summary>
      [Description("bolt-lightning")]
      public const string BoltLightning = "\ue0b7";

      /// <summary>
      /// fa-bolt-slash unicode value ("\ue0b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bolt-slash
      /// </summary>
      [Description("bolt-slash")]
      public const string BoltSlash = "\ue0b8";

      /// <summary>
      /// fa-bomb unicode value ("\uf1e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bomb
      /// </summary>
      [Description("bomb")]
      public const string Bomb = "\uf1e2";

      /// <summary>
      /// fa-bone unicode value ("\uf5d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bone
      /// </summary>
      [Description("bone")]
      public const string Bone = "\uf5d7";

      /// <summary>
      /// fa-bone-break unicode value ("\uf5d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bone-break
      /// </summary>
      [Description("bone-break")]
      public const string BoneBreak = "\uf5d8";

      /// <summary>
      /// fa-bong unicode value ("\uf55c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bong
      /// </summary>
      [Description("bong")]
      public const string Bong = "\uf55c";

      /// <summary>
      /// fa-book unicode value ("\uf02d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book
      /// </summary>
      [Description("book")]
      public const string Book = "\uf02d";

      /// <summary>
      /// fa-book-arrow-right unicode value ("\ue0b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-arrow-right
      /// </summary>
      [Description("book-arrow-right")]
      public const string BookArrowRight = "\ue0b9";

      /// <summary>
      /// fa-book-arrow-up unicode value ("\ue0ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-arrow-up
      /// </summary>
      [Description("book-arrow-up")]
      public const string BookArrowUp = "\ue0ba";

      /// <summary>
      /// fa-book-atlas unicode value ("\uf558").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-atlas
      /// </summary>
      [Description("book-atlas")]
      public const string BookAtlas = "\uf558";

      /// <summary>
      /// fa-book-bible unicode value ("\uf647").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-bible
      /// </summary>
      [Description("book-bible")]
      public const string BookBible = "\uf647";

      /// <summary>
      /// fa-book-blank unicode value ("\uf5d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-blank
      /// </summary>
      [Description("book-blank")]
      public const string BookBlank = "\uf5d9";

      /// <summary>
      /// fa-book-bookmark unicode value ("\ue0bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-bookmark
      /// </summary>
      [Description("book-bookmark")]
      public const string BookBookmark = "\ue0bb";

      /// <summary>
      /// fa-book-circle-arrow-right unicode value ("\ue0bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-circle-arrow-right
      /// </summary>
      [Description("book-circle-arrow-right")]
      public const string BookCircleArrowRight = "\ue0bc";

      /// <summary>
      /// fa-book-circle-arrow-up unicode value ("\ue0bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-circle-arrow-up
      /// </summary>
      [Description("book-circle-arrow-up")]
      public const string BookCircleArrowUp = "\ue0bd";

      /// <summary>
      /// fa-book-copy unicode value ("\ue0be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-copy
      /// </summary>
      [Description("book-copy")]
      public const string BookCopy = "\ue0be";

      /// <summary>
      /// fa-book-font unicode value ("\ue0bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-font
      /// </summary>
      [Description("book-font")]
      public const string BookFont = "\ue0bf";

      /// <summary>
      /// fa-book-heart unicode value ("\uf499").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-heart
      /// </summary>
      [Description("book-heart")]
      public const string BookHeart = "\uf499";

      /// <summary>
      /// fa-book-journal-whills unicode value ("\uf66a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-journal-whills
      /// </summary>
      [Description("book-journal-whills")]
      public const string BookJournalWhills = "\uf66a";

      /// <summary>
      /// fa-book-medical unicode value ("\uf7e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-medical
      /// </summary>
      [Description("book-medical")]
      public const string BookMedical = "\uf7e6";

      /// <summary>
      /// fa-book-open unicode value ("\uf518").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-open
      /// </summary>
      [Description("book-open")]
      public const string BookOpen = "\uf518";

      /// <summary>
      /// fa-book-open-cover unicode value ("\ue0c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-open-cover
      /// </summary>
      [Description("book-open-cover")]
      public const string BookOpenCover = "\ue0c0";

      /// <summary>
      /// fa-book-open-reader unicode value ("\uf5da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-open-reader
      /// </summary>
      [Description("book-open-reader")]
      public const string BookOpenReader = "\uf5da";

      /// <summary>
      /// fa-book-quran unicode value ("\uf687").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-quran
      /// </summary>
      [Description("book-quran")]
      public const string BookQuran = "\uf687";

      /// <summary>
      /// fa-book-section unicode value ("\ue0c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-section
      /// </summary>
      [Description("book-section")]
      public const string BookSection = "\ue0c1";

      /// <summary>
      /// fa-book-skull unicode value ("\uf6b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-skull
      /// </summary>
      [Description("book-skull")]
      public const string BookSkull = "\uf6b7";

      /// <summary>
      /// fa-book-sparkles unicode value ("\uf6b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-sparkles
      /// </summary>
      [Description("book-sparkles")]
      public const string BookSparkles = "\uf6b8";

      /// <summary>
      /// fa-book-tanakh unicode value ("\uf827").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-tanakh
      /// </summary>
      [Description("book-tanakh")]
      public const string BookTanakh = "\uf827";

      /// <summary>
      /// fa-book-user unicode value ("\uf7e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/book-user
      /// </summary>
      [Description("book-user")]
      public const string BookUser = "\uf7e7";

      /// <summary>
      /// fa-bookmark unicode value ("\uf02e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bookmark
      /// </summary>
      [Description("bookmark")]
      public const string Bookmark = "\uf02e";

      /// <summary>
      /// fa-bookmark-slash unicode value ("\ue0c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bookmark-slash
      /// </summary>
      [Description("bookmark-slash")]
      public const string BookmarkSlash = "\ue0c2";

      /// <summary>
      /// fa-books unicode value ("\uf5db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/books
      /// </summary>
      [Description("books")]
      public const string Books = "\uf5db";

      /// <summary>
      /// fa-books-medical unicode value ("\uf7e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/books-medical
      /// </summary>
      [Description("books-medical")]
      public const string BooksMedical = "\uf7e8";

      /// <summary>
      /// fa-boombox unicode value ("\uf8a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boombox
      /// </summary>
      [Description("boombox")]
      public const string Boombox = "\uf8a5";

      /// <summary>
      /// fa-boot unicode value ("\uf782").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boot
      /// </summary>
      [Description("boot")]
      public const string Boot = "\uf782";

      /// <summary>
      /// fa-boot-heeled unicode value ("\ue33f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boot-heeled
      /// </summary>
      [Description("boot-heeled")]
      public const string BootHeeled = "\ue33f";

      /// <summary>
      /// fa-booth-curtain unicode value ("\uf734").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/booth-curtain
      /// </summary>
      [Description("booth-curtain")]
      public const string BoothCurtain = "\uf734";

      /// <summary>
      /// fa-bootstrap unicode value ("\uf836").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/bootstrap
      /// </summary>
      [Description("bootstrap")]
      public const string Bootstrap = "\uf836";

      /// <summary>
      /// fa-border-all unicode value ("\uf84c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-all
      /// </summary>
      [Description("border-all")]
      public const string BorderAll = "\uf84c";

      /// <summary>
      /// fa-border-bottom unicode value ("\uf84d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-bottom
      /// </summary>
      [Description("border-bottom")]
      public const string BorderBottom = "\uf84d";

      /// <summary>
      /// fa-border-bottom-right unicode value ("\uf854").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-bottom-right
      /// </summary>
      [Description("border-bottom-right")]
      public const string BorderBottomRight = "\uf854";

      /// <summary>
      /// fa-border-center-h unicode value ("\uf89c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.10.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-center-h
      /// </summary>
      [Description("border-center-h")]
      public const string BorderCenterH = "\uf89c";

      /// <summary>
      /// fa-border-center-v unicode value ("\uf89d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.10.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-center-v
      /// </summary>
      [Description("border-center-v")]
      public const string BorderCenterV = "\uf89d";

      /// <summary>
      /// fa-border-inner unicode value ("\uf84e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-inner
      /// </summary>
      [Description("border-inner")]
      public const string BorderInner = "\uf84e";

      /// <summary>
      /// fa-border-left unicode value ("\uf84f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-left
      /// </summary>
      [Description("border-left")]
      public const string BorderLeft = "\uf84f";

      /// <summary>
      /// fa-border-none unicode value ("\uf850").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-none
      /// </summary>
      [Description("border-none")]
      public const string BorderNone = "\uf850";

      /// <summary>
      /// fa-border-outer unicode value ("\uf851").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-outer
      /// </summary>
      [Description("border-outer")]
      public const string BorderOuter = "\uf851";

      /// <summary>
      /// fa-border-right unicode value ("\uf852").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-right
      /// </summary>
      [Description("border-right")]
      public const string BorderRight = "\uf852";

      /// <summary>
      /// fa-border-top unicode value ("\uf855").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-top
      /// </summary>
      [Description("border-top")]
      public const string BorderTop = "\uf855";

      /// <summary>
      /// fa-border-top-left unicode value ("\uf853").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/border-top-left
      /// </summary>
      [Description("border-top-left")]
      public const string BorderTopLeft = "\uf853";

      /// <summary>
      /// fa-bore-hole unicode value ("\ue4c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bore-hole
      /// </summary>
      [Description("bore-hole")]
      public const string BoreHole = "\ue4c3";

      /// <summary>
      /// fa-bots unicode value ("\ue340").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/bots
      /// </summary>
      [Description("bots")]
      public const string Bots = "\ue340";

      /// <summary>
      /// fa-bottle-droplet unicode value ("\ue4c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bottle-droplet
      /// </summary>
      [Description("bottle-droplet")]
      public const string BottleDroplet = "\ue4c4";

      /// <summary>
      /// fa-bottle-water unicode value ("\ue4c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bottle-water
      /// </summary>
      [Description("bottle-water")]
      public const string BottleWater = "\ue4c5";

      /// <summary>
      /// fa-bow-arrow unicode value ("\uf6b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bow-arrow
      /// </summary>
      [Description("bow-arrow")]
      public const string BowArrow = "\uf6b9";

      /// <summary>
      /// fa-bowl-chopsticks unicode value ("\ue2e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-chopsticks
      /// </summary>
      [Description("bowl-chopsticks")]
      public const string BowlChopsticks = "\ue2e9";

      /// <summary>
      /// fa-bowl-chopsticks-noodles unicode value ("\ue2ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-chopsticks-noodles
      /// </summary>
      [Description("bowl-chopsticks-noodles")]
      public const string BowlChopsticksNoodles = "\ue2ea";

      /// <summary>
      /// fa-bowl-food unicode value ("\ue4c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-food
      /// </summary>
      [Description("bowl-food")]
      public const string BowlFood = "\ue4c6";

      /// <summary>
      /// fa-bowl-hot unicode value ("\uf823").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-hot
      /// </summary>
      [Description("bowl-hot")]
      public const string BowlHot = "\uf823";

      /// <summary>
      /// fa-bowl-rice unicode value ("\ue2eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-rice
      /// </summary>
      [Description("bowl-rice")]
      public const string BowlRice = "\ue2eb";

      /// <summary>
      /// fa-bowl-scoop unicode value ("\ue3de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-scoop
      /// </summary>
      [Description("bowl-scoop")]
      public const string BowlScoop = "\ue3de";

      /// <summary>
      /// fa-bowl-scoops unicode value ("\ue3df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-scoops
      /// </summary>
      [Description("bowl-scoops")]
      public const string BowlScoops = "\ue3df";

      /// <summary>
      /// fa-bowl-soft-serve unicode value ("\ue46b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-soft-serve
      /// </summary>
      [Description("bowl-soft-serve")]
      public const string BowlSoftServe = "\ue46b";

      /// <summary>
      /// fa-bowl-spoon unicode value ("\ue3e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowl-spoon
      /// </summary>
      [Description("bowl-spoon")]
      public const string BowlSpoon = "\ue3e0";

      /// <summary>
      /// fa-bowling-ball unicode value ("\uf436").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowling-ball
      /// </summary>
      [Description("bowling-ball")]
      public const string BowlingBall = "\uf436";

      /// <summary>
      /// fa-bowling-ball-pin unicode value ("\ue0c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowling-ball-pin
      /// </summary>
      [Description("bowling-ball-pin")]
      public const string BowlingBallPin = "\ue0c3";

      /// <summary>
      /// fa-bowling-pins unicode value ("\uf437").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bowling-pins
      /// </summary>
      [Description("bowling-pins")]
      public const string BowlingPins = "\uf437";

      /// <summary>
      /// fa-box unicode value ("\uf466").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box
      /// </summary>
      [Description("box")]
      public const string Box = "\uf466";

      /// <summary>
      /// fa-box-archive unicode value ("\uf187").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-archive
      /// </summary>
      [Description("box-archive")]
      public const string BoxArchive = "\uf187";

      /// <summary>
      /// fa-box-ballot unicode value ("\uf735").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-ballot
      /// </summary>
      [Description("box-ballot")]
      public const string BoxBallot = "\uf735";

      /// <summary>
      /// fa-box-check unicode value ("\uf467").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-check
      /// </summary>
      [Description("box-check")]
      public const string BoxCheck = "\uf467";

      /// <summary>
      /// fa-box-circle-check unicode value ("\ue0c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-circle-check
      /// </summary>
      [Description("box-circle-check")]
      public const string BoxCircleCheck = "\ue0c4";

      /// <summary>
      /// fa-box-dollar unicode value ("\uf4a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-dollar
      /// </summary>
      [Description("box-dollar")]
      public const string BoxDollar = "\uf4a0";

      /// <summary>
      /// fa-box-heart unicode value ("\uf49d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-heart
      /// </summary>
      [Description("box-heart")]
      public const string BoxHeart = "\uf49d";

      /// <summary>
      /// fa-box-open unicode value ("\uf49e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-open
      /// </summary>
      [Description("box-open")]
      public const string BoxOpen = "\uf49e";

      /// <summary>
      /// fa-box-open-full unicode value ("\uf49c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-open-full
      /// </summary>
      [Description("box-open-full")]
      public const string BoxOpenFull = "\uf49c";

      /// <summary>
      /// fa-box-taped unicode value ("\uf49a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-taped
      /// </summary>
      [Description("box-taped")]
      public const string BoxTaped = "\uf49a";

      /// <summary>
      /// fa-box-tissue unicode value ("\ue05b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/box-tissue
      /// </summary>
      [Description("box-tissue")]
      public const string BoxTissue = "\ue05b";

      /// <summary>
      /// fa-boxes-packing unicode value ("\ue4c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boxes-packing
      /// </summary>
      [Description("boxes-packing")]
      public const string BoxesPacking = "\ue4c7";

      /// <summary>
      /// fa-boxes-stacked unicode value ("\uf468").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boxes-stacked
      /// </summary>
      [Description("boxes-stacked")]
      public const string BoxesStacked = "\uf468";

      /// <summary>
      /// fa-boxing-glove unicode value ("\uf438").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/boxing-glove
      /// </summary>
      [Description("boxing-glove")]
      public const string BoxingGlove = "\uf438";

      /// <summary>
      /// fa-bracket-curly unicode value ("\u007b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-curly
      /// </summary>
      [Description("bracket-curly")]
      public const string BracketCurly = "\u007b";

      /// <summary>
      /// fa-bracket-curly-right unicode value ("\u007d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-curly-right
      /// </summary>
      [Description("bracket-curly-right")]
      public const string BracketCurlyRight = "\u007d";

      /// <summary>
      /// fa-bracket-round unicode value ("\u0028").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-round
      /// </summary>
      [Description("bracket-round")]
      public const string BracketRound = "\u0028";

      /// <summary>
      /// fa-bracket-round-right unicode value ("\u0029").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-round-right
      /// </summary>
      [Description("bracket-round-right")]
      public const string BracketRoundRight = "\u0029";

      /// <summary>
      /// fa-bracket-square unicode value ("\u005b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-square
      /// </summary>
      [Description("bracket-square")]
      public const string BracketSquare = "\u005b";

      /// <summary>
      /// fa-bracket-square-right unicode value ("\u005d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bracket-square-right
      /// </summary>
      [Description("bracket-square-right")]
      public const string BracketSquareRight = "\u005d";

      /// <summary>
      /// fa-brackets-curly unicode value ("\uf7ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brackets-curly
      /// </summary>
      [Description("brackets-curly")]
      public const string BracketsCurly = "\uf7ea";

      /// <summary>
      /// fa-brackets-round unicode value ("\ue0c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brackets-round
      /// </summary>
      [Description("brackets-round")]
      public const string BracketsRound = "\ue0c5";

      /// <summary>
      /// fa-brackets-square unicode value ("\uf7e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brackets-square
      /// </summary>
      [Description("brackets-square")]
      public const string BracketsSquare = "\uf7e9";

      /// <summary>
      /// fa-braille unicode value ("\uf2a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/braille
      /// </summary>
      [Description("braille")]
      public const string Braille = "\uf2a1";

      /// <summary>
      /// fa-brain unicode value ("\uf5dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brain
      /// </summary>
      [Description("brain")]
      public const string Brain = "\uf5dc";

      /// <summary>
      /// fa-brain-arrow-curved-right unicode value ("\uf677").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brain-arrow-curved-right
      /// </summary>
      [Description("brain-arrow-curved-right")]
      public const string BrainArrowCurvedRight = "\uf677";

      /// <summary>
      /// fa-brain-circuit unicode value ("\ue0c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brain-circuit
      /// </summary>
      [Description("brain-circuit")]
      public const string BrainCircuit = "\ue0c6";

      /// <summary>
      /// fa-brake-warning unicode value ("\ue0c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brake-warning
      /// </summary>
      [Description("brake-warning")]
      public const string BrakeWarning = "\ue0c7";

      /// <summary>
      /// fa-brazilian-real-sign unicode value ("\ue46c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brazilian-real-sign
      /// </summary>
      [Description("brazilian-real-sign")]
      public const string BrazilianRealSign = "\ue46c";

      /// <summary>
      /// fa-bread-loaf unicode value ("\uf7eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bread-loaf
      /// </summary>
      [Description("bread-loaf")]
      public const string BreadLoaf = "\uf7eb";

      /// <summary>
      /// fa-bread-slice unicode value ("\uf7ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bread-slice
      /// </summary>
      [Description("bread-slice")]
      public const string BreadSlice = "\uf7ec";

      /// <summary>
      /// fa-bread-slice-butter unicode value ("\ue3e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bread-slice-butter
      /// </summary>
      [Description("bread-slice-butter")]
      public const string BreadSliceButter = "\ue3e1";

      /// <summary>
      /// fa-bridge unicode value ("\ue4c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge
      /// </summary>
      [Description("bridge")]
      public const string Bridge = "\ue4c8";

      /// <summary>
      /// fa-bridge-circle-check unicode value ("\ue4c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-circle-check
      /// </summary>
      [Description("bridge-circle-check")]
      public const string BridgeCircleCheck = "\ue4c9";

      /// <summary>
      /// fa-bridge-circle-exclamation unicode value ("\ue4ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-circle-exclamation
      /// </summary>
      [Description("bridge-circle-exclamation")]
      public const string BridgeCircleExclamation = "\ue4ca";

      /// <summary>
      /// fa-bridge-circle-xmark unicode value ("\ue4cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-circle-xmark
      /// </summary>
      [Description("bridge-circle-xmark")]
      public const string BridgeCircleXmark = "\ue4cb";

      /// <summary>
      /// fa-bridge-lock unicode value ("\ue4cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-lock
      /// </summary>
      [Description("bridge-lock")]
      public const string BridgeLock = "\ue4cc";

      /// <summary>
      /// fa-bridge-suspension unicode value ("\ue4cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-suspension
      /// </summary>
      [Description("bridge-suspension")]
      public const string BridgeSuspension = "\ue4cd";

      /// <summary>
      /// fa-bridge-water unicode value ("\ue4ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bridge-water
      /// </summary>
      [Description("bridge-water")]
      public const string BridgeWater = "\ue4ce";

      /// <summary>
      /// fa-briefcase unicode value ("\uf0b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/briefcase
      /// </summary>
      [Description("briefcase")]
      public const string Briefcase = "\uf0b1";

      /// <summary>
      /// fa-briefcase-arrow-right unicode value ("\ue2f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/briefcase-arrow-right
      /// </summary>
      [Description("briefcase-arrow-right")]
      public const string BriefcaseArrowRight = "\ue2f2";

      /// <summary>
      /// fa-briefcase-blank unicode value ("\ue0c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/briefcase-blank
      /// </summary>
      [Description("briefcase-blank")]
      public const string BriefcaseBlank = "\ue0c8";

      /// <summary>
      /// fa-briefcase-medical unicode value ("\uf469").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/briefcase-medical
      /// </summary>
      [Description("briefcase-medical")]
      public const string BriefcaseMedical = "\uf469";

      /// <summary>
      /// fa-brightness unicode value ("\ue0c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brightness
      /// </summary>
      [Description("brightness")]
      public const string Brightness = "\ue0c9";

      /// <summary>
      /// fa-brightness-low unicode value ("\ue0ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brightness-low
      /// </summary>
      [Description("brightness-low")]
      public const string BrightnessLow = "\ue0ca";

      /// <summary>
      /// fa-bring-forward unicode value ("\uf856").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bring-forward
      /// </summary>
      [Description("bring-forward")]
      public const string BringForward = "\uf856";

      /// <summary>
      /// fa-bring-front unicode value ("\uf857").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bring-front
      /// </summary>
      [Description("bring-front")]
      public const string BringFront = "\uf857";

      /// <summary>
      /// fa-broccoli unicode value ("\ue3e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/broccoli
      /// </summary>
      [Description("broccoli")]
      public const string Broccoli = "\ue3e2";

      /// <summary>
      /// fa-broom unicode value ("\uf51a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/broom
      /// </summary>
      [Description("broom")]
      public const string Broom = "\uf51a";

      /// <summary>
      /// fa-broom-ball unicode value ("\uf458").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/broom-ball
      /// </summary>
      [Description("broom-ball")]
      public const string BroomBall = "\uf458";

      /// <summary>
      /// fa-broom-wide unicode value ("\ue5d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/broom-wide
      /// </summary>
      [Description("broom-wide")]
      public const string BroomWide = "\ue5d1";

      /// <summary>
      /// fa-browser unicode value ("\uf37e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/browser
      /// </summary>
      [Description("browser")]
      public const string Browser = "\uf37e";

      /// <summary>
      /// fa-browsers unicode value ("\ue0cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/browsers
      /// </summary>
      [Description("browsers")]
      public const string Browsers = "\ue0cb";

      /// <summary>
      /// fa-brush unicode value ("\uf55d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/brush
      /// </summary>
      [Description("brush")]
      public const string Brush = "\uf55d";

      /// <summary>
      /// fa-btc unicode value ("\uf15a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/btc
      /// </summary>
      [Description("btc")]
      public const string Btc = "\uf15a";

      /// <summary>
      /// fa-bucket unicode value ("\ue4cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bucket
      /// </summary>
      [Description("bucket")]
      public const string Bucket = "\ue4cf";

      /// <summary>
      /// fa-buffer unicode value ("\uf837").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buffer
      /// </summary>
      [Description("buffer")]
      public const string Buffer = "\uf837";

      /// <summary>
      /// fa-bug unicode value ("\uf188").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bug
      /// </summary>
      [Description("bug")]
      public const string Bug = "\uf188";

      /// <summary>
      /// fa-bug-slash unicode value ("\ue490").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bug-slash
      /// </summary>
      [Description("bug-slash")]
      public const string BugSlash = "\ue490";

      /// <summary>
      /// fa-bugs unicode value ("\ue4d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bugs
      /// </summary>
      [Description("bugs")]
      public const string Bugs = "\ue4d0";

      /// <summary>
      /// fa-building unicode value ("\uf1ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building
      /// </summary>
      [Description("building")]
      public const string Building = "\uf1ad";

      /// <summary>
      /// fa-building-circle-arrow-right unicode value ("\ue4d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-circle-arrow-right
      /// </summary>
      [Description("building-circle-arrow-right")]
      public const string BuildingCircleArrowRight = "\ue4d1";

      /// <summary>
      /// fa-building-circle-check unicode value ("\ue4d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-circle-check
      /// </summary>
      [Description("building-circle-check")]
      public const string BuildingCircleCheck = "\ue4d2";

      /// <summary>
      /// fa-building-circle-exclamation unicode value ("\ue4d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-circle-exclamation
      /// </summary>
      [Description("building-circle-exclamation")]
      public const string BuildingCircleExclamation = "\ue4d3";

      /// <summary>
      /// fa-building-circle-xmark unicode value ("\ue4d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-circle-xmark
      /// </summary>
      [Description("building-circle-xmark")]
      public const string BuildingCircleXmark = "\ue4d4";

      /// <summary>
      /// fa-building-columns unicode value ("\uf19c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-columns
      /// </summary>
      [Description("building-columns")]
      public const string BuildingColumns = "\uf19c";

      /// <summary>
      /// fa-building-flag unicode value ("\ue4d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-flag
      /// </summary>
      [Description("building-flag")]
      public const string BuildingFlag = "\ue4d5";

      /// <summary>
      /// fa-building-lock unicode value ("\ue4d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-lock
      /// </summary>
      [Description("building-lock")]
      public const string BuildingLock = "\ue4d6";

      /// <summary>
      /// fa-building-ngo unicode value ("\ue4d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-ngo
      /// </summary>
      [Description("building-ngo")]
      public const string BuildingNgo = "\ue4d7";

      /// <summary>
      /// fa-building-shield unicode value ("\ue4d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-shield
      /// </summary>
      [Description("building-shield")]
      public const string BuildingShield = "\ue4d8";

      /// <summary>
      /// fa-building-un unicode value ("\ue4d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-un
      /// </summary>
      [Description("building-un")]
      public const string BuildingUn = "\ue4d9";

      /// <summary>
      /// fa-building-user unicode value ("\ue4da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-user
      /// </summary>
      [Description("building-user")]
      public const string BuildingUser = "\ue4da";

      /// <summary>
      /// fa-building-wheat unicode value ("\ue4db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/building-wheat
      /// </summary>
      [Description("building-wheat")]
      public const string BuildingWheat = "\ue4db";

      /// <summary>
      /// fa-buildings unicode value ("\ue0cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buildings
      /// </summary>
      [Description("buildings")]
      public const string Buildings = "\ue0cc";

      /// <summary>
      /// fa-bullhorn unicode value ("\uf0a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bullhorn
      /// </summary>
      [Description("bullhorn")]
      public const string Bullhorn = "\uf0a1";

      /// <summary>
      /// fa-bullseye unicode value ("\uf140").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bullseye
      /// </summary>
      [Description("bullseye")]
      public const string Bullseye = "\uf140";

      /// <summary>
      /// fa-bullseye-arrow unicode value ("\uf648").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bullseye-arrow
      /// </summary>
      [Description("bullseye-arrow")]
      public const string BullseyeArrow = "\uf648";

      /// <summary>
      /// fa-bullseye-pointer unicode value ("\uf649").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bullseye-pointer
      /// </summary>
      [Description("bullseye-pointer")]
      public const string BullseyePointer = "\uf649";

      /// <summary>
      /// fa-buoy unicode value ("\ue5b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buoy
      /// </summary>
      [Description("buoy")]
      public const string Buoy = "\ue5b5";

      /// <summary>
      /// fa-buoy-mooring unicode value ("\ue5b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buoy-mooring
      /// </summary>
      [Description("buoy-mooring")]
      public const string BuoyMooring = "\ue5b6";

      /// <summary>
      /// fa-burger unicode value ("\uf805").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger
      /// </summary>
      [Description("burger")]
      public const string Burger = "\uf805";

      /// <summary>
      /// fa-burger-cheese unicode value ("\uf7f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger-cheese
      /// </summary>
      [Description("burger-cheese")]
      public const string BurgerCheese = "\uf7f1";

      /// <summary>
      /// fa-burger-fries unicode value ("\ue0cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger-fries
      /// </summary>
      [Description("burger-fries")]
      public const string BurgerFries = "\ue0cd";

      /// <summary>
      /// fa-burger-glass unicode value ("\ue0ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger-glass
      /// </summary>
      [Description("burger-glass")]
      public const string BurgerGlass = "\ue0ce";

      /// <summary>
      /// fa-burger-lettuce unicode value ("\ue3e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger-lettuce
      /// </summary>
      [Description("burger-lettuce")]
      public const string BurgerLettuce = "\ue3e3";

      /// <summary>
      /// fa-burger-soda unicode value ("\uf858").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burger-soda
      /// </summary>
      [Description("burger-soda")]
      public const string BurgerSoda = "\uf858";

      /// <summary>
      /// fa-buromobelexperte unicode value ("\uf37f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buromobelexperte
      /// </summary>
      [Description("buromobelexperte")]
      public const string Buromobelexperte = "\uf37f";

      /// <summary>
      /// fa-burrito unicode value ("\uf7ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burrito
      /// </summary>
      [Description("burrito")]
      public const string Burrito = "\uf7ed";

      /// <summary>
      /// fa-burst unicode value ("\ue4dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/burst
      /// </summary>
      [Description("burst")]
      public const string Burst = "\ue4dc";

      /// <summary>
      /// fa-bus unicode value ("\uf207").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bus
      /// </summary>
      [Description("bus")]
      public const string Bus = "\uf207";

      /// <summary>
      /// fa-bus-school unicode value ("\uf5dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bus-school
      /// </summary>
      [Description("bus-school")]
      public const string BusSchool = "\uf5dd";

      /// <summary>
      /// fa-bus-simple unicode value ("\uf55e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/bus-simple
      /// </summary>
      [Description("bus-simple")]
      public const string BusSimple = "\uf55e";

      /// <summary>
      /// fa-business-time unicode value ("\uf64a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/business-time
      /// </summary>
      [Description("business-time")]
      public const string BusinessTime = "\uf64a";

      /// <summary>
      /// fa-butter unicode value ("\ue3e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/butter
      /// </summary>
      [Description("butter")]
      public const string Butter = "\ue3e4";

      /// <summary>
      /// fa-buy-n-large unicode value ("\uf8a6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buy-n-large
      /// </summary>
      [Description("buy-n-large")]
      public const string BuyNLarge = "\uf8a6";

      /// <summary>
      /// fa-buysellads unicode value ("\uf20d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/buysellads
      /// </summary>
      [Description("buysellads")]
      public const string Buysellads = "\uf20d";

      /// <summary>
      /// fa-c unicode value ("\u0043").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/c
      /// </summary>
      [Description("c")]
      public const string C = "\u0043";

      /// <summary>
      /// fa-cabin unicode value ("\ue46d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cabin
      /// </summary>
      [Description("cabin")]
      public const string Cabin = "\ue46d";

      /// <summary>
      /// fa-cabinet-filing unicode value ("\uf64b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cabinet-filing
      /// </summary>
      [Description("cabinet-filing")]
      public const string CabinetFiling = "\uf64b";

      /// <summary>
      /// fa-cable-car unicode value ("\uf7da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cable-car
      /// </summary>
      [Description("cable-car")]
      public const string CableCar = "\uf7da";

      /// <summary>
      /// fa-cactus unicode value ("\uf8a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cactus
      /// </summary>
      [Description("cactus")]
      public const string Cactus = "\uf8a7";

      /// <summary>
      /// fa-cake-candles unicode value ("\uf1fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cake-candles
      /// </summary>
      [Description("cake-candles")]
      public const string CakeCandles = "\uf1fd";

      /// <summary>
      /// fa-cake-slice unicode value ("\ue3e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cake-slice
      /// </summary>
      [Description("cake-slice")]
      public const string CakeSlice = "\ue3e5";

      /// <summary>
      /// fa-calculator unicode value ("\uf1ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calculator
      /// </summary>
      [Description("calculator")]
      public const string Calculator = "\uf1ec";

      /// <summary>
      /// fa-calculator-simple unicode value ("\uf64c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calculator-simple
      /// </summary>
      [Description("calculator-simple")]
      public const string CalculatorSimple = "\uf64c";

      /// <summary>
      /// fa-calendar unicode value ("\uf133").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar
      /// </summary>
      [Description("calendar")]
      public const string Calendar = "\uf133";

      /// <summary>
      /// fa-calendar-arrow-down unicode value ("\ue0d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-arrow-down
      /// </summary>
      [Description("calendar-arrow-down")]
      public const string CalendarArrowDown = "\ue0d0";

      /// <summary>
      /// fa-calendar-arrow-up unicode value ("\ue0d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-arrow-up
      /// </summary>
      [Description("calendar-arrow-up")]
      public const string CalendarArrowUp = "\ue0d1";

      /// <summary>
      /// fa-calendar-check unicode value ("\uf274").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-check
      /// </summary>
      [Description("calendar-check")]
      public const string CalendarCheck = "\uf274";

      /// <summary>
      /// fa-calendar-circle-exclamation unicode value ("\ue46e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-circle-exclamation
      /// </summary>
      [Description("calendar-circle-exclamation")]
      public const string CalendarCircleExclamation = "\ue46e";

      /// <summary>
      /// fa-calendar-circle-minus unicode value ("\ue46f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-circle-minus
      /// </summary>
      [Description("calendar-circle-minus")]
      public const string CalendarCircleMinus = "\ue46f";

      /// <summary>
      /// fa-calendar-circle-plus unicode value ("\ue470").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-circle-plus
      /// </summary>
      [Description("calendar-circle-plus")]
      public const string CalendarCirclePlus = "\ue470";

      /// <summary>
      /// fa-calendar-circle-user unicode value ("\ue471").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-circle-user
      /// </summary>
      [Description("calendar-circle-user")]
      public const string CalendarCircleUser = "\ue471";

      /// <summary>
      /// fa-calendar-clock unicode value ("\ue0d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-clock
      /// </summary>
      [Description("calendar-clock")]
      public const string CalendarClock = "\ue0d2";

      /// <summary>
      /// fa-calendar-day unicode value ("\uf783").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-day
      /// </summary>
      [Description("calendar-day")]
      public const string CalendarDay = "\uf783";

      /// <summary>
      /// fa-calendar-days unicode value ("\uf073").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-days
      /// </summary>
      [Description("calendar-days")]
      public const string CalendarDays = "\uf073";

      /// <summary>
      /// fa-calendar-exclamation unicode value ("\uf334").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-exclamation
      /// </summary>
      [Description("calendar-exclamation")]
      public const string CalendarExclamation = "\uf334";

      /// <summary>
      /// fa-calendar-heart unicode value ("\ue0d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-heart
      /// </summary>
      [Description("calendar-heart")]
      public const string CalendarHeart = "\ue0d3";

      /// <summary>
      /// fa-calendar-image unicode value ("\ue0d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-image
      /// </summary>
      [Description("calendar-image")]
      public const string CalendarImage = "\ue0d4";

      /// <summary>
      /// fa-calendar-lines unicode value ("\ue0d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-lines
      /// </summary>
      [Description("calendar-lines")]
      public const string CalendarLines = "\ue0d5";

      /// <summary>
      /// fa-calendar-lines-pen unicode value ("\ue472").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-lines-pen
      /// </summary>
      [Description("calendar-lines-pen")]
      public const string CalendarLinesPen = "\ue472";

      /// <summary>
      /// fa-calendar-minus unicode value ("\uf272").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-minus
      /// </summary>
      [Description("calendar-minus")]
      public const string CalendarMinus = "\uf272";

      /// <summary>
      /// fa-calendar-pen unicode value ("\uf333").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-pen
      /// </summary>
      [Description("calendar-pen")]
      public const string CalendarPen = "\uf333";

      /// <summary>
      /// fa-calendar-plus unicode value ("\uf271").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-plus
      /// </summary>
      [Description("calendar-plus")]
      public const string CalendarPlus = "\uf271";

      /// <summary>
      /// fa-calendar-range unicode value ("\ue0d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-range
      /// </summary>
      [Description("calendar-range")]
      public const string CalendarRange = "\ue0d6";

      /// <summary>
      /// fa-calendar-star unicode value ("\uf736").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-star
      /// </summary>
      [Description("calendar-star")]
      public const string CalendarStar = "\uf736";

      /// <summary>
      /// fa-calendar-users unicode value ("\ue5e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-users
      /// </summary>
      [Description("calendar-users")]
      public const string CalendarUsers = "\ue5e2";

      /// <summary>
      /// fa-calendar-week unicode value ("\uf784").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-week
      /// </summary>
      [Description("calendar-week")]
      public const string CalendarWeek = "\uf784";

      /// <summary>
      /// fa-calendar-xmark unicode value ("\uf273").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendar-xmark
      /// </summary>
      [Description("calendar-xmark")]
      public const string CalendarXmark = "\uf273";

      /// <summary>
      /// fa-calendars unicode value ("\ue0d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/calendars
      /// </summary>
      [Description("calendars")]
      public const string Calendars = "\ue0d7";

      /// <summary>
      /// fa-camcorder unicode value ("\uf8a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camcorder
      /// </summary>
      [Description("camcorder")]
      public const string Camcorder = "\uf8a8";

      /// <summary>
      /// fa-camera unicode value ("\uf030").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera
      /// </summary>
      [Description("camera")]
      public const string Camera = "\uf030";

      /// <summary>
      /// fa-camera-cctv unicode value ("\uf8ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-cctv
      /// </summary>
      [Description("camera-cctv")]
      public const string CameraCctv = "\uf8ac";

      /// <summary>
      /// fa-camera-movie unicode value ("\uf8a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-movie
      /// </summary>
      [Description("camera-movie")]
      public const string CameraMovie = "\uf8a9";

      /// <summary>
      /// fa-camera-polaroid unicode value ("\uf8aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-polaroid
      /// </summary>
      [Description("camera-polaroid")]
      public const string CameraPolaroid = "\uf8aa";

      /// <summary>
      /// fa-camera-retro unicode value ("\uf083").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-retro
      /// </summary>
      [Description("camera-retro")]
      public const string CameraRetro = "\uf083";

      /// <summary>
      /// fa-camera-rotate unicode value ("\ue0d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-rotate
      /// </summary>
      [Description("camera-rotate")]
      public const string CameraRotate = "\ue0d8";

      /// <summary>
      /// fa-camera-security unicode value ("\uf8fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-security
      /// </summary>
      [Description("camera-security")]
      public const string CameraSecurity = "\uf8fe";

      /// <summary>
      /// fa-camera-slash unicode value ("\ue0d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-slash
      /// </summary>
      [Description("camera-slash")]
      public const string CameraSlash = "\ue0d9";

      /// <summary>
      /// fa-camera-viewfinder unicode value ("\ue0da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-viewfinder
      /// </summary>
      [Description("camera-viewfinder")]
      public const string CameraViewfinder = "\ue0da";

      /// <summary>
      /// fa-camera-web unicode value ("\uf832").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-web
      /// </summary>
      [Description("camera-web")]
      public const string CameraWeb = "\uf832";

      /// <summary>
      /// fa-camera-web-slash unicode value ("\uf833").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/camera-web-slash
      /// </summary>
      [Description("camera-web-slash")]
      public const string CameraWebSlash = "\uf833";

      /// <summary>
      /// fa-campfire unicode value ("\uf6ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/campfire
      /// </summary>
      [Description("campfire")]
      public const string Campfire = "\uf6ba";

      /// <summary>
      /// fa-campground unicode value ("\uf6bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/campground
      /// </summary>
      [Description("campground")]
      public const string Campground = "\uf6bb";

      /// <summary>
      /// fa-can-food unicode value ("\ue3e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/can-food
      /// </summary>
      [Description("can-food")]
      public const string CanFood = "\ue3e6";

      /// <summary>
      /// fa-canadian-maple-leaf unicode value ("\uf785").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/canadian-maple-leaf
      /// </summary>
      [Description("canadian-maple-leaf")]
      public const string CanadianMapleLeaf = "\uf785";

      /// <summary>
      /// fa-candle-holder unicode value ("\uf6bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/candle-holder
      /// </summary>
      [Description("candle-holder")]
      public const string CandleHolder = "\uf6bc";

      /// <summary>
      /// fa-candy unicode value ("\ue3e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/candy
      /// </summary>
      [Description("candy")]
      public const string Candy = "\ue3e7";

      /// <summary>
      /// fa-candy-bar unicode value ("\ue3e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/candy-bar
      /// </summary>
      [Description("candy-bar")]
      public const string CandyBar = "\ue3e8";

      /// <summary>
      /// fa-candy-cane unicode value ("\uf786").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/candy-cane
      /// </summary>
      [Description("candy-cane")]
      public const string CandyCane = "\uf786";

      /// <summary>
      /// fa-candy-corn unicode value ("\uf6bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/candy-corn
      /// </summary>
      [Description("candy-corn")]
      public const string CandyCorn = "\uf6bd";

      /// <summary>
      /// fa-cannabis unicode value ("\uf55f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cannabis
      /// </summary>
      [Description("cannabis")]
      public const string Cannabis = "\uf55f";

      /// <summary>
      /// fa-capsules unicode value ("\uf46b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/capsules
      /// </summary>
      [Description("capsules")]
      public const string Capsules = "\uf46b";

      /// <summary>
      /// fa-car unicode value ("\uf1b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car
      /// </summary>
      [Description("car")]
      public const string Car = "\uf1b9";

      /// <summary>
      /// fa-car-battery unicode value ("\uf5df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-battery
      /// </summary>
      [Description("car-battery")]
      public const string CarBattery = "\uf5df";

      /// <summary>
      /// fa-car-bolt unicode value ("\ue341").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-bolt
      /// </summary>
      [Description("car-bolt")]
      public const string CarBolt = "\ue341";

      /// <summary>
      /// fa-car-building unicode value ("\uf859").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-building
      /// </summary>
      [Description("car-building")]
      public const string CarBuilding = "\uf859";

      /// <summary>
      /// fa-car-bump unicode value ("\uf5e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-bump
      /// </summary>
      [Description("car-bump")]
      public const string CarBump = "\uf5e0";

      /// <summary>
      /// fa-car-burst unicode value ("\uf5e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-burst
      /// </summary>
      [Description("car-burst")]
      public const string CarBurst = "\uf5e1";

      /// <summary>
      /// fa-car-bus unicode value ("\uf85a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-bus
      /// </summary>
      [Description("car-bus")]
      public const string CarBus = "\uf85a";

      /// <summary>
      /// fa-car-circle-bolt unicode value ("\ue342").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-circle-bolt
      /// </summary>
      [Description("car-circle-bolt")]
      public const string CarCircleBolt = "\ue342";

      /// <summary>
      /// fa-car-garage unicode value ("\uf5e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-garage
      /// </summary>
      [Description("car-garage")]
      public const string CarGarage = "\uf5e2";

      /// <summary>
      /// fa-car-mirrors unicode value ("\ue343").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-mirrors
      /// </summary>
      [Description("car-mirrors")]
      public const string CarMirrors = "\ue343";

      /// <summary>
      /// fa-car-on unicode value ("\ue4dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-on
      /// </summary>
      [Description("car-on")]
      public const string CarOn = "\ue4dd";

      /// <summary>
      /// fa-car-rear unicode value ("\uf5de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-rear
      /// </summary>
      [Description("car-rear")]
      public const string CarRear = "\uf5de";

      /// <summary>
      /// fa-car-side unicode value ("\uf5e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-side
      /// </summary>
      [Description("car-side")]
      public const string CarSide = "\uf5e4";

      /// <summary>
      /// fa-car-side-bolt unicode value ("\ue344").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-side-bolt
      /// </summary>
      [Description("car-side-bolt")]
      public const string CarSideBolt = "\ue344";

      /// <summary>
      /// fa-car-tilt unicode value ("\uf5e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-tilt
      /// </summary>
      [Description("car-tilt")]
      public const string CarTilt = "\uf5e5";

      /// <summary>
      /// fa-car-tunnel unicode value ("\ue4de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-tunnel
      /// </summary>
      [Description("car-tunnel")]
      public const string CarTunnel = "\ue4de";

      /// <summary>
      /// fa-car-wash unicode value ("\uf5e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-wash
      /// </summary>
      [Description("car-wash")]
      public const string CarWash = "\uf5e6";

      /// <summary>
      /// fa-car-wrench unicode value ("\uf5e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/car-wrench
      /// </summary>
      [Description("car-wrench")]
      public const string CarWrench = "\uf5e3";

      /// <summary>
      /// fa-caravan unicode value ("\uf8ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caravan
      /// </summary>
      [Description("caravan")]
      public const string Caravan = "\uf8ff";

      /// <summary>
      /// fa-caravan-simple unicode value ("\ue000").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caravan-simple
      /// </summary>
      [Description("caravan-simple")]
      public const string CaravanSimple = "\ue000";

      /// <summary>
      /// fa-card-club unicode value ("\ue3e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/card-club
      /// </summary>
      [Description("card-club")]
      public const string CardClub = "\ue3e9";

      /// <summary>
      /// fa-card-diamond unicode value ("\ue3ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/card-diamond
      /// </summary>
      [Description("card-diamond")]
      public const string CardDiamond = "\ue3ea";

      /// <summary>
      /// fa-card-heart unicode value ("\ue3eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/card-heart
      /// </summary>
      [Description("card-heart")]
      public const string CardHeart = "\ue3eb";

      /// <summary>
      /// fa-card-spade unicode value ("\ue3ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/card-spade
      /// </summary>
      [Description("card-spade")]
      public const string CardSpade = "\ue3ec";

      /// <summary>
      /// fa-cards unicode value ("\ue3ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cards
      /// </summary>
      [Description("cards")]
      public const string Cards = "\ue3ed";

      /// <summary>
      /// fa-cards-blank unicode value ("\ue4df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cards-blank
      /// </summary>
      [Description("cards-blank")]
      public const string CardsBlank = "\ue4df";

      /// <summary>
      /// fa-caret-down unicode value ("\uf0d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caret-down
      /// </summary>
      [Description("caret-down")]
      public const string CaretDown = "\uf0d7";

      /// <summary>
      /// fa-caret-left unicode value ("\uf0d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caret-left
      /// </summary>
      [Description("caret-left")]
      public const string CaretLeft = "\uf0d9";

      /// <summary>
      /// fa-caret-right unicode value ("\uf0da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caret-right
      /// </summary>
      [Description("caret-right")]
      public const string CaretRight = "\uf0da";

      /// <summary>
      /// fa-caret-up unicode value ("\uf0d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/caret-up
      /// </summary>
      [Description("caret-up")]
      public const string CaretUp = "\uf0d8";

      /// <summary>
      /// fa-carrot unicode value ("\uf787").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/carrot
      /// </summary>
      [Description("carrot")]
      public const string Carrot = "\uf787";

      /// <summary>
      /// fa-cars unicode value ("\uf85b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cars
      /// </summary>
      [Description("cars")]
      public const string Cars = "\uf85b";

      /// <summary>
      /// fa-cart-arrow-down unicode value ("\uf218").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-arrow-down
      /// </summary>
      [Description("cart-arrow-down")]
      public const string CartArrowDown = "\uf218";

      /// <summary>
      /// fa-cart-arrow-up unicode value ("\ue3ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-arrow-up
      /// </summary>
      [Description("cart-arrow-up")]
      public const string CartArrowUp = "\ue3ee";

      /// <summary>
      /// fa-cart-circle-arrow-down unicode value ("\ue3ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-arrow-down
      /// </summary>
      [Description("cart-circle-arrow-down")]
      public const string CartCircleArrowDown = "\ue3ef";

      /// <summary>
      /// fa-cart-circle-arrow-up unicode value ("\ue3f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-arrow-up
      /// </summary>
      [Description("cart-circle-arrow-up")]
      public const string CartCircleArrowUp = "\ue3f0";

      /// <summary>
      /// fa-cart-circle-check unicode value ("\ue3f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-check
      /// </summary>
      [Description("cart-circle-check")]
      public const string CartCircleCheck = "\ue3f1";

      /// <summary>
      /// fa-cart-circle-exclamation unicode value ("\ue3f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-exclamation
      /// </summary>
      [Description("cart-circle-exclamation")]
      public const string CartCircleExclamation = "\ue3f2";

      /// <summary>
      /// fa-cart-circle-plus unicode value ("\ue3f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-plus
      /// </summary>
      [Description("cart-circle-plus")]
      public const string CartCirclePlus = "\ue3f3";

      /// <summary>
      /// fa-cart-circle-xmark unicode value ("\ue3f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-circle-xmark
      /// </summary>
      [Description("cart-circle-xmark")]
      public const string CartCircleXmark = "\ue3f4";

      /// <summary>
      /// fa-cart-flatbed unicode value ("\uf474").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-flatbed
      /// </summary>
      [Description("cart-flatbed")]
      public const string CartFlatbed = "\uf474";

      /// <summary>
      /// fa-cart-flatbed-boxes unicode value ("\uf475").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-flatbed-boxes
      /// </summary>
      [Description("cart-flatbed-boxes")]
      public const string CartFlatbedBoxes = "\uf475";

      /// <summary>
      /// fa-cart-flatbed-empty unicode value ("\uf476").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-flatbed-empty
      /// </summary>
      [Description("cart-flatbed-empty")]
      public const string CartFlatbedEmpty = "\uf476";

      /// <summary>
      /// fa-cart-flatbed-suitcase unicode value ("\uf59d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-flatbed-suitcase
      /// </summary>
      [Description("cart-flatbed-suitcase")]
      public const string CartFlatbedSuitcase = "\uf59d";

      /// <summary>
      /// fa-cart-minus unicode value ("\ue0db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-minus
      /// </summary>
      [Description("cart-minus")]
      public const string CartMinus = "\ue0db";

      /// <summary>
      /// fa-cart-plus unicode value ("\uf217").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-plus
      /// </summary>
      [Description("cart-plus")]
      public const string CartPlus = "\uf217";

      /// <summary>
      /// fa-cart-shopping unicode value ("\uf07a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-shopping
      /// </summary>
      [Description("cart-shopping")]
      public const string CartShopping = "\uf07a";

      /// <summary>
      /// fa-cart-shopping-fast unicode value ("\ue0dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-shopping-fast
      /// </summary>
      [Description("cart-shopping-fast")]
      public const string CartShoppingFast = "\ue0dc";

      /// <summary>
      /// fa-cart-xmark unicode value ("\ue0dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cart-xmark
      /// </summary>
      [Description("cart-xmark")]
      public const string CartXmark = "\ue0dd";

      /// <summary>
      /// fa-cash-register unicode value ("\uf788").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cash-register
      /// </summary>
      [Description("cash-register")]
      public const string CashRegister = "\uf788";

      /// <summary>
      /// fa-cassette-betamax unicode value ("\uf8a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cassette-betamax
      /// </summary>
      [Description("cassette-betamax")]
      public const string CassetteBetamax = "\uf8a4";

      /// <summary>
      /// fa-cassette-tape unicode value ("\uf8ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cassette-tape
      /// </summary>
      [Description("cassette-tape")]
      public const string CassetteTape = "\uf8ab";

      /// <summary>
      /// fa-cassette-vhs unicode value ("\uf8ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cassette-vhs
      /// </summary>
      [Description("cassette-vhs")]
      public const string CassetteVhs = "\uf8ec";

      /// <summary>
      /// fa-castle unicode value ("\ue0de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/castle
      /// </summary>
      [Description("castle")]
      public const string Castle = "\ue0de";

      /// <summary>
      /// fa-cat unicode value ("\uf6be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cat
      /// </summary>
      [Description("cat")]
      public const string Cat = "\uf6be";

      /// <summary>
      /// fa-cat-space unicode value ("\ue001").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cat-space
      /// </summary>
      [Description("cat-space")]
      public const string CatSpace = "\ue001";

      /// <summary>
      /// fa-cauldron unicode value ("\uf6bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cauldron
      /// </summary>
      [Description("cauldron")]
      public const string Cauldron = "\uf6bf";

      /// <summary>
      /// fa-cc-amazon-pay unicode value ("\uf42d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.2', Last Modified in '5.0.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-amazon-pay
      /// </summary>
      [Description("cc-amazon-pay")]
      public const string CcAmazonPay = "\uf42d";

      /// <summary>
      /// fa-cc-amex unicode value ("\uf1f3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-amex
      /// </summary>
      [Description("cc-amex")]
      public const string CcAmex = "\uf1f3";

      /// <summary>
      /// fa-cc-apple-pay unicode value ("\uf416").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-apple-pay
      /// </summary>
      [Description("cc-apple-pay")]
      public const string CcApplePay = "\uf416";

      /// <summary>
      /// fa-cc-diners-club unicode value ("\uf24c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-diners-club
      /// </summary>
      [Description("cc-diners-club")]
      public const string CcDinersClub = "\uf24c";

      /// <summary>
      /// fa-cc-discover unicode value ("\uf1f2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-discover
      /// </summary>
      [Description("cc-discover")]
      public const string CcDiscover = "\uf1f2";

      /// <summary>
      /// fa-cc-jcb unicode value ("\uf24b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-jcb
      /// </summary>
      [Description("cc-jcb")]
      public const string CcJcb = "\uf24b";

      /// <summary>
      /// fa-cc-mastercard unicode value ("\uf1f1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-mastercard
      /// </summary>
      [Description("cc-mastercard")]
      public const string CcMastercard = "\uf1f1";

      /// <summary>
      /// fa-cc-paypal unicode value ("\uf1f4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-paypal
      /// </summary>
      [Description("cc-paypal")]
      public const string CcPaypal = "\uf1f4";

      /// <summary>
      /// fa-cc-stripe unicode value ("\uf1f5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-stripe
      /// </summary>
      [Description("cc-stripe")]
      public const string CcStripe = "\uf1f5";

      /// <summary>
      /// fa-cc-visa unicode value ("\uf1f0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cc-visa
      /// </summary>
      [Description("cc-visa")]
      public const string CcVisa = "\uf1f0";

      /// <summary>
      /// fa-cedi-sign unicode value ("\ue0df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cedi-sign
      /// </summary>
      [Description("cedi-sign")]
      public const string CediSign = "\ue0df";

      /// <summary>
      /// fa-cent-sign unicode value ("\ue3f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cent-sign
      /// </summary>
      [Description("cent-sign")]
      public const string CentSign = "\ue3f5";

      /// <summary>
      /// fa-centercode unicode value ("\uf380").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/centercode
      /// </summary>
      [Description("centercode")]
      public const string Centercode = "\uf380";

      /// <summary>
      /// fa-centos unicode value ("\uf789").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/centos
      /// </summary>
      [Description("centos")]
      public const string Centos = "\uf789";

      /// <summary>
      /// fa-certificate unicode value ("\uf0a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/certificate
      /// </summary>
      [Description("certificate")]
      public const string Certificate = "\uf0a3";

      /// <summary>
      /// fa-chair unicode value ("\uf6c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chair
      /// </summary>
      [Description("chair")]
      public const string Chair = "\uf6c0";

      /// <summary>
      /// fa-chair-office unicode value ("\uf6c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chair-office
      /// </summary>
      [Description("chair-office")]
      public const string ChairOffice = "\uf6c1";

      /// <summary>
      /// fa-chalkboard unicode value ("\uf51b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chalkboard
      /// </summary>
      [Description("chalkboard")]
      public const string Chalkboard = "\uf51b";

      /// <summary>
      /// fa-chalkboard-user unicode value ("\uf51c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chalkboard-user
      /// </summary>
      [Description("chalkboard-user")]
      public const string ChalkboardUser = "\uf51c";

      /// <summary>
      /// fa-champagne-glass unicode value ("\uf79e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/champagne-glass
      /// </summary>
      [Description("champagne-glass")]
      public const string ChampagneGlass = "\uf79e";

      /// <summary>
      /// fa-champagne-glasses unicode value ("\uf79f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/champagne-glasses
      /// </summary>
      [Description("champagne-glasses")]
      public const string ChampagneGlasses = "\uf79f";

      /// <summary>
      /// fa-charging-station unicode value ("\uf5e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/charging-station
      /// </summary>
      [Description("charging-station")]
      public const string ChargingStation = "\uf5e7";

      /// <summary>
      /// fa-chart-area unicode value ("\uf1fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-area
      /// </summary>
      [Description("chart-area")]
      public const string ChartArea = "\uf1fe";

      /// <summary>
      /// fa-chart-bar unicode value ("\uf080").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-bar
      /// </summary>
      [Description("chart-bar")]
      public const string ChartBar = "\uf080";

      /// <summary>
      /// fa-chart-bullet unicode value ("\ue0e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-bullet
      /// </summary>
      [Description("chart-bullet")]
      public const string ChartBullet = "\ue0e1";

      /// <summary>
      /// fa-chart-candlestick unicode value ("\ue0e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-candlestick
      /// </summary>
      [Description("chart-candlestick")]
      public const string ChartCandlestick = "\ue0e2";

      /// <summary>
      /// fa-chart-column unicode value ("\ue0e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-column
      /// </summary>
      [Description("chart-column")]
      public const string ChartColumn = "\ue0e3";

      /// <summary>
      /// fa-chart-gantt unicode value ("\ue0e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-gantt
      /// </summary>
      [Description("chart-gantt")]
      public const string ChartGantt = "\ue0e4";

      /// <summary>
      /// fa-chart-line unicode value ("\uf201").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-line
      /// </summary>
      [Description("chart-line")]
      public const string ChartLine = "\uf201";

      /// <summary>
      /// fa-chart-line-down unicode value ("\uf64d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-line-down
      /// </summary>
      [Description("chart-line-down")]
      public const string ChartLineDown = "\uf64d";

      /// <summary>
      /// fa-chart-line-up unicode value ("\ue0e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-line-up
      /// </summary>
      [Description("chart-line-up")]
      public const string ChartLineUp = "\ue0e5";

      /// <summary>
      /// fa-chart-line-up-down unicode value ("\ue5d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-line-up-down
      /// </summary>
      [Description("chart-line-up-down")]
      public const string ChartLineUpDown = "\ue5d7";

      /// <summary>
      /// fa-chart-mixed unicode value ("\uf643").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-mixed
      /// </summary>
      [Description("chart-mixed")]
      public const string ChartMixed = "\uf643";

      /// <summary>
      /// fa-chart-mixed-up-circle-currency unicode value ("\ue5d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-mixed-up-circle-currency
      /// </summary>
      [Description("chart-mixed-up-circle-currency")]
      public const string ChartMixedUpCircleCurrency = "\ue5d8";

      /// <summary>
      /// fa-chart-mixed-up-circle-dollar unicode value ("\ue5d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-mixed-up-circle-dollar
      /// </summary>
      [Description("chart-mixed-up-circle-dollar")]
      public const string ChartMixedUpCircleDollar = "\ue5d9";

      /// <summary>
      /// fa-chart-network unicode value ("\uf78a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-network
      /// </summary>
      [Description("chart-network")]
      public const string ChartNetwork = "\uf78a";

      /// <summary>
      /// fa-chart-pie unicode value ("\uf200").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-pie
      /// </summary>
      [Description("chart-pie")]
      public const string ChartPie = "\uf200";

      /// <summary>
      /// fa-chart-pie-simple unicode value ("\uf64e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-pie-simple
      /// </summary>
      [Description("chart-pie-simple")]
      public const string ChartPieSimple = "\uf64e";

      /// <summary>
      /// fa-chart-pie-simple-circle-currency unicode value ("\ue604").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-pie-simple-circle-currency
      /// </summary>
      [Description("chart-pie-simple-circle-currency")]
      public const string ChartPieSimpleCircleCurrency = "\ue604";

      /// <summary>
      /// fa-chart-pie-simple-circle-dollar unicode value ("\ue605").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-pie-simple-circle-dollar
      /// </summary>
      [Description("chart-pie-simple-circle-dollar")]
      public const string ChartPieSimpleCircleDollar = "\ue605";

      /// <summary>
      /// fa-chart-pyramid unicode value ("\ue0e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-pyramid
      /// </summary>
      [Description("chart-pyramid")]
      public const string ChartPyramid = "\ue0e6";

      /// <summary>
      /// fa-chart-radar unicode value ("\ue0e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-radar
      /// </summary>
      [Description("chart-radar")]
      public const string ChartRadar = "\ue0e7";

      /// <summary>
      /// fa-chart-scatter unicode value ("\uf7ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-scatter
      /// </summary>
      [Description("chart-scatter")]
      public const string ChartScatter = "\uf7ee";

      /// <summary>
      /// fa-chart-scatter-3d unicode value ("\ue0e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-scatter-3d
      /// </summary>
      [Description("chart-scatter-3d")]
      public const string ChartScatter3d = "\ue0e8";

      /// <summary>
      /// fa-chart-scatter-bubble unicode value ("\ue0e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-scatter-bubble
      /// </summary>
      [Description("chart-scatter-bubble")]
      public const string ChartScatterBubble = "\ue0e9";

      /// <summary>
      /// fa-chart-simple unicode value ("\ue473").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-simple
      /// </summary>
      [Description("chart-simple")]
      public const string ChartSimple = "\ue473";

      /// <summary>
      /// fa-chart-simple-horizontal unicode value ("\ue474").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-simple-horizontal
      /// </summary>
      [Description("chart-simple-horizontal")]
      public const string ChartSimpleHorizontal = "\ue474";

      /// <summary>
      /// fa-chart-tree-map unicode value ("\ue0ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-tree-map
      /// </summary>
      [Description("chart-tree-map")]
      public const string ChartTreeMap = "\ue0ea";

      /// <summary>
      /// fa-chart-user unicode value ("\uf6a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-user
      /// </summary>
      [Description("chart-user")]
      public const string ChartUser = "\uf6a3";

      /// <summary>
      /// fa-chart-waterfall unicode value ("\ue0eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chart-waterfall
      /// </summary>
      [Description("chart-waterfall")]
      public const string ChartWaterfall = "\ue0eb";

      /// <summary>
      /// fa-check unicode value ("\uf00c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/check
      /// </summary>
      [Description("check")]
      public const string Check = "\uf00c";

      /// <summary>
      /// fa-check-double unicode value ("\uf560").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/check-double
      /// </summary>
      [Description("check-double")]
      public const string CheckDouble = "\uf560";

      /// <summary>
      /// fa-check-to-slot unicode value ("\uf772").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/check-to-slot
      /// </summary>
      [Description("check-to-slot")]
      public const string CheckToSlot = "\uf772";

      /// <summary>
      /// fa-cheese unicode value ("\uf7ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cheese
      /// </summary>
      [Description("cheese")]
      public const string Cheese = "\uf7ef";

      /// <summary>
      /// fa-cheese-swiss unicode value ("\uf7f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cheese-swiss
      /// </summary>
      [Description("cheese-swiss")]
      public const string CheeseSwiss = "\uf7f0";

      /// <summary>
      /// fa-cherries unicode value ("\ue0ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cherries
      /// </summary>
      [Description("cherries")]
      public const string Cherries = "\ue0ec";

      /// <summary>
      /// fa-chess unicode value ("\uf439").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess
      /// </summary>
      [Description("chess")]
      public const string Chess = "\uf439";

      /// <summary>
      /// fa-chess-bishop unicode value ("\uf43a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-bishop
      /// </summary>
      [Description("chess-bishop")]
      public const string ChessBishop = "\uf43a";

      /// <summary>
      /// fa-chess-bishop-piece unicode value ("\uf43b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-bishop-piece
      /// </summary>
      [Description("chess-bishop-piece")]
      public const string ChessBishopPiece = "\uf43b";

      /// <summary>
      /// fa-chess-board unicode value ("\uf43c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-board
      /// </summary>
      [Description("chess-board")]
      public const string ChessBoard = "\uf43c";

      /// <summary>
      /// fa-chess-clock unicode value ("\uf43d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-clock
      /// </summary>
      [Description("chess-clock")]
      public const string ChessClock = "\uf43d";

      /// <summary>
      /// fa-chess-clock-flip unicode value ("\uf43e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-clock-flip
      /// </summary>
      [Description("chess-clock-flip")]
      public const string ChessClockFlip = "\uf43e";

      /// <summary>
      /// fa-chess-king unicode value ("\uf43f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-king
      /// </summary>
      [Description("chess-king")]
      public const string ChessKing = "\uf43f";

      /// <summary>
      /// fa-chess-king-piece unicode value ("\uf440").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-king-piece
      /// </summary>
      [Description("chess-king-piece")]
      public const string ChessKingPiece = "\uf440";

      /// <summary>
      /// fa-chess-knight unicode value ("\uf441").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-knight
      /// </summary>
      [Description("chess-knight")]
      public const string ChessKnight = "\uf441";

      /// <summary>
      /// fa-chess-knight-piece unicode value ("\uf442").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-knight-piece
      /// </summary>
      [Description("chess-knight-piece")]
      public const string ChessKnightPiece = "\uf442";

      /// <summary>
      /// fa-chess-pawn unicode value ("\uf443").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-pawn
      /// </summary>
      [Description("chess-pawn")]
      public const string ChessPawn = "\uf443";

      /// <summary>
      /// fa-chess-pawn-piece unicode value ("\uf444").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-pawn-piece
      /// </summary>
      [Description("chess-pawn-piece")]
      public const string ChessPawnPiece = "\uf444";

      /// <summary>
      /// fa-chess-queen unicode value ("\uf445").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-queen
      /// </summary>
      [Description("chess-queen")]
      public const string ChessQueen = "\uf445";

      /// <summary>
      /// fa-chess-queen-piece unicode value ("\uf446").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-queen-piece
      /// </summary>
      [Description("chess-queen-piece")]
      public const string ChessQueenPiece = "\uf446";

      /// <summary>
      /// fa-chess-rook unicode value ("\uf447").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-rook
      /// </summary>
      [Description("chess-rook")]
      public const string ChessRook = "\uf447";

      /// <summary>
      /// fa-chess-rook-piece unicode value ("\uf448").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chess-rook-piece
      /// </summary>
      [Description("chess-rook-piece")]
      public const string ChessRookPiece = "\uf448";

      /// <summary>
      /// fa-chestnut unicode value ("\ue3f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chestnut
      /// </summary>
      [Description("chestnut")]
      public const string Chestnut = "\ue3f6";

      /// <summary>
      /// fa-chevron-down unicode value ("\uf078").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevron-down
      /// </summary>
      [Description("chevron-down")]
      public const string ChevronDown = "\uf078";

      /// <summary>
      /// fa-chevron-left unicode value ("\uf053").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevron-left
      /// </summary>
      [Description("chevron-left")]
      public const string ChevronLeft = "\uf053";

      /// <summary>
      /// fa-chevron-right unicode value ("\uf054").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevron-right
      /// </summary>
      [Description("chevron-right")]
      public const string ChevronRight = "\uf054";

      /// <summary>
      /// fa-chevron-up unicode value ("\uf077").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevron-up
      /// </summary>
      [Description("chevron-up")]
      public const string ChevronUp = "\uf077";

      /// <summary>
      /// fa-chevrons-down unicode value ("\uf322").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevrons-down
      /// </summary>
      [Description("chevrons-down")]
      public const string ChevronsDown = "\uf322";

      /// <summary>
      /// fa-chevrons-left unicode value ("\uf323").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevrons-left
      /// </summary>
      [Description("chevrons-left")]
      public const string ChevronsLeft = "\uf323";

      /// <summary>
      /// fa-chevrons-right unicode value ("\uf324").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevrons-right
      /// </summary>
      [Description("chevrons-right")]
      public const string ChevronsRight = "\uf324";

      /// <summary>
      /// fa-chevrons-up unicode value ("\uf325").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chevrons-up
      /// </summary>
      [Description("chevrons-up")]
      public const string ChevronsUp = "\uf325";

      /// <summary>
      /// fa-chf-sign unicode value ("\ue602").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chf-sign
      /// </summary>
      [Description("chf-sign")]
      public const string ChfSign = "\ue602";

      /// <summary>
      /// fa-child unicode value ("\uf1ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/child
      /// </summary>
      [Description("child")]
      public const string Child = "\uf1ae";

      /// <summary>
      /// fa-child-combatant unicode value ("\ue4e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/child-combatant
      /// </summary>
      [Description("child-combatant")]
      public const string ChildCombatant = "\ue4e0";

      /// <summary>
      /// fa-child-dress unicode value ("\ue59c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/child-dress
      /// </summary>
      [Description("child-dress")]
      public const string ChildDress = "\ue59c";

      /// <summary>
      /// fa-child-reaching unicode value ("\ue59d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/child-reaching
      /// </summary>
      [Description("child-reaching")]
      public const string ChildReaching = "\ue59d";

      /// <summary>
      /// fa-children unicode value ("\ue4e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/children
      /// </summary>
      [Description("children")]
      public const string Children = "\ue4e1";

      /// <summary>
      /// fa-chimney unicode value ("\uf78b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chimney
      /// </summary>
      [Description("chimney")]
      public const string Chimney = "\uf78b";

      /// <summary>
      /// fa-chopsticks unicode value ("\ue3f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chopsticks
      /// </summary>
      [Description("chopsticks")]
      public const string Chopsticks = "\ue3f7";

      /// <summary>
      /// fa-chrome unicode value ("\uf268").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/chrome
      /// </summary>
      [Description("chrome")]
      public const string Chrome = "\uf268";

      /// <summary>
      /// fa-chromecast unicode value ("\uf838").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/chromecast
      /// </summary>
      [Description("chromecast")]
      public const string Chromecast = "\uf838";

      /// <summary>
      /// fa-church unicode value ("\uf51d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/church
      /// </summary>
      [Description("church")]
      public const string Church = "\uf51d";

      /// <summary>
      /// fa-circle unicode value ("\uf111").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle
      /// </summary>
      [Description("circle")]
      public const string Circle = "\uf111";

      /// <summary>
      /// fa-circle-0 unicode value ("\ue0ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-0
      /// </summary>
      [Description("circle-0")]
      public const string Circle0 = "\ue0ed";

      /// <summary>
      /// fa-circle-1 unicode value ("\ue0ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-1
      /// </summary>
      [Description("circle-1")]
      public const string Circle1 = "\ue0ee";

      /// <summary>
      /// fa-circle-2 unicode value ("\ue0ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-2
      /// </summary>
      [Description("circle-2")]
      public const string Circle2 = "\ue0ef";

      /// <summary>
      /// fa-circle-3 unicode value ("\ue0f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-3
      /// </summary>
      [Description("circle-3")]
      public const string Circle3 = "\ue0f0";

      /// <summary>
      /// fa-circle-4 unicode value ("\ue0f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-4
      /// </summary>
      [Description("circle-4")]
      public const string Circle4 = "\ue0f1";

      /// <summary>
      /// fa-circle-5 unicode value ("\ue0f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-5
      /// </summary>
      [Description("circle-5")]
      public const string Circle5 = "\ue0f2";

      /// <summary>
      /// fa-circle-6 unicode value ("\ue0f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-6
      /// </summary>
      [Description("circle-6")]
      public const string Circle6 = "\ue0f3";

      /// <summary>
      /// fa-circle-7 unicode value ("\ue0f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-7
      /// </summary>
      [Description("circle-7")]
      public const string Circle7 = "\ue0f4";

      /// <summary>
      /// fa-circle-8 unicode value ("\ue0f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-8
      /// </summary>
      [Description("circle-8")]
      public const string Circle8 = "\ue0f5";

      /// <summary>
      /// fa-circle-9 unicode value ("\ue0f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-9
      /// </summary>
      [Description("circle-9")]
      public const string Circle9 = "\ue0f6";

      /// <summary>
      /// fa-circle-a unicode value ("\ue0f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-a
      /// </summary>
      [Description("circle-a")]
      public const string CircleA = "\ue0f7";

      /// <summary>
      /// fa-circle-ampersand unicode value ("\ue0f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-ampersand
      /// </summary>
      [Description("circle-ampersand")]
      public const string CircleAmpersand = "\ue0f8";

      /// <summary>
      /// fa-circle-arrow-down unicode value ("\uf0ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-down
      /// </summary>
      [Description("circle-arrow-down")]
      public const string CircleArrowDown = "\uf0ab";

      /// <summary>
      /// fa-circle-arrow-down-left unicode value ("\ue0f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-down-left
      /// </summary>
      [Description("circle-arrow-down-left")]
      public const string CircleArrowDownLeft = "\ue0f9";

      /// <summary>
      /// fa-circle-arrow-down-right unicode value ("\ue0fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-down-right
      /// </summary>
      [Description("circle-arrow-down-right")]
      public const string CircleArrowDownRight = "\ue0fa";

      /// <summary>
      /// fa-circle-arrow-left unicode value ("\uf0a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-left
      /// </summary>
      [Description("circle-arrow-left")]
      public const string CircleArrowLeft = "\uf0a8";

      /// <summary>
      /// fa-circle-arrow-right unicode value ("\uf0a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-right
      /// </summary>
      [Description("circle-arrow-right")]
      public const string CircleArrowRight = "\uf0a9";

      /// <summary>
      /// fa-circle-arrow-up unicode value ("\uf0aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-up
      /// </summary>
      [Description("circle-arrow-up")]
      public const string CircleArrowUp = "\uf0aa";

      /// <summary>
      /// fa-circle-arrow-up-left unicode value ("\ue0fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-up-left
      /// </summary>
      [Description("circle-arrow-up-left")]
      public const string CircleArrowUpLeft = "\ue0fb";

      /// <summary>
      /// fa-circle-arrow-up-right unicode value ("\ue0fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-arrow-up-right
      /// </summary>
      [Description("circle-arrow-up-right")]
      public const string CircleArrowUpRight = "\ue0fc";

      /// <summary>
      /// fa-circle-b unicode value ("\ue0fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-b
      /// </summary>
      [Description("circle-b")]
      public const string CircleB = "\ue0fd";

      /// <summary>
      /// fa-circle-bolt unicode value ("\ue0fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-bolt
      /// </summary>
      [Description("circle-bolt")]
      public const string CircleBolt = "\ue0fe";

      /// <summary>
      /// fa-circle-book-open unicode value ("\ue0ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-book-open
      /// </summary>
      [Description("circle-book-open")]
      public const string CircleBookOpen = "\ue0ff";

      /// <summary>
      /// fa-circle-bookmark unicode value ("\ue100").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-bookmark
      /// </summary>
      [Description("circle-bookmark")]
      public const string CircleBookmark = "\ue100";

      /// <summary>
      /// fa-circle-c unicode value ("\ue101").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-c
      /// </summary>
      [Description("circle-c")]
      public const string CircleC = "\ue101";

      /// <summary>
      /// fa-circle-calendar unicode value ("\ue102").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-calendar
      /// </summary>
      [Description("circle-calendar")]
      public const string CircleCalendar = "\ue102";

      /// <summary>
      /// fa-circle-camera unicode value ("\ue103").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-camera
      /// </summary>
      [Description("circle-camera")]
      public const string CircleCamera = "\ue103";

      /// <summary>
      /// fa-circle-caret-down unicode value ("\uf32d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-caret-down
      /// </summary>
      [Description("circle-caret-down")]
      public const string CircleCaretDown = "\uf32d";

      /// <summary>
      /// fa-circle-caret-left unicode value ("\uf32e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-caret-left
      /// </summary>
      [Description("circle-caret-left")]
      public const string CircleCaretLeft = "\uf32e";

      /// <summary>
      /// fa-circle-caret-right unicode value ("\uf330").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-caret-right
      /// </summary>
      [Description("circle-caret-right")]
      public const string CircleCaretRight = "\uf330";

      /// <summary>
      /// fa-circle-caret-up unicode value ("\uf331").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-caret-up
      /// </summary>
      [Description("circle-caret-up")]
      public const string CircleCaretUp = "\uf331";

      /// <summary>
      /// fa-circle-check unicode value ("\uf058").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-check
      /// </summary>
      [Description("circle-check")]
      public const string CircleCheck = "\uf058";

      /// <summary>
      /// fa-circle-chevron-down unicode value ("\uf13a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-chevron-down
      /// </summary>
      [Description("circle-chevron-down")]
      public const string CircleChevronDown = "\uf13a";

      /// <summary>
      /// fa-circle-chevron-left unicode value ("\uf137").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-chevron-left
      /// </summary>
      [Description("circle-chevron-left")]
      public const string CircleChevronLeft = "\uf137";

      /// <summary>
      /// fa-circle-chevron-right unicode value ("\uf138").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-chevron-right
      /// </summary>
      [Description("circle-chevron-right")]
      public const string CircleChevronRight = "\uf138";

      /// <summary>
      /// fa-circle-chevron-up unicode value ("\uf139").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-chevron-up
      /// </summary>
      [Description("circle-chevron-up")]
      public const string CircleChevronUp = "\uf139";

      /// <summary>
      /// fa-circle-d unicode value ("\ue104").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-d
      /// </summary>
      [Description("circle-d")]
      public const string CircleD = "\ue104";

      /// <summary>
      /// fa-circle-dashed unicode value ("\ue105").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-dashed
      /// </summary>
      [Description("circle-dashed")]
      public const string CircleDashed = "\ue105";

      /// <summary>
      /// fa-circle-divide unicode value ("\ue106").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-divide
      /// </summary>
      [Description("circle-divide")]
      public const string CircleDivide = "\ue106";

      /// <summary>
      /// fa-circle-dollar unicode value ("\uf2e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-dollar
      /// </summary>
      [Description("circle-dollar")]
      public const string CircleDollar = "\uf2e8";

      /// <summary>
      /// fa-circle-dollar-to-slot unicode value ("\uf4b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-dollar-to-slot
      /// </summary>
      [Description("circle-dollar-to-slot")]
      public const string CircleDollarToSlot = "\uf4b9";

      /// <summary>
      /// fa-circle-dot unicode value ("\uf192").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-dot
      /// </summary>
      [Description("circle-dot")]
      public const string CircleDot = "\uf192";

      /// <summary>
      /// fa-circle-down unicode value ("\uf358").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-down
      /// </summary>
      [Description("circle-down")]
      public const string CircleDown = "\uf358";

      /// <summary>
      /// fa-circle-down-left unicode value ("\ue107").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-down-left
      /// </summary>
      [Description("circle-down-left")]
      public const string CircleDownLeft = "\ue107";

      /// <summary>
      /// fa-circle-down-right unicode value ("\ue108").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-down-right
      /// </summary>
      [Description("circle-down-right")]
      public const string CircleDownRight = "\ue108";

      /// <summary>
      /// fa-circle-e unicode value ("\ue109").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-e
      /// </summary>
      [Description("circle-e")]
      public const string CircleE = "\ue109";

      /// <summary>
      /// fa-circle-ellipsis unicode value ("\ue10a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-ellipsis
      /// </summary>
      [Description("circle-ellipsis")]
      public const string CircleEllipsis = "\ue10a";

      /// <summary>
      /// fa-circle-ellipsis-vertical unicode value ("\ue10b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-ellipsis-vertical
      /// </summary>
      [Description("circle-ellipsis-vertical")]
      public const string CircleEllipsisVertical = "\ue10b";

      /// <summary>
      /// fa-circle-envelope unicode value ("\ue10c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-envelope
      /// </summary>
      [Description("circle-envelope")]
      public const string CircleEnvelope = "\ue10c";

      /// <summary>
      /// fa-circle-euro unicode value ("\ue5ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-euro
      /// </summary>
      [Description("circle-euro")]
      public const string CircleEuro = "\ue5ce";

      /// <summary>
      /// fa-circle-exclamation unicode value ("\uf06a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-exclamation
      /// </summary>
      [Description("circle-exclamation")]
      public const string CircleExclamation = "\uf06a";

      /// <summary>
      /// fa-circle-exclamation-check unicode value ("\ue10d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-exclamation-check
      /// </summary>
      [Description("circle-exclamation-check")]
      public const string CircleExclamationCheck = "\ue10d";

      /// <summary>
      /// fa-circle-f unicode value ("\ue10e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-f
      /// </summary>
      [Description("circle-f")]
      public const string CircleF = "\ue10e";

      /// <summary>
      /// fa-circle-g unicode value ("\ue10f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-g
      /// </summary>
      [Description("circle-g")]
      public const string CircleG = "\ue10f";

      /// <summary>
      /// fa-circle-h unicode value ("\uf47e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-h
      /// </summary>
      [Description("circle-h")]
      public const string CircleH = "\uf47e";

      /// <summary>
      /// fa-circle-half unicode value ("\ue110").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-half
      /// </summary>
      [Description("circle-half")]
      public const string CircleHalf = "\ue110";

      /// <summary>
      /// fa-circle-half-stroke unicode value ("\uf042").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-half-stroke
      /// </summary>
      [Description("circle-half-stroke")]
      public const string CircleHalfStroke = "\uf042";

      /// <summary>
      /// fa-circle-heart unicode value ("\uf4c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-heart
      /// </summary>
      [Description("circle-heart")]
      public const string CircleHeart = "\uf4c7";

      /// <summary>
      /// fa-circle-i unicode value ("\ue111").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-i
      /// </summary>
      [Description("circle-i")]
      public const string CircleI = "\ue111";

      /// <summary>
      /// fa-circle-info unicode value ("\uf05a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-info
      /// </summary>
      [Description("circle-info")]
      public const string CircleInfo = "\uf05a";

      /// <summary>
      /// fa-circle-j unicode value ("\ue112").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-j
      /// </summary>
      [Description("circle-j")]
      public const string CircleJ = "\ue112";

      /// <summary>
      /// fa-circle-k unicode value ("\ue113").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-k
      /// </summary>
      [Description("circle-k")]
      public const string CircleK = "\ue113";

      /// <summary>
      /// fa-circle-l unicode value ("\ue114").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-l
      /// </summary>
      [Description("circle-l")]
      public const string CircleL = "\ue114";

      /// <summary>
      /// fa-circle-left unicode value ("\uf359").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-left
      /// </summary>
      [Description("circle-left")]
      public const string CircleLeft = "\uf359";

      /// <summary>
      /// fa-circle-location-arrow unicode value ("\uf602").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-location-arrow
      /// </summary>
      [Description("circle-location-arrow")]
      public const string CircleLocationArrow = "\uf602";

      /// <summary>
      /// fa-circle-m unicode value ("\ue115").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-m
      /// </summary>
      [Description("circle-m")]
      public const string CircleM = "\ue115";

      /// <summary>
      /// fa-circle-microphone unicode value ("\ue116").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-microphone
      /// </summary>
      [Description("circle-microphone")]
      public const string CircleMicrophone = "\ue116";

      /// <summary>
      /// fa-circle-microphone-lines unicode value ("\ue117").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-microphone-lines
      /// </summary>
      [Description("circle-microphone-lines")]
      public const string CircleMicrophoneLines = "\ue117";

      /// <summary>
      /// fa-circle-minus unicode value ("\uf056").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-minus
      /// </summary>
      [Description("circle-minus")]
      public const string CircleMinus = "\uf056";

      /// <summary>
      /// fa-circle-n unicode value ("\ue118").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-n
      /// </summary>
      [Description("circle-n")]
      public const string CircleN = "\ue118";

      /// <summary>
      /// fa-circle-nodes unicode value ("\ue4e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-nodes
      /// </summary>
      [Description("circle-nodes")]
      public const string CircleNodes = "\ue4e2";

      /// <summary>
      /// fa-circle-notch unicode value ("\uf1ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-notch
      /// </summary>
      [Description("circle-notch")]
      public const string CircleNotch = "\uf1ce";

      /// <summary>
      /// fa-circle-o unicode value ("\ue119").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-o
      /// </summary>
      [Description("circle-o")]
      public const string CircleO = "\ue119";

      /// <summary>
      /// fa-circle-p unicode value ("\ue11a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-p
      /// </summary>
      [Description("circle-p")]
      public const string CircleP = "\ue11a";

      /// <summary>
      /// fa-circle-parking unicode value ("\uf615").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-parking
      /// </summary>
      [Description("circle-parking")]
      public const string CircleParking = "\uf615";

      /// <summary>
      /// fa-circle-pause unicode value ("\uf28b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-pause
      /// </summary>
      [Description("circle-pause")]
      public const string CirclePause = "\uf28b";

      /// <summary>
      /// fa-circle-phone unicode value ("\ue11b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-phone
      /// </summary>
      [Description("circle-phone")]
      public const string CirclePhone = "\ue11b";

      /// <summary>
      /// fa-circle-phone-flip unicode value ("\ue11c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-phone-flip
      /// </summary>
      [Description("circle-phone-flip")]
      public const string CirclePhoneFlip = "\ue11c";

      /// <summary>
      /// fa-circle-phone-hangup unicode value ("\ue11d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-phone-hangup
      /// </summary>
      [Description("circle-phone-hangup")]
      public const string CirclePhoneHangup = "\ue11d";

      /// <summary>
      /// fa-circle-play unicode value ("\uf144").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-play
      /// </summary>
      [Description("circle-play")]
      public const string CirclePlay = "\uf144";

      /// <summary>
      /// fa-circle-plus unicode value ("\uf055").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-plus
      /// </summary>
      [Description("circle-plus")]
      public const string CirclePlus = "\uf055";

      /// <summary>
      /// fa-circle-q unicode value ("\ue11e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-q
      /// </summary>
      [Description("circle-q")]
      public const string CircleQ = "\ue11e";

      /// <summary>
      /// fa-circle-quarter unicode value ("\ue11f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-quarter
      /// </summary>
      [Description("circle-quarter")]
      public const string CircleQuarter = "\ue11f";

      /// <summary>
      /// fa-circle-quarter-stroke unicode value ("\ue5d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-quarter-stroke
      /// </summary>
      [Description("circle-quarter-stroke")]
      public const string CircleQuarterStroke = "\ue5d3";

      /// <summary>
      /// fa-circle-quarters unicode value ("\ue3f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-quarters
      /// </summary>
      [Description("circle-quarters")]
      public const string CircleQuarters = "\ue3f8";

      /// <summary>
      /// fa-circle-question unicode value ("\uf059").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-question
      /// </summary>
      [Description("circle-question")]
      public const string CircleQuestion = "\uf059";

      /// <summary>
      /// fa-circle-r unicode value ("\ue120").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-r
      /// </summary>
      [Description("circle-r")]
      public const string CircleR = "\ue120";

      /// <summary>
      /// fa-circle-radiation unicode value ("\uf7ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-radiation
      /// </summary>
      [Description("circle-radiation")]
      public const string CircleRadiation = "\uf7ba";

      /// <summary>
      /// fa-circle-right unicode value ("\uf35a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-right
      /// </summary>
      [Description("circle-right")]
      public const string CircleRight = "\uf35a";

      /// <summary>
      /// fa-circle-s unicode value ("\ue121").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-s
      /// </summary>
      [Description("circle-s")]
      public const string CircleS = "\ue121";

      /// <summary>
      /// fa-circle-small unicode value ("\ue122").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-small
      /// </summary>
      [Description("circle-small")]
      public const string CircleSmall = "\ue122";

      /// <summary>
      /// fa-circle-sort unicode value ("\ue030").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-sort
      /// </summary>
      [Description("circle-sort")]
      public const string CircleSort = "\ue030";

      /// <summary>
      /// fa-circle-sort-down unicode value ("\ue031").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-sort-down
      /// </summary>
      [Description("circle-sort-down")]
      public const string CircleSortDown = "\ue031";

      /// <summary>
      /// fa-circle-sort-up unicode value ("\ue032").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-sort-up
      /// </summary>
      [Description("circle-sort-up")]
      public const string CircleSortUp = "\ue032";

      /// <summary>
      /// fa-circle-star unicode value ("\ue123").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-star
      /// </summary>
      [Description("circle-star")]
      public const string CircleStar = "\ue123";

      /// <summary>
      /// fa-circle-sterling unicode value ("\ue5cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-sterling
      /// </summary>
      [Description("circle-sterling")]
      public const string CircleSterling = "\ue5cf";

      /// <summary>
      /// fa-circle-stop unicode value ("\uf28d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-stop
      /// </summary>
      [Description("circle-stop")]
      public const string CircleStop = "\uf28d";

      /// <summary>
      /// fa-circle-t unicode value ("\ue124").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-t
      /// </summary>
      [Description("circle-t")]
      public const string CircleT = "\ue124";

      /// <summary>
      /// fa-circle-three-quarters unicode value ("\ue125").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-three-quarters
      /// </summary>
      [Description("circle-three-quarters")]
      public const string CircleThreeQuarters = "\ue125";

      /// <summary>
      /// fa-circle-three-quarters-stroke unicode value ("\ue5d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-three-quarters-stroke
      /// </summary>
      [Description("circle-three-quarters-stroke")]
      public const string CircleThreeQuartersStroke = "\ue5d4";

      /// <summary>
      /// fa-circle-trash unicode value ("\ue126").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-trash
      /// </summary>
      [Description("circle-trash")]
      public const string CircleTrash = "\ue126";

      /// <summary>
      /// fa-circle-u unicode value ("\ue127").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-u
      /// </summary>
      [Description("circle-u")]
      public const string CircleU = "\ue127";

      /// <summary>
      /// fa-circle-up unicode value ("\uf35b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-up
      /// </summary>
      [Description("circle-up")]
      public const string CircleUp = "\uf35b";

      /// <summary>
      /// fa-circle-up-left unicode value ("\ue128").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-up-left
      /// </summary>
      [Description("circle-up-left")]
      public const string CircleUpLeft = "\ue128";

      /// <summary>
      /// fa-circle-up-right unicode value ("\ue129").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-up-right
      /// </summary>
      [Description("circle-up-right")]
      public const string CircleUpRight = "\ue129";

      /// <summary>
      /// fa-circle-user unicode value ("\uf2bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-user
      /// </summary>
      [Description("circle-user")]
      public const string CircleUser = "\uf2bd";

      /// <summary>
      /// fa-circle-v unicode value ("\ue12a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-v
      /// </summary>
      [Description("circle-v")]
      public const string CircleV = "\ue12a";

      /// <summary>
      /// fa-circle-video unicode value ("\ue12b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-video
      /// </summary>
      [Description("circle-video")]
      public const string CircleVideo = "\ue12b";

      /// <summary>
      /// fa-circle-w unicode value ("\ue12c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-w
      /// </summary>
      [Description("circle-w")]
      public const string CircleW = "\ue12c";

      /// <summary>
      /// fa-circle-waveform-lines unicode value ("\ue12d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-waveform-lines
      /// </summary>
      [Description("circle-waveform-lines")]
      public const string CircleWaveformLines = "\ue12d";

      /// <summary>
      /// fa-circle-x unicode value ("\ue12e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-x
      /// </summary>
      [Description("circle-x")]
      public const string CircleX = "\ue12e";

      /// <summary>
      /// fa-circle-xmark unicode value ("\uf057").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-xmark
      /// </summary>
      [Description("circle-xmark")]
      public const string CircleXmark = "\uf057";

      /// <summary>
      /// fa-circle-y unicode value ("\ue12f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-y
      /// </summary>
      [Description("circle-y")]
      public const string CircleY = "\ue12f";

      /// <summary>
      /// fa-circle-yen unicode value ("\ue5d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-yen
      /// </summary>
      [Description("circle-yen")]
      public const string CircleYen = "\ue5d0";

      /// <summary>
      /// fa-circle-z unicode value ("\ue130").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circle-z
      /// </summary>
      [Description("circle-z")]
      public const string CircleZ = "\ue130";

      /// <summary>
      /// fa-circles-overlap unicode value ("\ue600").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/circles-overlap
      /// </summary>
      [Description("circles-overlap")]
      public const string CirclesOverlap = "\ue600";

      /// <summary>
      /// fa-citrus unicode value ("\ue2f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/citrus
      /// </summary>
      [Description("citrus")]
      public const string Citrus = "\ue2f4";

      /// <summary>
      /// fa-citrus-slice unicode value ("\ue2f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/citrus-slice
      /// </summary>
      [Description("citrus-slice")]
      public const string CitrusSlice = "\ue2f5";

      /// <summary>
      /// fa-city unicode value ("\uf64f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/city
      /// </summary>
      [Description("city")]
      public const string City = "\uf64f";

      /// <summary>
      /// fa-clapperboard unicode value ("\ue131").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clapperboard
      /// </summary>
      [Description("clapperboard")]
      public const string Clapperboard = "\ue131";

      /// <summary>
      /// fa-clapperboard-play unicode value ("\ue132").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clapperboard-play
      /// </summary>
      [Description("clapperboard-play")]
      public const string ClapperboardPlay = "\ue132";

      /// <summary>
      /// fa-clarinet unicode value ("\uf8ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clarinet
      /// </summary>
      [Description("clarinet")]
      public const string Clarinet = "\uf8ad";

      /// <summary>
      /// fa-claw-marks unicode value ("\uf6c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/claw-marks
      /// </summary>
      [Description("claw-marks")]
      public const string ClawMarks = "\uf6c2";

      /// <summary>
      /// fa-clipboard unicode value ("\uf328").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard
      /// </summary>
      [Description("clipboard")]
      public const string Clipboard = "\uf328";

      /// <summary>
      /// fa-clipboard-check unicode value ("\uf46c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-check
      /// </summary>
      [Description("clipboard-check")]
      public const string ClipboardCheck = "\uf46c";

      /// <summary>
      /// fa-clipboard-list unicode value ("\uf46d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-list
      /// </summary>
      [Description("clipboard-list")]
      public const string ClipboardList = "\uf46d";

      /// <summary>
      /// fa-clipboard-list-check unicode value ("\uf737").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-list-check
      /// </summary>
      [Description("clipboard-list-check")]
      public const string ClipboardListCheck = "\uf737";

      /// <summary>
      /// fa-clipboard-medical unicode value ("\ue133").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-medical
      /// </summary>
      [Description("clipboard-medical")]
      public const string ClipboardMedical = "\ue133";

      /// <summary>
      /// fa-clipboard-prescription unicode value ("\uf5e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-prescription
      /// </summary>
      [Description("clipboard-prescription")]
      public const string ClipboardPrescription = "\uf5e8";

      /// <summary>
      /// fa-clipboard-question unicode value ("\ue4e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-question
      /// </summary>
      [Description("clipboard-question")]
      public const string ClipboardQuestion = "\ue4e3";

      /// <summary>
      /// fa-clipboard-user unicode value ("\uf7f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clipboard-user
      /// </summary>
      [Description("clipboard-user")]
      public const string ClipboardUser = "\uf7f3";

      /// <summary>
      /// fa-clock unicode value ("\uf017").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock
      /// </summary>
      [Description("clock")]
      public const string Clock = "\uf017";

      /// <summary>
      /// fa-clock-desk unicode value ("\ue134").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-desk
      /// </summary>
      [Description("clock-desk")]
      public const string ClockDesk = "\ue134";

      /// <summary>
      /// fa-clock-eight unicode value ("\ue345").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-eight
      /// </summary>
      [Description("clock-eight")]
      public const string ClockEight = "\ue345";

      /// <summary>
      /// fa-clock-eight-thirty unicode value ("\ue346").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-eight-thirty
      /// </summary>
      [Description("clock-eight-thirty")]
      public const string ClockEightThirty = "\ue346";

      /// <summary>
      /// fa-clock-eleven unicode value ("\ue347").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-eleven
      /// </summary>
      [Description("clock-eleven")]
      public const string ClockEleven = "\ue347";

      /// <summary>
      /// fa-clock-eleven-thirty unicode value ("\ue348").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-eleven-thirty
      /// </summary>
      [Description("clock-eleven-thirty")]
      public const string ClockElevenThirty = "\ue348";

      /// <summary>
      /// fa-clock-five unicode value ("\ue349").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-five
      /// </summary>
      [Description("clock-five")]
      public const string ClockFive = "\ue349";

      /// <summary>
      /// fa-clock-five-thirty unicode value ("\ue34a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-five-thirty
      /// </summary>
      [Description("clock-five-thirty")]
      public const string ClockFiveThirty = "\ue34a";

      /// <summary>
      /// fa-clock-four-thirty unicode value ("\ue34b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-four-thirty
      /// </summary>
      [Description("clock-four-thirty")]
      public const string ClockFourThirty = "\ue34b";

      /// <summary>
      /// fa-clock-nine unicode value ("\ue34c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-nine
      /// </summary>
      [Description("clock-nine")]
      public const string ClockNine = "\ue34c";

      /// <summary>
      /// fa-clock-nine-thirty unicode value ("\ue34d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-nine-thirty
      /// </summary>
      [Description("clock-nine-thirty")]
      public const string ClockNineThirty = "\ue34d";

      /// <summary>
      /// fa-clock-one unicode value ("\ue34e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-one
      /// </summary>
      [Description("clock-one")]
      public const string ClockOne = "\ue34e";

      /// <summary>
      /// fa-clock-one-thirty unicode value ("\ue34f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-one-thirty
      /// </summary>
      [Description("clock-one-thirty")]
      public const string ClockOneThirty = "\ue34f";

      /// <summary>
      /// fa-clock-rotate-left unicode value ("\uf1da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-rotate-left
      /// </summary>
      [Description("clock-rotate-left")]
      public const string ClockRotateLeft = "\uf1da";

      /// <summary>
      /// fa-clock-seven unicode value ("\ue350").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-seven
      /// </summary>
      [Description("clock-seven")]
      public const string ClockSeven = "\ue350";

      /// <summary>
      /// fa-clock-seven-thirty unicode value ("\ue351").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-seven-thirty
      /// </summary>
      [Description("clock-seven-thirty")]
      public const string ClockSevenThirty = "\ue351";

      /// <summary>
      /// fa-clock-six unicode value ("\ue352").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-six
      /// </summary>
      [Description("clock-six")]
      public const string ClockSix = "\ue352";

      /// <summary>
      /// fa-clock-six-thirty unicode value ("\ue353").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-six-thirty
      /// </summary>
      [Description("clock-six-thirty")]
      public const string ClockSixThirty = "\ue353";

      /// <summary>
      /// fa-clock-ten unicode value ("\ue354").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-ten
      /// </summary>
      [Description("clock-ten")]
      public const string ClockTen = "\ue354";

      /// <summary>
      /// fa-clock-ten-thirty unicode value ("\ue355").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-ten-thirty
      /// </summary>
      [Description("clock-ten-thirty")]
      public const string ClockTenThirty = "\ue355";

      /// <summary>
      /// fa-clock-three unicode value ("\ue356").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-three
      /// </summary>
      [Description("clock-three")]
      public const string ClockThree = "\ue356";

      /// <summary>
      /// fa-clock-three-thirty unicode value ("\ue357").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-three-thirty
      /// </summary>
      [Description("clock-three-thirty")]
      public const string ClockThreeThirty = "\ue357";

      /// <summary>
      /// fa-clock-twelve unicode value ("\ue358").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-twelve
      /// </summary>
      [Description("clock-twelve")]
      public const string ClockTwelve = "\ue358";

      /// <summary>
      /// fa-clock-twelve-thirty unicode value ("\ue359").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-twelve-thirty
      /// </summary>
      [Description("clock-twelve-thirty")]
      public const string ClockTwelveThirty = "\ue359";

      /// <summary>
      /// fa-clock-two unicode value ("\ue35a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-two
      /// </summary>
      [Description("clock-two")]
      public const string ClockTwo = "\ue35a";

      /// <summary>
      /// fa-clock-two-thirty unicode value ("\ue35b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clock-two-thirty
      /// </summary>
      [Description("clock-two-thirty")]
      public const string ClockTwoThirty = "\ue35b";

      /// <summary>
      /// fa-clone unicode value ("\uf24d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clone
      /// </summary>
      [Description("clone")]
      public const string Clone = "\uf24d";

      /// <summary>
      /// fa-closed-captioning unicode value ("\uf20a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/closed-captioning
      /// </summary>
      [Description("closed-captioning")]
      public const string ClosedCaptioning = "\uf20a";

      /// <summary>
      /// fa-closed-captioning-slash unicode value ("\ue135").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/closed-captioning-slash
      /// </summary>
      [Description("closed-captioning-slash")]
      public const string ClosedCaptioningSlash = "\ue135";

      /// <summary>
      /// fa-clothes-hanger unicode value ("\ue136").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clothes-hanger
      /// </summary>
      [Description("clothes-hanger")]
      public const string ClothesHanger = "\ue136";

      /// <summary>
      /// fa-cloud unicode value ("\uf0c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud
      /// </summary>
      [Description("cloud")]
      public const string Cloud = "\uf0c2";

      /// <summary>
      /// fa-cloud-arrow-down unicode value ("\uf0ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-arrow-down
      /// </summary>
      [Description("cloud-arrow-down")]
      public const string CloudArrowDown = "\uf0ed";

      /// <summary>
      /// fa-cloud-arrow-up unicode value ("\uf0ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-arrow-up
      /// </summary>
      [Description("cloud-arrow-up")]
      public const string CloudArrowUp = "\uf0ee";

      /// <summary>
      /// fa-cloud-binary unicode value ("\ue601").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-binary
      /// </summary>
      [Description("cloud-binary")]
      public const string CloudBinary = "\ue601";

      /// <summary>
      /// fa-cloud-bolt unicode value ("\uf76c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-bolt
      /// </summary>
      [Description("cloud-bolt")]
      public const string CloudBolt = "\uf76c";

      /// <summary>
      /// fa-cloud-bolt-moon unicode value ("\uf76d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-bolt-moon
      /// </summary>
      [Description("cloud-bolt-moon")]
      public const string CloudBoltMoon = "\uf76d";

      /// <summary>
      /// fa-cloud-bolt-sun unicode value ("\uf76e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-bolt-sun
      /// </summary>
      [Description("cloud-bolt-sun")]
      public const string CloudBoltSun = "\uf76e";

      /// <summary>
      /// fa-cloud-check unicode value ("\ue35c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-check
      /// </summary>
      [Description("cloud-check")]
      public const string CloudCheck = "\ue35c";

      /// <summary>
      /// fa-cloud-drizzle unicode value ("\uf738").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-drizzle
      /// </summary>
      [Description("cloud-drizzle")]
      public const string CloudDrizzle = "\uf738";

      /// <summary>
      /// fa-cloud-exclamation unicode value ("\ue491").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-exclamation
      /// </summary>
      [Description("cloud-exclamation")]
      public const string CloudExclamation = "\ue491";

      /// <summary>
      /// fa-cloud-fog unicode value ("\uf74e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-fog
      /// </summary>
      [Description("cloud-fog")]
      public const string CloudFog = "\uf74e";

      /// <summary>
      /// fa-cloud-hail unicode value ("\uf739").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-hail
      /// </summary>
      [Description("cloud-hail")]
      public const string CloudHail = "\uf739";

      /// <summary>
      /// fa-cloud-hail-mixed unicode value ("\uf73a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-hail-mixed
      /// </summary>
      [Description("cloud-hail-mixed")]
      public const string CloudHailMixed = "\uf73a";

      /// <summary>
      /// fa-cloud-meatball unicode value ("\uf73b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-meatball
      /// </summary>
      [Description("cloud-meatball")]
      public const string CloudMeatball = "\uf73b";

      /// <summary>
      /// fa-cloud-minus unicode value ("\ue35d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-minus
      /// </summary>
      [Description("cloud-minus")]
      public const string CloudMinus = "\ue35d";

      /// <summary>
      /// fa-cloud-moon unicode value ("\uf6c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-moon
      /// </summary>
      [Description("cloud-moon")]
      public const string CloudMoon = "\uf6c3";

      /// <summary>
      /// fa-cloud-moon-rain unicode value ("\uf73c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-moon-rain
      /// </summary>
      [Description("cloud-moon-rain")]
      public const string CloudMoonRain = "\uf73c";

      /// <summary>
      /// fa-cloud-music unicode value ("\uf8ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-music
      /// </summary>
      [Description("cloud-music")]
      public const string CloudMusic = "\uf8ae";

      /// <summary>
      /// fa-cloud-plus unicode value ("\ue35e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-plus
      /// </summary>
      [Description("cloud-plus")]
      public const string CloudPlus = "\ue35e";

      /// <summary>
      /// fa-cloud-question unicode value ("\ue492").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-question
      /// </summary>
      [Description("cloud-question")]
      public const string CloudQuestion = "\ue492";

      /// <summary>
      /// fa-cloud-rain unicode value ("\uf73d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-rain
      /// </summary>
      [Description("cloud-rain")]
      public const string CloudRain = "\uf73d";

      /// <summary>
      /// fa-cloud-rainbow unicode value ("\uf73e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-rainbow
      /// </summary>
      [Description("cloud-rainbow")]
      public const string CloudRainbow = "\uf73e";

      /// <summary>
      /// fa-cloud-showers unicode value ("\uf73f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-showers
      /// </summary>
      [Description("cloud-showers")]
      public const string CloudShowers = "\uf73f";

      /// <summary>
      /// fa-cloud-showers-heavy unicode value ("\uf740").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-showers-heavy
      /// </summary>
      [Description("cloud-showers-heavy")]
      public const string CloudShowersHeavy = "\uf740";

      /// <summary>
      /// fa-cloud-showers-water unicode value ("\ue4e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-showers-water
      /// </summary>
      [Description("cloud-showers-water")]
      public const string CloudShowersWater = "\ue4e4";

      /// <summary>
      /// fa-cloud-slash unicode value ("\ue137").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-slash
      /// </summary>
      [Description("cloud-slash")]
      public const string CloudSlash = "\ue137";

      /// <summary>
      /// fa-cloud-sleet unicode value ("\uf741").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-sleet
      /// </summary>
      [Description("cloud-sleet")]
      public const string CloudSleet = "\uf741";

      /// <summary>
      /// fa-cloud-snow unicode value ("\uf742").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-snow
      /// </summary>
      [Description("cloud-snow")]
      public const string CloudSnow = "\uf742";

      /// <summary>
      /// fa-cloud-sun unicode value ("\uf6c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-sun
      /// </summary>
      [Description("cloud-sun")]
      public const string CloudSun = "\uf6c4";

      /// <summary>
      /// fa-cloud-sun-rain unicode value ("\uf743").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-sun-rain
      /// </summary>
      [Description("cloud-sun-rain")]
      public const string CloudSunRain = "\uf743";

      /// <summary>
      /// fa-cloud-word unicode value ("\ue138").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-word
      /// </summary>
      [Description("cloud-word")]
      public const string CloudWord = "\ue138";

      /// <summary>
      /// fa-cloud-xmark unicode value ("\ue35f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloud-xmark
      /// </summary>
      [Description("cloud-xmark")]
      public const string CloudXmark = "\ue35f";

      /// <summary>
      /// fa-cloudflare unicode value ("\ue07d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloudflare
      /// </summary>
      [Description("cloudflare")]
      public const string Cloudflare = "\ue07d";

      /// <summary>
      /// fa-clouds unicode value ("\uf744").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clouds
      /// </summary>
      [Description("clouds")]
      public const string Clouds = "\uf744";

      /// <summary>
      /// fa-clouds-moon unicode value ("\uf745").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clouds-moon
      /// </summary>
      [Description("clouds-moon")]
      public const string CloudsMoon = "\uf745";

      /// <summary>
      /// fa-clouds-sun unicode value ("\uf746").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clouds-sun
      /// </summary>
      [Description("clouds-sun")]
      public const string CloudsSun = "\uf746";

      /// <summary>
      /// fa-cloudscale unicode value ("\uf383").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloudscale
      /// </summary>
      [Description("cloudscale")]
      public const string Cloudscale = "\uf383";

      /// <summary>
      /// fa-cloudsmith unicode value ("\uf384").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloudsmith
      /// </summary>
      [Description("cloudsmith")]
      public const string Cloudsmith = "\uf384";

      /// <summary>
      /// fa-cloudversify unicode value ("\uf385").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cloudversify
      /// </summary>
      [Description("cloudversify")]
      public const string Cloudversify = "\uf385";

      /// <summary>
      /// fa-clover unicode value ("\ue139").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/clover
      /// </summary>
      [Description("clover")]
      public const string Clover = "\ue139";

      /// <summary>
      /// fa-club unicode value ("\uf327").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/club
      /// </summary>
      [Description("club")]
      public const string Club = "\uf327";

      /// <summary>
      /// fa-cmplid unicode value ("\ue360").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/cmplid
      /// </summary>
      [Description("cmplid")]
      public const string Cmplid = "\ue360";

      /// <summary>
      /// fa-coconut unicode value ("\ue2f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coconut
      /// </summary>
      [Description("coconut")]
      public const string Coconut = "\ue2f6";

      /// <summary>
      /// fa-code unicode value ("\uf121").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code
      /// </summary>
      [Description("code")]
      public const string Code = "\uf121";

      /// <summary>
      /// fa-code-branch unicode value ("\uf126").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-branch
      /// </summary>
      [Description("code-branch")]
      public const string CodeBranch = "\uf126";

      /// <summary>
      /// fa-code-commit unicode value ("\uf386").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-commit
      /// </summary>
      [Description("code-commit")]
      public const string CodeCommit = "\uf386";

      /// <summary>
      /// fa-code-compare unicode value ("\ue13a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-compare
      /// </summary>
      [Description("code-compare")]
      public const string CodeCompare = "\ue13a";

      /// <summary>
      /// fa-code-fork unicode value ("\ue13b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-fork
      /// </summary>
      [Description("code-fork")]
      public const string CodeFork = "\ue13b";

      /// <summary>
      /// fa-code-merge unicode value ("\uf387").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-merge
      /// </summary>
      [Description("code-merge")]
      public const string CodeMerge = "\uf387";

      /// <summary>
      /// fa-code-pull-request unicode value ("\ue13c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-pull-request
      /// </summary>
      [Description("code-pull-request")]
      public const string CodePullRequest = "\ue13c";

      /// <summary>
      /// fa-code-pull-request-closed unicode value ("\ue3f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-pull-request-closed
      /// </summary>
      [Description("code-pull-request-closed")]
      public const string CodePullRequestClosed = "\ue3f9";

      /// <summary>
      /// fa-code-pull-request-draft unicode value ("\ue3fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-pull-request-draft
      /// </summary>
      [Description("code-pull-request-draft")]
      public const string CodePullRequestDraft = "\ue3fa";

      /// <summary>
      /// fa-code-simple unicode value ("\ue13d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/code-simple
      /// </summary>
      [Description("code-simple")]
      public const string CodeSimple = "\ue13d";

      /// <summary>
      /// fa-codepen unicode value ("\uf1cb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/codepen
      /// </summary>
      [Description("codepen")]
      public const string Codepen = "\uf1cb";

      /// <summary>
      /// fa-codiepie unicode value ("\uf284").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/codiepie
      /// </summary>
      [Description("codiepie")]
      public const string Codiepie = "\uf284";

      /// <summary>
      /// fa-coffee-bean unicode value ("\ue13e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coffee-bean
      /// </summary>
      [Description("coffee-bean")]
      public const string CoffeeBean = "\ue13e";

      /// <summary>
      /// fa-coffee-beans unicode value ("\ue13f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coffee-beans
      /// </summary>
      [Description("coffee-beans")]
      public const string CoffeeBeans = "\ue13f";

      /// <summary>
      /// fa-coffee-pot unicode value ("\ue002").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coffee-pot
      /// </summary>
      [Description("coffee-pot")]
      public const string CoffeePot = "\ue002";

      /// <summary>
      /// fa-coffin unicode value ("\uf6c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coffin
      /// </summary>
      [Description("coffin")]
      public const string Coffin = "\uf6c6";

      /// <summary>
      /// fa-coffin-cross unicode value ("\ue051").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coffin-cross
      /// </summary>
      [Description("coffin-cross")]
      public const string CoffinCross = "\ue051";

      /// <summary>
      /// fa-coin unicode value ("\uf85c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coin
      /// </summary>
      [Description("coin")]
      public const string Coin = "\uf85c";

      /// <summary>
      /// fa-coin-blank unicode value ("\ue3fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coin-blank
      /// </summary>
      [Description("coin-blank")]
      public const string CoinBlank = "\ue3fb";

      /// <summary>
      /// fa-coin-front unicode value ("\ue3fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coin-front
      /// </summary>
      [Description("coin-front")]
      public const string CoinFront = "\ue3fc";

      /// <summary>
      /// fa-coin-vertical unicode value ("\ue3fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coin-vertical
      /// </summary>
      [Description("coin-vertical")]
      public const string CoinVertical = "\ue3fd";

      /// <summary>
      /// fa-coins unicode value ("\uf51e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/coins
      /// </summary>
      [Description("coins")]
      public const string Coins = "\uf51e";

      /// <summary>
      /// fa-colon unicode value ("\u003a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/colon
      /// </summary>
      [Description("colon")]
      public const string Colon = "\u003a";

      /// <summary>
      /// fa-colon-sign unicode value ("\ue140").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/colon-sign
      /// </summary>
      [Description("colon-sign")]
      public const string ColonSign = "\ue140";

      /// <summary>
      /// fa-columns-3 unicode value ("\ue361").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/columns-3
      /// </summary>
      [Description("columns-3")]
      public const string Columns3 = "\ue361";

      /// <summary>
      /// fa-comet unicode value ("\ue003").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comet
      /// </summary>
      [Description("comet")]
      public const string Comet = "\ue003";

      /// <summary>
      /// fa-comma unicode value ("\u002c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comma
      /// </summary>
      [Description("comma")]
      public const string Comma = "\u002c";

      /// <summary>
      /// fa-command unicode value ("\ue142").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/command
      /// </summary>
      [Description("command")]
      public const string Command = "\ue142";

      /// <summary>
      /// fa-comment unicode value ("\uf075").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment
      /// </summary>
      [Description("comment")]
      public const string Comment = "\uf075";

      /// <summary>
      /// fa-comment-arrow-down unicode value ("\ue143").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-arrow-down
      /// </summary>
      [Description("comment-arrow-down")]
      public const string CommentArrowDown = "\ue143";

      /// <summary>
      /// fa-comment-arrow-up unicode value ("\ue144").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-arrow-up
      /// </summary>
      [Description("comment-arrow-up")]
      public const string CommentArrowUp = "\ue144";

      /// <summary>
      /// fa-comment-arrow-up-right unicode value ("\ue145").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-arrow-up-right
      /// </summary>
      [Description("comment-arrow-up-right")]
      public const string CommentArrowUpRight = "\ue145";

      /// <summary>
      /// fa-comment-captions unicode value ("\ue146").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-captions
      /// </summary>
      [Description("comment-captions")]
      public const string CommentCaptions = "\ue146";

      /// <summary>
      /// fa-comment-check unicode value ("\uf4ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-check
      /// </summary>
      [Description("comment-check")]
      public const string CommentCheck = "\uf4ac";

      /// <summary>
      /// fa-comment-code unicode value ("\ue147").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-code
      /// </summary>
      [Description("comment-code")]
      public const string CommentCode = "\ue147";

      /// <summary>
      /// fa-comment-dollar unicode value ("\uf651").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-dollar
      /// </summary>
      [Description("comment-dollar")]
      public const string CommentDollar = "\uf651";

      /// <summary>
      /// fa-comment-dots unicode value ("\uf4ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-dots
      /// </summary>
      [Description("comment-dots")]
      public const string CommentDots = "\uf4ad";

      /// <summary>
      /// fa-comment-exclamation unicode value ("\uf4af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-exclamation
      /// </summary>
      [Description("comment-exclamation")]
      public const string CommentExclamation = "\uf4af";

      /// <summary>
      /// fa-comment-heart unicode value ("\ue5c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-heart
      /// </summary>
      [Description("comment-heart")]
      public const string CommentHeart = "\ue5c8";

      /// <summary>
      /// fa-comment-image unicode value ("\ue148").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-image
      /// </summary>
      [Description("comment-image")]
      public const string CommentImage = "\ue148";

      /// <summary>
      /// fa-comment-lines unicode value ("\uf4b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-lines
      /// </summary>
      [Description("comment-lines")]
      public const string CommentLines = "\uf4b0";

      /// <summary>
      /// fa-comment-medical unicode value ("\uf7f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-medical
      /// </summary>
      [Description("comment-medical")]
      public const string CommentMedical = "\uf7f5";

      /// <summary>
      /// fa-comment-middle unicode value ("\ue149").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-middle
      /// </summary>
      [Description("comment-middle")]
      public const string CommentMiddle = "\ue149";

      /// <summary>
      /// fa-comment-middle-top unicode value ("\ue14a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-middle-top
      /// </summary>
      [Description("comment-middle-top")]
      public const string CommentMiddleTop = "\ue14a";

      /// <summary>
      /// fa-comment-minus unicode value ("\uf4b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-minus
      /// </summary>
      [Description("comment-minus")]
      public const string CommentMinus = "\uf4b1";

      /// <summary>
      /// fa-comment-music unicode value ("\uf8b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-music
      /// </summary>
      [Description("comment-music")]
      public const string CommentMusic = "\uf8b0";

      /// <summary>
      /// fa-comment-pen unicode value ("\uf4ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-pen
      /// </summary>
      [Description("comment-pen")]
      public const string CommentPen = "\uf4ae";

      /// <summary>
      /// fa-comment-plus unicode value ("\uf4b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-plus
      /// </summary>
      [Description("comment-plus")]
      public const string CommentPlus = "\uf4b2";

      /// <summary>
      /// fa-comment-question unicode value ("\ue14b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-question
      /// </summary>
      [Description("comment-question")]
      public const string CommentQuestion = "\ue14b";

      /// <summary>
      /// fa-comment-quote unicode value ("\ue14c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-quote
      /// </summary>
      [Description("comment-quote")]
      public const string CommentQuote = "\ue14c";

      /// <summary>
      /// fa-comment-slash unicode value ("\uf4b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-slash
      /// </summary>
      [Description("comment-slash")]
      public const string CommentSlash = "\uf4b3";

      /// <summary>
      /// fa-comment-smile unicode value ("\uf4b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-smile
      /// </summary>
      [Description("comment-smile")]
      public const string CommentSmile = "\uf4b4";

      /// <summary>
      /// fa-comment-sms unicode value ("\uf7cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-sms
      /// </summary>
      [Description("comment-sms")]
      public const string CommentSms = "\uf7cd";

      /// <summary>
      /// fa-comment-text unicode value ("\ue14d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-text
      /// </summary>
      [Description("comment-text")]
      public const string CommentText = "\ue14d";

      /// <summary>
      /// fa-comment-xmark unicode value ("\uf4b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comment-xmark
      /// </summary>
      [Description("comment-xmark")]
      public const string CommentXmark = "\uf4b5";

      /// <summary>
      /// fa-comments unicode value ("\uf086").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comments
      /// </summary>
      [Description("comments")]
      public const string Comments = "\uf086";

      /// <summary>
      /// fa-comments-dollar unicode value ("\uf653").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comments-dollar
      /// </summary>
      [Description("comments-dollar")]
      public const string CommentsDollar = "\uf653";

      /// <summary>
      /// fa-comments-question unicode value ("\ue14e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comments-question
      /// </summary>
      [Description("comments-question")]
      public const string CommentsQuestion = "\ue14e";

      /// <summary>
      /// fa-comments-question-check unicode value ("\ue14f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/comments-question-check
      /// </summary>
      [Description("comments-question-check")]
      public const string CommentsQuestionCheck = "\ue14f";

      /// <summary>
      /// fa-compact-disc unicode value ("\uf51f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compact-disc
      /// </summary>
      [Description("compact-disc")]
      public const string CompactDisc = "\uf51f";

      /// <summary>
      /// fa-compass unicode value ("\uf14e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compass
      /// </summary>
      [Description("compass")]
      public const string Compass = "\uf14e";

      /// <summary>
      /// fa-compass-drafting unicode value ("\uf568").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compass-drafting
      /// </summary>
      [Description("compass-drafting")]
      public const string CompassDrafting = "\uf568";

      /// <summary>
      /// fa-compass-slash unicode value ("\uf5e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compass-slash
      /// </summary>
      [Description("compass-slash")]
      public const string CompassSlash = "\uf5e9";

      /// <summary>
      /// fa-compress unicode value ("\uf066").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compress
      /// </summary>
      [Description("compress")]
      public const string Compress = "\uf066";

      /// <summary>
      /// fa-compress-wide unicode value ("\uf326").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/compress-wide
      /// </summary>
      [Description("compress-wide")]
      public const string CompressWide = "\uf326";

      /// <summary>
      /// fa-computer unicode value ("\ue4e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/computer
      /// </summary>
      [Description("computer")]
      public const string Computer = "\ue4e5";

      /// <summary>
      /// fa-computer-classic unicode value ("\uf8b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/computer-classic
      /// </summary>
      [Description("computer-classic")]
      public const string ComputerClassic = "\uf8b1";

      /// <summary>
      /// fa-computer-mouse unicode value ("\uf8cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/computer-mouse
      /// </summary>
      [Description("computer-mouse")]
      public const string ComputerMouse = "\uf8cc";

      /// <summary>
      /// fa-computer-mouse-scrollwheel unicode value ("\uf8cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/computer-mouse-scrollwheel
      /// </summary>
      [Description("computer-mouse-scrollwheel")]
      public const string ComputerMouseScrollwheel = "\uf8cd";

      /// <summary>
      /// fa-computer-speaker unicode value ("\uf8b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/computer-speaker
      /// </summary>
      [Description("computer-speaker")]
      public const string ComputerSpeaker = "\uf8b2";

      /// <summary>
      /// fa-confluence unicode value ("\uf78d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/confluence
      /// </summary>
      [Description("confluence")]
      public const string Confluence = "\uf78d";

      /// <summary>
      /// fa-connectdevelop unicode value ("\uf20e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/connectdevelop
      /// </summary>
      [Description("connectdevelop")]
      public const string Connectdevelop = "\uf20e";

      /// <summary>
      /// fa-container-storage unicode value ("\uf4b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/container-storage
      /// </summary>
      [Description("container-storage")]
      public const string ContainerStorage = "\uf4b7";

      /// <summary>
      /// fa-contao unicode value ("\uf26d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/contao
      /// </summary>
      [Description("contao")]
      public const string Contao = "\uf26d";

      /// <summary>
      /// fa-conveyor-belt unicode value ("\uf46e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/conveyor-belt
      /// </summary>
      [Description("conveyor-belt")]
      public const string ConveyorBelt = "\uf46e";

      /// <summary>
      /// fa-conveyor-belt-arm unicode value ("\ue5f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/conveyor-belt-arm
      /// </summary>
      [Description("conveyor-belt-arm")]
      public const string ConveyorBeltArm = "\ue5f8";

      /// <summary>
      /// fa-conveyor-belt-boxes unicode value ("\uf46f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/conveyor-belt-boxes
      /// </summary>
      [Description("conveyor-belt-boxes")]
      public const string ConveyorBeltBoxes = "\uf46f";

      /// <summary>
      /// fa-conveyor-belt-empty unicode value ("\ue150").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/conveyor-belt-empty
      /// </summary>
      [Description("conveyor-belt-empty")]
      public const string ConveyorBeltEmpty = "\ue150";

      /// <summary>
      /// fa-cookie unicode value ("\uf563").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cookie
      /// </summary>
      [Description("cookie")]
      public const string Cookie = "\uf563";

      /// <summary>
      /// fa-cookie-bite unicode value ("\uf564").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cookie-bite
      /// </summary>
      [Description("cookie-bite")]
      public const string CookieBite = "\uf564";

      /// <summary>
      /// fa-copy unicode value ("\uf0c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/copy
      /// </summary>
      [Description("copy")]
      public const string Copy = "\uf0c5";

      /// <summary>
      /// fa-copyright unicode value ("\uf1f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/copyright
      /// </summary>
      [Description("copyright")]
      public const string Copyright = "\uf1f9";

      /// <summary>
      /// fa-corn unicode value ("\uf6c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/corn
      /// </summary>
      [Description("corn")]
      public const string Corn = "\uf6c7";

      /// <summary>
      /// fa-corner unicode value ("\ue3fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/corner
      /// </summary>
      [Description("corner")]
      public const string Corner = "\ue3fe";

      /// <summary>
      /// fa-cotton-bureau unicode value ("\uf89e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.10.0', Last Modified in '5.10.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cotton-bureau
      /// </summary>
      [Description("cotton-bureau")]
      public const string CottonBureau = "\uf89e";

      /// <summary>
      /// fa-couch unicode value ("\uf4b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/couch
      /// </summary>
      [Description("couch")]
      public const string Couch = "\uf4b8";

      /// <summary>
      /// fa-cow unicode value ("\uf6c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cow
      /// </summary>
      [Description("cow")]
      public const string Cow = "\uf6c8";

      /// <summary>
      /// fa-cowbell unicode value ("\uf8b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cowbell
      /// </summary>
      [Description("cowbell")]
      public const string Cowbell = "\uf8b3";

      /// <summary>
      /// fa-cowbell-circle-plus unicode value ("\uf8b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cowbell-circle-plus
      /// </summary>
      [Description("cowbell-circle-plus")]
      public const string CowbellCirclePlus = "\uf8b4";

      /// <summary>
      /// fa-cpanel unicode value ("\uf388").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cpanel
      /// </summary>
      [Description("cpanel")]
      public const string Cpanel = "\uf388";

      /// <summary>
      /// fa-crab unicode value ("\ue3ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crab
      /// </summary>
      [Description("crab")]
      public const string Crab = "\ue3ff";

      /// <summary>
      /// fa-crate-apple unicode value ("\uf6b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crate-apple
      /// </summary>
      [Description("crate-apple")]
      public const string CrateApple = "\uf6b1";

      /// <summary>
      /// fa-crate-empty unicode value ("\ue151").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crate-empty
      /// </summary>
      [Description("crate-empty")]
      public const string CrateEmpty = "\ue151";

      /// <summary>
      /// fa-creative-commons unicode value ("\uf25e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons
      /// </summary>
      [Description("creative-commons")]
      public const string CreativeCommons = "\uf25e";

      /// <summary>
      /// fa-creative-commons-by unicode value ("\uf4e7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-by
      /// </summary>
      [Description("creative-commons-by")]
      public const string CreativeCommonsBy = "\uf4e7";

      /// <summary>
      /// fa-creative-commons-nc unicode value ("\uf4e8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-nc
      /// </summary>
      [Description("creative-commons-nc")]
      public const string CreativeCommonsNc = "\uf4e8";

      /// <summary>
      /// fa-creative-commons-nc-eu unicode value ("\uf4e9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-nc-eu
      /// </summary>
      [Description("creative-commons-nc-eu")]
      public const string CreativeCommonsNcEu = "\uf4e9";

      /// <summary>
      /// fa-creative-commons-nc-jp unicode value ("\uf4ea").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-nc-jp
      /// </summary>
      [Description("creative-commons-nc-jp")]
      public const string CreativeCommonsNcJp = "\uf4ea";

      /// <summary>
      /// fa-creative-commons-nd unicode value ("\uf4eb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-nd
      /// </summary>
      [Description("creative-commons-nd")]
      public const string CreativeCommonsNd = "\uf4eb";

      /// <summary>
      /// fa-creative-commons-pd unicode value ("\uf4ec").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-pd
      /// </summary>
      [Description("creative-commons-pd")]
      public const string CreativeCommonsPd = "\uf4ec";

      /// <summary>
      /// fa-creative-commons-pd-alt unicode value ("\uf4ed").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-pd-alt
      /// </summary>
      [Description("creative-commons-pd-alt")]
      public const string CreativeCommonsPdAlt = "\uf4ed";

      /// <summary>
      /// fa-creative-commons-remix unicode value ("\uf4ee").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-remix
      /// </summary>
      [Description("creative-commons-remix")]
      public const string CreativeCommonsRemix = "\uf4ee";

      /// <summary>
      /// fa-creative-commons-sa unicode value ("\uf4ef").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-sa
      /// </summary>
      [Description("creative-commons-sa")]
      public const string CreativeCommonsSa = "\uf4ef";

      /// <summary>
      /// fa-creative-commons-sampling unicode value ("\uf4f0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-sampling
      /// </summary>
      [Description("creative-commons-sampling")]
      public const string CreativeCommonsSampling = "\uf4f0";

      /// <summary>
      /// fa-creative-commons-sampling-plus unicode value ("\uf4f1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-sampling-plus
      /// </summary>
      [Description("creative-commons-sampling-plus")]
      public const string CreativeCommonsSamplingPlus = "\uf4f1";

      /// <summary>
      /// fa-creative-commons-share unicode value ("\uf4f2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-share
      /// </summary>
      [Description("creative-commons-share")]
      public const string CreativeCommonsShare = "\uf4f2";

      /// <summary>
      /// fa-creative-commons-zero unicode value ("\uf4f3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/creative-commons-zero
      /// </summary>
      [Description("creative-commons-zero")]
      public const string CreativeCommonsZero = "\uf4f3";

      /// <summary>
      /// fa-credit-card unicode value ("\uf09d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/credit-card
      /// </summary>
      [Description("credit-card")]
      public const string CreditCard = "\uf09d";

      /// <summary>
      /// fa-credit-card-blank unicode value ("\uf389").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/credit-card-blank
      /// </summary>
      [Description("credit-card-blank")]
      public const string CreditCardBlank = "\uf389";

      /// <summary>
      /// fa-credit-card-front unicode value ("\uf38a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/credit-card-front
      /// </summary>
      [Description("credit-card-front")]
      public const string CreditCardFront = "\uf38a";

      /// <summary>
      /// fa-cricket-bat-ball unicode value ("\uf449").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cricket-bat-ball
      /// </summary>
      [Description("cricket-bat-ball")]
      public const string CricketBatBall = "\uf449";

      /// <summary>
      /// fa-critical-role unicode value ("\uf6c9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/critical-role
      /// </summary>
      [Description("critical-role")]
      public const string CriticalRole = "\uf6c9";

      /// <summary>
      /// fa-croissant unicode value ("\uf7f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/croissant
      /// </summary>
      [Description("croissant")]
      public const string Croissant = "\uf7f6";

      /// <summary>
      /// fa-crop unicode value ("\uf125").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crop
      /// </summary>
      [Description("crop")]
      public const string Crop = "\uf125";

      /// <summary>
      /// fa-crop-simple unicode value ("\uf565").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crop-simple
      /// </summary>
      [Description("crop-simple")]
      public const string CropSimple = "\uf565";

      /// <summary>
      /// fa-cross unicode value ("\uf654").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cross
      /// </summary>
      [Description("cross")]
      public const string Cross = "\uf654";

      /// <summary>
      /// fa-crosshairs unicode value ("\uf05b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crosshairs
      /// </summary>
      [Description("crosshairs")]
      public const string Crosshairs = "\uf05b";

      /// <summary>
      /// fa-crosshairs-simple unicode value ("\ue59f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crosshairs-simple
      /// </summary>
      [Description("crosshairs-simple")]
      public const string CrosshairsSimple = "\ue59f";

      /// <summary>
      /// fa-crow unicode value ("\uf520").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crow
      /// </summary>
      [Description("crow")]
      public const string Crow = "\uf520";

      /// <summary>
      /// fa-crown unicode value ("\uf521").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crown
      /// </summary>
      [Description("crown")]
      public const string Crown = "\uf521";

      /// <summary>
      /// fa-crutch unicode value ("\uf7f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crutch
      /// </summary>
      [Description("crutch")]
      public const string Crutch = "\uf7f7";

      /// <summary>
      /// fa-crutches unicode value ("\uf7f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crutches
      /// </summary>
      [Description("crutches")]
      public const string Crutches = "\uf7f8";

      /// <summary>
      /// fa-cruzeiro-sign unicode value ("\ue152").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cruzeiro-sign
      /// </summary>
      [Description("cruzeiro-sign")]
      public const string CruzeiroSign = "\ue152";

      /// <summary>
      /// fa-crystal-ball unicode value ("\ue362").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/crystal-ball
      /// </summary>
      [Description("crystal-ball")]
      public const string CrystalBall = "\ue362";

      /// <summary>
      /// fa-css3 unicode value ("\uf13c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/css3
      /// </summary>
      [Description("css3")]
      public const string Css3 = "\uf13c";

      /// <summary>
      /// fa-css3-alt unicode value ("\uf38b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/css3-alt
      /// </summary>
      [Description("css3-alt")]
      public const string Css3Alt = "\uf38b";

      /// <summary>
      /// fa-cube unicode value ("\uf1b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cube
      /// </summary>
      [Description("cube")]
      public const string Cube = "\uf1b2";

      /// <summary>
      /// fa-cubes unicode value ("\uf1b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cubes
      /// </summary>
      [Description("cubes")]
      public const string Cubes = "\uf1b3";

      /// <summary>
      /// fa-cubes-stacked unicode value ("\ue4e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cubes-stacked
      /// </summary>
      [Description("cubes-stacked")]
      public const string CubesStacked = "\ue4e6";

      /// <summary>
      /// fa-cucumber unicode value ("\ue401").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cucumber
      /// </summary>
      [Description("cucumber")]
      public const string Cucumber = "\ue401";

      /// <summary>
      /// fa-cup-straw unicode value ("\ue363").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cup-straw
      /// </summary>
      [Description("cup-straw")]
      public const string CupStraw = "\ue363";

      /// <summary>
      /// fa-cup-straw-swoosh unicode value ("\ue364").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cup-straw-swoosh
      /// </summary>
      [Description("cup-straw-swoosh")]
      public const string CupStrawSwoosh = "\ue364";

      /// <summary>
      /// fa-cup-togo unicode value ("\uf6c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cup-togo
      /// </summary>
      [Description("cup-togo")]
      public const string CupTogo = "\uf6c5";

      /// <summary>
      /// fa-cupcake unicode value ("\ue402").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cupcake
      /// </summary>
      [Description("cupcake")]
      public const string Cupcake = "\ue402";

      /// <summary>
      /// fa-curling-stone unicode value ("\uf44a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/curling-stone
      /// </summary>
      [Description("curling-stone")]
      public const string CurlingStone = "\uf44a";

      /// <summary>
      /// fa-custard unicode value ("\ue403").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/custard
      /// </summary>
      [Description("custard")]
      public const string Custard = "\ue403";

      /// <summary>
      /// fa-cuttlefish unicode value ("\uf38c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/cuttlefish
      /// </summary>
      [Description("cuttlefish")]
      public const string Cuttlefish = "\uf38c";

      /// <summary>
      /// fa-d unicode value ("\u0044").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/d
      /// </summary>
      [Description("d")]
      public const string D = "\u0044";

      /// <summary>
      /// fa-d-and-d unicode value ("\uf38d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/d-and-d
      /// </summary>
      [Description("d-and-d")]
      public const string DAndD = "\uf38d";

      /// <summary>
      /// fa-d-and-d-beyond unicode value ("\uf6ca").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '5.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/d-and-d-beyond
      /// </summary>
      [Description("d-and-d-beyond")]
      public const string DAndDBeyond = "\uf6ca";

      /// <summary>
      /// fa-dagger unicode value ("\uf6cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dagger
      /// </summary>
      [Description("dagger")]
      public const string Dagger = "\uf6cb";

      /// <summary>
      /// fa-dailymotion unicode value ("\ue052").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dailymotion
      /// </summary>
      [Description("dailymotion")]
      public const string Dailymotion = "\ue052";

      /// <summary>
      /// fa-dash unicode value ("\ue404").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dash
      /// </summary>
      [Description("dash")]
      public const string Dash = "\ue404";

      /// <summary>
      /// fa-dashcube unicode value ("\uf210").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/dashcube
      /// </summary>
      [Description("dashcube")]
      public const string Dashcube = "\uf210";

      /// <summary>
      /// fa-database unicode value ("\uf1c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/database
      /// </summary>
      [Description("database")]
      public const string Database = "\uf1c0";

      /// <summary>
      /// fa-deer unicode value ("\uf78e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deer
      /// </summary>
      [Description("deer")]
      public const string Deer = "\uf78e";

      /// <summary>
      /// fa-deer-rudolph unicode value ("\uf78f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deer-rudolph
      /// </summary>
      [Description("deer-rudolph")]
      public const string DeerRudolph = "\uf78f";

      /// <summary>
      /// fa-deezer unicode value ("\ue077").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deezer
      /// </summary>
      [Description("deezer")]
      public const string Deezer = "\ue077";

      /// <summary>
      /// fa-delete-left unicode value ("\uf55a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/delete-left
      /// </summary>
      [Description("delete-left")]
      public const string DeleteLeft = "\uf55a";

      /// <summary>
      /// fa-delete-right unicode value ("\ue154").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/delete-right
      /// </summary>
      [Description("delete-right")]
      public const string DeleteRight = "\ue154";

      /// <summary>
      /// fa-delicious unicode value ("\uf1a5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/delicious
      /// </summary>
      [Description("delicious")]
      public const string Delicious = "\uf1a5";

      /// <summary>
      /// fa-democrat unicode value ("\uf747").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/democrat
      /// </summary>
      [Description("democrat")]
      public const string Democrat = "\uf747";

      /// <summary>
      /// fa-deploydog unicode value ("\uf38e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deploydog
      /// </summary>
      [Description("deploydog")]
      public const string Deploydog = "\uf38e";

      /// <summary>
      /// fa-deskpro unicode value ("\uf38f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deskpro
      /// </summary>
      [Description("deskpro")]
      public const string Deskpro = "\uf38f";

      /// <summary>
      /// fa-desktop unicode value ("\uf390").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/desktop
      /// </summary>
      [Description("desktop")]
      public const string Desktop = "\uf390";

      /// <summary>
      /// fa-desktop-arrow-down unicode value ("\ue155").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/desktop-arrow-down
      /// </summary>
      [Description("desktop-arrow-down")]
      public const string DesktopArrowDown = "\ue155";

      /// <summary>
      /// fa-dev unicode value ("\uf6cc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '5.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dev
      /// </summary>
      [Description("dev")]
      public const string Dev = "\uf6cc";

      /// <summary>
      /// fa-deviantart unicode value ("\uf1bd").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/deviantart
      /// </summary>
      [Description("deviantart")]
      public const string Deviantart = "\uf1bd";

      /// <summary>
      /// fa-dharmachakra unicode value ("\uf655").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dharmachakra
      /// </summary>
      [Description("dharmachakra")]
      public const string Dharmachakra = "\uf655";

      /// <summary>
      /// fa-dhl unicode value ("\uf790").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dhl
      /// </summary>
      [Description("dhl")]
      public const string Dhl = "\uf790";

      /// <summary>
      /// fa-diagram-cells unicode value ("\ue475").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-cells
      /// </summary>
      [Description("diagram-cells")]
      public const string DiagramCells = "\ue475";

      /// <summary>
      /// fa-diagram-lean-canvas unicode value ("\ue156").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-lean-canvas
      /// </summary>
      [Description("diagram-lean-canvas")]
      public const string DiagramLeanCanvas = "\ue156";

      /// <summary>
      /// fa-diagram-nested unicode value ("\ue157").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-nested
      /// </summary>
      [Description("diagram-nested")]
      public const string DiagramNested = "\ue157";

      /// <summary>
      /// fa-diagram-next unicode value ("\ue476").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-next
      /// </summary>
      [Description("diagram-next")]
      public const string DiagramNext = "\ue476";

      /// <summary>
      /// fa-diagram-predecessor unicode value ("\ue477").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-predecessor
      /// </summary>
      [Description("diagram-predecessor")]
      public const string DiagramPredecessor = "\ue477";

      /// <summary>
      /// fa-diagram-previous unicode value ("\ue478").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-previous
      /// </summary>
      [Description("diagram-previous")]
      public const string DiagramPrevious = "\ue478";

      /// <summary>
      /// fa-diagram-project unicode value ("\uf542").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-project
      /// </summary>
      [Description("diagram-project")]
      public const string DiagramProject = "\uf542";

      /// <summary>
      /// fa-diagram-sankey unicode value ("\ue158").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-sankey
      /// </summary>
      [Description("diagram-sankey")]
      public const string DiagramSankey = "\ue158";

      /// <summary>
      /// fa-diagram-subtask unicode value ("\ue479").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-subtask
      /// </summary>
      [Description("diagram-subtask")]
      public const string DiagramSubtask = "\ue479";

      /// <summary>
      /// fa-diagram-successor unicode value ("\ue47a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-successor
      /// </summary>
      [Description("diagram-successor")]
      public const string DiagramSuccessor = "\ue47a";

      /// <summary>
      /// fa-diagram-venn unicode value ("\ue15a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diagram-venn
      /// </summary>
      [Description("diagram-venn")]
      public const string DiagramVenn = "\ue15a";

      /// <summary>
      /// fa-dial unicode value ("\ue15b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial
      /// </summary>
      [Description("dial")]
      public const string Dial = "\ue15b";

      /// <summary>
      /// fa-dial-high unicode value ("\ue15c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-high
      /// </summary>
      [Description("dial-high")]
      public const string DialHigh = "\ue15c";

      /// <summary>
      /// fa-dial-low unicode value ("\ue15d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-low
      /// </summary>
      [Description("dial-low")]
      public const string DialLow = "\ue15d";

      /// <summary>
      /// fa-dial-max unicode value ("\ue15e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-max
      /// </summary>
      [Description("dial-max")]
      public const string DialMax = "\ue15e";

      /// <summary>
      /// fa-dial-med unicode value ("\ue15f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-med
      /// </summary>
      [Description("dial-med")]
      public const string DialMed = "\ue15f";

      /// <summary>
      /// fa-dial-med-low unicode value ("\ue160").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-med-low
      /// </summary>
      [Description("dial-med-low")]
      public const string DialMedLow = "\ue160";

      /// <summary>
      /// fa-dial-min unicode value ("\ue161").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-min
      /// </summary>
      [Description("dial-min")]
      public const string DialMin = "\ue161";

      /// <summary>
      /// fa-dial-off unicode value ("\ue162").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dial-off
      /// </summary>
      [Description("dial-off")]
      public const string DialOff = "\ue162";

      /// <summary>
      /// fa-diamond unicode value ("\uf219").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diamond
      /// </summary>
      [Description("diamond")]
      public const string Diamond = "\uf219";

      /// <summary>
      /// fa-diamond-exclamation unicode value ("\ue405").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diamond-exclamation
      /// </summary>
      [Description("diamond-exclamation")]
      public const string DiamondExclamation = "\ue405";

      /// <summary>
      /// fa-diamond-half unicode value ("\ue5b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diamond-half
      /// </summary>
      [Description("diamond-half")]
      public const string DiamondHalf = "\ue5b7";

      /// <summary>
      /// fa-diamond-half-stroke unicode value ("\ue5b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diamond-half-stroke
      /// </summary>
      [Description("diamond-half-stroke")]
      public const string DiamondHalfStroke = "\ue5b8";

      /// <summary>
      /// fa-diamond-turn-right unicode value ("\uf5eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diamond-turn-right
      /// </summary>
      [Description("diamond-turn-right")]
      public const string DiamondTurnRight = "\uf5eb";

      /// <summary>
      /// fa-diaspora unicode value ("\uf791").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diaspora
      /// </summary>
      [Description("diaspora")]
      public const string Diaspora = "\uf791";

      /// <summary>
      /// fa-dice unicode value ("\uf522").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice
      /// </summary>
      [Description("dice")]
      public const string Dice = "\uf522";

      /// <summary>
      /// fa-dice-d10 unicode value ("\uf6cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d10
      /// </summary>
      [Description("dice-d10")]
      public const string DiceD10 = "\uf6cd";

      /// <summary>
      /// fa-dice-d12 unicode value ("\uf6ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d12
      /// </summary>
      [Description("dice-d12")]
      public const string DiceD12 = "\uf6ce";

      /// <summary>
      /// fa-dice-d20 unicode value ("\uf6cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d20
      /// </summary>
      [Description("dice-d20")]
      public const string DiceD20 = "\uf6cf";

      /// <summary>
      /// fa-dice-d4 unicode value ("\uf6d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d4
      /// </summary>
      [Description("dice-d4")]
      public const string DiceD4 = "\uf6d0";

      /// <summary>
      /// fa-dice-d6 unicode value ("\uf6d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d6
      /// </summary>
      [Description("dice-d6")]
      public const string DiceD6 = "\uf6d1";

      /// <summary>
      /// fa-dice-d8 unicode value ("\uf6d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-d8
      /// </summary>
      [Description("dice-d8")]
      public const string DiceD8 = "\uf6d2";

      /// <summary>
      /// fa-dice-five unicode value ("\uf523").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-five
      /// </summary>
      [Description("dice-five")]
      public const string DiceFive = "\uf523";

      /// <summary>
      /// fa-dice-four unicode value ("\uf524").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-four
      /// </summary>
      [Description("dice-four")]
      public const string DiceFour = "\uf524";

      /// <summary>
      /// fa-dice-one unicode value ("\uf525").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-one
      /// </summary>
      [Description("dice-one")]
      public const string DiceOne = "\uf525";

      /// <summary>
      /// fa-dice-six unicode value ("\uf526").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-six
      /// </summary>
      [Description("dice-six")]
      public const string DiceSix = "\uf526";

      /// <summary>
      /// fa-dice-three unicode value ("\uf527").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-three
      /// </summary>
      [Description("dice-three")]
      public const string DiceThree = "\uf527";

      /// <summary>
      /// fa-dice-two unicode value ("\uf528").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dice-two
      /// </summary>
      [Description("dice-two")]
      public const string DiceTwo = "\uf528";

      /// <summary>
      /// fa-digg unicode value ("\uf1a6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/digg
      /// </summary>
      [Description("digg")]
      public const string Digg = "\uf1a6";

      /// <summary>
      /// fa-digital-ocean unicode value ("\uf391").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/digital-ocean
      /// </summary>
      [Description("digital-ocean")]
      public const string DigitalOcean = "\uf391";

      /// <summary>
      /// fa-dinosaur unicode value ("\ue5fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dinosaur
      /// </summary>
      [Description("dinosaur")]
      public const string Dinosaur = "\ue5fe";

      /// <summary>
      /// fa-diploma unicode value ("\uf5ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/diploma
      /// </summary>
      [Description("diploma")]
      public const string Diploma = "\uf5ea";

      /// <summary>
      /// fa-disc-drive unicode value ("\uf8b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/disc-drive
      /// </summary>
      [Description("disc-drive")]
      public const string DiscDrive = "\uf8b5";

      /// <summary>
      /// fa-discord unicode value ("\uf392").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/discord
      /// </summary>
      [Description("discord")]
      public const string Discord = "\uf392";

      /// <summary>
      /// fa-discourse unicode value ("\uf393").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/discourse
      /// </summary>
      [Description("discourse")]
      public const string Discourse = "\uf393";

      /// <summary>
      /// fa-disease unicode value ("\uf7fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/disease
      /// </summary>
      [Description("disease")]
      public const string Disease = "\uf7fa";

      /// <summary>
      /// fa-display unicode value ("\ue163").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display
      /// </summary>
      [Description("display")]
      public const string Display = "\ue163";

      /// <summary>
      /// fa-display-arrow-down unicode value ("\ue164").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-arrow-down
      /// </summary>
      [Description("display-arrow-down")]
      public const string DisplayArrowDown = "\ue164";

      /// <summary>
      /// fa-display-chart-up unicode value ("\ue5e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-chart-up
      /// </summary>
      [Description("display-chart-up")]
      public const string DisplayChartUp = "\ue5e3";

      /// <summary>
      /// fa-display-chart-up-circle-currency unicode value ("\ue5e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-chart-up-circle-currency
      /// </summary>
      [Description("display-chart-up-circle-currency")]
      public const string DisplayChartUpCircleCurrency = "\ue5e5";

      /// <summary>
      /// fa-display-chart-up-circle-dollar unicode value ("\ue5e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-chart-up-circle-dollar
      /// </summary>
      [Description("display-chart-up-circle-dollar")]
      public const string DisplayChartUpCircleDollar = "\ue5e6";

      /// <summary>
      /// fa-display-code unicode value ("\ue165").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-code
      /// </summary>
      [Description("display-code")]
      public const string DisplayCode = "\ue165";

      /// <summary>
      /// fa-display-medical unicode value ("\ue166").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-medical
      /// </summary>
      [Description("display-medical")]
      public const string DisplayMedical = "\ue166";

      /// <summary>
      /// fa-display-slash unicode value ("\ue2fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/display-slash
      /// </summary>
      [Description("display-slash")]
      public const string DisplaySlash = "\ue2fa";

      /// <summary>
      /// fa-distribute-spacing-horizontal unicode value ("\ue365").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/distribute-spacing-horizontal
      /// </summary>
      [Description("distribute-spacing-horizontal")]
      public const string DistributeSpacingHorizontal = "\ue365";

      /// <summary>
      /// fa-distribute-spacing-vertical unicode value ("\ue366").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/distribute-spacing-vertical
      /// </summary>
      [Description("distribute-spacing-vertical")]
      public const string DistributeSpacingVertical = "\ue366";

      /// <summary>
      /// fa-ditto unicode value ("\u0022").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ditto
      /// </summary>
      [Description("ditto")]
      public const string Ditto = "\u0022";

      /// <summary>
      /// fa-divide unicode value ("\uf529").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/divide
      /// </summary>
      [Description("divide")]
      public const string Divide = "\uf529";

      /// <summary>
      /// fa-dna unicode value ("\uf471").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dna
      /// </summary>
      [Description("dna")]
      public const string Dna = "\uf471";

      /// <summary>
      /// fa-do-not-enter unicode value ("\uf5ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/do-not-enter
      /// </summary>
      [Description("do-not-enter")]
      public const string DoNotEnter = "\uf5ec";

      /// <summary>
      /// fa-dochub unicode value ("\uf394").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dochub
      /// </summary>
      [Description("dochub")]
      public const string Dochub = "\uf394";

      /// <summary>
      /// fa-docker unicode value ("\uf395").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/docker
      /// </summary>
      [Description("docker")]
      public const string Docker = "\uf395";

      /// <summary>
      /// fa-dog unicode value ("\uf6d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dog
      /// </summary>
      [Description("dog")]
      public const string Dog = "\uf6d3";

      /// <summary>
      /// fa-dog-leashed unicode value ("\uf6d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dog-leashed
      /// </summary>
      [Description("dog-leashed")]
      public const string DogLeashed = "\uf6d4";

      /// <summary>
      /// fa-dollar-sign unicode value ("\u0024").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dollar-sign
      /// </summary>
      [Description("dollar-sign")]
      public const string DollarSign = "\u0024";

      /// <summary>
      /// fa-dolly unicode value ("\uf472").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dolly
      /// </summary>
      [Description("dolly")]
      public const string Dolly = "\uf472";

      /// <summary>
      /// fa-dolly-empty unicode value ("\uf473").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dolly-empty
      /// </summary>
      [Description("dolly-empty")]
      public const string DollyEmpty = "\uf473";

      /// <summary>
      /// fa-dolphin unicode value ("\ue168").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dolphin
      /// </summary>
      [Description("dolphin")]
      public const string Dolphin = "\ue168";

      /// <summary>
      /// fa-dong-sign unicode value ("\ue169").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dong-sign
      /// </summary>
      [Description("dong-sign")]
      public const string DongSign = "\ue169";

      /// <summary>
      /// fa-donut unicode value ("\ue406").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/donut
      /// </summary>
      [Description("donut")]
      public const string Donut = "\ue406";

      /// <summary>
      /// fa-door-closed unicode value ("\uf52a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/door-closed
      /// </summary>
      [Description("door-closed")]
      public const string DoorClosed = "\uf52a";

      /// <summary>
      /// fa-door-open unicode value ("\uf52b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/door-open
      /// </summary>
      [Description("door-open")]
      public const string DoorOpen = "\uf52b";

      /// <summary>
      /// fa-dove unicode value ("\uf4ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dove
      /// </summary>
      [Description("dove")]
      public const string Dove = "\uf4ba";

      /// <summary>
      /// fa-down unicode value ("\uf354").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down
      /// </summary>
      [Description("down")]
      public const string Down = "\uf354";

      /// <summary>
      /// fa-down-from-dotted-line unicode value ("\ue407").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-from-dotted-line
      /// </summary>
      [Description("down-from-dotted-line")]
      public const string DownFromDottedLine = "\ue407";

      /// <summary>
      /// fa-down-from-line unicode value ("\uf349").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-from-line
      /// </summary>
      [Description("down-from-line")]
      public const string DownFromLine = "\uf349";

      /// <summary>
      /// fa-down-left unicode value ("\ue16a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-left
      /// </summary>
      [Description("down-left")]
      public const string DownLeft = "\ue16a";

      /// <summary>
      /// fa-down-left-and-up-right-to-center unicode value ("\uf422").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-left-and-up-right-to-center
      /// </summary>
      [Description("down-left-and-up-right-to-center")]
      public const string DownLeftAndUpRightToCenter = "\uf422";

      /// <summary>
      /// fa-down-long unicode value ("\uf309").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-long
      /// </summary>
      [Description("down-long")]
      public const string DownLong = "\uf309";

      /// <summary>
      /// fa-down-right unicode value ("\ue16b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-right
      /// </summary>
      [Description("down-right")]
      public const string DownRight = "\ue16b";

      /// <summary>
      /// fa-down-to-bracket unicode value ("\ue4e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-to-bracket
      /// </summary>
      [Description("down-to-bracket")]
      public const string DownToBracket = "\ue4e7";

      /// <summary>
      /// fa-down-to-dotted-line unicode value ("\ue408").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-to-dotted-line
      /// </summary>
      [Description("down-to-dotted-line")]
      public const string DownToDottedLine = "\ue408";

      /// <summary>
      /// fa-down-to-line unicode value ("\uf34a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/down-to-line
      /// </summary>
      [Description("down-to-line")]
      public const string DownToLine = "\uf34a";

      /// <summary>
      /// fa-download unicode value ("\uf019").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/download
      /// </summary>
      [Description("download")]
      public const string Download = "\uf019";

      /// <summary>
      /// fa-draft2digital unicode value ("\uf396").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/draft2digital
      /// </summary>
      [Description("draft2digital")]
      public const string Draft2digital = "\uf396";

      /// <summary>
      /// fa-dragon unicode value ("\uf6d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dragon
      /// </summary>
      [Description("dragon")]
      public const string Dragon = "\uf6d5";

      /// <summary>
      /// fa-draw-circle unicode value ("\uf5ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/draw-circle
      /// </summary>
      [Description("draw-circle")]
      public const string DrawCircle = "\uf5ed";

      /// <summary>
      /// fa-draw-polygon unicode value ("\uf5ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/draw-polygon
      /// </summary>
      [Description("draw-polygon")]
      public const string DrawPolygon = "\uf5ee";

      /// <summary>
      /// fa-draw-square unicode value ("\uf5ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/draw-square
      /// </summary>
      [Description("draw-square")]
      public const string DrawSquare = "\uf5ef";

      /// <summary>
      /// fa-dreidel unicode value ("\uf792").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dreidel
      /// </summary>
      [Description("dreidel")]
      public const string Dreidel = "\uf792";

      /// <summary>
      /// fa-dribbble unicode value ("\uf17d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dribbble
      /// </summary>
      [Description("dribbble")]
      public const string Dribbble = "\uf17d";

      /// <summary>
      /// fa-drone unicode value ("\uf85f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drone
      /// </summary>
      [Description("drone")]
      public const string Drone = "\uf85f";

      /// <summary>
      /// fa-drone-front unicode value ("\uf860").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drone-front
      /// </summary>
      [Description("drone-front")]
      public const string DroneFront = "\uf860";

      /// <summary>
      /// fa-dropbox unicode value ("\uf16b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/dropbox
      /// </summary>
      [Description("dropbox")]
      public const string Dropbox = "\uf16b";

      /// <summary>
      /// fa-droplet unicode value ("\uf043").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/droplet
      /// </summary>
      [Description("droplet")]
      public const string Droplet = "\uf043";

      /// <summary>
      /// fa-droplet-degree unicode value ("\uf748").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/droplet-degree
      /// </summary>
      [Description("droplet-degree")]
      public const string DropletDegree = "\uf748";

      /// <summary>
      /// fa-droplet-percent unicode value ("\uf750").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/droplet-percent
      /// </summary>
      [Description("droplet-percent")]
      public const string DropletPercent = "\uf750";

      /// <summary>
      /// fa-droplet-slash unicode value ("\uf5c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/droplet-slash
      /// </summary>
      [Description("droplet-slash")]
      public const string DropletSlash = "\uf5c7";

      /// <summary>
      /// fa-drum unicode value ("\uf569").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drum
      /// </summary>
      [Description("drum")]
      public const string Drum = "\uf569";

      /// <summary>
      /// fa-drum-steelpan unicode value ("\uf56a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drum-steelpan
      /// </summary>
      [Description("drum-steelpan")]
      public const string DrumSteelpan = "\uf56a";

      /// <summary>
      /// fa-drumstick unicode value ("\uf6d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drumstick
      /// </summary>
      [Description("drumstick")]
      public const string Drumstick = "\uf6d6";

      /// <summary>
      /// fa-drumstick-bite unicode value ("\uf6d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/drumstick-bite
      /// </summary>
      [Description("drumstick-bite")]
      public const string DrumstickBite = "\uf6d7";

      /// <summary>
      /// fa-drupal unicode value ("\uf1a9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/drupal
      /// </summary>
      [Description("drupal")]
      public const string Drupal = "\uf1a9";

      /// <summary>
      /// fa-dryer unicode value ("\uf861").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dryer
      /// </summary>
      [Description("dryer")]
      public const string Dryer = "\uf861";

      /// <summary>
      /// fa-dryer-heat unicode value ("\uf862").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dryer-heat
      /// </summary>
      [Description("dryer-heat")]
      public const string DryerHeat = "\uf862";

      /// <summary>
      /// fa-duck unicode value ("\uf6d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/duck
      /// </summary>
      [Description("duck")]
      public const string Duck = "\uf6d8";

      /// <summary>
      /// fa-dumbbell unicode value ("\uf44b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dumbbell
      /// </summary>
      [Description("dumbbell")]
      public const string Dumbbell = "\uf44b";

      /// <summary>
      /// fa-dumpster unicode value ("\uf793").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dumpster
      /// </summary>
      [Description("dumpster")]
      public const string Dumpster = "\uf793";

      /// <summary>
      /// fa-dumpster-fire unicode value ("\uf794").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dumpster-fire
      /// </summary>
      [Description("dumpster-fire")]
      public const string DumpsterFire = "\uf794";

      /// <summary>
      /// fa-dungeon unicode value ("\uf6d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dungeon
      /// </summary>
      [Description("dungeon")]
      public const string Dungeon = "\uf6d9";

      /// <summary>
      /// fa-dyalog unicode value ("\uf399").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/dyalog
      /// </summary>
      [Description("dyalog")]
      public const string Dyalog = "\uf399";

      /// <summary>
      /// fa-e unicode value ("\u0045").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/e
      /// </summary>
      [Description("e")]
      public const string E = "\u0045";

      /// <summary>
      /// fa-ear unicode value ("\uf5f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ear
      /// </summary>
      [Description("ear")]
      public const string Ear = "\uf5f0";

      /// <summary>
      /// fa-ear-deaf unicode value ("\uf2a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ear-deaf
      /// </summary>
      [Description("ear-deaf")]
      public const string EarDeaf = "\uf2a4";

      /// <summary>
      /// fa-ear-listen unicode value ("\uf2a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ear-listen
      /// </summary>
      [Description("ear-listen")]
      public const string EarListen = "\uf2a2";

      /// <summary>
      /// fa-ear-muffs unicode value ("\uf795").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ear-muffs
      /// </summary>
      [Description("ear-muffs")]
      public const string EarMuffs = "\uf795";

      /// <summary>
      /// fa-earlybirds unicode value ("\uf39a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earlybirds
      /// </summary>
      [Description("earlybirds")]
      public const string Earlybirds = "\uf39a";

      /// <summary>
      /// fa-earth-africa unicode value ("\uf57c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earth-africa
      /// </summary>
      [Description("earth-africa")]
      public const string EarthAfrica = "\uf57c";

      /// <summary>
      /// fa-earth-americas unicode value ("\uf57d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earth-americas
      /// </summary>
      [Description("earth-americas")]
      public const string EarthAmericas = "\uf57d";

      /// <summary>
      /// fa-earth-asia unicode value ("\uf57e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earth-asia
      /// </summary>
      [Description("earth-asia")]
      public const string EarthAsia = "\uf57e";

      /// <summary>
      /// fa-earth-europe unicode value ("\uf7a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earth-europe
      /// </summary>
      [Description("earth-europe")]
      public const string EarthEurope = "\uf7a2";

      /// <summary>
      /// fa-earth-oceania unicode value ("\ue47b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/earth-oceania
      /// </summary>
      [Description("earth-oceania")]
      public const string EarthOceania = "\ue47b";

      /// <summary>
      /// fa-ebay unicode value ("\uf4f4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ebay
      /// </summary>
      [Description("ebay")]
      public const string Ebay = "\uf4f4";

      /// <summary>
      /// fa-eclipse unicode value ("\uf749").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eclipse
      /// </summary>
      [Description("eclipse")]
      public const string Eclipse = "\uf749";

      /// <summary>
      /// fa-edge unicode value ("\uf282").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/edge
      /// </summary>
      [Description("edge")]
      public const string Edge = "\uf282";

      /// <summary>
      /// fa-edge-legacy unicode value ("\ue078").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/edge-legacy
      /// </summary>
      [Description("edge-legacy")]
      public const string EdgeLegacy = "\ue078";

      /// <summary>
      /// fa-egg unicode value ("\uf7fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/egg
      /// </summary>
      [Description("egg")]
      public const string Egg = "\uf7fb";

      /// <summary>
      /// fa-egg-fried unicode value ("\uf7fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/egg-fried
      /// </summary>
      [Description("egg-fried")]
      public const string EggFried = "\uf7fc";

      /// <summary>
      /// fa-eggplant unicode value ("\ue16c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eggplant
      /// </summary>
      [Description("eggplant")]
      public const string Eggplant = "\ue16c";

      /// <summary>
      /// fa-eject unicode value ("\uf052").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eject
      /// </summary>
      [Description("eject")]
      public const string Eject = "\uf052";

      /// <summary>
      /// fa-elementor unicode value ("\uf430").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.3', Last Modified in '6.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/elementor
      /// </summary>
      [Description("elementor")]
      public const string Elementor = "\uf430";

      /// <summary>
      /// fa-elephant unicode value ("\uf6da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/elephant
      /// </summary>
      [Description("elephant")]
      public const string Elephant = "\uf6da";

      /// <summary>
      /// fa-elevator unicode value ("\ue16d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/elevator
      /// </summary>
      [Description("elevator")]
      public const string Elevator = "\ue16d";

      /// <summary>
      /// fa-ellipsis unicode value ("\uf141").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ellipsis
      /// </summary>
      [Description("ellipsis")]
      public const string Ellipsis = "\uf141";

      /// <summary>
      /// fa-ellipsis-stroke unicode value ("\uf39b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ellipsis-stroke
      /// </summary>
      [Description("ellipsis-stroke")]
      public const string EllipsisStroke = "\uf39b";

      /// <summary>
      /// fa-ellipsis-stroke-vertical unicode value ("\uf39c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ellipsis-stroke-vertical
      /// </summary>
      [Description("ellipsis-stroke-vertical")]
      public const string EllipsisStrokeVertical = "\uf39c";

      /// <summary>
      /// fa-ellipsis-vertical unicode value ("\uf142").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ellipsis-vertical
      /// </summary>
      [Description("ellipsis-vertical")]
      public const string EllipsisVertical = "\uf142";

      /// <summary>
      /// fa-ello unicode value ("\uf5f1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.2.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ello
      /// </summary>
      [Description("ello")]
      public const string Ello = "\uf5f1";

      /// <summary>
      /// fa-ember unicode value ("\uf423").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/ember
      /// </summary>
      [Description("ember")]
      public const string Ember = "\uf423";

      /// <summary>
      /// fa-empire unicode value ("\uf1d1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/empire
      /// </summary>
      [Description("empire")]
      public const string Empire = "\uf1d1";

      /// <summary>
      /// fa-empty-set unicode value ("\uf656").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/empty-set
      /// </summary>
      [Description("empty-set")]
      public const string EmptySet = "\uf656";

      /// <summary>
      /// fa-engine unicode value ("\ue16e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/engine
      /// </summary>
      [Description("engine")]
      public const string Engine = "\ue16e";

      /// <summary>
      /// fa-engine-warning unicode value ("\uf5f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/engine-warning
      /// </summary>
      [Description("engine-warning")]
      public const string EngineWarning = "\uf5f2";

      /// <summary>
      /// fa-envelope unicode value ("\uf0e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope
      /// </summary>
      [Description("envelope")]
      public const string Envelope = "\uf0e0";

      /// <summary>
      /// fa-envelope-circle-check unicode value ("\ue4e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope-circle-check
      /// </summary>
      [Description("envelope-circle-check")]
      public const string EnvelopeCircleCheck = "\ue4e8";

      /// <summary>
      /// fa-envelope-dot unicode value ("\ue16f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope-dot
      /// </summary>
      [Description("envelope-dot")]
      public const string EnvelopeDot = "\ue16f";

      /// <summary>
      /// fa-envelope-open unicode value ("\uf2b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope-open
      /// </summary>
      [Description("envelope-open")]
      public const string EnvelopeOpen = "\uf2b6";

      /// <summary>
      /// fa-envelope-open-dollar unicode value ("\uf657").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope-open-dollar
      /// </summary>
      [Description("envelope-open-dollar")]
      public const string EnvelopeOpenDollar = "\uf657";

      /// <summary>
      /// fa-envelope-open-text unicode value ("\uf658").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelope-open-text
      /// </summary>
      [Description("envelope-open-text")]
      public const string EnvelopeOpenText = "\uf658";

      /// <summary>
      /// fa-envelopes unicode value ("\ue170").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelopes
      /// </summary>
      [Description("envelopes")]
      public const string Envelopes = "\ue170";

      /// <summary>
      /// fa-envelopes-bulk unicode value ("\uf674").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envelopes-bulk
      /// </summary>
      [Description("envelopes-bulk")]
      public const string EnvelopesBulk = "\uf674";

      /// <summary>
      /// fa-envira unicode value ("\uf299").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/envira
      /// </summary>
      [Description("envira")]
      public const string Envira = "\uf299";

      /// <summary>
      /// fa-equals unicode value ("\u003d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/equals
      /// </summary>
      [Description("equals")]
      public const string Equal = "\u003d";

      /// <summary>
      /// fa-eraser unicode value ("\uf12d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eraser
      /// </summary>
      [Description("eraser")]
      public const string Eraser = "\uf12d";

      /// <summary>
      /// fa-erlang unicode value ("\uf39d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/erlang
      /// </summary>
      [Description("erlang")]
      public const string Erlang = "\uf39d";

      /// <summary>
      /// fa-escalator unicode value ("\ue171").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/escalator
      /// </summary>
      [Description("escalator")]
      public const string Escalator = "\ue171";

      /// <summary>
      /// fa-ethereum unicode value ("\uf42e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.2', Last Modified in '5.0.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/ethereum
      /// </summary>
      [Description("ethereum")]
      public const string Ethereum = "\uf42e";

      /// <summary>
      /// fa-ethernet unicode value ("\uf796").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ethernet
      /// </summary>
      [Description("ethernet")]
      public const string Ethernet = "\uf796";

      /// <summary>
      /// fa-etsy unicode value ("\uf2d7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/etsy
      /// </summary>
      [Description("etsy")]
      public const string Etsy = "\uf2d7";

      /// <summary>
      /// fa-euro-sign unicode value ("\uf153").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/euro-sign
      /// </summary>
      [Description("euro-sign")]
      public const string EuroSign = "\uf153";

      /// <summary>
      /// fa-evernote unicode value ("\uf839").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/evernote
      /// </summary>
      [Description("evernote")]
      public const string Evernote = "\uf839";

      /// <summary>
      /// fa-exclamation unicode value ("\u0021").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/exclamation
      /// </summary>
      [Description("exclamation")]
      public const string Exclamation = "\u0021";

      /// <summary>
      /// fa-expand unicode value ("\uf065").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/expand
      /// </summary>
      [Description("expand")]
      public const string Expand = "\uf065";

      /// <summary>
      /// fa-expand-wide unicode value ("\uf320").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/expand-wide
      /// </summary>
      [Description("expand-wide")]
      public const string ExpandWide = "\uf320";

      /// <summary>
      /// fa-expeditedssl unicode value ("\uf23e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/expeditedssl
      /// </summary>
      [Description("expeditedssl")]
      public const string Expeditedssl = "\uf23e";

      /// <summary>
      /// fa-explosion unicode value ("\ue4e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/explosion
      /// </summary>
      [Description("explosion")]
      public const string Explosion = "\ue4e9";

      /// <summary>
      /// fa-eye unicode value ("\uf06e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye
      /// </summary>
      [Description("eye")]
      public const string Eye = "\uf06e";

      /// <summary>
      /// fa-eye-dropper unicode value ("\uf1fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-dropper
      /// </summary>
      [Description("eye-dropper")]
      public const string EyeDropper = "\uf1fb";

      /// <summary>
      /// fa-eye-dropper-full unicode value ("\ue172").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-dropper-full
      /// </summary>
      [Description("eye-dropper-full")]
      public const string EyeDropperFull = "\ue172";

      /// <summary>
      /// fa-eye-dropper-half unicode value ("\ue173").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-dropper-half
      /// </summary>
      [Description("eye-dropper-half")]
      public const string EyeDropperHalf = "\ue173";

      /// <summary>
      /// fa-eye-evil unicode value ("\uf6db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-evil
      /// </summary>
      [Description("eye-evil")]
      public const string EyeEvil = "\uf6db";

      /// <summary>
      /// fa-eye-low-vision unicode value ("\uf2a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-low-vision
      /// </summary>
      [Description("eye-low-vision")]
      public const string EyeLowVision = "\uf2a8";

      /// <summary>
      /// fa-eye-slash unicode value ("\uf070").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eye-slash
      /// </summary>
      [Description("eye-slash")]
      public const string EyeSlash = "\uf070";

      /// <summary>
      /// fa-eyes unicode value ("\ue367").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/eyes
      /// </summary>
      [Description("eyes")]
      public const string Eyes = "\ue367";

      /// <summary>
      /// fa-f unicode value ("\u0046").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/f
      /// </summary>
      [Description("f")]
      public const string F = "\u0046";

      /// <summary>
      /// fa-face-angry unicode value ("\uf556").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-angry
      /// </summary>
      [Description("face-angry")]
      public const string FaceAngry = "\uf556";

      /// <summary>
      /// fa-face-angry-horns unicode value ("\ue368").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-angry-horns
      /// </summary>
      [Description("face-angry-horns")]
      public const string FaceAngryHorns = "\ue368";

      /// <summary>
      /// fa-face-anguished unicode value ("\ue369").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-anguished
      /// </summary>
      [Description("face-anguished")]
      public const string FaceAnguished = "\ue369";

      /// <summary>
      /// fa-face-anxious-sweat unicode value ("\ue36a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-anxious-sweat
      /// </summary>
      [Description("face-anxious-sweat")]
      public const string FaceAnxiousSweat = "\ue36a";

      /// <summary>
      /// fa-face-astonished unicode value ("\ue36b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-astonished
      /// </summary>
      [Description("face-astonished")]
      public const string FaceAstonished = "\ue36b";

      /// <summary>
      /// fa-face-awesome unicode value ("\ue409").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-awesome
      /// </summary>
      [Description("face-awesome")]
      public const string FaceAwesome = "\ue409";

      /// <summary>
      /// fa-face-beam-hand-over-mouth unicode value ("\ue47c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-beam-hand-over-mouth
      /// </summary>
      [Description("face-beam-hand-over-mouth")]
      public const string FaceBeamHandOverMouth = "\ue47c";

      /// <summary>
      /// fa-face-clouds unicode value ("\ue47d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-clouds
      /// </summary>
      [Description("face-clouds")]
      public const string FaceClouds = "\ue47d";

      /// <summary>
      /// fa-face-confounded unicode value ("\ue36c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-confounded
      /// </summary>
      [Description("face-confounded")]
      public const string FaceConfounded = "\ue36c";

      /// <summary>
      /// fa-face-confused unicode value ("\ue36d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-confused
      /// </summary>
      [Description("face-confused")]
      public const string FaceConfused = "\ue36d";

      /// <summary>
      /// fa-face-cowboy-hat unicode value ("\ue36e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-cowboy-hat
      /// </summary>
      [Description("face-cowboy-hat")]
      public const string FaceCowboyHat = "\ue36e";

      /// <summary>
      /// fa-face-diagonal-mouth unicode value ("\ue47e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-diagonal-mouth
      /// </summary>
      [Description("face-diagonal-mouth")]
      public const string FaceDiagonalMouth = "\ue47e";

      /// <summary>
      /// fa-face-disappointed unicode value ("\ue36f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-disappointed
      /// </summary>
      [Description("face-disappointed")]
      public const string FaceDisappointed = "\ue36f";

      /// <summary>
      /// fa-face-disguise unicode value ("\ue370").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-disguise
      /// </summary>
      [Description("face-disguise")]
      public const string FaceDisguise = "\ue370";

      /// <summary>
      /// fa-face-dizzy unicode value ("\uf567").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-dizzy
      /// </summary>
      [Description("face-dizzy")]
      public const string FaceDizzy = "\uf567";

      /// <summary>
      /// fa-face-dotted unicode value ("\ue47f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-dotted
      /// </summary>
      [Description("face-dotted")]
      public const string FaceDotted = "\ue47f";

      /// <summary>
      /// fa-face-downcast-sweat unicode value ("\ue371").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-downcast-sweat
      /// </summary>
      [Description("face-downcast-sweat")]
      public const string FaceDowncastSweat = "\ue371";

      /// <summary>
      /// fa-face-drooling unicode value ("\ue372").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-drooling
      /// </summary>
      [Description("face-drooling")]
      public const string FaceDrooling = "\ue372";

      /// <summary>
      /// fa-face-exhaling unicode value ("\ue480").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-exhaling
      /// </summary>
      [Description("face-exhaling")]
      public const string FaceExhaling = "\ue480";

      /// <summary>
      /// fa-face-explode unicode value ("\ue2fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-explode
      /// </summary>
      [Description("face-explode")]
      public const string FaceExplode = "\ue2fe";

      /// <summary>
      /// fa-face-expressionless unicode value ("\ue373").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-expressionless
      /// </summary>
      [Description("face-expressionless")]
      public const string FaceExpressionless = "\ue373";

      /// <summary>
      /// fa-face-eyes-xmarks unicode value ("\ue374").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-eyes-xmarks
      /// </summary>
      [Description("face-eyes-xmarks")]
      public const string FaceEyesXmarks = "\ue374";

      /// <summary>
      /// fa-face-fearful unicode value ("\ue375").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-fearful
      /// </summary>
      [Description("face-fearful")]
      public const string FaceFearful = "\ue375";

      /// <summary>
      /// fa-face-flushed unicode value ("\uf579").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-flushed
      /// </summary>
      [Description("face-flushed")]
      public const string FaceFlushed = "\uf579";

      /// <summary>
      /// fa-face-frown unicode value ("\uf119").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-frown
      /// </summary>
      [Description("face-frown")]
      public const string FaceFrown = "\uf119";

      /// <summary>
      /// fa-face-frown-open unicode value ("\uf57a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-frown-open
      /// </summary>
      [Description("face-frown-open")]
      public const string FaceFrownOpen = "\uf57a";

      /// <summary>
      /// fa-face-frown-slight unicode value ("\ue376").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-frown-slight
      /// </summary>
      [Description("face-frown-slight")]
      public const string FaceFrownSlight = "\ue376";

      /// <summary>
      /// fa-face-glasses unicode value ("\ue377").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-glasses
      /// </summary>
      [Description("face-glasses")]
      public const string FaceGlasses = "\ue377";

      /// <summary>
      /// fa-face-grimace unicode value ("\uf57f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grimace
      /// </summary>
      [Description("face-grimace")]
      public const string FaceGrimace = "\uf57f";

      /// <summary>
      /// fa-face-grin unicode value ("\uf580").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin
      /// </summary>
      [Description("face-grin")]
      public const string FaceGrin = "\uf580";

      /// <summary>
      /// fa-face-grin-beam unicode value ("\uf582").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-beam
      /// </summary>
      [Description("face-grin-beam")]
      public const string FaceGrinBeam = "\uf582";

      /// <summary>
      /// fa-face-grin-beam-sweat unicode value ("\uf583").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-beam-sweat
      /// </summary>
      [Description("face-grin-beam-sweat")]
      public const string FaceGrinBeamSweat = "\uf583";

      /// <summary>
      /// fa-face-grin-hearts unicode value ("\uf584").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-hearts
      /// </summary>
      [Description("face-grin-hearts")]
      public const string FaceGrinHearts = "\uf584";

      /// <summary>
      /// fa-face-grin-squint unicode value ("\uf585").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-squint
      /// </summary>
      [Description("face-grin-squint")]
      public const string FaceGrinSquint = "\uf585";

      /// <summary>
      /// fa-face-grin-squint-tears unicode value ("\uf586").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-squint-tears
      /// </summary>
      [Description("face-grin-squint-tears")]
      public const string FaceGrinSquintTears = "\uf586";

      /// <summary>
      /// fa-face-grin-stars unicode value ("\uf587").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-stars
      /// </summary>
      [Description("face-grin-stars")]
      public const string FaceGrinStars = "\uf587";

      /// <summary>
      /// fa-face-grin-tears unicode value ("\uf588").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-tears
      /// </summary>
      [Description("face-grin-tears")]
      public const string FaceGrinTears = "\uf588";

      /// <summary>
      /// fa-face-grin-tongue unicode value ("\uf589").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-tongue
      /// </summary>
      [Description("face-grin-tongue")]
      public const string FaceGrinTongue = "\uf589";

      /// <summary>
      /// fa-face-grin-tongue-squint unicode value ("\uf58a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-tongue-squint
      /// </summary>
      [Description("face-grin-tongue-squint")]
      public const string FaceGrinTongueSquint = "\uf58a";

      /// <summary>
      /// fa-face-grin-tongue-wink unicode value ("\uf58b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-tongue-wink
      /// </summary>
      [Description("face-grin-tongue-wink")]
      public const string FaceGrinTongueWink = "\uf58b";

      /// <summary>
      /// fa-face-grin-wide unicode value ("\uf581").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-wide
      /// </summary>
      [Description("face-grin-wide")]
      public const string FaceGrinWide = "\uf581";

      /// <summary>
      /// fa-face-grin-wink unicode value ("\uf58c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-grin-wink
      /// </summary>
      [Description("face-grin-wink")]
      public const string FaceGrinWink = "\uf58c";

      /// <summary>
      /// fa-face-hand-over-mouth unicode value ("\ue378").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-hand-over-mouth
      /// </summary>
      [Description("face-hand-over-mouth")]
      public const string FaceHandOverMouth = "\ue378";

      /// <summary>
      /// fa-face-hand-peeking unicode value ("\ue481").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-hand-peeking
      /// </summary>
      [Description("face-hand-peeking")]
      public const string FaceHandPeeking = "\ue481";

      /// <summary>
      /// fa-face-hand-yawn unicode value ("\ue379").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-hand-yawn
      /// </summary>
      [Description("face-hand-yawn")]
      public const string FaceHandYawn = "\ue379";

      /// <summary>
      /// fa-face-head-bandage unicode value ("\ue37a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-head-bandage
      /// </summary>
      [Description("face-head-bandage")]
      public const string FaceHeadBandage = "\ue37a";

      /// <summary>
      /// fa-face-holding-back-tears unicode value ("\ue482").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-holding-back-tears
      /// </summary>
      [Description("face-holding-back-tears")]
      public const string FaceHoldingBackTears = "\ue482";

      /// <summary>
      /// fa-face-hushed unicode value ("\ue37b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-hushed
      /// </summary>
      [Description("face-hushed")]
      public const string FaceHushed = "\ue37b";

      /// <summary>
      /// fa-face-icicles unicode value ("\ue37c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-icicles
      /// </summary>
      [Description("face-icicles")]
      public const string FaceIcicles = "\ue37c";

      /// <summary>
      /// fa-face-kiss unicode value ("\uf596").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-kiss
      /// </summary>
      [Description("face-kiss")]
      public const string FaceKiss = "\uf596";

      /// <summary>
      /// fa-face-kiss-beam unicode value ("\uf597").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-kiss-beam
      /// </summary>
      [Description("face-kiss-beam")]
      public const string FaceKissBeam = "\uf597";

      /// <summary>
      /// fa-face-kiss-closed-eyes unicode value ("\ue37d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-kiss-closed-eyes
      /// </summary>
      [Description("face-kiss-closed-eyes")]
      public const string FaceKissClosedEyes = "\ue37d";

      /// <summary>
      /// fa-face-kiss-wink-heart unicode value ("\uf598").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-kiss-wink-heart
      /// </summary>
      [Description("face-kiss-wink-heart")]
      public const string FaceKissWinkHeart = "\uf598";

      /// <summary>
      /// fa-face-laugh unicode value ("\uf599").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-laugh
      /// </summary>
      [Description("face-laugh")]
      public const string FaceLaugh = "\uf599";

      /// <summary>
      /// fa-face-laugh-beam unicode value ("\uf59a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-laugh-beam
      /// </summary>
      [Description("face-laugh-beam")]
      public const string FaceLaughBeam = "\uf59a";

      /// <summary>
      /// fa-face-laugh-squint unicode value ("\uf59b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-laugh-squint
      /// </summary>
      [Description("face-laugh-squint")]
      public const string FaceLaughSquint = "\uf59b";

      /// <summary>
      /// fa-face-laugh-wink unicode value ("\uf59c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-laugh-wink
      /// </summary>
      [Description("face-laugh-wink")]
      public const string FaceLaughWink = "\uf59c";

      /// <summary>
      /// fa-face-lying unicode value ("\ue37e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-lying
      /// </summary>
      [Description("face-lying")]
      public const string FaceLying = "\ue37e";

      /// <summary>
      /// fa-face-mask unicode value ("\ue37f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-mask
      /// </summary>
      [Description("face-mask")]
      public const string FaceMask = "\ue37f";

      /// <summary>
      /// fa-face-meh unicode value ("\uf11a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-meh
      /// </summary>
      [Description("face-meh")]
      public const string FaceMeh = "\uf11a";

      /// <summary>
      /// fa-face-meh-blank unicode value ("\uf5a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-meh-blank
      /// </summary>
      [Description("face-meh-blank")]
      public const string FaceMehBlank = "\uf5a4";

      /// <summary>
      /// fa-face-melting unicode value ("\ue483").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-melting
      /// </summary>
      [Description("face-melting")]
      public const string FaceMelting = "\ue483";

      /// <summary>
      /// fa-face-monocle unicode value ("\ue380").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-monocle
      /// </summary>
      [Description("face-monocle")]
      public const string FaceMonocle = "\ue380";

      /// <summary>
      /// fa-face-nauseated unicode value ("\ue381").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-nauseated
      /// </summary>
      [Description("face-nauseated")]
      public const string FaceNauseated = "\ue381";

      /// <summary>
      /// fa-face-nose-steam unicode value ("\ue382").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-nose-steam
      /// </summary>
      [Description("face-nose-steam")]
      public const string FaceNoseSteam = "\ue382";

      /// <summary>
      /// fa-face-party unicode value ("\ue383").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-party
      /// </summary>
      [Description("face-party")]
      public const string FaceParty = "\ue383";

      /// <summary>
      /// fa-face-pensive unicode value ("\ue384").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-pensive
      /// </summary>
      [Description("face-pensive")]
      public const string FacePensive = "\ue384";

      /// <summary>
      /// fa-face-persevering unicode value ("\ue385").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-persevering
      /// </summary>
      [Description("face-persevering")]
      public const string FacePersevering = "\ue385";

      /// <summary>
      /// fa-face-pleading unicode value ("\ue386").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-pleading
      /// </summary>
      [Description("face-pleading")]
      public const string FacePleading = "\ue386";

      /// <summary>
      /// fa-face-pouting unicode value ("\ue387").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-pouting
      /// </summary>
      [Description("face-pouting")]
      public const string FacePouting = "\ue387";

      /// <summary>
      /// fa-face-raised-eyebrow unicode value ("\ue388").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-raised-eyebrow
      /// </summary>
      [Description("face-raised-eyebrow")]
      public const string FaceRaisedEyebrow = "\ue388";

      /// <summary>
      /// fa-face-relieved unicode value ("\ue389").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-relieved
      /// </summary>
      [Description("face-relieved")]
      public const string FaceRelieved = "\ue389";

      /// <summary>
      /// fa-face-rolling-eyes unicode value ("\uf5a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-rolling-eyes
      /// </summary>
      [Description("face-rolling-eyes")]
      public const string FaceRollingEyes = "\uf5a5";

      /// <summary>
      /// fa-face-sad-cry unicode value ("\uf5b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sad-cry
      /// </summary>
      [Description("face-sad-cry")]
      public const string FaceSadCry = "\uf5b3";

      /// <summary>
      /// fa-face-sad-sweat unicode value ("\ue38a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sad-sweat
      /// </summary>
      [Description("face-sad-sweat")]
      public const string FaceSadSweat = "\ue38a";

      /// <summary>
      /// fa-face-sad-tear unicode value ("\uf5b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sad-tear
      /// </summary>
      [Description("face-sad-tear")]
      public const string FaceSadTear = "\uf5b4";

      /// <summary>
      /// fa-face-saluting unicode value ("\ue484").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-saluting
      /// </summary>
      [Description("face-saluting")]
      public const string FaceSaluting = "\ue484";

      /// <summary>
      /// fa-face-scream unicode value ("\ue38b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-scream
      /// </summary>
      [Description("face-scream")]
      public const string FaceScream = "\ue38b";

      /// <summary>
      /// fa-face-shush unicode value ("\ue38c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-shush
      /// </summary>
      [Description("face-shush")]
      public const string FaceShush = "\ue38c";

      /// <summary>
      /// fa-face-sleeping unicode value ("\ue38d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sleeping
      /// </summary>
      [Description("face-sleeping")]
      public const string FaceSleeping = "\ue38d";

      /// <summary>
      /// fa-face-sleepy unicode value ("\ue38e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sleepy
      /// </summary>
      [Description("face-sleepy")]
      public const string FaceSleepy = "\ue38e";

      /// <summary>
      /// fa-face-smile unicode value ("\uf118").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile
      /// </summary>
      [Description("face-smile")]
      public const string FaceSmile = "\uf118";

      /// <summary>
      /// fa-face-smile-beam unicode value ("\uf5b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-beam
      /// </summary>
      [Description("face-smile-beam")]
      public const string FaceSmileBeam = "\uf5b8";

      /// <summary>
      /// fa-face-smile-halo unicode value ("\ue38f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-halo
      /// </summary>
      [Description("face-smile-halo")]
      public const string FaceSmileHalo = "\ue38f";

      /// <summary>
      /// fa-face-smile-hearts unicode value ("\ue390").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-hearts
      /// </summary>
      [Description("face-smile-hearts")]
      public const string FaceSmileHearts = "\ue390";

      /// <summary>
      /// fa-face-smile-horns unicode value ("\ue391").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-horns
      /// </summary>
      [Description("face-smile-horns")]
      public const string FaceSmileHorns = "\ue391";

      /// <summary>
      /// fa-face-smile-plus unicode value ("\uf5b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-plus
      /// </summary>
      [Description("face-smile-plus")]
      public const string FaceSmilePlus = "\uf5b9";

      /// <summary>
      /// fa-face-smile-relaxed unicode value ("\ue392").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-relaxed
      /// </summary>
      [Description("face-smile-relaxed")]
      public const string FaceSmileRelaxed = "\ue392";

      /// <summary>
      /// fa-face-smile-tear unicode value ("\ue393").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-tear
      /// </summary>
      [Description("face-smile-tear")]
      public const string FaceSmileTear = "\ue393";

      /// <summary>
      /// fa-face-smile-tongue unicode value ("\ue394").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-tongue
      /// </summary>
      [Description("face-smile-tongue")]
      public const string FaceSmileTongue = "\ue394";

      /// <summary>
      /// fa-face-smile-upside-down unicode value ("\ue395").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-upside-down
      /// </summary>
      [Description("face-smile-upside-down")]
      public const string FaceSmileUpsideDown = "\ue395";

      /// <summary>
      /// fa-face-smile-wink unicode value ("\uf4da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smile-wink
      /// </summary>
      [Description("face-smile-wink")]
      public const string FaceSmileWink = "\uf4da";

      /// <summary>
      /// fa-face-smiling-hands unicode value ("\ue396").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smiling-hands
      /// </summary>
      [Description("face-smiling-hands")]
      public const string FaceSmilingHands = "\ue396";

      /// <summary>
      /// fa-face-smirking unicode value ("\ue397").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-smirking
      /// </summary>
      [Description("face-smirking")]
      public const string FaceSmirking = "\ue397";

      /// <summary>
      /// fa-face-spiral-eyes unicode value ("\ue485").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-spiral-eyes
      /// </summary>
      [Description("face-spiral-eyes")]
      public const string FaceSpiralEyes = "\ue485";

      /// <summary>
      /// fa-face-sunglasses unicode value ("\ue398").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-sunglasses
      /// </summary>
      [Description("face-sunglasses")]
      public const string FaceSunglasses = "\ue398";

      /// <summary>
      /// fa-face-surprise unicode value ("\uf5c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-surprise
      /// </summary>
      [Description("face-surprise")]
      public const string FaceSurprise = "\uf5c2";

      /// <summary>
      /// fa-face-swear unicode value ("\ue399").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-swear
      /// </summary>
      [Description("face-swear")]
      public const string FaceSwear = "\ue399";

      /// <summary>
      /// fa-face-thermometer unicode value ("\ue39a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-thermometer
      /// </summary>
      [Description("face-thermometer")]
      public const string FaceThermometer = "\ue39a";

      /// <summary>
      /// fa-face-thinking unicode value ("\ue39b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-thinking
      /// </summary>
      [Description("face-thinking")]
      public const string FaceThinking = "\ue39b";

      /// <summary>
      /// fa-face-tired unicode value ("\uf5c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-tired
      /// </summary>
      [Description("face-tired")]
      public const string FaceTired = "\uf5c8";

      /// <summary>
      /// fa-face-tissue unicode value ("\ue39c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-tissue
      /// </summary>
      [Description("face-tissue")]
      public const string FaceTissue = "\ue39c";

      /// <summary>
      /// fa-face-tongue-money unicode value ("\ue39d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-tongue-money
      /// </summary>
      [Description("face-tongue-money")]
      public const string FaceTongueMoney = "\ue39d";

      /// <summary>
      /// fa-face-tongue-sweat unicode value ("\ue39e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-tongue-sweat
      /// </summary>
      [Description("face-tongue-sweat")]
      public const string FaceTongueSweat = "\ue39e";

      /// <summary>
      /// fa-face-unamused unicode value ("\ue39f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-unamused
      /// </summary>
      [Description("face-unamused")]
      public const string FaceUnamused = "\ue39f";

      /// <summary>
      /// fa-face-viewfinder unicode value ("\ue2ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-viewfinder
      /// </summary>
      [Description("face-viewfinder")]
      public const string FaceViewfinder = "\ue2ff";

      /// <summary>
      /// fa-face-vomit unicode value ("\ue3a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-vomit
      /// </summary>
      [Description("face-vomit")]
      public const string FaceVomit = "\ue3a0";

      /// <summary>
      /// fa-face-weary unicode value ("\ue3a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-weary
      /// </summary>
      [Description("face-weary")]
      public const string FaceWeary = "\ue3a1";

      /// <summary>
      /// fa-face-woozy unicode value ("\ue3a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-woozy
      /// </summary>
      [Description("face-woozy")]
      public const string FaceWoozy = "\ue3a2";

      /// <summary>
      /// fa-face-worried unicode value ("\ue3a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-worried
      /// </summary>
      [Description("face-worried")]
      public const string FaceWorried = "\ue3a3";

      /// <summary>
      /// fa-face-zany unicode value ("\ue3a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-zany
      /// </summary>
      [Description("face-zany")]
      public const string FaceZany = "\ue3a4";

      /// <summary>
      /// fa-face-zipper unicode value ("\ue3a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/face-zipper
      /// </summary>
      [Description("face-zipper")]
      public const string FaceZipper = "\ue3a5";

      /// <summary>
      /// fa-facebook unicode value ("\uf09a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/facebook
      /// </summary>
      [Description("facebook")]
      public const string Facebook = "\uf09a";

      /// <summary>
      /// fa-facebook-f unicode value ("\uf39e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/facebook-f
      /// </summary>
      [Description("facebook-f")]
      public const string FacebookF = "\uf39e";

      /// <summary>
      /// fa-facebook-messenger unicode value ("\uf39f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.9.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/facebook-messenger
      /// </summary>
      [Description("facebook-messenger")]
      public const string FacebookMessenger = "\uf39f";

      /// <summary>
      /// fa-falafel unicode value ("\ue40a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/falafel
      /// </summary>
      [Description("falafel")]
      public const string Falafel = "\ue40a";

      /// <summary>
      /// fa-family unicode value ("\ue300").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/family
      /// </summary>
      [Description("family")]
      public const string Family = "\ue300";

      /// <summary>
      /// fa-family-dress unicode value ("\ue301").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/family-dress
      /// </summary>
      [Description("family-dress")]
      public const string FamilyDress = "\ue301";

      /// <summary>
      /// fa-family-pants unicode value ("\ue302").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/family-pants
      /// </summary>
      [Description("family-pants")]
      public const string FamilyPants = "\ue302";

      /// <summary>
      /// fa-fan unicode value ("\uf863").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fan
      /// </summary>
      [Description("fan")]
      public const string Fan = "\uf863";

      /// <summary>
      /// fa-fan-table unicode value ("\ue004").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fan-table
      /// </summary>
      [Description("fan-table")]
      public const string FanTable = "\ue004";

      /// <summary>
      /// fa-fantasy-flight-games unicode value ("\uf6dc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '5.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fantasy-flight-games
      /// </summary>
      [Description("fantasy-flight-games")]
      public const string FantasyFlightGames = "\uf6dc";

      /// <summary>
      /// fa-farm unicode value ("\uf864").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/farm
      /// </summary>
      [Description("farm")]
      public const string Farm = "\uf864";

      /// <summary>
      /// fa-faucet unicode value ("\ue005").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/faucet
      /// </summary>
      [Description("faucet")]
      public const string Faucet = "\ue005";

      /// <summary>
      /// fa-faucet-drip unicode value ("\ue006").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/faucet-drip
      /// </summary>
      [Description("faucet-drip")]
      public const string FaucetDrip = "\ue006";

      /// <summary>
      /// fa-fax unicode value ("\uf1ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fax
      /// </summary>
      [Description("fax")]
      public const string Fax = "\uf1ac";

      /// <summary>
      /// fa-feather unicode value ("\uf52d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/feather
      /// </summary>
      [Description("feather")]
      public const string Feather = "\uf52d";

      /// <summary>
      /// fa-feather-pointed unicode value ("\uf56b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/feather-pointed
      /// </summary>
      [Description("feather-pointed")]
      public const string FeatherPointed = "\uf56b";

      /// <summary>
      /// fa-fedex unicode value ("\uf797").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fedex
      /// </summary>
      [Description("fedex")]
      public const string Fedex = "\uf797";

      /// <summary>
      /// fa-fedora unicode value ("\uf798").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fedora
      /// </summary>
      [Description("fedora")]
      public const string Fedora = "\uf798";

      /// <summary>
      /// fa-fence unicode value ("\ue303").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fence
      /// </summary>
      [Description("fence")]
      public const string Fence = "\ue303";

      /// <summary>
      /// fa-ferris-wheel unicode value ("\ue174").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ferris-wheel
      /// </summary>
      [Description("ferris-wheel")]
      public const string FerrisWheel = "\ue174";

      /// <summary>
      /// fa-ferry unicode value ("\ue4ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ferry
      /// </summary>
      [Description("ferry")]
      public const string Ferry = "\ue4ea";

      /// <summary>
      /// fa-field-hockey-stick-ball unicode value ("\uf44c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/field-hockey-stick-ball
      /// </summary>
      [Description("field-hockey-stick-ball")]
      public const string FieldHockeyStickBall = "\uf44c";

      /// <summary>
      /// fa-figma unicode value ("\uf799").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/figma
      /// </summary>
      [Description("figma")]
      public const string Figma = "\uf799";

      /// <summary>
      /// fa-file unicode value ("\uf15b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file
      /// </summary>
      [Description("file")]
      public const string File = "\uf15b";

      /// <summary>
      /// fa-file-arrow-down unicode value ("\uf56d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-arrow-down
      /// </summary>
      [Description("file-arrow-down")]
      public const string FileArrowDown = "\uf56d";

      /// <summary>
      /// fa-file-arrow-up unicode value ("\uf574").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-arrow-up
      /// </summary>
      [Description("file-arrow-up")]
      public const string FileArrowUp = "\uf574";

      /// <summary>
      /// fa-file-audio unicode value ("\uf1c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-audio
      /// </summary>
      [Description("file-audio")]
      public const string FileAudio = "\uf1c7";

      /// <summary>
      /// fa-file-binary unicode value ("\ue175").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-binary
      /// </summary>
      [Description("file-binary")]
      public const string FileBinary = "\ue175";

      /// <summary>
      /// fa-file-certificate unicode value ("\uf5f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-certificate
      /// </summary>
      [Description("file-certificate")]
      public const string FileCertificate = "\uf5f3";

      /// <summary>
      /// fa-file-chart-column unicode value ("\uf659").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-chart-column
      /// </summary>
      [Description("file-chart-column")]
      public const string FileChartColumn = "\uf659";

      /// <summary>
      /// fa-file-chart-pie unicode value ("\uf65a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-chart-pie
      /// </summary>
      [Description("file-chart-pie")]
      public const string FileChartPie = "\uf65a";

      /// <summary>
      /// fa-file-check unicode value ("\uf316").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-check
      /// </summary>
      [Description("file-check")]
      public const string FileCheck = "\uf316";

      /// <summary>
      /// fa-file-circle-check unicode value ("\ue5a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-check
      /// </summary>
      [Description("file-circle-check")]
      public const string FileCircleCheck = "\ue5a0";

      /// <summary>
      /// fa-file-circle-exclamation unicode value ("\ue4eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-exclamation
      /// </summary>
      [Description("file-circle-exclamation")]
      public const string FileCircleExclamation = "\ue4eb";

      /// <summary>
      /// fa-file-circle-info unicode value ("\ue493").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-info
      /// </summary>
      [Description("file-circle-info")]
      public const string FileCircleInfo = "\ue493";

      /// <summary>
      /// fa-file-circle-minus unicode value ("\ue4ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-minus
      /// </summary>
      [Description("file-circle-minus")]
      public const string FileCircleMinus = "\ue4ed";

      /// <summary>
      /// fa-file-circle-plus unicode value ("\ue494").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-plus
      /// </summary>
      [Description("file-circle-plus")]
      public const string FileCirclePlus = "\ue494";

      /// <summary>
      /// fa-file-circle-question unicode value ("\ue4ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-question
      /// </summary>
      [Description("file-circle-question")]
      public const string FileCircleQuestion = "\ue4ef";

      /// <summary>
      /// fa-file-circle-xmark unicode value ("\ue5a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-circle-xmark
      /// </summary>
      [Description("file-circle-xmark")]
      public const string FileCircleXmark = "\ue5a1";

      /// <summary>
      /// fa-file-code unicode value ("\uf1c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-code
      /// </summary>
      [Description("file-code")]
      public const string FileCode = "\uf1c9";

      /// <summary>
      /// fa-file-contract unicode value ("\uf56c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-contract
      /// </summary>
      [Description("file-contract")]
      public const string FileContract = "\uf56c";

      /// <summary>
      /// fa-file-csv unicode value ("\uf6dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-csv
      /// </summary>
      [Description("file-csv")]
      public const string FileCsv = "\uf6dd";

      /// <summary>
      /// fa-file-dashed-line unicode value ("\uf877").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-dashed-line
      /// </summary>
      [Description("file-dashed-line")]
      public const string FileDashedLine = "\uf877";

      /// <summary>
      /// fa-file-doc unicode value ("\ue5ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-doc
      /// </summary>
      [Description("file-doc")]
      public const string FileDoc = "\ue5ed";

      /// <summary>
      /// fa-file-excel unicode value ("\uf1c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-excel
      /// </summary>
      [Description("file-excel")]
      public const string FileExcel = "\uf1c3";

      /// <summary>
      /// fa-file-exclamation unicode value ("\uf31a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-exclamation
      /// </summary>
      [Description("file-exclamation")]
      public const string FileExclamation = "\uf31a";

      /// <summary>
      /// fa-file-export unicode value ("\uf56e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-export
      /// </summary>
      [Description("file-export")]
      public const string FileExport = "\uf56e";

      /// <summary>
      /// fa-file-heart unicode value ("\ue176").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-heart
      /// </summary>
      [Description("file-heart")]
      public const string FileHeart = "\ue176";

      /// <summary>
      /// fa-file-image unicode value ("\uf1c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-image
      /// </summary>
      [Description("file-image")]
      public const string FileImage = "\uf1c5";

      /// <summary>
      /// fa-file-import unicode value ("\uf56f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-import
      /// </summary>
      [Description("file-import")]
      public const string FileImport = "\uf56f";

      /// <summary>
      /// fa-file-invoice unicode value ("\uf570").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-invoice
      /// </summary>
      [Description("file-invoice")]
      public const string FileInvoice = "\uf570";

      /// <summary>
      /// fa-file-invoice-dollar unicode value ("\uf571").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-invoice-dollar
      /// </summary>
      [Description("file-invoice-dollar")]
      public const string FileInvoiceDollar = "\uf571";

      /// <summary>
      /// fa-file-lines unicode value ("\uf15c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-lines
      /// </summary>
      [Description("file-lines")]
      public const string FileLines = "\uf15c";

      /// <summary>
      /// fa-file-lock unicode value ("\ue3a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-lock
      /// </summary>
      [Description("file-lock")]
      public const string FileLock = "\ue3a6";

      /// <summary>
      /// fa-file-magnifying-glass unicode value ("\uf865").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-magnifying-glass
      /// </summary>
      [Description("file-magnifying-glass")]
      public const string FileMagnifyingGlass = "\uf865";

      /// <summary>
      /// fa-file-medical unicode value ("\uf477").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-medical
      /// </summary>
      [Description("file-medical")]
      public const string FileMedical = "\uf477";

      /// <summary>
      /// fa-file-minus unicode value ("\uf318").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-minus
      /// </summary>
      [Description("file-minus")]
      public const string FileMinus = "\uf318";

      /// <summary>
      /// fa-file-music unicode value ("\uf8b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-music
      /// </summary>
      [Description("file-music")]
      public const string FileMusic = "\uf8b6";

      /// <summary>
      /// fa-file-pdf unicode value ("\uf1c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-pdf
      /// </summary>
      [Description("file-pdf")]
      public const string FilePdf = "\uf1c1";

      /// <summary>
      /// fa-file-pen unicode value ("\uf31c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-pen
      /// </summary>
      [Description("file-pen")]
      public const string FilePen = "\uf31c";

      /// <summary>
      /// fa-file-plus unicode value ("\uf319").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-plus
      /// </summary>
      [Description("file-plus")]
      public const string FilePlus = "\uf319";

      /// <summary>
      /// fa-file-plus-minus unicode value ("\ue177").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-plus-minus
      /// </summary>
      [Description("file-plus-minus")]
      public const string FilePlusMinus = "\ue177";

      /// <summary>
      /// fa-file-powerpoint unicode value ("\uf1c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-powerpoint
      /// </summary>
      [Description("file-powerpoint")]
      public const string FilePowerpoint = "\uf1c4";

      /// <summary>
      /// fa-file-prescription unicode value ("\uf572").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-prescription
      /// </summary>
      [Description("file-prescription")]
      public const string FilePrescription = "\uf572";

      /// <summary>
      /// fa-file-shield unicode value ("\ue4f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-shield
      /// </summary>
      [Description("file-shield")]
      public const string FileShield = "\ue4f0";

      /// <summary>
      /// fa-file-signature unicode value ("\uf573").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-signature
      /// </summary>
      [Description("file-signature")]
      public const string FileSignature = "\uf573";

      /// <summary>
      /// fa-file-slash unicode value ("\ue3a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-slash
      /// </summary>
      [Description("file-slash")]
      public const string FileSlash = "\ue3a7";

      /// <summary>
      /// fa-file-spreadsheet unicode value ("\uf65b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-spreadsheet
      /// </summary>
      [Description("file-spreadsheet")]
      public const string FileSpreadsheet = "\uf65b";

      /// <summary>
      /// fa-file-user unicode value ("\uf65c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-user
      /// </summary>
      [Description("file-user")]
      public const string FileUser = "\uf65c";

      /// <summary>
      /// fa-file-video unicode value ("\uf1c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-video
      /// </summary>
      [Description("file-video")]
      public const string FileVideo = "\uf1c8";

      /// <summary>
      /// fa-file-waveform unicode value ("\uf478").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-waveform
      /// </summary>
      [Description("file-waveform")]
      public const string FileWaveform = "\uf478";

      /// <summary>
      /// fa-file-word unicode value ("\uf1c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-word
      /// </summary>
      [Description("file-word")]
      public const string FileWord = "\uf1c2";

      /// <summary>
      /// fa-file-xmark unicode value ("\uf317").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-xmark
      /// </summary>
      [Description("file-xmark")]
      public const string FileXmark = "\uf317";

      /// <summary>
      /// fa-file-zip unicode value ("\ue5ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-zip
      /// </summary>
      [Description("file-zip")]
      public const string FileZip = "\ue5ee";

      /// <summary>
      /// fa-file-zipper unicode value ("\uf1c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/file-zipper
      /// </summary>
      [Description("file-zipper")]
      public const string FileZipper = "\uf1c6";

      /// <summary>
      /// fa-files unicode value ("\ue178").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/files
      /// </summary>
      [Description("files")]
      public const string Files = "\ue178";

      /// <summary>
      /// fa-files-medical unicode value ("\uf7fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/files-medical
      /// </summary>
      [Description("files-medical")]
      public const string FilesMedical = "\uf7fd";

      /// <summary>
      /// fa-fill unicode value ("\uf575").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fill
      /// </summary>
      [Description("fill")]
      public const string Fill = "\uf575";

      /// <summary>
      /// fa-fill-drip unicode value ("\uf576").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fill-drip
      /// </summary>
      [Description("fill-drip")]
      public const string FillDrip = "\uf576";

      /// <summary>
      /// fa-film unicode value ("\uf008").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/film
      /// </summary>
      [Description("film")]
      public const string Film = "\uf008";

      /// <summary>
      /// fa-film-canister unicode value ("\uf8b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/film-canister
      /// </summary>
      [Description("film-canister")]
      public const string FilmCanister = "\uf8b7";

      /// <summary>
      /// fa-film-simple unicode value ("\uf3a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/film-simple
      /// </summary>
      [Description("film-simple")]
      public const string FilmSimple = "\uf3a0";

      /// <summary>
      /// fa-film-slash unicode value ("\ue179").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/film-slash
      /// </summary>
      [Description("film-slash")]
      public const string FilmSlash = "\ue179";

      /// <summary>
      /// fa-films unicode value ("\ue17a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/films
      /// </summary>
      [Description("films")]
      public const string Films = "\ue17a";

      /// <summary>
      /// fa-filter unicode value ("\uf0b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filter
      /// </summary>
      [Description("filter")]
      public const string Filter = "\uf0b0";

      /// <summary>
      /// fa-filter-circle-dollar unicode value ("\uf662").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filter-circle-dollar
      /// </summary>
      [Description("filter-circle-dollar")]
      public const string FilterCircleDollar = "\uf662";

      /// <summary>
      /// fa-filter-circle-xmark unicode value ("\ue17b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filter-circle-xmark
      /// </summary>
      [Description("filter-circle-xmark")]
      public const string FilterCircleXmark = "\ue17b";

      /// <summary>
      /// fa-filter-list unicode value ("\ue17c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filter-list
      /// </summary>
      [Description("filter-list")]
      public const string FilterList = "\ue17c";

      /// <summary>
      /// fa-filter-slash unicode value ("\ue17d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filter-slash
      /// </summary>
      [Description("filter-slash")]
      public const string FilterSlash = "\ue17d";

      /// <summary>
      /// fa-filters unicode value ("\ue17e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/filters
      /// </summary>
      [Description("filters")]
      public const string Filters = "\ue17e";

      /// <summary>
      /// fa-fingerprint unicode value ("\uf577").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fingerprint
      /// </summary>
      [Description("fingerprint")]
      public const string Fingerprint = "\uf577";

      /// <summary>
      /// fa-fire unicode value ("\uf06d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire
      /// </summary>
      [Description("fire")]
      public const string Fire = "\uf06d";

      /// <summary>
      /// fa-fire-burner unicode value ("\ue4f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-burner
      /// </summary>
      [Description("fire-burner")]
      public const string FireBurner = "\ue4f1";

      /// <summary>
      /// fa-fire-extinguisher unicode value ("\uf134").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-extinguisher
      /// </summary>
      [Description("fire-extinguisher")]
      public const string FireExtinguisher = "\uf134";

      /// <summary>
      /// fa-fire-flame unicode value ("\uf6df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-flame
      /// </summary>
      [Description("fire-flame")]
      public const string FireFlame = "\uf6df";

      /// <summary>
      /// fa-fire-flame-curved unicode value ("\uf7e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-flame-curved
      /// </summary>
      [Description("fire-flame-curved")]
      public const string FireFlameCurved = "\uf7e4";

      /// <summary>
      /// fa-fire-flame-simple unicode value ("\uf46a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-flame-simple
      /// </summary>
      [Description("fire-flame-simple")]
      public const string FireFlameSimple = "\uf46a";

      /// <summary>
      /// fa-fire-hydrant unicode value ("\ue17f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-hydrant
      /// </summary>
      [Description("fire-hydrant")]
      public const string FireHydrant = "\ue17f";

      /// <summary>
      /// fa-fire-smoke unicode value ("\uf74b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fire-smoke
      /// </summary>
      [Description("fire-smoke")]
      public const string FireSmoke = "\uf74b";

      /// <summary>
      /// fa-firefox unicode value ("\uf269").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/firefox
      /// </summary>
      [Description("firefox")]
      public const string Firefox = "\uf269";

      /// <summary>
      /// fa-firefox-browser unicode value ("\ue007").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/firefox-browser
      /// </summary>
      [Description("firefox-browser")]
      public const string FirefoxBrowser = "\ue007";

      /// <summary>
      /// fa-fireplace unicode value ("\uf79a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fireplace
      /// </summary>
      [Description("fireplace")]
      public const string Fireplace = "\uf79a";

      /// <summary>
      /// fa-first-order unicode value ("\uf2b0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/first-order
      /// </summary>
      [Description("first-order")]
      public const string FirstOrder = "\uf2b0";

      /// <summary>
      /// fa-first-order-alt unicode value ("\uf50a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/first-order-alt
      /// </summary>
      [Description("first-order-alt")]
      public const string FirstOrderAlt = "\uf50a";

      /// <summary>
      /// fa-firstdraft unicode value ("\uf3a1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/firstdraft
      /// </summary>
      [Description("firstdraft")]
      public const string Firstdraft = "\uf3a1";

      /// <summary>
      /// fa-fish unicode value ("\uf578").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fish
      /// </summary>
      [Description("fish")]
      public const string Fish = "\uf578";

      /// <summary>
      /// fa-fish-bones unicode value ("\ue304").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fish-bones
      /// </summary>
      [Description("fish-bones")]
      public const string FishBones = "\ue304";

      /// <summary>
      /// fa-fish-cooked unicode value ("\uf7fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fish-cooked
      /// </summary>
      [Description("fish-cooked")]
      public const string FishCooked = "\uf7fe";

      /// <summary>
      /// fa-fish-fins unicode value ("\ue4f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fish-fins
      /// </summary>
      [Description("fish-fins")]
      public const string FishFins = "\ue4f2";

      /// <summary>
      /// fa-fishing-rod unicode value ("\ue3a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fishing-rod
      /// </summary>
      [Description("fishing-rod")]
      public const string FishingRod = "\ue3a8";

      /// <summary>
      /// fa-flag unicode value ("\uf024").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flag
      /// </summary>
      [Description("flag")]
      public const string Flag = "\uf024";

      /// <summary>
      /// fa-flag-checkered unicode value ("\uf11e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flag-checkered
      /// </summary>
      [Description("flag-checkered")]
      public const string FlagCheckered = "\uf11e";

      /// <summary>
      /// fa-flag-pennant unicode value ("\uf456").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flag-pennant
      /// </summary>
      [Description("flag-pennant")]
      public const string FlagPennant = "\uf456";

      /// <summary>
      /// fa-flag-swallowtail unicode value ("\uf74c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flag-swallowtail
      /// </summary>
      [Description("flag-swallowtail")]
      public const string FlagSwallowtail = "\uf74c";

      /// <summary>
      /// fa-flag-usa unicode value ("\uf74d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flag-usa
      /// </summary>
      [Description("flag-usa")]
      public const string FlagUsa = "\uf74d";

      /// <summary>
      /// fa-flashlight unicode value ("\uf8b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flashlight
      /// </summary>
      [Description("flashlight")]
      public const string Flashlight = "\uf8b8";

      /// <summary>
      /// fa-flask unicode value ("\uf0c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flask
      /// </summary>
      [Description("flask")]
      public const string Flask = "\uf0c3";

      /// <summary>
      /// fa-flask-gear unicode value ("\ue5f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flask-gear
      /// </summary>
      [Description("flask-gear")]
      public const string FlaskGear = "\ue5f1";

      /// <summary>
      /// fa-flask-round-poison unicode value ("\uf6e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flask-round-poison
      /// </summary>
      [Description("flask-round-poison")]
      public const string FlaskRoundPoison = "\uf6e0";

      /// <summary>
      /// fa-flask-round-potion unicode value ("\uf6e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flask-round-potion
      /// </summary>
      [Description("flask-round-potion")]
      public const string FlaskRoundPotion = "\uf6e1";

      /// <summary>
      /// fa-flask-vial unicode value ("\ue4f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flask-vial
      /// </summary>
      [Description("flask-vial")]
      public const string FlaskVial = "\ue4f3";

      /// <summary>
      /// fa-flatbread unicode value ("\ue40b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flatbread
      /// </summary>
      [Description("flatbread")]
      public const string Flatbread = "\ue40b";

      /// <summary>
      /// fa-flatbread-stuffed unicode value ("\ue40c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flatbread-stuffed
      /// </summary>
      [Description("flatbread-stuffed")]
      public const string FlatbreadStuffed = "\ue40c";

      /// <summary>
      /// fa-flickr unicode value ("\uf16e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flickr
      /// </summary>
      [Description("flickr")]
      public const string Flickr = "\uf16e";

      /// <summary>
      /// fa-flipboard unicode value ("\uf44d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '5.0.9'.
      /// <para/>
      /// See https://fontawesome.com/icons/flipboard
      /// </summary>
      [Description("flipboard")]
      public const string Flipboard = "\uf44d";

      /// <summary>
      /// fa-floppy-disk unicode value ("\uf0c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/floppy-disk
      /// </summary>
      [Description("floppy-disk")]
      public const string FloppyDisk = "\uf0c7";

      /// <summary>
      /// fa-floppy-disk-circle-arrow-right unicode value ("\ue180").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/floppy-disk-circle-arrow-right
      /// </summary>
      [Description("floppy-disk-circle-arrow-right")]
      public const string FloppyDiskCircleArrowRight = "\ue180";

      /// <summary>
      /// fa-floppy-disk-circle-xmark unicode value ("\ue181").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/floppy-disk-circle-xmark
      /// </summary>
      [Description("floppy-disk-circle-xmark")]
      public const string FloppyDiskCircleXmark = "\ue181";

      /// <summary>
      /// fa-floppy-disk-pen unicode value ("\ue182").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/floppy-disk-pen
      /// </summary>
      [Description("floppy-disk-pen")]
      public const string FloppyDiskPen = "\ue182";

      /// <summary>
      /// fa-floppy-disks unicode value ("\ue183").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/floppy-disks
      /// </summary>
      [Description("floppy-disks")]
      public const string FloppyDisks = "\ue183";

      /// <summary>
      /// fa-florin-sign unicode value ("\ue184").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/florin-sign
      /// </summary>
      [Description("florin-sign")]
      public const string FlorinSign = "\ue184";

      /// <summary>
      /// fa-flower unicode value ("\uf7ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flower
      /// </summary>
      [Description("flower")]
      public const string Flower = "\uf7ff";

      /// <summary>
      /// fa-flower-daffodil unicode value ("\uf800").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flower-daffodil
      /// </summary>
      [Description("flower-daffodil")]
      public const string FlowerDaffodil = "\uf800";

      /// <summary>
      /// fa-flower-tulip unicode value ("\uf801").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flower-tulip
      /// </summary>
      [Description("flower-tulip")]
      public const string FlowerTulip = "\uf801";

      /// <summary>
      /// fa-flute unicode value ("\uf8b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flute
      /// </summary>
      [Description("flute")]
      public const string Flute = "\uf8b9";

      /// <summary>
      /// fa-flux-capacitor unicode value ("\uf8ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flux-capacitor
      /// </summary>
      [Description("flux-capacitor")]
      public const string FluxCapacitor = "\uf8ba";

      /// <summary>
      /// fa-fly unicode value ("\uf417").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fly
      /// </summary>
      [Description("fly")]
      public const string Fly = "\uf417";

      /// <summary>
      /// fa-flying-disc unicode value ("\ue3a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/flying-disc
      /// </summary>
      [Description("flying-disc")]
      public const string FlyingDisc = "\ue3a9";

      /// <summary>
      /// fa-folder unicode value ("\uf07b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder
      /// </summary>
      [Description("folder")]
      public const string Folder = "\uf07b";

      /// <summary>
      /// fa-folder-arrow-down unicode value ("\ue053").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-arrow-down
      /// </summary>
      [Description("folder-arrow-down")]
      public const string FolderArrowDown = "\ue053";

      /// <summary>
      /// fa-folder-arrow-up unicode value ("\ue054").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-arrow-up
      /// </summary>
      [Description("folder-arrow-up")]
      public const string FolderArrowUp = "\ue054";

      /// <summary>
      /// fa-folder-bookmark unicode value ("\ue186").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-bookmark
      /// </summary>
      [Description("folder-bookmark")]
      public const string FolderBookmark = "\ue186";

      /// <summary>
      /// fa-folder-closed unicode value ("\ue185").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-closed
      /// </summary>
      [Description("folder-closed")]
      public const string FolderClosed = "\ue185";

      /// <summary>
      /// fa-folder-gear unicode value ("\ue187").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-gear
      /// </summary>
      [Description("folder-gear")]
      public const string FolderGear = "\ue187";

      /// <summary>
      /// fa-folder-grid unicode value ("\ue188").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-grid
      /// </summary>
      [Description("folder-grid")]
      public const string FolderGrid = "\ue188";

      /// <summary>
      /// fa-folder-heart unicode value ("\ue189").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-heart
      /// </summary>
      [Description("folder-heart")]
      public const string FolderHeart = "\ue189";

      /// <summary>
      /// fa-folder-image unicode value ("\ue18a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-image
      /// </summary>
      [Description("folder-image")]
      public const string FolderImage = "\ue18a";

      /// <summary>
      /// fa-folder-magnifying-glass unicode value ("\ue18b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-magnifying-glass
      /// </summary>
      [Description("folder-magnifying-glass")]
      public const string FolderMagnifyingGlass = "\ue18b";

      /// <summary>
      /// fa-folder-medical unicode value ("\ue18c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-medical
      /// </summary>
      [Description("folder-medical")]
      public const string FolderMedical = "\ue18c";

      /// <summary>
      /// fa-folder-minus unicode value ("\uf65d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-minus
      /// </summary>
      [Description("folder-minus")]
      public const string FolderMinus = "\uf65d";

      /// <summary>
      /// fa-folder-music unicode value ("\ue18d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-music
      /// </summary>
      [Description("folder-music")]
      public const string FolderMusic = "\ue18d";

      /// <summary>
      /// fa-folder-open unicode value ("\uf07c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-open
      /// </summary>
      [Description("folder-open")]
      public const string FolderOpen = "\uf07c";

      /// <summary>
      /// fa-folder-plus unicode value ("\uf65e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-plus
      /// </summary>
      [Description("folder-plus")]
      public const string FolderPlus = "\uf65e";

      /// <summary>
      /// fa-folder-tree unicode value ("\uf802").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-tree
      /// </summary>
      [Description("folder-tree")]
      public const string FolderTree = "\uf802";

      /// <summary>
      /// fa-folder-user unicode value ("\ue18e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-user
      /// </summary>
      [Description("folder-user")]
      public const string FolderUser = "\ue18e";

      /// <summary>
      /// fa-folder-xmark unicode value ("\uf65f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folder-xmark
      /// </summary>
      [Description("folder-xmark")]
      public const string FolderXmark = "\uf65f";

      /// <summary>
      /// fa-folders unicode value ("\uf660").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/folders
      /// </summary>
      [Description("folders")]
      public const string Folders = "\uf660";

      /// <summary>
      /// fa-fondue-pot unicode value ("\ue40d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fondue-pot
      /// </summary>
      [Description("fondue-pot")]
      public const string FonduePot = "\ue40d";

      /// <summary>
      /// fa-font unicode value ("\uf031").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/font
      /// </summary>
      [Description("font")]
      public const string Font = "\uf031";

      /// <summary>
      /// fa-font-awesome unicode value ("\uf2b4").
      /// <para/>
      /// This icon supports the following styles: Brands, Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/font-awesome
      /// </summary>
      [Description("font-awesome")]
      public const string FontAwesome = "\uf2b4";

      /// <summary>
      /// fa-font-case unicode value ("\uf866").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/font-case
      /// </summary>
      [Description("font-case")]
      public const string FontCase = "\uf866";

      /// <summary>
      /// fa-fonticons unicode value ("\uf280").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fonticons
      /// </summary>
      [Description("fonticons")]
      public const string Fonticons = "\uf280";

      /// <summary>
      /// fa-fonticons-fi unicode value ("\uf3a2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fonticons-fi
      /// </summary>
      [Description("fonticons-fi")]
      public const string FonticonsFi = "\uf3a2";

      /// <summary>
      /// fa-football unicode value ("\uf44e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/football
      /// </summary>
      [Description("football")]
      public const string Football = "\uf44e";

      /// <summary>
      /// fa-football-helmet unicode value ("\uf44f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/football-helmet
      /// </summary>
      [Description("football-helmet")]
      public const string FootballHelmet = "\uf44f";

      /// <summary>
      /// fa-fork unicode value ("\uf2e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fork
      /// </summary>
      [Description("fork")]
      public const string Fork = "\uf2e3";

      /// <summary>
      /// fa-fork-knife unicode value ("\uf2e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fork-knife
      /// </summary>
      [Description("fork-knife")]
      public const string ForkKnife = "\uf2e6";

      /// <summary>
      /// fa-forklift unicode value ("\uf47a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/forklift
      /// </summary>
      [Description("forklift")]
      public const string Forklift = "\uf47a";

      /// <summary>
      /// fa-fort unicode value ("\ue486").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fort
      /// </summary>
      [Description("fort")]
      public const string Fort = "\ue486";

      /// <summary>
      /// fa-fort-awesome unicode value ("\uf286").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/fort-awesome
      /// </summary>
      [Description("fort-awesome")]
      public const string FortAwesome = "\uf286";

      /// <summary>
      /// fa-fort-awesome-alt unicode value ("\uf3a3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fort-awesome-alt
      /// </summary>
      [Description("fort-awesome-alt")]
      public const string FortAwesomeAlt = "\uf3a3";

      /// <summary>
      /// fa-forumbee unicode value ("\uf211").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/forumbee
      /// </summary>
      [Description("forumbee")]
      public const string Forumbee = "\uf211";

      /// <summary>
      /// fa-forward unicode value ("\uf04e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/forward
      /// </summary>
      [Description("forward")]
      public const string Forward = "\uf04e";

      /// <summary>
      /// fa-forward-fast unicode value ("\uf050").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/forward-fast
      /// </summary>
      [Description("forward-fast")]
      public const string ForwardFast = "\uf050";

      /// <summary>
      /// fa-forward-step unicode value ("\uf051").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/forward-step
      /// </summary>
      [Description("forward-step")]
      public const string ForwardStep = "\uf051";

      /// <summary>
      /// fa-foursquare unicode value ("\uf180").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/foursquare
      /// </summary>
      [Description("foursquare")]
      public const string Foursquare = "\uf180";

      /// <summary>
      /// fa-frame unicode value ("\ue495").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/frame
      /// </summary>
      [Description("frame")]
      public const string Frame = "\ue495";

      /// <summary>
      /// fa-franc-sign unicode value ("\ue18f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/franc-sign
      /// </summary>
      [Description("franc-sign")]
      public const string FrancSign = "\ue18f";

      /// <summary>
      /// fa-free-code-camp unicode value ("\uf2c5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/free-code-camp
      /// </summary>
      [Description("free-code-camp")]
      public const string FreeCodeCamp = "\uf2c5";

      /// <summary>
      /// fa-freebsd unicode value ("\uf3a4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/freebsd
      /// </summary>
      [Description("freebsd")]
      public const string Freebsd = "\uf3a4";

      /// <summary>
      /// fa-french-fries unicode value ("\uf803").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/french-fries
      /// </summary>
      [Description("french-fries")]
      public const string FrenchFries = "\uf803";

      /// <summary>
      /// fa-frog unicode value ("\uf52e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/frog
      /// </summary>
      [Description("frog")]
      public const string Frog = "\uf52e";

      /// <summary>
      /// fa-fulcrum unicode value ("\uf50b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/fulcrum
      /// </summary>
      [Description("fulcrum")]
      public const string Fulcrum = "\uf50b";

      /// <summary>
      /// fa-function unicode value ("\uf661").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/function
      /// </summary>
      [Description("function")]
      public const string Function = "\uf661";

      /// <summary>
      /// fa-futbol unicode value ("\uf1e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/futbol
      /// </summary>
      [Description("futbol")]
      public const string Futbol = "\uf1e3";

      /// <summary>
      /// fa-g unicode value ("\u0047").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/g
      /// </summary>
      [Description("g")]
      public const string G = "\u0047";

      /// <summary>
      /// fa-galactic-republic unicode value ("\uf50c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/galactic-republic
      /// </summary>
      [Description("galactic-republic")]
      public const string GalacticRepublic = "\uf50c";

      /// <summary>
      /// fa-galactic-senate unicode value ("\uf50d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/galactic-senate
      /// </summary>
      [Description("galactic-senate")]
      public const string GalacticSenate = "\uf50d";

      /// <summary>
      /// fa-galaxy unicode value ("\ue008").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/galaxy
      /// </summary>
      [Description("galaxy")]
      public const string Galaxy = "\ue008";

      /// <summary>
      /// fa-gallery-thumbnails unicode value ("\ue3aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gallery-thumbnails
      /// </summary>
      [Description("gallery-thumbnails")]
      public const string GalleryThumbnails = "\ue3aa";

      /// <summary>
      /// fa-game-board unicode value ("\uf867").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/game-board
      /// </summary>
      [Description("game-board")]
      public const string GameBoard = "\uf867";

      /// <summary>
      /// fa-game-board-simple unicode value ("\uf868").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/game-board-simple
      /// </summary>
      [Description("game-board-simple")]
      public const string GameBoardSimple = "\uf868";

      /// <summary>
      /// fa-game-console-handheld unicode value ("\uf8bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/game-console-handheld
      /// </summary>
      [Description("game-console-handheld")]
      public const string GameConsoleHandheld = "\uf8bb";

      /// <summary>
      /// fa-game-console-handheld-crank unicode value ("\ue5b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/game-console-handheld-crank
      /// </summary>
      [Description("game-console-handheld-crank")]
      public const string GameConsoleHandheldCrank = "\ue5b9";

      /// <summary>
      /// fa-gamepad unicode value ("\uf11b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gamepad
      /// </summary>
      [Description("gamepad")]
      public const string Gamepad = "\uf11b";

      /// <summary>
      /// fa-gamepad-modern unicode value ("\ue5a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gamepad-modern
      /// </summary>
      [Description("gamepad-modern")]
      public const string GamepadModern = "\ue5a2";

      /// <summary>
      /// fa-garage unicode value ("\ue009").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/garage
      /// </summary>
      [Description("garage")]
      public const string Garage = "\ue009";

      /// <summary>
      /// fa-garage-car unicode value ("\ue00a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/garage-car
      /// </summary>
      [Description("garage-car")]
      public const string GarageCar = "\ue00a";

      /// <summary>
      /// fa-garage-open unicode value ("\ue00b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/garage-open
      /// </summary>
      [Description("garage-open")]
      public const string GarageOpen = "\ue00b";

      /// <summary>
      /// fa-garlic unicode value ("\ue40e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/garlic
      /// </summary>
      [Description("garlic")]
      public const string Garlic = "\ue40e";

      /// <summary>
      /// fa-gas-pump unicode value ("\uf52f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gas-pump
      /// </summary>
      [Description("gas-pump")]
      public const string GasPump = "\uf52f";

      /// <summary>
      /// fa-gas-pump-slash unicode value ("\uf5f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gas-pump-slash
      /// </summary>
      [Description("gas-pump-slash")]
      public const string GasPumpSlash = "\uf5f4";

      /// <summary>
      /// fa-gauge unicode value ("\uf624").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge
      /// </summary>
      [Description("gauge")]
      public const string Gauge = "\uf624";

      /// <summary>
      /// fa-gauge-circle-bolt unicode value ("\ue496").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-circle-bolt
      /// </summary>
      [Description("gauge-circle-bolt")]
      public const string GaugeCircleBolt = "\ue496";

      /// <summary>
      /// fa-gauge-circle-minus unicode value ("\ue497").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-circle-minus
      /// </summary>
      [Description("gauge-circle-minus")]
      public const string GaugeCircleMinus = "\ue497";

      /// <summary>
      /// fa-gauge-circle-plus unicode value ("\ue498").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-circle-plus
      /// </summary>
      [Description("gauge-circle-plus")]
      public const string GaugeCirclePlus = "\ue498";

      /// <summary>
      /// fa-gauge-high unicode value ("\uf625").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-high
      /// </summary>
      [Description("gauge-high")]
      public const string GaugeHigh = "\uf625";

      /// <summary>
      /// fa-gauge-low unicode value ("\uf627").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-low
      /// </summary>
      [Description("gauge-low")]
      public const string GaugeLow = "\uf627";

      /// <summary>
      /// fa-gauge-max unicode value ("\uf626").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-max
      /// </summary>
      [Description("gauge-max")]
      public const string GaugeMax = "\uf626";

      /// <summary>
      /// fa-gauge-min unicode value ("\uf628").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-min
      /// </summary>
      [Description("gauge-min")]
      public const string GaugeMin = "\uf628";

      /// <summary>
      /// fa-gauge-simple unicode value ("\uf629").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-simple
      /// </summary>
      [Description("gauge-simple")]
      public const string GaugeSimple = "\uf629";

      /// <summary>
      /// fa-gauge-simple-high unicode value ("\uf62a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-simple-high
      /// </summary>
      [Description("gauge-simple-high")]
      public const string GaugeSimpleHigh = "\uf62a";

      /// <summary>
      /// fa-gauge-simple-low unicode value ("\uf62c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-simple-low
      /// </summary>
      [Description("gauge-simple-low")]
      public const string GaugeSimpleLow = "\uf62c";

      /// <summary>
      /// fa-gauge-simple-max unicode value ("\uf62b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-simple-max
      /// </summary>
      [Description("gauge-simple-max")]
      public const string GaugeSimpleMax = "\uf62b";

      /// <summary>
      /// fa-gauge-simple-min unicode value ("\uf62d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gauge-simple-min
      /// </summary>
      [Description("gauge-simple-min")]
      public const string GaugeSimpleMin = "\uf62d";

      /// <summary>
      /// fa-gavel unicode value ("\uf0e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gavel
      /// </summary>
      [Description("gavel")]
      public const string Gavel = "\uf0e3";

      /// <summary>
      /// fa-gear unicode value ("\uf013").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gear
      /// </summary>
      [Description("gear")]
      public const string Gear = "\uf013";

      /// <summary>
      /// fa-gear-code unicode value ("\ue5e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gear-code
      /// </summary>
      [Description("gear-code")]
      public const string GearCode = "\ue5e8";

      /// <summary>
      /// fa-gear-complex unicode value ("\ue5e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gear-complex
      /// </summary>
      [Description("gear-complex")]
      public const string GearComplex = "\ue5e9";

      /// <summary>
      /// fa-gear-complex-code unicode value ("\ue5eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gear-complex-code
      /// </summary>
      [Description("gear-complex-code")]
      public const string GearComplexCode = "\ue5eb";

      /// <summary>
      /// fa-gears unicode value ("\uf085").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gears
      /// </summary>
      [Description("gears")]
      public const string Gears = "\uf085";

      /// <summary>
      /// fa-gem unicode value ("\uf3a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gem
      /// </summary>
      [Description("gem")]
      public const string Gem = "\uf3a5";

      /// <summary>
      /// fa-genderless unicode value ("\uf22d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/genderless
      /// </summary>
      [Description("genderless")]
      public const string Genderless = "\uf22d";

      /// <summary>
      /// fa-get-pocket unicode value ("\uf265").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/get-pocket
      /// </summary>
      [Description("get-pocket")]
      public const string GetPocket = "\uf265";

      /// <summary>
      /// fa-gg unicode value ("\uf260").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gg
      /// </summary>
      [Description("gg")]
      public const string Gg = "\uf260";

      /// <summary>
      /// fa-gg-circle unicode value ("\uf261").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gg-circle
      /// </summary>
      [Description("gg-circle")]
      public const string GgCircle = "\uf261";

      /// <summary>
      /// fa-ghost unicode value ("\uf6e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ghost
      /// </summary>
      [Description("ghost")]
      public const string Ghost = "\uf6e2";

      /// <summary>
      /// fa-gif unicode value ("\ue190").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gif
      /// </summary>
      [Description("gif")]
      public const string Gif = "\ue190";

      /// <summary>
      /// fa-gift unicode value ("\uf06b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gift
      /// </summary>
      [Description("gift")]
      public const string Gift = "\uf06b";

      /// <summary>
      /// fa-gift-card unicode value ("\uf663").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gift-card
      /// </summary>
      [Description("gift-card")]
      public const string GiftCard = "\uf663";

      /// <summary>
      /// fa-gifts unicode value ("\uf79c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gifts
      /// </summary>
      [Description("gifts")]
      public const string Gifts = "\uf79c";

      /// <summary>
      /// fa-gingerbread-man unicode value ("\uf79d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gingerbread-man
      /// </summary>
      [Description("gingerbread-man")]
      public const string GingerbreadMan = "\uf79d";

      /// <summary>
      /// fa-git unicode value ("\uf1d3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/git
      /// </summary>
      [Description("git")]
      public const string Git = "\uf1d3";

      /// <summary>
      /// fa-git-alt unicode value ("\uf841").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.2', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/git-alt
      /// </summary>
      [Description("git-alt")]
      public const string GitAlt = "\uf841";

      /// <summary>
      /// fa-github unicode value ("\uf09b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/github
      /// </summary>
      [Description("github")]
      public const string Github = "\uf09b";

      /// <summary>
      /// fa-github-alt unicode value ("\uf113").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/github-alt
      /// </summary>
      [Description("github-alt")]
      public const string GithubAlt = "\uf113";

      /// <summary>
      /// fa-gitkraken unicode value ("\uf3a6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gitkraken
      /// </summary>
      [Description("gitkraken")]
      public const string Gitkraken = "\uf3a6";

      /// <summary>
      /// fa-gitlab unicode value ("\uf296").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/gitlab
      /// </summary>
      [Description("gitlab")]
      public const string Gitlab = "\uf296";

      /// <summary>
      /// fa-gitter unicode value ("\uf426").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gitter
      /// </summary>
      [Description("gitter")]
      public const string Gitter = "\uf426";

      /// <summary>
      /// fa-glass unicode value ("\uf804").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass
      /// </summary>
      [Description("glass")]
      public const string Glass = "\uf804";

      /// <summary>
      /// fa-glass-citrus unicode value ("\uf869").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass-citrus
      /// </summary>
      [Description("glass-citrus")]
      public const string GlassCitrus = "\uf869";

      /// <summary>
      /// fa-glass-empty unicode value ("\ue191").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass-empty
      /// </summary>
      [Description("glass-empty")]
      public const string GlassEmpty = "\ue191";

      /// <summary>
      /// fa-glass-half unicode value ("\ue192").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass-half
      /// </summary>
      [Description("glass-half")]
      public const string GlassHalf = "\ue192";

      /// <summary>
      /// fa-glass-water unicode value ("\ue4f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass-water
      /// </summary>
      [Description("glass-water")]
      public const string GlassWater = "\ue4f4";

      /// <summary>
      /// fa-glass-water-droplet unicode value ("\ue4f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glass-water-droplet
      /// </summary>
      [Description("glass-water-droplet")]
      public const string GlassWaterDroplet = "\ue4f5";

      /// <summary>
      /// fa-glasses unicode value ("\uf530").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glasses
      /// </summary>
      [Description("glasses")]
      public const string Glasses = "\uf530";

      /// <summary>
      /// fa-glasses-round unicode value ("\uf5f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glasses-round
      /// </summary>
      [Description("glasses-round")]
      public const string GlassesRound = "\uf5f5";

      /// <summary>
      /// fa-glide unicode value ("\uf2a5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glide
      /// </summary>
      [Description("glide")]
      public const string Glide = "\uf2a5";

      /// <summary>
      /// fa-glide-g unicode value ("\uf2a6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/glide-g
      /// </summary>
      [Description("glide-g")]
      public const string GlideG = "\uf2a6";

      /// <summary>
      /// fa-globe unicode value ("\uf0ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/globe
      /// </summary>
      [Description("globe")]
      public const string Globe = "\uf0ac";

      /// <summary>
      /// fa-globe-snow unicode value ("\uf7a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/globe-snow
      /// </summary>
      [Description("globe-snow")]
      public const string GlobeSnow = "\uf7a3";

      /// <summary>
      /// fa-globe-stand unicode value ("\uf5f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/globe-stand
      /// </summary>
      [Description("globe-stand")]
      public const string GlobeStand = "\uf5f6";

      /// <summary>
      /// fa-goal-net unicode value ("\ue3ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/goal-net
      /// </summary>
      [Description("goal-net")]
      public const string GoalNet = "\ue3ab";

      /// <summary>
      /// fa-gofore unicode value ("\uf3a7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gofore
      /// </summary>
      [Description("gofore")]
      public const string Gofore = "\uf3a7";

      /// <summary>
      /// fa-golang unicode value ("\ue40f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/golang
      /// </summary>
      [Description("golang")]
      public const string Golang = "\ue40f";

      /// <summary>
      /// fa-golf-ball-tee unicode value ("\uf450").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/golf-ball-tee
      /// </summary>
      [Description("golf-ball-tee")]
      public const string GolfBallTee = "\uf450";

      /// <summary>
      /// fa-golf-club unicode value ("\uf451").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/golf-club
      /// </summary>
      [Description("golf-club")]
      public const string GolfClub = "\uf451";

      /// <summary>
      /// fa-golf-flag-hole unicode value ("\ue3ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/golf-flag-hole
      /// </summary>
      [Description("golf-flag-hole")]
      public const string GolfFlagHole = "\ue3ac";

      /// <summary>
      /// fa-goodreads unicode value ("\uf3a8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/goodreads
      /// </summary>
      [Description("goodreads")]
      public const string Goodreads = "\uf3a8";

      /// <summary>
      /// fa-goodreads-g unicode value ("\uf3a9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/goodreads-g
      /// </summary>
      [Description("goodreads-g")]
      public const string GoodreadsG = "\uf3a9";

      /// <summary>
      /// fa-google unicode value ("\uf1a0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google
      /// </summary>
      [Description("google")]
      public const string Google = "\uf1a0";

      /// <summary>
      /// fa-google-drive unicode value ("\uf3aa").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-drive
      /// </summary>
      [Description("google-drive")]
      public const string GoogleDrive = "\uf3aa";

      /// <summary>
      /// fa-google-pay unicode value ("\ue079").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-pay
      /// </summary>
      [Description("google-pay")]
      public const string GooglePay = "\ue079";

      /// <summary>
      /// fa-google-play unicode value ("\uf3ab").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-play
      /// </summary>
      [Description("google-play")]
      public const string GooglePlay = "\uf3ab";

      /// <summary>
      /// fa-google-plus unicode value ("\uf2b3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.13.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-plus
      /// </summary>
      [Description("google-plus")]
      public const string GooglePlus = "\uf2b3";

      /// <summary>
      /// fa-google-plus-g unicode value ("\uf0d5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-plus-g
      /// </summary>
      [Description("google-plus-g")]
      public const string GooglePlusG = "\uf0d5";

      /// <summary>
      /// fa-google-wallet unicode value ("\uf1ee").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/google-wallet
      /// </summary>
      [Description("google-wallet")]
      public const string GoogleWallet = "\uf1ee";

      /// <summary>
      /// fa-gopuram unicode value ("\uf664").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gopuram
      /// </summary>
      [Description("gopuram")]
      public const string Gopuram = "\uf664";

      /// <summary>
      /// fa-graduation-cap unicode value ("\uf19d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/graduation-cap
      /// </summary>
      [Description("graduation-cap")]
      public const string GraduationCap = "\uf19d";

      /// <summary>
      /// fa-gramophone unicode value ("\uf8bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gramophone
      /// </summary>
      [Description("gramophone")]
      public const string Gramophone = "\uf8bd";

      /// <summary>
      /// fa-grapes unicode value ("\ue306").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grapes
      /// </summary>
      [Description("grapes")]
      public const string Grapes = "\ue306";

      /// <summary>
      /// fa-grate unicode value ("\ue193").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grate
      /// </summary>
      [Description("grate")]
      public const string Grate = "\ue193";

      /// <summary>
      /// fa-grate-droplet unicode value ("\ue194").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grate-droplet
      /// </summary>
      [Description("grate-droplet")]
      public const string GrateDroplet = "\ue194";

      /// <summary>
      /// fa-gratipay unicode value ("\uf184").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gratipay
      /// </summary>
      [Description("gratipay")]
      public const string Gratipay = "\uf184";

      /// <summary>
      /// fa-grav unicode value ("\uf2d6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grav
      /// </summary>
      [Description("grav")]
      public const string Grav = "\uf2d6";

      /// <summary>
      /// fa-greater-than unicode value ("\u003e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/greater-than
      /// </summary>
      [Description("greater-than")]
      public const string GreaterThan = "\u003e";

      /// <summary>
      /// fa-greater-than-equal unicode value ("\uf532").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/greater-than-equal
      /// </summary>
      [Description("greater-than-equal")]
      public const string GreaterThanEqual = "\uf532";

      /// <summary>
      /// fa-grid unicode value ("\ue195").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid
      /// </summary>
      [Description("grid")]
      public const string Grid = "\ue195";

      /// <summary>
      /// fa-grid-2 unicode value ("\ue196").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-2
      /// </summary>
      [Description("grid-2")]
      public const string Grid2 = "\ue196";

      /// <summary>
      /// fa-grid-2-plus unicode value ("\ue197").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-2-plus
      /// </summary>
      [Description("grid-2-plus")]
      public const string Grid2Plus = "\ue197";

      /// <summary>
      /// fa-grid-4 unicode value ("\ue198").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-4
      /// </summary>
      [Description("grid-4")]
      public const string Grid4 = "\ue198";

      /// <summary>
      /// fa-grid-5 unicode value ("\ue199").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-5
      /// </summary>
      [Description("grid-5")]
      public const string Grid5 = "\ue199";

      /// <summary>
      /// fa-grid-dividers unicode value ("\ue3ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-dividers
      /// </summary>
      [Description("grid-dividers")]
      public const string GridDividers = "\ue3ad";

      /// <summary>
      /// fa-grid-horizontal unicode value ("\ue307").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-horizontal
      /// </summary>
      [Description("grid-horizontal")]
      public const string GridHorizontal = "\ue307";

      /// <summary>
      /// fa-grid-round unicode value ("\ue5da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-round
      /// </summary>
      [Description("grid-round")]
      public const string GridRound = "\ue5da";

      /// <summary>
      /// fa-grid-round-2 unicode value ("\ue5db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-round-2
      /// </summary>
      [Description("grid-round-2")]
      public const string GridRound2 = "\ue5db";

      /// <summary>
      /// fa-grid-round-2-plus unicode value ("\ue5dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-round-2-plus
      /// </summary>
      [Description("grid-round-2-plus")]
      public const string GridRound2Plus = "\ue5dc";

      /// <summary>
      /// fa-grid-round-4 unicode value ("\ue5dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-round-4
      /// </summary>
      [Description("grid-round-4")]
      public const string GridRound4 = "\ue5dd";

      /// <summary>
      /// fa-grid-round-5 unicode value ("\ue5de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grid-round-5
      /// </summary>
      [Description("grid-round-5")]
      public const string GridRound5 = "\ue5de";

      /// <summary>
      /// fa-grill unicode value ("\ue5a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grill
      /// </summary>
      [Description("grill")]
      public const string Grill = "\ue5a3";

      /// <summary>
      /// fa-grill-fire unicode value ("\ue5a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grill-fire
      /// </summary>
      [Description("grill-fire")]
      public const string GrillFire = "\ue5a4";

      /// <summary>
      /// fa-grill-hot unicode value ("\ue5a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grill-hot
      /// </summary>
      [Description("grill-hot")]
      public const string GrillHot = "\ue5a5";

      /// <summary>
      /// fa-grip unicode value ("\uf58d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip
      /// </summary>
      [Description("grip")]
      public const string Grip = "\uf58d";

      /// <summary>
      /// fa-grip-dots unicode value ("\ue410").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip-dots
      /// </summary>
      [Description("grip-dots")]
      public const string GripDots = "\ue410";

      /// <summary>
      /// fa-grip-dots-vertical unicode value ("\ue411").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip-dots-vertical
      /// </summary>
      [Description("grip-dots-vertical")]
      public const string GripDotsVertical = "\ue411";

      /// <summary>
      /// fa-grip-lines unicode value ("\uf7a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip-lines
      /// </summary>
      [Description("grip-lines")]
      public const string GripLines = "\uf7a4";

      /// <summary>
      /// fa-grip-lines-vertical unicode value ("\uf7a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip-lines-vertical
      /// </summary>
      [Description("grip-lines-vertical")]
      public const string GripLinesVertical = "\uf7a5";

      /// <summary>
      /// fa-grip-vertical unicode value ("\uf58e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grip-vertical
      /// </summary>
      [Description("grip-vertical")]
      public const string GripVertical = "\uf58e";

      /// <summary>
      /// fa-gripfire unicode value ("\uf3ac").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gripfire
      /// </summary>
      [Description("gripfire")]
      public const string Gripfire = "\uf3ac";

      /// <summary>
      /// fa-group-arrows-rotate unicode value ("\ue4f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/group-arrows-rotate
      /// </summary>
      [Description("group-arrows-rotate")]
      public const string GroupArrowsRotate = "\ue4f6";

      /// <summary>
      /// fa-grunt unicode value ("\uf3ad").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/grunt
      /// </summary>
      [Description("grunt")]
      public const string Grunt = "\uf3ad";

      /// <summary>
      /// fa-guarani-sign unicode value ("\ue19a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/guarani-sign
      /// </summary>
      [Description("guarani-sign")]
      public const string GuaraniSign = "\ue19a";

      /// <summary>
      /// fa-guilded unicode value ("\ue07e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/guilded
      /// </summary>
      [Description("guilded")]
      public const string Guilded = "\ue07e";

      /// <summary>
      /// fa-guitar unicode value ("\uf7a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/guitar
      /// </summary>
      [Description("guitar")]
      public const string Guitar = "\uf7a6";

      /// <summary>
      /// fa-guitar-electric unicode value ("\uf8be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/guitar-electric
      /// </summary>
      [Description("guitar-electric")]
      public const string GuitarElectric = "\uf8be";

      /// <summary>
      /// fa-guitars unicode value ("\uf8bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/guitars
      /// </summary>
      [Description("guitars")]
      public const string Guitars = "\uf8bf";

      /// <summary>
      /// fa-gulp unicode value ("\uf3ae").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gulp
      /// </summary>
      [Description("gulp")]
      public const string Gulp = "\uf3ae";

      /// <summary>
      /// fa-gun unicode value ("\ue19b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gun
      /// </summary>
      [Description("gun")]
      public const string Gun = "\ue19b";

      /// <summary>
      /// fa-gun-slash unicode value ("\ue19c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gun-slash
      /// </summary>
      [Description("gun-slash")]
      public const string GunSlash = "\ue19c";

      /// <summary>
      /// fa-gun-squirt unicode value ("\ue19d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/gun-squirt
      /// </summary>
      [Description("gun-squirt")]
      public const string GunSquirt = "\ue19d";

      /// <summary>
      /// fa-h unicode value ("\u0048").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h
      /// </summary>
      [Description("h")]
      public const string H = "\u0048";

      /// <summary>
      /// fa-h1 unicode value ("\uf313").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h1
      /// </summary>
      [Description("h1")]
      public const string H1 = "\uf313";

      /// <summary>
      /// fa-h2 unicode value ("\uf314").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h2
      /// </summary>
      [Description("h2")]
      public const string H2 = "\uf314";

      /// <summary>
      /// fa-h3 unicode value ("\uf315").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h3
      /// </summary>
      [Description("h3")]
      public const string H3 = "\uf315";

      /// <summary>
      /// fa-h4 unicode value ("\uf86a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h4
      /// </summary>
      [Description("h4")]
      public const string H4 = "\uf86a";

      /// <summary>
      /// fa-h5 unicode value ("\ue412").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h5
      /// </summary>
      [Description("h5")]
      public const string H5 = "\ue412";

      /// <summary>
      /// fa-h6 unicode value ("\ue413").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/h6
      /// </summary>
      [Description("h6")]
      public const string H6 = "\ue413";

      /// <summary>
      /// fa-hacker-news unicode value ("\uf1d4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hacker-news
      /// </summary>
      [Description("hacker-news")]
      public const string HackerNews = "\uf1d4";

      /// <summary>
      /// fa-hackerrank unicode value ("\uf5f7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hackerrank
      /// </summary>
      [Description("hackerrank")]
      public const string Hackerrank = "\uf5f7";

      /// <summary>
      /// fa-hammer unicode value ("\uf6e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hammer
      /// </summary>
      [Description("hammer")]
      public const string Hammer = "\uf6e3";

      /// <summary>
      /// fa-hammer-crash unicode value ("\ue414").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hammer-crash
      /// </summary>
      [Description("hammer-crash")]
      public const string HammerCrash = "\ue414";

      /// <summary>
      /// fa-hammer-war unicode value ("\uf6e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hammer-war
      /// </summary>
      [Description("hammer-war")]
      public const string HammerWar = "\uf6e4";

      /// <summary>
      /// fa-hamsa unicode value ("\uf665").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hamsa
      /// </summary>
      [Description("hamsa")]
      public const string Hamsa = "\uf665";

      /// <summary>
      /// fa-hand unicode value ("\uf256").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand
      /// </summary>
      [Description("hand")]
      public const string Hand = "\uf256";

      /// <summary>
      /// fa-hand-back-fist unicode value ("\uf255").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-fist
      /// </summary>
      [Description("hand-back-fist")]
      public const string HandBackFist = "\uf255";

      /// <summary>
      /// fa-hand-back-point-down unicode value ("\ue19e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-point-down
      /// </summary>
      [Description("hand-back-point-down")]
      public const string HandBackPointDown = "\ue19e";

      /// <summary>
      /// fa-hand-back-point-left unicode value ("\ue19f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-point-left
      /// </summary>
      [Description("hand-back-point-left")]
      public const string HandBackPointLeft = "\ue19f";

      /// <summary>
      /// fa-hand-back-point-ribbon unicode value ("\ue1a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-point-ribbon
      /// </summary>
      [Description("hand-back-point-ribbon")]
      public const string HandBackPointRibbon = "\ue1a0";

      /// <summary>
      /// fa-hand-back-point-right unicode value ("\ue1a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-point-right
      /// </summary>
      [Description("hand-back-point-right")]
      public const string HandBackPointRight = "\ue1a1";

      /// <summary>
      /// fa-hand-back-point-up unicode value ("\ue1a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-back-point-up
      /// </summary>
      [Description("hand-back-point-up")]
      public const string HandBackPointUp = "\ue1a2";

      /// <summary>
      /// fa-hand-dots unicode value ("\uf461").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-dots
      /// </summary>
      [Description("hand-dots")]
      public const string HandDots = "\uf461";

      /// <summary>
      /// fa-hand-fingers-crossed unicode value ("\ue1a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-fingers-crossed
      /// </summary>
      [Description("hand-fingers-crossed")]
      public const string HandFingersCrossed = "\ue1a3";

      /// <summary>
      /// fa-hand-fist unicode value ("\uf6de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-fist
      /// </summary>
      [Description("hand-fist")]
      public const string HandFist = "\uf6de";

      /// <summary>
      /// fa-hand-heart unicode value ("\uf4bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-heart
      /// </summary>
      [Description("hand-heart")]
      public const string HandHeart = "\uf4bc";

      /// <summary>
      /// fa-hand-holding unicode value ("\uf4bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding
      /// </summary>
      [Description("hand-holding")]
      public const string HandHolding = "\uf4bd";

      /// <summary>
      /// fa-hand-holding-box unicode value ("\uf47b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-box
      /// </summary>
      [Description("hand-holding-box")]
      public const string HandHoldingBox = "\uf47b";

      /// <summary>
      /// fa-hand-holding-dollar unicode value ("\uf4c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-dollar
      /// </summary>
      [Description("hand-holding-dollar")]
      public const string HandHoldingDollar = "\uf4c0";

      /// <summary>
      /// fa-hand-holding-droplet unicode value ("\uf4c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-droplet
      /// </summary>
      [Description("hand-holding-droplet")]
      public const string HandHoldingDroplet = "\uf4c1";

      /// <summary>
      /// fa-hand-holding-hand unicode value ("\ue4f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-hand
      /// </summary>
      [Description("hand-holding-hand")]
      public const string HandHoldingHand = "\ue4f7";

      /// <summary>
      /// fa-hand-holding-heart unicode value ("\uf4be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-heart
      /// </summary>
      [Description("hand-holding-heart")]
      public const string HandHoldingHeart = "\uf4be";

      /// <summary>
      /// fa-hand-holding-magic unicode value ("\uf6e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-magic
      /// </summary>
      [Description("hand-holding-magic")]
      public const string HandHoldingMagic = "\uf6e5";

      /// <summary>
      /// fa-hand-holding-medical unicode value ("\ue05c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-medical
      /// </summary>
      [Description("hand-holding-medical")]
      public const string HandHoldingMedical = "\ue05c";

      /// <summary>
      /// fa-hand-holding-seedling unicode value ("\uf4bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-seedling
      /// </summary>
      [Description("hand-holding-seedling")]
      public const string HandHoldingSeedling = "\uf4bf";

      /// <summary>
      /// fa-hand-holding-skull unicode value ("\ue1a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-holding-skull
      /// </summary>
      [Description("hand-holding-skull")]
      public const string HandHoldingSkull = "\ue1a4";

      /// <summary>
      /// fa-hand-horns unicode value ("\ue1a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-horns
      /// </summary>
      [Description("hand-horns")]
      public const string HandHorns = "\ue1a9";

      /// <summary>
      /// fa-hand-lizard unicode value ("\uf258").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-lizard
      /// </summary>
      [Description("hand-lizard")]
      public const string HandLizard = "\uf258";

      /// <summary>
      /// fa-hand-love unicode value ("\ue1a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-love
      /// </summary>
      [Description("hand-love")]
      public const string HandLove = "\ue1a5";

      /// <summary>
      /// fa-hand-middle-finger unicode value ("\uf806").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-middle-finger
      /// </summary>
      [Description("hand-middle-finger")]
      public const string HandMiddleFinger = "\uf806";

      /// <summary>
      /// fa-hand-peace unicode value ("\uf25b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-peace
      /// </summary>
      [Description("hand-peace")]
      public const string HandPeace = "\uf25b";

      /// <summary>
      /// fa-hand-point-down unicode value ("\uf0a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-point-down
      /// </summary>
      [Description("hand-point-down")]
      public const string HandPointDown = "\uf0a7";

      /// <summary>
      /// fa-hand-point-left unicode value ("\uf0a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-point-left
      /// </summary>
      [Description("hand-point-left")]
      public const string HandPointLeft = "\uf0a5";

      /// <summary>
      /// fa-hand-point-ribbon unicode value ("\ue1a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-point-ribbon
      /// </summary>
      [Description("hand-point-ribbon")]
      public const string HandPointRibbon = "\ue1a6";

      /// <summary>
      /// fa-hand-point-right unicode value ("\uf0a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-point-right
      /// </summary>
      [Description("hand-point-right")]
      public const string HandPointRight = "\uf0a4";

      /// <summary>
      /// fa-hand-point-up unicode value ("\uf0a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-point-up
      /// </summary>
      [Description("hand-point-up")]
      public const string HandPointUp = "\uf0a6";

      /// <summary>
      /// fa-hand-pointer unicode value ("\uf25a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-pointer
      /// </summary>
      [Description("hand-pointer")]
      public const string HandPointer = "\uf25a";

      /// <summary>
      /// fa-hand-scissors unicode value ("\uf257").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-scissors
      /// </summary>
      [Description("hand-scissors")]
      public const string HandScissors = "\uf257";

      /// <summary>
      /// fa-hand-sparkles unicode value ("\ue05d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-sparkles
      /// </summary>
      [Description("hand-sparkles")]
      public const string HandSparkles = "\ue05d";

      /// <summary>
      /// fa-hand-spock unicode value ("\uf259").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-spock
      /// </summary>
      [Description("hand-spock")]
      public const string HandSpock = "\uf259";

      /// <summary>
      /// fa-hand-wave unicode value ("\ue1a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hand-wave
      /// </summary>
      [Description("hand-wave")]
      public const string HandWave = "\ue1a7";

      /// <summary>
      /// fa-handcuffs unicode value ("\ue4f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handcuffs
      /// </summary>
      [Description("handcuffs")]
      public const string Handcuffs = "\ue4f8";

      /// <summary>
      /// fa-hands unicode value ("\uf2a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands
      /// </summary>
      [Description("hands")]
      public const string Hands = "\uf2a7";

      /// <summary>
      /// fa-hands-asl-interpreting unicode value ("\uf2a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-asl-interpreting
      /// </summary>
      [Description("hands-asl-interpreting")]
      public const string HandsAslInterpreting = "\uf2a3";

      /// <summary>
      /// fa-hands-bound unicode value ("\ue4f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-bound
      /// </summary>
      [Description("hands-bound")]
      public const string HandsBound = "\ue4f9";

      /// <summary>
      /// fa-hands-bubbles unicode value ("\ue05e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-bubbles
      /// </summary>
      [Description("hands-bubbles")]
      public const string HandsBubbles = "\ue05e";

      /// <summary>
      /// fa-hands-clapping unicode value ("\ue1a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-clapping
      /// </summary>
      [Description("hands-clapping")]
      public const string HandsClapping = "\ue1a8";

      /// <summary>
      /// fa-hands-holding unicode value ("\uf4c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding
      /// </summary>
      [Description("hands-holding")]
      public const string HandsHolding = "\uf4c2";

      /// <summary>
      /// fa-hands-holding-child unicode value ("\ue4fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding-child
      /// </summary>
      [Description("hands-holding-child")]
      public const string HandsHoldingChild = "\ue4fa";

      /// <summary>
      /// fa-hands-holding-circle unicode value ("\ue4fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding-circle
      /// </summary>
      [Description("hands-holding-circle")]
      public const string HandsHoldingCircle = "\ue4fb";

      /// <summary>
      /// fa-hands-holding-diamond unicode value ("\uf47c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding-diamond
      /// </summary>
      [Description("hands-holding-diamond")]
      public const string HandsHoldingDiamond = "\uf47c";

      /// <summary>
      /// fa-hands-holding-dollar unicode value ("\uf4c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding-dollar
      /// </summary>
      [Description("hands-holding-dollar")]
      public const string HandsHoldingDollar = "\uf4c5";

      /// <summary>
      /// fa-hands-holding-heart unicode value ("\uf4c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-holding-heart
      /// </summary>
      [Description("hands-holding-heart")]
      public const string HandsHoldingHeart = "\uf4c3";

      /// <summary>
      /// fa-hands-praying unicode value ("\uf684").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hands-praying
      /// </summary>
      [Description("hands-praying")]
      public const string HandsPraying = "\uf684";

      /// <summary>
      /// fa-handshake unicode value ("\uf2b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handshake
      /// </summary>
      [Description("handshake")]
      public const string Handshake = "\uf2b5";

      /// <summary>
      /// fa-handshake-angle unicode value ("\uf4c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handshake-angle
      /// </summary>
      [Description("handshake-angle")]
      public const string HandshakeAngle = "\uf4c4";

      /// <summary>
      /// fa-handshake-simple unicode value ("\uf4c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handshake-simple
      /// </summary>
      [Description("handshake-simple")]
      public const string HandshakeSimple = "\uf4c6";

      /// <summary>
      /// fa-handshake-simple-slash unicode value ("\ue05f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handshake-simple-slash
      /// </summary>
      [Description("handshake-simple-slash")]
      public const string HandshakeSimpleSlash = "\ue05f";

      /// <summary>
      /// fa-handshake-slash unicode value ("\ue060").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/handshake-slash
      /// </summary>
      [Description("handshake-slash")]
      public const string HandshakeSlash = "\ue060";

      /// <summary>
      /// fa-hanukiah unicode value ("\uf6e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hanukiah
      /// </summary>
      [Description("hanukiah")]
      public const string Hanukiah = "\uf6e6";

      /// <summary>
      /// fa-hard-drive unicode value ("\uf0a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hard-drive
      /// </summary>
      [Description("hard-drive")]
      public const string HardDrive = "\uf0a0";

      /// <summary>
      /// fa-hashnode unicode value ("\ue499").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hashnode
      /// </summary>
      [Description("hashnode")]
      public const string Hashnode = "\ue499";

      /// <summary>
      /// fa-hashtag unicode value ("\u0023").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hashtag
      /// </summary>
      [Description("hashtag")]
      public const string Hashtag = "\u0023";

      /// <summary>
      /// fa-hashtag-lock unicode value ("\ue415").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hashtag-lock
      /// </summary>
      [Description("hashtag-lock")]
      public const string HashtagLock = "\ue415";

      /// <summary>
      /// fa-hat-beach unicode value ("\ue606").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-beach
      /// </summary>
      [Description("hat-beach")]
      public const string HatBeach = "\ue606";

      /// <summary>
      /// fa-hat-chef unicode value ("\uf86b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-chef
      /// </summary>
      [Description("hat-chef")]
      public const string HatChef = "\uf86b";

      /// <summary>
      /// fa-hat-cowboy unicode value ("\uf8c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-cowboy
      /// </summary>
      [Description("hat-cowboy")]
      public const string HatCowboy = "\uf8c0";

      /// <summary>
      /// fa-hat-cowboy-side unicode value ("\uf8c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-cowboy-side
      /// </summary>
      [Description("hat-cowboy-side")]
      public const string HatCowboySide = "\uf8c1";

      /// <summary>
      /// fa-hat-santa unicode value ("\uf7a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-santa
      /// </summary>
      [Description("hat-santa")]
      public const string HatSanta = "\uf7a7";

      /// <summary>
      /// fa-hat-winter unicode value ("\uf7a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-winter
      /// </summary>
      [Description("hat-winter")]
      public const string HatWinter = "\uf7a8";

      /// <summary>
      /// fa-hat-witch unicode value ("\uf6e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-witch
      /// </summary>
      [Description("hat-witch")]
      public const string HatWitch = "\uf6e7";

      /// <summary>
      /// fa-hat-wizard unicode value ("\uf6e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hat-wizard
      /// </summary>
      [Description("hat-wizard")]
      public const string HatWizard = "\uf6e8";

      /// <summary>
      /// fa-head-side unicode value ("\uf6e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side
      /// </summary>
      [Description("head-side")]
      public const string HeadSide = "\uf6e9";

      /// <summary>
      /// fa-head-side-brain unicode value ("\uf808").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-brain
      /// </summary>
      [Description("head-side-brain")]
      public const string HeadSideBrain = "\uf808";

      /// <summary>
      /// fa-head-side-cough unicode value ("\ue061").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-cough
      /// </summary>
      [Description("head-side-cough")]
      public const string HeadSideCough = "\ue061";

      /// <summary>
      /// fa-head-side-cough-slash unicode value ("\ue062").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-cough-slash
      /// </summary>
      [Description("head-side-cough-slash")]
      public const string HeadSideCoughSlash = "\ue062";

      /// <summary>
      /// fa-head-side-goggles unicode value ("\uf6ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-goggles
      /// </summary>
      [Description("head-side-goggles")]
      public const string HeadSideGoggles = "\uf6ea";

      /// <summary>
      /// fa-head-side-headphones unicode value ("\uf8c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-headphones
      /// </summary>
      [Description("head-side-headphones")]
      public const string HeadSideHeadphones = "\uf8c2";

      /// <summary>
      /// fa-head-side-heart unicode value ("\ue1aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-heart
      /// </summary>
      [Description("head-side-heart")]
      public const string HeadSideHeart = "\ue1aa";

      /// <summary>
      /// fa-head-side-mask unicode value ("\ue063").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-mask
      /// </summary>
      [Description("head-side-mask")]
      public const string HeadSideMask = "\ue063";

      /// <summary>
      /// fa-head-side-medical unicode value ("\uf809").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-medical
      /// </summary>
      [Description("head-side-medical")]
      public const string HeadSideMedical = "\uf809";

      /// <summary>
      /// fa-head-side-virus unicode value ("\ue064").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/head-side-virus
      /// </summary>
      [Description("head-side-virus")]
      public const string HeadSideVirus = "\ue064";

      /// <summary>
      /// fa-heading unicode value ("\uf1dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heading
      /// </summary>
      [Description("heading")]
      public const string Heading = "\uf1dc";

      /// <summary>
      /// fa-headphones unicode value ("\uf025").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/headphones
      /// </summary>
      [Description("headphones")]
      public const string Headphones = "\uf025";

      /// <summary>
      /// fa-headphones-simple unicode value ("\uf58f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/headphones-simple
      /// </summary>
      [Description("headphones-simple")]
      public const string HeadphonesSimple = "\uf58f";

      /// <summary>
      /// fa-headset unicode value ("\uf590").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/headset
      /// </summary>
      [Description("headset")]
      public const string Headset = "\uf590";

      /// <summary>
      /// fa-heart unicode value ("\uf004").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart
      /// </summary>
      [Description("heart")]
      public const string Heart = "\uf004";

      /// <summary>
      /// fa-heart-circle-bolt unicode value ("\ue4fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-bolt
      /// </summary>
      [Description("heart-circle-bolt")]
      public const string HeartCircleBolt = "\ue4fc";

      /// <summary>
      /// fa-heart-circle-check unicode value ("\ue4fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-check
      /// </summary>
      [Description("heart-circle-check")]
      public const string HeartCircleCheck = "\ue4fd";

      /// <summary>
      /// fa-heart-circle-exclamation unicode value ("\ue4fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-exclamation
      /// </summary>
      [Description("heart-circle-exclamation")]
      public const string HeartCircleExclamation = "\ue4fe";

      /// <summary>
      /// fa-heart-circle-minus unicode value ("\ue4ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-minus
      /// </summary>
      [Description("heart-circle-minus")]
      public const string HeartCircleMinus = "\ue4ff";

      /// <summary>
      /// fa-heart-circle-plus unicode value ("\ue500").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-plus
      /// </summary>
      [Description("heart-circle-plus")]
      public const string HeartCirclePlus = "\ue500";

      /// <summary>
      /// fa-heart-circle-xmark unicode value ("\ue501").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-circle-xmark
      /// </summary>
      [Description("heart-circle-xmark")]
      public const string HeartCircleXmark = "\ue501";

      /// <summary>
      /// fa-heart-crack unicode value ("\uf7a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-crack
      /// </summary>
      [Description("heart-crack")]
      public const string HeartCrack = "\uf7a9";

      /// <summary>
      /// fa-heart-half unicode value ("\ue1ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-half
      /// </summary>
      [Description("heart-half")]
      public const string HeartHalf = "\ue1ab";

      /// <summary>
      /// fa-heart-half-stroke unicode value ("\ue1ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-half-stroke
      /// </summary>
      [Description("heart-half-stroke")]
      public const string HeartHalfStroke = "\ue1ac";

      /// <summary>
      /// fa-heart-pulse unicode value ("\uf21e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heart-pulse
      /// </summary>
      [Description("heart-pulse")]
      public const string HeartPulse = "\uf21e";

      /// <summary>
      /// fa-heat unicode value ("\ue00c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/heat
      /// </summary>
      [Description("heat")]
      public const string Heat = "\ue00c";

      /// <summary>
      /// fa-helicopter unicode value ("\uf533").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/helicopter
      /// </summary>
      [Description("helicopter")]
      public const string Helicopter = "\uf533";

      /// <summary>
      /// fa-helicopter-symbol unicode value ("\ue502").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/helicopter-symbol
      /// </summary>
      [Description("helicopter-symbol")]
      public const string HelicopterSymbol = "\ue502";

      /// <summary>
      /// fa-helmet-battle unicode value ("\uf6eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/helmet-battle
      /// </summary>
      [Description("helmet-battle")]
      public const string HelmetBattle = "\uf6eb";

      /// <summary>
      /// fa-helmet-safety unicode value ("\uf807").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/helmet-safety
      /// </summary>
      [Description("helmet-safety")]
      public const string HelmetSafety = "\uf807";

      /// <summary>
      /// fa-helmet-un unicode value ("\ue503").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/helmet-un
      /// </summary>
      [Description("helmet-un")]
      public const string HelmetUn = "\ue503";

      /// <summary>
      /// fa-hexagon unicode value ("\uf312").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon
      /// </summary>
      [Description("hexagon")]
      public const string Hexagon = "\uf312";

      /// <summary>
      /// fa-hexagon-check unicode value ("\ue416").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-check
      /// </summary>
      [Description("hexagon-check")]
      public const string HexagonCheck = "\ue416";

      /// <summary>
      /// fa-hexagon-divide unicode value ("\ue1ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-divide
      /// </summary>
      [Description("hexagon-divide")]
      public const string HexagonDivide = "\ue1ad";

      /// <summary>
      /// fa-hexagon-exclamation unicode value ("\ue417").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-exclamation
      /// </summary>
      [Description("hexagon-exclamation")]
      public const string HexagonExclamation = "\ue417";

      /// <summary>
      /// fa-hexagon-image unicode value ("\ue504").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-image
      /// </summary>
      [Description("hexagon-image")]
      public const string HexagonImage = "\ue504";

      /// <summary>
      /// fa-hexagon-minus unicode value ("\uf307").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-minus
      /// </summary>
      [Description("hexagon-minus")]
      public const string HexagonMinus = "\uf307";

      /// <summary>
      /// fa-hexagon-plus unicode value ("\uf300").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-plus
      /// </summary>
      [Description("hexagon-plus")]
      public const string HexagonPlus = "\uf300";

      /// <summary>
      /// fa-hexagon-vertical-nft unicode value ("\ue505").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-vertical-nft
      /// </summary>
      [Description("hexagon-vertical-nft")]
      public const string HexagonVerticalNft = "\ue505";

      /// <summary>
      /// fa-hexagon-vertical-nft-slanted unicode value ("\ue506").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-vertical-nft-slanted
      /// </summary>
      [Description("hexagon-vertical-nft-slanted")]
      public const string HexagonVerticalNftSlanted = "\ue506";

      /// <summary>
      /// fa-hexagon-xmark unicode value ("\uf2ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hexagon-xmark
      /// </summary>
      [Description("hexagon-xmark")]
      public const string HexagonXmark = "\uf2ee";

      /// <summary>
      /// fa-high-definition unicode value ("\ue1ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/high-definition
      /// </summary>
      [Description("high-definition")]
      public const string HighDefinition = "\ue1ae";

      /// <summary>
      /// fa-highlighter unicode value ("\uf591").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/highlighter
      /// </summary>
      [Description("highlighter")]
      public const string Highlighter = "\uf591";

      /// <summary>
      /// fa-highlighter-line unicode value ("\ue1af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/highlighter-line
      /// </summary>
      [Description("highlighter-line")]
      public const string HighlighterLine = "\ue1af";

      /// <summary>
      /// fa-hill-avalanche unicode value ("\ue507").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hill-avalanche
      /// </summary>
      [Description("hill-avalanche")]
      public const string HillAvalanche = "\ue507";

      /// <summary>
      /// fa-hill-rockslide unicode value ("\ue508").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hill-rockslide
      /// </summary>
      [Description("hill-rockslide")]
      public const string HillRockslide = "\ue508";

      /// <summary>
      /// fa-hippo unicode value ("\uf6ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hippo
      /// </summary>
      [Description("hippo")]
      public const string Hippo = "\uf6ed";

      /// <summary>
      /// fa-hips unicode value ("\uf452").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '5.0.5'.
      /// <para/>
      /// See https://fontawesome.com/icons/hips
      /// </summary>
      [Description("hips")]
      public const string Hips = "\uf452";

      /// <summary>
      /// fa-hire-a-helper unicode value ("\uf3b0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hire-a-helper
      /// </summary>
      [Description("hire-a-helper")]
      public const string HireAHelper = "\uf3b0";

      /// <summary>
      /// fa-hive unicode value ("\ue07f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hive
      /// </summary>
      [Description("hive")]
      public const string Hive = "\ue07f";

      /// <summary>
      /// fa-hockey-mask unicode value ("\uf6ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hockey-mask
      /// </summary>
      [Description("hockey-mask")]
      public const string HockeyMask = "\uf6ee";

      /// <summary>
      /// fa-hockey-puck unicode value ("\uf453").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hockey-puck
      /// </summary>
      [Description("hockey-puck")]
      public const string HockeyPuck = "\uf453";

      /// <summary>
      /// fa-hockey-stick-puck unicode value ("\ue3ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hockey-stick-puck
      /// </summary>
      [Description("hockey-stick-puck")]
      public const string HockeyStickPuck = "\ue3ae";

      /// <summary>
      /// fa-hockey-sticks unicode value ("\uf454").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hockey-sticks
      /// </summary>
      [Description("hockey-sticks")]
      public const string HockeySticks = "\uf454";

      /// <summary>
      /// fa-holly-berry unicode value ("\uf7aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/holly-berry
      /// </summary>
      [Description("holly-berry")]
      public const string HollyBerry = "\uf7aa";

      /// <summary>
      /// fa-honey-pot unicode value ("\ue418").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/honey-pot
      /// </summary>
      [Description("honey-pot")]
      public const string HoneyPot = "\ue418";

      /// <summary>
      /// fa-hood-cloak unicode value ("\uf6ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hood-cloak
      /// </summary>
      [Description("hood-cloak")]
      public const string HoodCloak = "\uf6ef";

      /// <summary>
      /// fa-hooli unicode value ("\uf427").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hooli
      /// </summary>
      [Description("hooli")]
      public const string Hooli = "\uf427";

      /// <summary>
      /// fa-horizontal-rule unicode value ("\uf86c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/horizontal-rule
      /// </summary>
      [Description("horizontal-rule")]
      public const string HorizontalRule = "\uf86c";

      /// <summary>
      /// fa-hornbill unicode value ("\uf592").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hornbill
      /// </summary>
      [Description("hornbill")]
      public const string Hornbill = "\uf592";

      /// <summary>
      /// fa-horse unicode value ("\uf6f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/horse
      /// </summary>
      [Description("horse")]
      public const string Horse = "\uf6f0";

      /// <summary>
      /// fa-horse-head unicode value ("\uf7ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/horse-head
      /// </summary>
      [Description("horse-head")]
      public const string HorseHead = "\uf7ab";

      /// <summary>
      /// fa-horse-saddle unicode value ("\uf8c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/horse-saddle
      /// </summary>
      [Description("horse-saddle")]
      public const string HorseSaddle = "\uf8c3";

      /// <summary>
      /// fa-hose unicode value ("\ue419").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hose
      /// </summary>
      [Description("hose")]
      public const string Hose = "\ue419";

      /// <summary>
      /// fa-hose-reel unicode value ("\ue41a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hose-reel
      /// </summary>
      [Description("hose-reel")]
      public const string HoseReel = "\ue41a";

      /// <summary>
      /// fa-hospital unicode value ("\uf0f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hospital
      /// </summary>
      [Description("hospital")]
      public const string Hospital = "\uf0f8";

      /// <summary>
      /// fa-hospital-user unicode value ("\uf80d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hospital-user
      /// </summary>
      [Description("hospital-user")]
      public const string HospitalUser = "\uf80d";

      /// <summary>
      /// fa-hospitals unicode value ("\uf80e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hospitals
      /// </summary>
      [Description("hospitals")]
      public const string Hospitals = "\uf80e";

      /// <summary>
      /// fa-hot-tub-person unicode value ("\uf593").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hot-tub-person
      /// </summary>
      [Description("hot-tub-person")]
      public const string HotTubPerson = "\uf593";

      /// <summary>
      /// fa-hotdog unicode value ("\uf80f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hotdog
      /// </summary>
      [Description("hotdog")]
      public const string Hotdog = "\uf80f";

      /// <summary>
      /// fa-hotel unicode value ("\uf594").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hotel
      /// </summary>
      [Description("hotel")]
      public const string Hotel = "\uf594";

      /// <summary>
      /// fa-hotjar unicode value ("\uf3b1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hotjar
      /// </summary>
      [Description("hotjar")]
      public const string Hotjar = "\uf3b1";

      /// <summary>
      /// fa-hourglass unicode value ("\uf254").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hourglass
      /// </summary>
      [Description("hourglass")]
      public const string Hourglass = "\uf254";

      /// <summary>
      /// fa-hourglass-clock unicode value ("\ue41b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hourglass-clock
      /// </summary>
      [Description("hourglass-clock")]
      public const string HourglassClock = "\ue41b";

      /// <summary>
      /// fa-hourglass-end unicode value ("\uf253").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hourglass-end
      /// </summary>
      [Description("hourglass-end")]
      public const string HourglassEnd = "\uf253";

      /// <summary>
      /// fa-hourglass-half unicode value ("\uf252").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hourglass-half
      /// </summary>
      [Description("hourglass-half")]
      public const string HourglassHalf = "\uf252";

      /// <summary>
      /// fa-hourglass-start unicode value ("\uf251").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hourglass-start
      /// </summary>
      [Description("hourglass-start")]
      public const string HourglassStart = "\uf251";

      /// <summary>
      /// fa-house unicode value ("\uf015").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house
      /// </summary>
      [Description("house")]
      public const string House = "\uf015";

      /// <summary>
      /// fa-house-blank unicode value ("\ue487").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-blank
      /// </summary>
      [Description("house-blank")]
      public const string HouseBlank = "\ue487";

      /// <summary>
      /// fa-house-building unicode value ("\ue1b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-building
      /// </summary>
      [Description("house-building")]
      public const string HouseBuilding = "\ue1b1";

      /// <summary>
      /// fa-house-chimney unicode value ("\ue3af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney
      /// </summary>
      [Description("house-chimney")]
      public const string HouseChimney = "\ue3af";

      /// <summary>
      /// fa-house-chimney-blank unicode value ("\ue3b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-blank
      /// </summary>
      [Description("house-chimney-blank")]
      public const string HouseChimneyBlank = "\ue3b0";

      /// <summary>
      /// fa-house-chimney-crack unicode value ("\uf6f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-crack
      /// </summary>
      [Description("house-chimney-crack")]
      public const string HouseChimneyCrack = "\uf6f1";

      /// <summary>
      /// fa-house-chimney-heart unicode value ("\ue1b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-heart
      /// </summary>
      [Description("house-chimney-heart")]
      public const string HouseChimneyHeart = "\ue1b2";

      /// <summary>
      /// fa-house-chimney-medical unicode value ("\uf7f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-medical
      /// </summary>
      [Description("house-chimney-medical")]
      public const string HouseChimneyMedical = "\uf7f2";

      /// <summary>
      /// fa-house-chimney-user unicode value ("\ue065").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-user
      /// </summary>
      [Description("house-chimney-user")]
      public const string HouseChimneyUser = "\ue065";

      /// <summary>
      /// fa-house-chimney-window unicode value ("\ue00d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-chimney-window
      /// </summary>
      [Description("house-chimney-window")]
      public const string HouseChimneyWindow = "\ue00d";

      /// <summary>
      /// fa-house-circle-check unicode value ("\ue509").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-circle-check
      /// </summary>
      [Description("house-circle-check")]
      public const string HouseCircleCheck = "\ue509";

      /// <summary>
      /// fa-house-circle-exclamation unicode value ("\ue50a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-circle-exclamation
      /// </summary>
      [Description("house-circle-exclamation")]
      public const string HouseCircleExclamation = "\ue50a";

      /// <summary>
      /// fa-house-circle-xmark unicode value ("\ue50b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-circle-xmark
      /// </summary>
      [Description("house-circle-xmark")]
      public const string HouseCircleXmark = "\ue50b";

      /// <summary>
      /// fa-house-crack unicode value ("\ue3b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-crack
      /// </summary>
      [Description("house-crack")]
      public const string HouseCrack = "\ue3b1";

      /// <summary>
      /// fa-house-day unicode value ("\ue00e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-day
      /// </summary>
      [Description("house-day")]
      public const string HouseDay = "\ue00e";

      /// <summary>
      /// fa-house-fire unicode value ("\ue50c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-fire
      /// </summary>
      [Description("house-fire")]
      public const string HouseFire = "\ue50c";

      /// <summary>
      /// fa-house-flag unicode value ("\ue50d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-flag
      /// </summary>
      [Description("house-flag")]
      public const string HouseFlag = "\ue50d";

      /// <summary>
      /// fa-house-flood-water unicode value ("\ue50e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-flood-water
      /// </summary>
      [Description("house-flood-water")]
      public const string HouseFloodWater = "\ue50e";

      /// <summary>
      /// fa-house-flood-water-circle-arrow-right unicode value ("\ue50f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-flood-water-circle-arrow-right
      /// </summary>
      [Description("house-flood-water-circle-arrow-right")]
      public const string HouseFloodWaterCircleArrowRight = "\ue50f";

      /// <summary>
      /// fa-house-heart unicode value ("\uf4c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-heart
      /// </summary>
      [Description("house-heart")]
      public const string HouseHeart = "\uf4c9";

      /// <summary>
      /// fa-house-laptop unicode value ("\ue066").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-laptop
      /// </summary>
      [Description("house-laptop")]
      public const string HouseLaptop = "\ue066";

      /// <summary>
      /// fa-house-lock unicode value ("\ue510").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-lock
      /// </summary>
      [Description("house-lock")]
      public const string HouseLock = "\ue510";

      /// <summary>
      /// fa-house-medical unicode value ("\ue3b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-medical
      /// </summary>
      [Description("house-medical")]
      public const string HouseMedical = "\ue3b2";

      /// <summary>
      /// fa-house-medical-circle-check unicode value ("\ue511").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-medical-circle-check
      /// </summary>
      [Description("house-medical-circle-check")]
      public const string HouseMedicalCircleCheck = "\ue511";

      /// <summary>
      /// fa-house-medical-circle-exclamation unicode value ("\ue512").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-medical-circle-exclamation
      /// </summary>
      [Description("house-medical-circle-exclamation")]
      public const string HouseMedicalCircleExclamation = "\ue512";

      /// <summary>
      /// fa-house-medical-circle-xmark unicode value ("\ue513").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-medical-circle-xmark
      /// </summary>
      [Description("house-medical-circle-xmark")]
      public const string HouseMedicalCircleXmark = "\ue513";

      /// <summary>
      /// fa-house-medical-flag unicode value ("\ue514").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-medical-flag
      /// </summary>
      [Description("house-medical-flag")]
      public const string HouseMedicalFlag = "\ue514";

      /// <summary>
      /// fa-house-night unicode value ("\ue010").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-night
      /// </summary>
      [Description("house-night")]
      public const string HouseNight = "\ue010";

      /// <summary>
      /// fa-house-person-leave unicode value ("\ue00f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-person-leave
      /// </summary>
      [Description("house-person-leave")]
      public const string HousePersonLeave = "\ue00f";

      /// <summary>
      /// fa-house-person-return unicode value ("\ue011").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-person-return
      /// </summary>
      [Description("house-person-return")]
      public const string HousePersonReturn = "\ue011";

      /// <summary>
      /// fa-house-signal unicode value ("\ue012").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-signal
      /// </summary>
      [Description("house-signal")]
      public const string HouseSignal = "\ue012";

      /// <summary>
      /// fa-house-tree unicode value ("\ue1b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-tree
      /// </summary>
      [Description("house-tree")]
      public const string HouseTree = "\ue1b3";

      /// <summary>
      /// fa-house-tsunami unicode value ("\ue515").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-tsunami
      /// </summary>
      [Description("house-tsunami")]
      public const string HouseTsunami = "\ue515";

      /// <summary>
      /// fa-house-turret unicode value ("\ue1b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-turret
      /// </summary>
      [Description("house-turret")]
      public const string HouseTurret = "\ue1b4";

      /// <summary>
      /// fa-house-user unicode value ("\ue1b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-user
      /// </summary>
      [Description("house-user")]
      public const string HouseUser = "\ue1b0";

      /// <summary>
      /// fa-house-water unicode value ("\uf74f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-water
      /// </summary>
      [Description("house-water")]
      public const string HouseWater = "\uf74f";

      /// <summary>
      /// fa-house-window unicode value ("\ue3b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/house-window
      /// </summary>
      [Description("house-window")]
      public const string HouseWindow = "\ue3b3";

      /// <summary>
      /// fa-houzz unicode value ("\uf27c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/houzz
      /// </summary>
      [Description("houzz")]
      public const string Houzz = "\uf27c";

      /// <summary>
      /// fa-hryvnia-sign unicode value ("\uf6f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hryvnia-sign
      /// </summary>
      [Description("hryvnia-sign")]
      public const string HryvniaSign = "\uf6f2";

      /// <summary>
      /// fa-html5 unicode value ("\uf13b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/html5
      /// </summary>
      [Description("html5")]
      public const string Html5 = "\uf13b";

      /// <summary>
      /// fa-hubspot unicode value ("\uf3b2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hubspot
      /// </summary>
      [Description("hubspot")]
      public const string Hubspot = "\uf3b2";

      /// <summary>
      /// fa-hundred-points unicode value ("\ue41c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hundred-points
      /// </summary>
      [Description("hundred-points")]
      public const string HundredPoints = "\ue41c";

      /// <summary>
      /// fa-hurricane unicode value ("\uf751").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hurricane
      /// </summary>
      [Description("hurricane")]
      public const string Hurricane = "\uf751";

      /// <summary>
      /// fa-hyphen unicode value ("\u002d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/hyphen
      /// </summary>
      [Description("hyphen")]
      public const string Hyphen = "\u002d";

      /// <summary>
      /// fa-i unicode value ("\u0049").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/i
      /// </summary>
      [Description("i")]
      public const string I = "\u0049";

      /// <summary>
      /// fa-i-cursor unicode value ("\uf246").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/i-cursor
      /// </summary>
      [Description("i-cursor")]
      public const string ICursor = "\uf246";

      /// <summary>
      /// fa-ice-cream unicode value ("\uf810").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ice-cream
      /// </summary>
      [Description("ice-cream")]
      public const string IceCream = "\uf810";

      /// <summary>
      /// fa-ice-skate unicode value ("\uf7ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ice-skate
      /// </summary>
      [Description("ice-skate")]
      public const string IceSkate = "\uf7ac";

      /// <summary>
      /// fa-icicles unicode value ("\uf7ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/icicles
      /// </summary>
      [Description("icicles")]
      public const string Icicles = "\uf7ad";

      /// <summary>
      /// fa-icons unicode value ("\uf86d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/icons
      /// </summary>
      [Description("icons")]
      public const string _Icons = "\uf86d";

      /// <summary>
      /// fa-id-badge unicode value ("\uf2c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/id-badge
      /// </summary>
      [Description("id-badge")]
      public const string IdBadge = "\uf2c1";

      /// <summary>
      /// fa-id-card unicode value ("\uf2c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/id-card
      /// </summary>
      [Description("id-card")]
      public const string IdCard = "\uf2c2";

      /// <summary>
      /// fa-id-card-clip unicode value ("\uf47f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/id-card-clip
      /// </summary>
      [Description("id-card-clip")]
      public const string IdCardClip = "\uf47f";

      /// <summary>
      /// fa-ideal unicode value ("\ue013").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ideal
      /// </summary>
      [Description("ideal")]
      public const string Ideal = "\ue013";

      /// <summary>
      /// fa-igloo unicode value ("\uf7ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/igloo
      /// </summary>
      [Description("igloo")]
      public const string Igloo = "\uf7ae";

      /// <summary>
      /// fa-image unicode value ("\uf03e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image
      /// </summary>
      [Description("image")]
      public const string Image = "\uf03e";

      /// <summary>
      /// fa-image-landscape unicode value ("\ue1b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-landscape
      /// </summary>
      [Description("image-landscape")]
      public const string ImageLandscape = "\ue1b5";

      /// <summary>
      /// fa-image-polaroid unicode value ("\uf8c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-polaroid
      /// </summary>
      [Description("image-polaroid")]
      public const string ImagePolaroid = "\uf8c4";

      /// <summary>
      /// fa-image-polaroid-user unicode value ("\ue1b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-polaroid-user
      /// </summary>
      [Description("image-polaroid-user")]
      public const string ImagePolaroidUser = "\ue1b6";

      /// <summary>
      /// fa-image-portrait unicode value ("\uf3e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-portrait
      /// </summary>
      [Description("image-portrait")]
      public const string ImagePortrait = "\uf3e0";

      /// <summary>
      /// fa-image-slash unicode value ("\ue1b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-slash
      /// </summary>
      [Description("image-slash")]
      public const string ImageSlash = "\ue1b7";

      /// <summary>
      /// fa-image-user unicode value ("\ue1b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/image-user
      /// </summary>
      [Description("image-user")]
      public const string ImageUser = "\ue1b8";

      /// <summary>
      /// fa-images unicode value ("\uf302").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/images
      /// </summary>
      [Description("images")]
      public const string Images = "\uf302";

      /// <summary>
      /// fa-images-user unicode value ("\ue1b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/images-user
      /// </summary>
      [Description("images-user")]
      public const string ImagesUser = "\ue1b9";

      /// <summary>
      /// fa-imdb unicode value ("\uf2d8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/imdb
      /// </summary>
      [Description("imdb")]
      public const string Imdb = "\uf2d8";

      /// <summary>
      /// fa-inbox unicode value ("\uf01c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inbox
      /// </summary>
      [Description("inbox")]
      public const string Inbox = "\uf01c";

      /// <summary>
      /// fa-inbox-full unicode value ("\ue1ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inbox-full
      /// </summary>
      [Description("inbox-full")]
      public const string InboxFull = "\ue1ba";

      /// <summary>
      /// fa-inbox-in unicode value ("\uf310").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inbox-in
      /// </summary>
      [Description("inbox-in")]
      public const string InboxIn = "\uf310";

      /// <summary>
      /// fa-inbox-out unicode value ("\uf311").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inbox-out
      /// </summary>
      [Description("inbox-out")]
      public const string InboxOut = "\uf311";

      /// <summary>
      /// fa-inboxes unicode value ("\ue1bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inboxes
      /// </summary>
      [Description("inboxes")]
      public const string Inboxes = "\ue1bb";

      /// <summary>
      /// fa-indent unicode value ("\uf03c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/indent
      /// </summary>
      [Description("indent")]
      public const string Indent = "\uf03c";

      /// <summary>
      /// fa-indian-rupee-sign unicode value ("\ue1bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/indian-rupee-sign
      /// </summary>
      [Description("indian-rupee-sign")]
      public const string IndianRupeeSign = "\ue1bc";

      /// <summary>
      /// fa-industry unicode value ("\uf275").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/industry
      /// </summary>
      [Description("industry")]
      public const string Industry = "\uf275";

      /// <summary>
      /// fa-industry-windows unicode value ("\uf3b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/industry-windows
      /// </summary>
      [Description("industry-windows")]
      public const string IndustryWindows = "\uf3b3";

      /// <summary>
      /// fa-infinity unicode value ("\uf534").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/infinity
      /// </summary>
      [Description("infinity")]
      public const string Infinity = "\uf534";

      /// <summary>
      /// fa-info unicode value ("\uf129").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/info
      /// </summary>
      [Description("info")]
      public const string Info = "\uf129";

      /// <summary>
      /// fa-inhaler unicode value ("\uf5f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/inhaler
      /// </summary>
      [Description("inhaler")]
      public const string Inhaler = "\uf5f9";

      /// <summary>
      /// fa-input-numeric unicode value ("\ue1bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/input-numeric
      /// </summary>
      [Description("input-numeric")]
      public const string InputNumeric = "\ue1bd";

      /// <summary>
      /// fa-input-pipe unicode value ("\ue1be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/input-pipe
      /// </summary>
      [Description("input-pipe")]
      public const string InputPipe = "\ue1be";

      /// <summary>
      /// fa-input-text unicode value ("\ue1bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/input-text
      /// </summary>
      [Description("input-text")]
      public const string InputText = "\ue1bf";

      /// <summary>
      /// fa-instagram unicode value ("\uf16d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/instagram
      /// </summary>
      [Description("instagram")]
      public const string Instagram = "\uf16d";

      /// <summary>
      /// fa-instalod unicode value ("\ue081").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/instalod
      /// </summary>
      [Description("instalod")]
      public const string Instalod = "\ue081";

      /// <summary>
      /// fa-integral unicode value ("\uf667").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/integral
      /// </summary>
      [Description("integral")]
      public const string Integral = "\uf667";

      /// <summary>
      /// fa-intercom unicode value ("\uf7af").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/intercom
      /// </summary>
      [Description("intercom")]
      public const string Intercom = "\uf7af";

      /// <summary>
      /// fa-internet-explorer unicode value ("\uf26b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/internet-explorer
      /// </summary>
      [Description("internet-explorer")]
      public const string InternetExplorer = "\uf26b";

      /// <summary>
      /// fa-interrobang unicode value ("\ue5ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/interrobang
      /// </summary>
      [Description("interrobang")]
      public const string Interrobang = "\ue5ba";

      /// <summary>
      /// fa-intersection unicode value ("\uf668").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/intersection
      /// </summary>
      [Description("intersection")]
      public const string Intersection = "\uf668";

      /// <summary>
      /// fa-invision unicode value ("\uf7b0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/invision
      /// </summary>
      [Description("invision")]
      public const string Invision = "\uf7b0";

      /// <summary>
      /// fa-ioxhost unicode value ("\uf208").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ioxhost
      /// </summary>
      [Description("ioxhost")]
      public const string Ioxhost = "\uf208";

      /// <summary>
      /// fa-island-tropical unicode value ("\uf811").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/island-tropical
      /// </summary>
      [Description("island-tropical")]
      public const string IslandTropical = "\uf811";

      /// <summary>
      /// fa-italic unicode value ("\uf033").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/italic
      /// </summary>
      [Description("italic")]
      public const string Italic = "\uf033";

      /// <summary>
      /// fa-itch-io unicode value ("\uf83a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/itch-io
      /// </summary>
      [Description("itch-io")]
      public const string ItchIo = "\uf83a";

      /// <summary>
      /// fa-itunes unicode value ("\uf3b4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/itunes
      /// </summary>
      [Description("itunes")]
      public const string Itunes = "\uf3b4";

      /// <summary>
      /// fa-itunes-note unicode value ("\uf3b5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/itunes-note
      /// </summary>
      [Description("itunes-note")]
      public const string ItunesNote = "\uf3b5";

      /// <summary>
      /// fa-j unicode value ("\u004a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/j
      /// </summary>
      [Description("j")]
      public const string J = "\u004a";

      /// <summary>
      /// fa-jack-o-lantern unicode value ("\uf30e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jack-o-lantern
      /// </summary>
      [Description("jack-o-lantern")]
      public const string JackOLantern = "\uf30e";

      /// <summary>
      /// fa-jar unicode value ("\ue516").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jar
      /// </summary>
      [Description("jar")]
      public const string Jar = "\ue516";

      /// <summary>
      /// fa-jar-wheat unicode value ("\ue517").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jar-wheat
      /// </summary>
      [Description("jar-wheat")]
      public const string JarWheat = "\ue517";

      /// <summary>
      /// fa-java unicode value ("\uf4e4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.10', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/java
      /// </summary>
      [Description("java")]
      public const string Java = "\uf4e4";

      /// <summary>
      /// fa-jedi unicode value ("\uf669").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jedi
      /// </summary>
      [Description("jedi")]
      public const string Jedi = "\uf669";

      /// <summary>
      /// fa-jedi-order unicode value ("\uf50e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jedi-order
      /// </summary>
      [Description("jedi-order")]
      public const string JediOrder = "\uf50e";

      /// <summary>
      /// fa-jenkins unicode value ("\uf3b6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jenkins
      /// </summary>
      [Description("jenkins")]
      public const string Jenkins = "\uf3b6";

      /// <summary>
      /// fa-jet-fighter unicode value ("\uf0fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jet-fighter
      /// </summary>
      [Description("jet-fighter")]
      public const string JetFighter = "\uf0fb";

      /// <summary>
      /// fa-jet-fighter-up unicode value ("\ue518").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jet-fighter-up
      /// </summary>
      [Description("jet-fighter-up")]
      public const string JetFighterUp = "\ue518";

      /// <summary>
      /// fa-jira unicode value ("\uf7b1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jira
      /// </summary>
      [Description("jira")]
      public const string Jira = "\uf7b1";

      /// <summary>
      /// fa-joget unicode value ("\uf3b7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/joget
      /// </summary>
      [Description("joget")]
      public const string Joget = "\uf3b7";

      /// <summary>
      /// fa-joint unicode value ("\uf595").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/joint
      /// </summary>
      [Description("joint")]
      public const string Joint = "\uf595";

      /// <summary>
      /// fa-joomla unicode value ("\uf1aa").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/joomla
      /// </summary>
      [Description("joomla")]
      public const string Joomla = "\uf1aa";

      /// <summary>
      /// fa-joystick unicode value ("\uf8c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/joystick
      /// </summary>
      [Description("joystick")]
      public const string Joystick = "\uf8c5";

      /// <summary>
      /// fa-js unicode value ("\uf3b8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/js
      /// </summary>
      [Description("js")]
      public const string Js = "\uf3b8";

      /// <summary>
      /// fa-jsfiddle unicode value ("\uf1cc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jsfiddle
      /// </summary>
      [Description("jsfiddle")]
      public const string Jsfiddle = "\uf1cc";

      /// <summary>
      /// fa-jug unicode value ("\uf8c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jug
      /// </summary>
      [Description("jug")]
      public const string Jug = "\uf8c6";

      /// <summary>
      /// fa-jug-bottle unicode value ("\ue5fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jug-bottle
      /// </summary>
      [Description("jug-bottle")]
      public const string JugBottle = "\ue5fb";

      /// <summary>
      /// fa-jug-detergent unicode value ("\ue519").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/jug-detergent
      /// </summary>
      [Description("jug-detergent")]
      public const string JugDetergent = "\ue519";

      /// <summary>
      /// fa-k unicode value ("\u004b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/k
      /// </summary>
      [Description("k")]
      public const string K = "\u004b";

      /// <summary>
      /// fa-kaaba unicode value ("\uf66b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kaaba
      /// </summary>
      [Description("kaaba")]
      public const string Kaaba = "\uf66b";

      /// <summary>
      /// fa-kaggle unicode value ("\uf5fa").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kaggle
      /// </summary>
      [Description("kaggle")]
      public const string Kaggle = "\uf5fa";

      /// <summary>
      /// fa-kazoo unicode value ("\uf8c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kazoo
      /// </summary>
      [Description("kazoo")]
      public const string Kazoo = "\uf8c7";

      /// <summary>
      /// fa-kerning unicode value ("\uf86f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kerning
      /// </summary>
      [Description("kerning")]
      public const string Kerning = "\uf86f";

      /// <summary>
      /// fa-key unicode value ("\uf084").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/key
      /// </summary>
      [Description("key")]
      public const string Key = "\uf084";

      /// <summary>
      /// fa-key-skeleton unicode value ("\uf6f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/key-skeleton
      /// </summary>
      [Description("key-skeleton")]
      public const string KeySkeleton = "\uf6f3";

      /// <summary>
      /// fa-key-skeleton-left-right unicode value ("\ue3b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/key-skeleton-left-right
      /// </summary>
      [Description("key-skeleton-left-right")]
      public const string KeySkeletonLeftRight = "\ue3b4";

      /// <summary>
      /// fa-keybase unicode value ("\uf4f5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keybase
      /// </summary>
      [Description("keybase")]
      public const string Keybase = "\uf4f5";

      /// <summary>
      /// fa-keyboard unicode value ("\uf11c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keyboard
      /// </summary>
      [Description("keyboard")]
      public const string Keyboard = "\uf11c";

      /// <summary>
      /// fa-keyboard-brightness unicode value ("\ue1c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keyboard-brightness
      /// </summary>
      [Description("keyboard-brightness")]
      public const string KeyboardBrightness = "\ue1c0";

      /// <summary>
      /// fa-keyboard-brightness-low unicode value ("\ue1c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keyboard-brightness-low
      /// </summary>
      [Description("keyboard-brightness-low")]
      public const string KeyboardBrightnessLow = "\ue1c1";

      /// <summary>
      /// fa-keyboard-down unicode value ("\ue1c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keyboard-down
      /// </summary>
      [Description("keyboard-down")]
      public const string KeyboardDown = "\ue1c2";

      /// <summary>
      /// fa-keyboard-left unicode value ("\ue1c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keyboard-left
      /// </summary>
      [Description("keyboard-left")]
      public const string KeyboardLeft = "\ue1c3";

      /// <summary>
      /// fa-keycdn unicode value ("\uf3ba").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keycdn
      /// </summary>
      [Description("keycdn")]
      public const string Keycdn = "\uf3ba";

      /// <summary>
      /// fa-keynote unicode value ("\uf66c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/keynote
      /// </summary>
      [Description("keynote")]
      public const string Keynote = "\uf66c";

      /// <summary>
      /// fa-khanda unicode value ("\uf66d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/khanda
      /// </summary>
      [Description("khanda")]
      public const string Khanda = "\uf66d";

      /// <summary>
      /// fa-kickstarter unicode value ("\uf3bb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kickstarter
      /// </summary>
      [Description("kickstarter")]
      public const string Kickstarter = "\uf3bb";

      /// <summary>
      /// fa-kickstarter-k unicode value ("\uf3bc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kickstarter-k
      /// </summary>
      [Description("kickstarter-k")]
      public const string KickstarterK = "\uf3bc";

      /// <summary>
      /// fa-kidneys unicode value ("\uf5fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kidneys
      /// </summary>
      [Description("kidneys")]
      public const string Kidneys = "\uf5fb";

      /// <summary>
      /// fa-kip-sign unicode value ("\ue1c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kip-sign
      /// </summary>
      [Description("kip-sign")]
      public const string KipSign = "\ue1c4";

      /// <summary>
      /// fa-kit-medical unicode value ("\uf479").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kit-medical
      /// </summary>
      [Description("kit-medical")]
      public const string KitMedical = "\uf479";

      /// <summary>
      /// fa-kitchen-set unicode value ("\ue51a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kitchen-set
      /// </summary>
      [Description("kitchen-set")]
      public const string KitchenSet = "\ue51a";

      /// <summary>
      /// fa-kite unicode value ("\uf6f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kite
      /// </summary>
      [Description("kite")]
      public const string Kite = "\uf6f4";

      /// <summary>
      /// fa-kiwi-bird unicode value ("\uf535").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kiwi-bird
      /// </summary>
      [Description("kiwi-bird")]
      public const string KiwiBird = "\uf535";

      /// <summary>
      /// fa-kiwi-fruit unicode value ("\ue30c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/kiwi-fruit
      /// </summary>
      [Description("kiwi-fruit")]
      public const string KiwiFruit = "\ue30c";

      /// <summary>
      /// fa-knife unicode value ("\uf2e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/knife
      /// </summary>
      [Description("knife")]
      public const string Knife = "\uf2e4";

      /// <summary>
      /// fa-knife-kitchen unicode value ("\uf6f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/knife-kitchen
      /// </summary>
      [Description("knife-kitchen")]
      public const string KnifeKitchen = "\uf6f5";

      /// <summary>
      /// fa-korvue unicode value ("\uf42f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.2', Last Modified in '5.0.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/korvue
      /// </summary>
      [Description("korvue")]
      public const string Korvue = "\uf42f";

      /// <summary>
      /// fa-l unicode value ("\u004c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/l
      /// </summary>
      [Description("l")]
      public const string L = "\u004c";

      /// <summary>
      /// fa-lacrosse-stick unicode value ("\ue3b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lacrosse-stick
      /// </summary>
      [Description("lacrosse-stick")]
      public const string LacrosseStick = "\ue3b5";

      /// <summary>
      /// fa-lacrosse-stick-ball unicode value ("\ue3b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lacrosse-stick-ball
      /// </summary>
      [Description("lacrosse-stick-ball")]
      public const string LacrosseStickBall = "\ue3b6";

      /// <summary>
      /// fa-lambda unicode value ("\uf66e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lambda
      /// </summary>
      [Description("lambda")]
      public const string Lambda = "\uf66e";

      /// <summary>
      /// fa-lamp unicode value ("\uf4ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lamp
      /// </summary>
      [Description("lamp")]
      public const string Lamp = "\uf4ca";

      /// <summary>
      /// fa-lamp-desk unicode value ("\ue014").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lamp-desk
      /// </summary>
      [Description("lamp-desk")]
      public const string LampDesk = "\ue014";

      /// <summary>
      /// fa-lamp-floor unicode value ("\ue015").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lamp-floor
      /// </summary>
      [Description("lamp-floor")]
      public const string LampFloor = "\ue015";

      /// <summary>
      /// fa-lamp-street unicode value ("\ue1c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lamp-street
      /// </summary>
      [Description("lamp-street")]
      public const string LampStreet = "\ue1c5";

      /// <summary>
      /// fa-land-mine-on unicode value ("\ue51b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/land-mine-on
      /// </summary>
      [Description("land-mine-on")]
      public const string LandMineOn = "\ue51b";

      /// <summary>
      /// fa-landmark unicode value ("\uf66f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/landmark
      /// </summary>
      [Description("landmark")]
      public const string Landmark = "\uf66f";

      /// <summary>
      /// fa-landmark-dome unicode value ("\uf752").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/landmark-dome
      /// </summary>
      [Description("landmark-dome")]
      public const string LandmarkDome = "\uf752";

      /// <summary>
      /// fa-landmark-flag unicode value ("\ue51c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/landmark-flag
      /// </summary>
      [Description("landmark-flag")]
      public const string LandmarkFlag = "\ue51c";

      /// <summary>
      /// fa-language unicode value ("\uf1ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/language
      /// </summary>
      [Description("language")]
      public const string Language = "\uf1ab";

      /// <summary>
      /// fa-laptop unicode value ("\uf109").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop
      /// </summary>
      [Description("laptop")]
      public const string Laptop = "\uf109";

      /// <summary>
      /// fa-laptop-arrow-down unicode value ("\ue1c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-arrow-down
      /// </summary>
      [Description("laptop-arrow-down")]
      public const string LaptopArrowDown = "\ue1c6";

      /// <summary>
      /// fa-laptop-binary unicode value ("\ue5e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-binary
      /// </summary>
      [Description("laptop-binary")]
      public const string LaptopBinary = "\ue5e7";

      /// <summary>
      /// fa-laptop-code unicode value ("\uf5fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-code
      /// </summary>
      [Description("laptop-code")]
      public const string LaptopCode = "\uf5fc";

      /// <summary>
      /// fa-laptop-file unicode value ("\ue51d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-file
      /// </summary>
      [Description("laptop-file")]
      public const string LaptopFile = "\ue51d";

      /// <summary>
      /// fa-laptop-medical unicode value ("\uf812").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-medical
      /// </summary>
      [Description("laptop-medical")]
      public const string LaptopMedical = "\uf812";

      /// <summary>
      /// fa-laptop-mobile unicode value ("\uf87a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-mobile
      /// </summary>
      [Description("laptop-mobile")]
      public const string LaptopMobile = "\uf87a";

      /// <summary>
      /// fa-laptop-slash unicode value ("\ue1c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/laptop-slash
      /// </summary>
      [Description("laptop-slash")]
      public const string LaptopSlash = "\ue1c7";

      /// <summary>
      /// fa-laravel unicode value ("\uf3bd").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.11.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/laravel
      /// </summary>
      [Description("laravel")]
      public const string Laravel = "\uf3bd";

      /// <summary>
      /// fa-lari-sign unicode value ("\ue1c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lari-sign
      /// </summary>
      [Description("lari-sign")]
      public const string LariSign = "\ue1c8";

      /// <summary>
      /// fa-lasso unicode value ("\uf8c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lasso
      /// </summary>
      [Description("lasso")]
      public const string Lasso = "\uf8c8";

      /// <summary>
      /// fa-lasso-sparkles unicode value ("\ue1c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lasso-sparkles
      /// </summary>
      [Description("lasso-sparkles")]
      public const string LassoSparkles = "\ue1c9";

      /// <summary>
      /// fa-lastfm unicode value ("\uf202").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lastfm
      /// </summary>
      [Description("lastfm")]
      public const string Lastfm = "\uf202";

      /// <summary>
      /// fa-layer-group unicode value ("\uf5fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/layer-group
      /// </summary>
      [Description("layer-group")]
      public const string LayerGroup = "\uf5fd";

      /// <summary>
      /// fa-layer-minus unicode value ("\uf5fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/layer-minus
      /// </summary>
      [Description("layer-minus")]
      public const string LayerMinus = "\uf5fe";

      /// <summary>
      /// fa-layer-plus unicode value ("\uf5ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/layer-plus
      /// </summary>
      [Description("layer-plus")]
      public const string LayerPlus = "\uf5ff";

      /// <summary>
      /// fa-leaf unicode value ("\uf06c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leaf
      /// </summary>
      [Description("leaf")]
      public const string Leaf = "\uf06c";

      /// <summary>
      /// fa-leaf-heart unicode value ("\uf4cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leaf-heart
      /// </summary>
      [Description("leaf-heart")]
      public const string LeafHeart = "\uf4cb";

      /// <summary>
      /// fa-leaf-maple unicode value ("\uf6f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leaf-maple
      /// </summary>
      [Description("leaf-maple")]
      public const string LeafMaple = "\uf6f6";

      /// <summary>
      /// fa-leaf-oak unicode value ("\uf6f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leaf-oak
      /// </summary>
      [Description("leaf-oak")]
      public const string LeafOak = "\uf6f7";

      /// <summary>
      /// fa-leafy-green unicode value ("\ue41d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leafy-green
      /// </summary>
      [Description("leafy-green")]
      public const string LeafyGreen = "\ue41d";

      /// <summary>
      /// fa-leanpub unicode value ("\uf212").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/leanpub
      /// </summary>
      [Description("leanpub")]
      public const string Leanpub = "\uf212";

      /// <summary>
      /// fa-left unicode value ("\uf355").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left
      /// </summary>
      [Description("left")]
      public const string Left = "\uf355";

      /// <summary>
      /// fa-left-from-line unicode value ("\uf348").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left-from-line
      /// </summary>
      [Description("left-from-line")]
      public const string LeftFromLine = "\uf348";

      /// <summary>
      /// fa-left-long unicode value ("\uf30a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left-long
      /// </summary>
      [Description("left-long")]
      public const string LeftLong = "\uf30a";

      /// <summary>
      /// fa-left-long-to-line unicode value ("\ue41e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left-long-to-line
      /// </summary>
      [Description("left-long-to-line")]
      public const string LeftLongToLine = "\ue41e";

      /// <summary>
      /// fa-left-right unicode value ("\uf337").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left-right
      /// </summary>
      [Description("left-right")]
      public const string LeftRight = "\uf337";

      /// <summary>
      /// fa-left-to-line unicode value ("\uf34b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/left-to-line
      /// </summary>
      [Description("left-to-line")]
      public const string LeftToLine = "\uf34b";

      /// <summary>
      /// fa-lemon unicode value ("\uf094").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lemon
      /// </summary>
      [Description("lemon")]
      public const string Lemon = "\uf094";

      /// <summary>
      /// fa-less unicode value ("\uf41d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/less
      /// </summary>
      [Description("less")]
      public const string Less = "\uf41d";

      /// <summary>
      /// fa-less-than unicode value ("\u003c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/less-than
      /// </summary>
      [Description("less-than")]
      public const string LessThan = "\u003c";

      /// <summary>
      /// fa-less-than-equal unicode value ("\uf537").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/less-than-equal
      /// </summary>
      [Description("less-than-equal")]
      public const string LessThanEqual = "\uf537";

      /// <summary>
      /// fa-life-ring unicode value ("\uf1cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/life-ring
      /// </summary>
      [Description("life-ring")]
      public const string LifeRing = "\uf1cd";

      /// <summary>
      /// fa-light-ceiling unicode value ("\ue016").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-ceiling
      /// </summary>
      [Description("light-ceiling")]
      public const string LightCeiling = "\ue016";

      /// <summary>
      /// fa-light-emergency unicode value ("\ue41f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-emergency
      /// </summary>
      [Description("light-emergency")]
      public const string LightEmergency = "\ue41f";

      /// <summary>
      /// fa-light-emergency-on unicode value ("\ue420").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-emergency-on
      /// </summary>
      [Description("light-emergency-on")]
      public const string LightEmergencyOn = "\ue420";

      /// <summary>
      /// fa-light-switch unicode value ("\ue017").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-switch
      /// </summary>
      [Description("light-switch")]
      public const string LightSwitch = "\ue017";

      /// <summary>
      /// fa-light-switch-off unicode value ("\ue018").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-switch-off
      /// </summary>
      [Description("light-switch-off")]
      public const string LightSwitchOff = "\ue018";

      /// <summary>
      /// fa-light-switch-on unicode value ("\ue019").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/light-switch-on
      /// </summary>
      [Description("light-switch-on")]
      public const string LightSwitchOn = "\ue019";

      /// <summary>
      /// fa-lightbulb unicode value ("\uf0eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb
      /// </summary>
      [Description("lightbulb")]
      public const string Lightbulb = "\uf0eb";

      /// <summary>
      /// fa-lightbulb-cfl unicode value ("\ue5a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-cfl
      /// </summary>
      [Description("lightbulb-cfl")]
      public const string LightbulbCfl = "\ue5a6";

      /// <summary>
      /// fa-lightbulb-cfl-on unicode value ("\ue5a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-cfl-on
      /// </summary>
      [Description("lightbulb-cfl-on")]
      public const string LightbulbCflOn = "\ue5a7";

      /// <summary>
      /// fa-lightbulb-dollar unicode value ("\uf670").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-dollar
      /// </summary>
      [Description("lightbulb-dollar")]
      public const string LightbulbDollar = "\uf670";

      /// <summary>
      /// fa-lightbulb-exclamation unicode value ("\uf671").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-exclamation
      /// </summary>
      [Description("lightbulb-exclamation")]
      public const string LightbulbExclamation = "\uf671";

      /// <summary>
      /// fa-lightbulb-exclamation-on unicode value ("\ue1ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-exclamation-on
      /// </summary>
      [Description("lightbulb-exclamation-on")]
      public const string LightbulbExclamationOn = "\ue1ca";

      /// <summary>
      /// fa-lightbulb-gear unicode value ("\ue5fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-gear
      /// </summary>
      [Description("lightbulb-gear")]
      public const string LightbulbGear = "\ue5fd";

      /// <summary>
      /// fa-lightbulb-on unicode value ("\uf672").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-on
      /// </summary>
      [Description("lightbulb-on")]
      public const string LightbulbOn = "\uf672";

      /// <summary>
      /// fa-lightbulb-slash unicode value ("\uf673").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lightbulb-slash
      /// </summary>
      [Description("lightbulb-slash")]
      public const string LightbulbSlash = "\uf673";

      /// <summary>
      /// fa-lights-holiday unicode value ("\uf7b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lights-holiday
      /// </summary>
      [Description("lights-holiday")]
      public const string LightsHoliday = "\uf7b2";

      /// <summary>
      /// fa-line unicode value ("\uf3c0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.3.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/line
      /// </summary>
      [Description("line")]
      public const string Line = "\uf3c0";

      /// <summary>
      /// fa-line-columns unicode value ("\uf870").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/line-columns
      /// </summary>
      [Description("line-columns")]
      public const string LineColumns = "\uf870";

      /// <summary>
      /// fa-line-height unicode value ("\uf871").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/line-height
      /// </summary>
      [Description("line-height")]
      public const string LineHeight = "\uf871";

      /// <summary>
      /// fa-lines-leaning unicode value ("\ue51e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lines-leaning
      /// </summary>
      [Description("lines-leaning")]
      public const string LinesLeaning = "\ue51e";

      /// <summary>
      /// fa-link unicode value ("\uf0c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link
      /// </summary>
      [Description("link")]
      public const string Link = "\uf0c1";

      /// <summary>
      /// fa-link-horizontal unicode value ("\ue1cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link-horizontal
      /// </summary>
      [Description("link-horizontal")]
      public const string LinkHorizontal = "\ue1cb";

      /// <summary>
      /// fa-link-horizontal-slash unicode value ("\ue1cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link-horizontal-slash
      /// </summary>
      [Description("link-horizontal-slash")]
      public const string LinkHorizontalSlash = "\ue1cc";

      /// <summary>
      /// fa-link-simple unicode value ("\ue1cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link-simple
      /// </summary>
      [Description("link-simple")]
      public const string LinkSimple = "\ue1cd";

      /// <summary>
      /// fa-link-simple-slash unicode value ("\ue1ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link-simple-slash
      /// </summary>
      [Description("link-simple-slash")]
      public const string LinkSimpleSlash = "\ue1ce";

      /// <summary>
      /// fa-link-slash unicode value ("\uf127").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/link-slash
      /// </summary>
      [Description("link-slash")]
      public const string LinkSlash = "\uf127";

      /// <summary>
      /// fa-linkedin unicode value ("\uf08c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/linkedin
      /// </summary>
      [Description("linkedin")]
      public const string Linkedin = "\uf08c";

      /// <summary>
      /// fa-linkedin-in unicode value ("\uf0e1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.8.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/linkedin-in
      /// </summary>
      [Description("linkedin-in")]
      public const string LinkedinIn = "\uf0e1";

      /// <summary>
      /// fa-linode unicode value ("\uf2b8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/linode
      /// </summary>
      [Description("linode")]
      public const string Linode = "\uf2b8";

      /// <summary>
      /// fa-linux unicode value ("\uf17c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/linux
      /// </summary>
      [Description("linux")]
      public const string Linux = "\uf17c";

      /// <summary>
      /// fa-lips unicode value ("\uf600").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lips
      /// </summary>
      [Description("lips")]
      public const string Lips = "\uf600";

      /// <summary>
      /// fa-lira-sign unicode value ("\uf195").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lira-sign
      /// </summary>
      [Description("lira-sign")]
      public const string LiraSign = "\uf195";

      /// <summary>
      /// fa-list unicode value ("\uf03a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list
      /// </summary>
      [Description("list")]
      public const string List = "\uf03a";

      /// <summary>
      /// fa-list-check unicode value ("\uf0ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-check
      /// </summary>
      [Description("list-check")]
      public const string ListCheck = "\uf0ae";

      /// <summary>
      /// fa-list-dropdown unicode value ("\ue1cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-dropdown
      /// </summary>
      [Description("list-dropdown")]
      public const string ListDropdown = "\ue1cf";

      /// <summary>
      /// fa-list-music unicode value ("\uf8c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-music
      /// </summary>
      [Description("list-music")]
      public const string ListMusic = "\uf8c9";

      /// <summary>
      /// fa-list-ol unicode value ("\uf0cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-ol
      /// </summary>
      [Description("list-ol")]
      public const string ListOl = "\uf0cb";

      /// <summary>
      /// fa-list-radio unicode value ("\ue1d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-radio
      /// </summary>
      [Description("list-radio")]
      public const string ListRadio = "\ue1d0";

      /// <summary>
      /// fa-list-timeline unicode value ("\ue1d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-timeline
      /// </summary>
      [Description("list-timeline")]
      public const string ListTimeline = "\ue1d1";

      /// <summary>
      /// fa-list-tree unicode value ("\ue1d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-tree
      /// </summary>
      [Description("list-tree")]
      public const string ListTree = "\ue1d2";

      /// <summary>
      /// fa-list-ul unicode value ("\uf0ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/list-ul
      /// </summary>
      [Description("list-ul")]
      public const string ListUl = "\uf0ca";

      /// <summary>
      /// fa-litecoin-sign unicode value ("\ue1d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/litecoin-sign
      /// </summary>
      [Description("litecoin-sign")]
      public const string LitecoinSign = "\ue1d3";

      /// <summary>
      /// fa-loader unicode value ("\ue1d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/loader
      /// </summary>
      [Description("loader")]
      public const string Loader = "\ue1d4";

      /// <summary>
      /// fa-lobster unicode value ("\ue421").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lobster
      /// </summary>
      [Description("lobster")]
      public const string Lobster = "\ue421";

      /// <summary>
      /// fa-location-arrow unicode value ("\uf124").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-arrow
      /// </summary>
      [Description("location-arrow")]
      public const string LocationArrow = "\uf124";

      /// <summary>
      /// fa-location-check unicode value ("\uf606").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-check
      /// </summary>
      [Description("location-check")]
      public const string LocationCheck = "\uf606";

      /// <summary>
      /// fa-location-crosshairs unicode value ("\uf601").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-crosshairs
      /// </summary>
      [Description("location-crosshairs")]
      public const string LocationCrosshairs = "\uf601";

      /// <summary>
      /// fa-location-crosshairs-slash unicode value ("\uf603").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-crosshairs-slash
      /// </summary>
      [Description("location-crosshairs-slash")]
      public const string LocationCrosshairsSlash = "\uf603";

      /// <summary>
      /// fa-location-dot unicode value ("\uf3c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-dot
      /// </summary>
      [Description("location-dot")]
      public const string LocationDot = "\uf3c5";

      /// <summary>
      /// fa-location-dot-slash unicode value ("\uf605").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-dot-slash
      /// </summary>
      [Description("location-dot-slash")]
      public const string LocationDotSlash = "\uf605";

      /// <summary>
      /// fa-location-exclamation unicode value ("\uf608").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-exclamation
      /// </summary>
      [Description("location-exclamation")]
      public const string LocationExclamation = "\uf608";

      /// <summary>
      /// fa-location-minus unicode value ("\uf609").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-minus
      /// </summary>
      [Description("location-minus")]
      public const string LocationMinus = "\uf609";

      /// <summary>
      /// fa-location-pen unicode value ("\uf607").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-pen
      /// </summary>
      [Description("location-pen")]
      public const string LocationPen = "\uf607";

      /// <summary>
      /// fa-location-pin unicode value ("\uf041").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-pin
      /// </summary>
      [Description("location-pin")]
      public const string LocationPin = "\uf041";

      /// <summary>
      /// fa-location-pin-lock unicode value ("\ue51f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-pin-lock
      /// </summary>
      [Description("location-pin-lock")]
      public const string LocationPinLock = "\ue51f";

      /// <summary>
      /// fa-location-pin-slash unicode value ("\uf60c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-pin-slash
      /// </summary>
      [Description("location-pin-slash")]
      public const string LocationPinSlash = "\uf60c";

      /// <summary>
      /// fa-location-plus unicode value ("\uf60a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-plus
      /// </summary>
      [Description("location-plus")]
      public const string LocationPlus = "\uf60a";

      /// <summary>
      /// fa-location-question unicode value ("\uf60b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-question
      /// </summary>
      [Description("location-question")]
      public const string LocationQuestion = "\uf60b";

      /// <summary>
      /// fa-location-smile unicode value ("\uf60d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-smile
      /// </summary>
      [Description("location-smile")]
      public const string LocationSmile = "\uf60d";

      /// <summary>
      /// fa-location-xmark unicode value ("\uf60e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/location-xmark
      /// </summary>
      [Description("location-xmark")]
      public const string LocationXmark = "\uf60e";

      /// <summary>
      /// fa-lock unicode value ("\uf023").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock
      /// </summary>
      [Description("lock")]
      public const string Lock = "\uf023";

      /// <summary>
      /// fa-lock-a unicode value ("\ue422").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock-a
      /// </summary>
      [Description("lock-a")]
      public const string LockA = "\ue422";

      /// <summary>
      /// fa-lock-hashtag unicode value ("\ue423").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock-hashtag
      /// </summary>
      [Description("lock-hashtag")]
      public const string LockHashtag = "\ue423";

      /// <summary>
      /// fa-lock-keyhole unicode value ("\uf30d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock-keyhole
      /// </summary>
      [Description("lock-keyhole")]
      public const string LockKeyhole = "\uf30d";

      /// <summary>
      /// fa-lock-keyhole-open unicode value ("\uf3c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock-keyhole-open
      /// </summary>
      [Description("lock-keyhole-open")]
      public const string LockKeyholeOpen = "\uf3c2";

      /// <summary>
      /// fa-lock-open unicode value ("\uf3c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lock-open
      /// </summary>
      [Description("lock-open")]
      public const string LockOpen = "\uf3c1";

      /// <summary>
      /// fa-locust unicode value ("\ue520").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/locust
      /// </summary>
      [Description("locust")]
      public const string Locust = "\ue520";

      /// <summary>
      /// fa-lollipop unicode value ("\ue424").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lollipop
      /// </summary>
      [Description("lollipop")]
      public const string Lollipop = "\ue424";

      /// <summary>
      /// fa-loveseat unicode value ("\uf4cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/loveseat
      /// </summary>
      [Description("loveseat")]
      public const string Loveseat = "\uf4cc";

      /// <summary>
      /// fa-luchador-mask unicode value ("\uf455").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/luchador-mask
      /// </summary>
      [Description("luchador-mask")]
      public const string LuchadorMask = "\uf455";

      /// <summary>
      /// fa-lungs unicode value ("\uf604").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lungs
      /// </summary>
      [Description("lungs")]
      public const string Lungs = "\uf604";

      /// <summary>
      /// fa-lungs-virus unicode value ("\ue067").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lungs-virus
      /// </summary>
      [Description("lungs-virus")]
      public const string LungsVirus = "\ue067";

      /// <summary>
      /// fa-lyft unicode value ("\uf3c3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/lyft
      /// </summary>
      [Description("lyft")]
      public const string Lyft = "\uf3c3";

      /// <summary>
      /// fa-m unicode value ("\u004d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/m
      /// </summary>
      [Description("m")]
      public const string M = "\u004d";

      /// <summary>
      /// fa-mace unicode value ("\uf6f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mace
      /// </summary>
      [Description("mace")]
      public const string Mace = "\uf6f8";

      /// <summary>
      /// fa-magento unicode value ("\uf3c4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magento
      /// </summary>
      [Description("magento")]
      public const string Magento = "\uf3c4";

      /// <summary>
      /// fa-magnet unicode value ("\uf076").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnet
      /// </summary>
      [Description("magnet")]
      public const string Magnet = "\uf076";

      /// <summary>
      /// fa-magnifying-glass unicode value ("\uf002").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass
      /// </summary>
      [Description("magnifying-glass")]
      public const string MagnifyingGlass = "\uf002";

      /// <summary>
      /// fa-magnifying-glass-arrow-right unicode value ("\ue521").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-arrow-right
      /// </summary>
      [Description("magnifying-glass-arrow-right")]
      public const string MagnifyingGlassArrowRight = "\ue521";

      /// <summary>
      /// fa-magnifying-glass-chart unicode value ("\ue522").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-chart
      /// </summary>
      [Description("magnifying-glass-chart")]
      public const string MagnifyingGlassChart = "\ue522";

      /// <summary>
      /// fa-magnifying-glass-dollar unicode value ("\uf688").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-dollar
      /// </summary>
      [Description("magnifying-glass-dollar")]
      public const string MagnifyingGlassDollar = "\uf688";

      /// <summary>
      /// fa-magnifying-glass-location unicode value ("\uf689").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-location
      /// </summary>
      [Description("magnifying-glass-location")]
      public const string MagnifyingGlassLocation = "\uf689";

      /// <summary>
      /// fa-magnifying-glass-minus unicode value ("\uf010").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-minus
      /// </summary>
      [Description("magnifying-glass-minus")]
      public const string MagnifyingGlassMinus = "\uf010";

      /// <summary>
      /// fa-magnifying-glass-plus unicode value ("\uf00e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/magnifying-glass-plus
      /// </summary>
      [Description("magnifying-glass-plus")]
      public const string MagnifyingGlassPlus = "\uf00e";

      /// <summary>
      /// fa-mailbox unicode value ("\uf813").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mailbox
      /// </summary>
      [Description("mailbox")]
      public const string Mailbox = "\uf813";

      /// <summary>
      /// fa-mailbox-flag-up unicode value ("\ue5bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mailbox-flag-up
      /// </summary>
      [Description("mailbox-flag-up")]
      public const string MailboxFlagUp = "\ue5bb";

      /// <summary>
      /// fa-mailchimp unicode value ("\uf59e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/mailchimp
      /// </summary>
      [Description("mailchimp")]
      public const string Mailchimp = "\uf59e";

      /// <summary>
      /// fa-manat-sign unicode value ("\ue1d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/manat-sign
      /// </summary>
      [Description("manat-sign")]
      public const string ManatSign = "\ue1d5";

      /// <summary>
      /// fa-mandalorian unicode value ("\uf50f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mandalorian
      /// </summary>
      [Description("mandalorian")]
      public const string Mandalorian = "\uf50f";

      /// <summary>
      /// fa-mandolin unicode value ("\uf6f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mandolin
      /// </summary>
      [Description("mandolin")]
      public const string Mandolin = "\uf6f9";

      /// <summary>
      /// fa-mango unicode value ("\ue30f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mango
      /// </summary>
      [Description("mango")]
      public const string Mango = "\ue30f";

      /// <summary>
      /// fa-manhole unicode value ("\ue1d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/manhole
      /// </summary>
      [Description("manhole")]
      public const string Manhole = "\ue1d6";

      /// <summary>
      /// fa-map unicode value ("\uf279").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/map
      /// </summary>
      [Description("map")]
      public const string Map = "\uf279";

      /// <summary>
      /// fa-map-location unicode value ("\uf59f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/map-location
      /// </summary>
      [Description("map-location")]
      public const string MapLocation = "\uf59f";

      /// <summary>
      /// fa-map-location-dot unicode value ("\uf5a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/map-location-dot
      /// </summary>
      [Description("map-location-dot")]
      public const string MapLocationDot = "\uf5a0";

      /// <summary>
      /// fa-map-pin unicode value ("\uf276").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/map-pin
      /// </summary>
      [Description("map-pin")]
      public const string MapPin = "\uf276";

      /// <summary>
      /// fa-markdown unicode value ("\uf60f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/markdown
      /// </summary>
      [Description("markdown")]
      public const string Markdown = "\uf60f";

      /// <summary>
      /// fa-marker unicode value ("\uf5a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/marker
      /// </summary>
      [Description("marker")]
      public const string Marker = "\uf5a1";

      /// <summary>
      /// fa-mars unicode value ("\uf222").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars
      /// </summary>
      [Description("mars")]
      public const string Mars = "\uf222";

      /// <summary>
      /// fa-mars-and-venus unicode value ("\uf224").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-and-venus
      /// </summary>
      [Description("mars-and-venus")]
      public const string MarsAndVenus = "\uf224";

      /// <summary>
      /// fa-mars-and-venus-burst unicode value ("\ue523").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-and-venus-burst
      /// </summary>
      [Description("mars-and-venus-burst")]
      public const string MarsAndVenusBurst = "\ue523";

      /// <summary>
      /// fa-mars-double unicode value ("\uf227").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-double
      /// </summary>
      [Description("mars-double")]
      public const string MarsDouble = "\uf227";

      /// <summary>
      /// fa-mars-stroke unicode value ("\uf229").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-stroke
      /// </summary>
      [Description("mars-stroke")]
      public const string MarsStroke = "\uf229";

      /// <summary>
      /// fa-mars-stroke-right unicode value ("\uf22b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-stroke-right
      /// </summary>
      [Description("mars-stroke-right")]
      public const string MarsStrokeRight = "\uf22b";

      /// <summary>
      /// fa-mars-stroke-up unicode value ("\uf22a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mars-stroke-up
      /// </summary>
      [Description("mars-stroke-up")]
      public const string MarsStrokeUp = "\uf22a";

      /// <summary>
      /// fa-martini-glass unicode value ("\uf57b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/martini-glass
      /// </summary>
      [Description("martini-glass")]
      public const string MartiniGlass = "\uf57b";

      /// <summary>
      /// fa-martini-glass-citrus unicode value ("\uf561").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/martini-glass-citrus
      /// </summary>
      [Description("martini-glass-citrus")]
      public const string MartiniGlassCitrus = "\uf561";

      /// <summary>
      /// fa-martini-glass-empty unicode value ("\uf000").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/martini-glass-empty
      /// </summary>
      [Description("martini-glass-empty")]
      public const string MartiniGlassEmpty = "\uf000";

      /// <summary>
      /// fa-mask unicode value ("\uf6fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mask
      /// </summary>
      [Description("mask")]
      public const string Mask = "\uf6fa";

      /// <summary>
      /// fa-mask-face unicode value ("\ue1d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mask-face
      /// </summary>
      [Description("mask-face")]
      public const string MaskFace = "\ue1d7";

      /// <summary>
      /// fa-mask-snorkel unicode value ("\ue3b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mask-snorkel
      /// </summary>
      [Description("mask-snorkel")]
      public const string MaskSnorkel = "\ue3b7";

      /// <summary>
      /// fa-mask-ventilator unicode value ("\ue524").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mask-ventilator
      /// </summary>
      [Description("mask-ventilator")]
      public const string MaskVentilator = "\ue524";

      /// <summary>
      /// fa-masks-theater unicode value ("\uf630").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/masks-theater
      /// </summary>
      [Description("masks-theater")]
      public const string MasksTheater = "\uf630";

      /// <summary>
      /// fa-mastodon unicode value ("\uf4f6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mastodon
      /// </summary>
      [Description("mastodon")]
      public const string Mastodon = "\uf4f6";

      /// <summary>
      /// fa-mattress-pillow unicode value ("\ue525").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mattress-pillow
      /// </summary>
      [Description("mattress-pillow")]
      public const string MattressPillow = "\ue525";

      /// <summary>
      /// fa-maxcdn unicode value ("\uf136").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/maxcdn
      /// </summary>
      [Description("maxcdn")]
      public const string Maxcdn = "\uf136";

      /// <summary>
      /// fa-maximize unicode value ("\uf31e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/maximize
      /// </summary>
      [Description("maximize")]
      public const string Maximize = "\uf31e";

      /// <summary>
      /// fa-mdb unicode value ("\uf8ca").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mdb
      /// </summary>
      [Description("mdb")]
      public const string Mdb = "\uf8ca";

      /// <summary>
      /// fa-meat unicode value ("\uf814").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meat
      /// </summary>
      [Description("meat")]
      public const string Meat = "\uf814";

      /// <summary>
      /// fa-medal unicode value ("\uf5a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/medal
      /// </summary>
      [Description("medal")]
      public const string Medal = "\uf5a2";

      /// <summary>
      /// fa-medapps unicode value ("\uf3c6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/medapps
      /// </summary>
      [Description("medapps")]
      public const string Medapps = "\uf3c6";

      /// <summary>
      /// fa-medium unicode value ("\uf23a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/medium
      /// </summary>
      [Description("medium")]
      public const string Medium = "\uf23a";

      /// <summary>
      /// fa-medrt unicode value ("\uf3c8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/medrt
      /// </summary>
      [Description("medrt")]
      public const string Medrt = "\uf3c8";

      /// <summary>
      /// fa-meetup unicode value ("\uf2e0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meetup
      /// </summary>
      [Description("meetup")]
      public const string Meetup = "\uf2e0";

      /// <summary>
      /// fa-megaphone unicode value ("\uf675").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/megaphone
      /// </summary>
      [Description("megaphone")]
      public const string Megaphone = "\uf675";

      /// <summary>
      /// fa-megaport unicode value ("\uf5a3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/megaport
      /// </summary>
      [Description("megaport")]
      public const string Megaport = "\uf5a3";

      /// <summary>
      /// fa-melon unicode value ("\ue310").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/melon
      /// </summary>
      [Description("melon")]
      public const string Melon = "\ue310";

      /// <summary>
      /// fa-melon-slice unicode value ("\ue311").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/melon-slice
      /// </summary>
      [Description("melon-slice")]
      public const string MelonSlice = "\ue311";

      /// <summary>
      /// fa-memo unicode value ("\ue1d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/memo
      /// </summary>
      [Description("memo")]
      public const string Memo = "\ue1d8";

      /// <summary>
      /// fa-memo-circle-check unicode value ("\ue1d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/memo-circle-check
      /// </summary>
      [Description("memo-circle-check")]
      public const string MemoCircleCheck = "\ue1d9";

      /// <summary>
      /// fa-memo-circle-info unicode value ("\ue49a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/memo-circle-info
      /// </summary>
      [Description("memo-circle-info")]
      public const string MemoCircleInfo = "\ue49a";

      /// <summary>
      /// fa-memo-pad unicode value ("\ue1da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/memo-pad
      /// </summary>
      [Description("memo-pad")]
      public const string MemoPad = "\ue1da";

      /// <summary>
      /// fa-memory unicode value ("\uf538").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/memory
      /// </summary>
      [Description("memory")]
      public const string Memory = "\uf538";

      /// <summary>
      /// fa-mendeley unicode value ("\uf7b3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mendeley
      /// </summary>
      [Description("mendeley")]
      public const string Mendeley = "\uf7b3";

      /// <summary>
      /// fa-menorah unicode value ("\uf676").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/menorah
      /// </summary>
      [Description("menorah")]
      public const string Menorah = "\uf676";

      /// <summary>
      /// fa-mercury unicode value ("\uf223").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mercury
      /// </summary>
      [Description("mercury")]
      public const string Mercury = "\uf223";

      /// <summary>
      /// fa-merge unicode value ("\ue526").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/merge
      /// </summary>
      [Description("merge")]
      public const string Merge = "\ue526";

      /// <summary>
      /// fa-message unicode value ("\uf27a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message
      /// </summary>
      [Description("message")]
      public const string Message = "\uf27a";

      /// <summary>
      /// fa-message-arrow-down unicode value ("\ue1db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-arrow-down
      /// </summary>
      [Description("message-arrow-down")]
      public const string MessageArrowDown = "\ue1db";

      /// <summary>
      /// fa-message-arrow-up unicode value ("\ue1dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-arrow-up
      /// </summary>
      [Description("message-arrow-up")]
      public const string MessageArrowUp = "\ue1dc";

      /// <summary>
      /// fa-message-arrow-up-right unicode value ("\ue1dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-arrow-up-right
      /// </summary>
      [Description("message-arrow-up-right")]
      public const string MessageArrowUpRight = "\ue1dd";

      /// <summary>
      /// fa-message-bot unicode value ("\ue3b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-bot
      /// </summary>
      [Description("message-bot")]
      public const string MessageBot = "\ue3b8";

      /// <summary>
      /// fa-message-captions unicode value ("\ue1de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-captions
      /// </summary>
      [Description("message-captions")]
      public const string MessageCaptions = "\ue1de";

      /// <summary>
      /// fa-message-check unicode value ("\uf4a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-check
      /// </summary>
      [Description("message-check")]
      public const string MessageCheck = "\uf4a2";

      /// <summary>
      /// fa-message-code unicode value ("\ue1df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-code
      /// </summary>
      [Description("message-code")]
      public const string MessageCode = "\ue1df";

      /// <summary>
      /// fa-message-dollar unicode value ("\uf650").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-dollar
      /// </summary>
      [Description("message-dollar")]
      public const string MessageDollar = "\uf650";

      /// <summary>
      /// fa-message-dots unicode value ("\uf4a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-dots
      /// </summary>
      [Description("message-dots")]
      public const string MessageDots = "\uf4a3";

      /// <summary>
      /// fa-message-exclamation unicode value ("\uf4a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-exclamation
      /// </summary>
      [Description("message-exclamation")]
      public const string MessageExclamation = "\uf4a5";

      /// <summary>
      /// fa-message-heart unicode value ("\ue5c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-heart
      /// </summary>
      [Description("message-heart")]
      public const string MessageHeart = "\ue5c9";

      /// <summary>
      /// fa-message-image unicode value ("\ue1e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-image
      /// </summary>
      [Description("message-image")]
      public const string MessageImage = "\ue1e0";

      /// <summary>
      /// fa-message-lines unicode value ("\uf4a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-lines
      /// </summary>
      [Description("message-lines")]
      public const string MessageLines = "\uf4a6";

      /// <summary>
      /// fa-message-medical unicode value ("\uf7f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-medical
      /// </summary>
      [Description("message-medical")]
      public const string MessageMedical = "\uf7f4";

      /// <summary>
      /// fa-message-middle unicode value ("\ue1e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-middle
      /// </summary>
      [Description("message-middle")]
      public const string MessageMiddle = "\ue1e1";

      /// <summary>
      /// fa-message-middle-top unicode value ("\ue1e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-middle-top
      /// </summary>
      [Description("message-middle-top")]
      public const string MessageMiddleTop = "\ue1e2";

      /// <summary>
      /// fa-message-minus unicode value ("\uf4a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-minus
      /// </summary>
      [Description("message-minus")]
      public const string MessageMinus = "\uf4a7";

      /// <summary>
      /// fa-message-music unicode value ("\uf8af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-music
      /// </summary>
      [Description("message-music")]
      public const string MessageMusic = "\uf8af";

      /// <summary>
      /// fa-message-pen unicode value ("\uf4a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-pen
      /// </summary>
      [Description("message-pen")]
      public const string MessagePen = "\uf4a4";

      /// <summary>
      /// fa-message-plus unicode value ("\uf4a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-plus
      /// </summary>
      [Description("message-plus")]
      public const string MessagePlus = "\uf4a8";

      /// <summary>
      /// fa-message-question unicode value ("\ue1e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-question
      /// </summary>
      [Description("message-question")]
      public const string MessageQuestion = "\ue1e3";

      /// <summary>
      /// fa-message-quote unicode value ("\ue1e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-quote
      /// </summary>
      [Description("message-quote")]
      public const string MessageQuote = "\ue1e4";

      /// <summary>
      /// fa-message-slash unicode value ("\uf4a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-slash
      /// </summary>
      [Description("message-slash")]
      public const string MessageSlash = "\uf4a9";

      /// <summary>
      /// fa-message-smile unicode value ("\uf4aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-smile
      /// </summary>
      [Description("message-smile")]
      public const string MessageSmile = "\uf4aa";

      /// <summary>
      /// fa-message-sms unicode value ("\ue1e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-sms
      /// </summary>
      [Description("message-sms")]
      public const string MessageSms = "\ue1e5";

      /// <summary>
      /// fa-message-text unicode value ("\ue1e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-text
      /// </summary>
      [Description("message-text")]
      public const string MessageText = "\ue1e6";

      /// <summary>
      /// fa-message-xmark unicode value ("\uf4ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/message-xmark
      /// </summary>
      [Description("message-xmark")]
      public const string MessageXmark = "\uf4ab";

      /// <summary>
      /// fa-messages unicode value ("\uf4b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/messages
      /// </summary>
      [Description("messages")]
      public const string Messages = "\uf4b6";

      /// <summary>
      /// fa-messages-dollar unicode value ("\uf652").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/messages-dollar
      /// </summary>
      [Description("messages-dollar")]
      public const string MessagesDollar = "\uf652";

      /// <summary>
      /// fa-messages-question unicode value ("\ue1e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/messages-question
      /// </summary>
      [Description("messages-question")]
      public const string MessagesQuestion = "\ue1e7";

      /// <summary>
      /// fa-meta unicode value ("\ue49b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/meta
      /// </summary>
      [Description("meta")]
      public const string Meta = "\ue49b";

      /// <summary>
      /// fa-meteor unicode value ("\uf753").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meteor
      /// </summary>
      [Description("meteor")]
      public const string Meteor = "\uf753";

      /// <summary>
      /// fa-meter unicode value ("\ue1e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meter
      /// </summary>
      [Description("meter")]
      public const string Meter = "\ue1e8";

      /// <summary>
      /// fa-meter-bolt unicode value ("\ue1e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meter-bolt
      /// </summary>
      [Description("meter-bolt")]
      public const string MeterBolt = "\ue1e9";

      /// <summary>
      /// fa-meter-droplet unicode value ("\ue1ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meter-droplet
      /// </summary>
      [Description("meter-droplet")]
      public const string MeterDroplet = "\ue1ea";

      /// <summary>
      /// fa-meter-fire unicode value ("\ue1eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/meter-fire
      /// </summary>
      [Description("meter-fire")]
      public const string MeterFire = "\ue1eb";

      /// <summary>
      /// fa-microblog unicode value ("\ue01a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microblog
      /// </summary>
      [Description("microblog")]
      public const string Microblog = "\ue01a";

      /// <summary>
      /// fa-microchip unicode value ("\uf2db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microchip
      /// </summary>
      [Description("microchip")]
      public const string Microchip = "\uf2db";

      /// <summary>
      /// fa-microchip-ai unicode value ("\ue1ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microchip-ai
      /// </summary>
      [Description("microchip-ai")]
      public const string MicrochipAi = "\ue1ec";

      /// <summary>
      /// fa-microphone unicode value ("\uf130").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microphone
      /// </summary>
      [Description("microphone")]
      public const string Microphone = "\uf130";

      /// <summary>
      /// fa-microphone-lines unicode value ("\uf3c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microphone-lines
      /// </summary>
      [Description("microphone-lines")]
      public const string MicrophoneLines = "\uf3c9";

      /// <summary>
      /// fa-microphone-lines-slash unicode value ("\uf539").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microphone-lines-slash
      /// </summary>
      [Description("microphone-lines-slash")]
      public const string MicrophoneLinesSlash = "\uf539";

      /// <summary>
      /// fa-microphone-slash unicode value ("\uf131").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microphone-slash
      /// </summary>
      [Description("microphone-slash")]
      public const string MicrophoneSlash = "\uf131";

      /// <summary>
      /// fa-microphone-stand unicode value ("\uf8cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microphone-stand
      /// </summary>
      [Description("microphone-stand")]
      public const string MicrophoneStand = "\uf8cb";

      /// <summary>
      /// fa-microscope unicode value ("\uf610").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microscope
      /// </summary>
      [Description("microscope")]
      public const string Microscope = "\uf610";

      /// <summary>
      /// fa-microsoft unicode value ("\uf3ca").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microsoft
      /// </summary>
      [Description("microsoft")]
      public const string Microsoft = "\uf3ca";

      /// <summary>
      /// fa-microwave unicode value ("\ue01b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/microwave
      /// </summary>
      [Description("microwave")]
      public const string Microwave = "\ue01b";

      /// <summary>
      /// fa-mill-sign unicode value ("\ue1ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mill-sign
      /// </summary>
      [Description("mill-sign")]
      public const string MillSign = "\ue1ed";

      /// <summary>
      /// fa-minimize unicode value ("\uf78c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/minimize
      /// </summary>
      [Description("minimize")]
      public const string Minimize = "\uf78c";

      /// <summary>
      /// fa-minus unicode value ("\uf068").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/minus
      /// </summary>
      [Description("minus")]
      public const string Minus = "\uf068";

      /// <summary>
      /// fa-mistletoe unicode value ("\uf7b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mistletoe
      /// </summary>
      [Description("mistletoe")]
      public const string Mistletoe = "\uf7b4";

      /// <summary>
      /// fa-mitten unicode value ("\uf7b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mitten
      /// </summary>
      [Description("mitten")]
      public const string Mitten = "\uf7b5";

      /// <summary>
      /// fa-mix unicode value ("\uf3cb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/mix
      /// </summary>
      [Description("mix")]
      public const string Mix = "\uf3cb";

      /// <summary>
      /// fa-mixcloud unicode value ("\uf289").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/mixcloud
      /// </summary>
      [Description("mixcloud")]
      public const string Mixcloud = "\uf289";

      /// <summary>
      /// fa-mixer unicode value ("\ue056").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mixer
      /// </summary>
      [Description("mixer")]
      public const string Mixer = "\ue056";

      /// <summary>
      /// fa-mizuni unicode value ("\uf3cc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mizuni
      /// </summary>
      [Description("mizuni")]
      public const string Mizuni = "\uf3cc";

      /// <summary>
      /// fa-mobile unicode value ("\uf3ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile
      /// </summary>
      [Description("mobile")]
      public const string Mobile = "\uf3ce";

      /// <summary>
      /// fa-mobile-button unicode value ("\uf10b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-button
      /// </summary>
      [Description("mobile-button")]
      public const string MobileButton = "\uf10b";

      /// <summary>
      /// fa-mobile-notch unicode value ("\ue1ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-notch
      /// </summary>
      [Description("mobile-notch")]
      public const string MobileNotch = "\ue1ee";

      /// <summary>
      /// fa-mobile-retro unicode value ("\ue527").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-retro
      /// </summary>
      [Description("mobile-retro")]
      public const string MobileRetro = "\ue527";

      /// <summary>
      /// fa-mobile-screen unicode value ("\uf3cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-screen
      /// </summary>
      [Description("mobile-screen")]
      public const string MobileScreen = "\uf3cf";

      /// <summary>
      /// fa-mobile-screen-button unicode value ("\uf3cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-screen-button
      /// </summary>
      [Description("mobile-screen-button")]
      public const string MobileScreenButton = "\uf3cd";

      /// <summary>
      /// fa-mobile-signal unicode value ("\ue1ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-signal
      /// </summary>
      [Description("mobile-signal")]
      public const string MobileSignal = "\ue1ef";

      /// <summary>
      /// fa-mobile-signal-out unicode value ("\ue1f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mobile-signal-out
      /// </summary>
      [Description("mobile-signal-out")]
      public const string MobileSignalOut = "\ue1f0";

      /// <summary>
      /// fa-modx unicode value ("\uf285").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/modx
      /// </summary>
      [Description("modx")]
      public const string Modx = "\uf285";

      /// <summary>
      /// fa-monero unicode value ("\uf3d0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/monero
      /// </summary>
      [Description("monero")]
      public const string Monero = "\uf3d0";

      /// <summary>
      /// fa-money-bill unicode value ("\uf0d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill
      /// </summary>
      [Description("money-bill")]
      public const string MoneyBill = "\uf0d6";

      /// <summary>
      /// fa-money-bill-1 unicode value ("\uf3d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-1
      /// </summary>
      [Description("money-bill-1")]
      public const string MoneyBill1 = "\uf3d1";

      /// <summary>
      /// fa-money-bill-1-wave unicode value ("\uf53b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-1-wave
      /// </summary>
      [Description("money-bill-1-wave")]
      public const string MoneyBill1Wave = "\uf53b";

      /// <summary>
      /// fa-money-bill-simple unicode value ("\ue1f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-simple
      /// </summary>
      [Description("money-bill-simple")]
      public const string MoneyBillSimple = "\ue1f1";

      /// <summary>
      /// fa-money-bill-simple-wave unicode value ("\ue1f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-simple-wave
      /// </summary>
      [Description("money-bill-simple-wave")]
      public const string MoneyBillSimpleWave = "\ue1f2";

      /// <summary>
      /// fa-money-bill-transfer unicode value ("\ue528").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-transfer
      /// </summary>
      [Description("money-bill-transfer")]
      public const string MoneyBillTransfer = "\ue528";

      /// <summary>
      /// fa-money-bill-trend-up unicode value ("\ue529").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-trend-up
      /// </summary>
      [Description("money-bill-trend-up")]
      public const string MoneyBillTrendUp = "\ue529";

      /// <summary>
      /// fa-money-bill-wave unicode value ("\uf53a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-wave
      /// </summary>
      [Description("money-bill-wave")]
      public const string MoneyBillWave = "\uf53a";

      /// <summary>
      /// fa-money-bill-wheat unicode value ("\ue52a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bill-wheat
      /// </summary>
      [Description("money-bill-wheat")]
      public const string MoneyBillWheat = "\ue52a";

      /// <summary>
      /// fa-money-bills unicode value ("\ue1f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bills
      /// </summary>
      [Description("money-bills")]
      public const string MoneyBills = "\ue1f3";

      /// <summary>
      /// fa-money-bills-simple unicode value ("\ue1f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-bills-simple
      /// </summary>
      [Description("money-bills-simple")]
      public const string MoneyBillsSimple = "\ue1f4";

      /// <summary>
      /// fa-money-check unicode value ("\uf53c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-check
      /// </summary>
      [Description("money-check")]
      public const string MoneyCheck = "\uf53c";

      /// <summary>
      /// fa-money-check-dollar unicode value ("\uf53d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-check-dollar
      /// </summary>
      [Description("money-check-dollar")]
      public const string MoneyCheckDollar = "\uf53d";

      /// <summary>
      /// fa-money-check-dollar-pen unicode value ("\uf873").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-check-dollar-pen
      /// </summary>
      [Description("money-check-dollar-pen")]
      public const string MoneyCheckDollarPen = "\uf873";

      /// <summary>
      /// fa-money-check-pen unicode value ("\uf872").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-check-pen
      /// </summary>
      [Description("money-check-pen")]
      public const string MoneyCheckPen = "\uf872";

      /// <summary>
      /// fa-money-from-bracket unicode value ("\ue312").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-from-bracket
      /// </summary>
      [Description("money-from-bracket")]
      public const string MoneyFromBracket = "\ue312";

      /// <summary>
      /// fa-money-simple-from-bracket unicode value ("\ue313").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/money-simple-from-bracket
      /// </summary>
      [Description("money-simple-from-bracket")]
      public const string MoneySimpleFromBracket = "\ue313";

      /// <summary>
      /// fa-monitor-waveform unicode value ("\uf611").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/monitor-waveform
      /// </summary>
      [Description("monitor-waveform")]
      public const string MonitorWaveform = "\uf611";

      /// <summary>
      /// fa-monkey unicode value ("\uf6fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/monkey
      /// </summary>
      [Description("monkey")]
      public const string Monkey = "\uf6fb";

      /// <summary>
      /// fa-monument unicode value ("\uf5a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/monument
      /// </summary>
      [Description("monument")]
      public const string Monument = "\uf5a6";

      /// <summary>
      /// fa-moon unicode value ("\uf186").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/moon
      /// </summary>
      [Description("moon")]
      public const string Moon = "\uf186";

      /// <summary>
      /// fa-moon-cloud unicode value ("\uf754").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/moon-cloud
      /// </summary>
      [Description("moon-cloud")]
      public const string MoonCloud = "\uf754";

      /// <summary>
      /// fa-moon-over-sun unicode value ("\uf74a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/moon-over-sun
      /// </summary>
      [Description("moon-over-sun")]
      public const string MoonOverSun = "\uf74a";

      /// <summary>
      /// fa-moon-stars unicode value ("\uf755").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/moon-stars
      /// </summary>
      [Description("moon-stars")]
      public const string MoonStars = "\uf755";

      /// <summary>
      /// fa-moped unicode value ("\ue3b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/moped
      /// </summary>
      [Description("moped")]
      public const string Moped = "\ue3b9";

      /// <summary>
      /// fa-mortar-pestle unicode value ("\uf5a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mortar-pestle
      /// </summary>
      [Description("mortar-pestle")]
      public const string MortarPestle = "\uf5a7";

      /// <summary>
      /// fa-mosque unicode value ("\uf678").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mosque
      /// </summary>
      [Description("mosque")]
      public const string Mosque = "\uf678";

      /// <summary>
      /// fa-mosquito unicode value ("\ue52b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mosquito
      /// </summary>
      [Description("mosquito")]
      public const string Mosquito = "\ue52b";

      /// <summary>
      /// fa-mosquito-net unicode value ("\ue52c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mosquito-net
      /// </summary>
      [Description("mosquito-net")]
      public const string MosquitoNet = "\ue52c";

      /// <summary>
      /// fa-motorcycle unicode value ("\uf21c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/motorcycle
      /// </summary>
      [Description("motorcycle")]
      public const string Motorcycle = "\uf21c";

      /// <summary>
      /// fa-mound unicode value ("\ue52d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mound
      /// </summary>
      [Description("mound")]
      public const string Mound = "\ue52d";

      /// <summary>
      /// fa-mountain unicode value ("\uf6fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mountain
      /// </summary>
      [Description("mountain")]
      public const string Mountain = "\uf6fc";

      /// <summary>
      /// fa-mountain-city unicode value ("\ue52e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mountain-city
      /// </summary>
      [Description("mountain-city")]
      public const string MountainCity = "\ue52e";

      /// <summary>
      /// fa-mountain-sun unicode value ("\ue52f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mountain-sun
      /// </summary>
      [Description("mountain-sun")]
      public const string MountainSun = "\ue52f";

      /// <summary>
      /// fa-mountains unicode value ("\uf6fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mountains
      /// </summary>
      [Description("mountains")]
      public const string Mountains = "\uf6fd";

      /// <summary>
      /// fa-mouse-field unicode value ("\ue5a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mouse-field
      /// </summary>
      [Description("mouse-field")]
      public const string MouseField = "\ue5a8";

      /// <summary>
      /// fa-mp3-player unicode value ("\uf8ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mp3-player
      /// </summary>
      [Description("mp3-player")]
      public const string Mp3Player = "\uf8ce";

      /// <summary>
      /// fa-mug unicode value ("\uf874").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug
      /// </summary>
      [Description("mug")]
      public const string Mug = "\uf874";

      /// <summary>
      /// fa-mug-hot unicode value ("\uf7b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug-hot
      /// </summary>
      [Description("mug-hot")]
      public const string MugHot = "\uf7b6";

      /// <summary>
      /// fa-mug-marshmallows unicode value ("\uf7b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug-marshmallows
      /// </summary>
      [Description("mug-marshmallows")]
      public const string MugMarshmallows = "\uf7b7";

      /// <summary>
      /// fa-mug-saucer unicode value ("\uf0f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug-saucer
      /// </summary>
      [Description("mug-saucer")]
      public const string MugSaucer = "\uf0f4";

      /// <summary>
      /// fa-mug-tea unicode value ("\uf875").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug-tea
      /// </summary>
      [Description("mug-tea")]
      public const string MugTea = "\uf875";

      /// <summary>
      /// fa-mug-tea-saucer unicode value ("\ue1f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mug-tea-saucer
      /// </summary>
      [Description("mug-tea-saucer")]
      public const string MugTeaSaucer = "\ue1f5";

      /// <summary>
      /// fa-mushroom unicode value ("\ue425").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mushroom
      /// </summary>
      [Description("mushroom")]
      public const string Mushroom = "\ue425";

      /// <summary>
      /// fa-music unicode value ("\uf001").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/music
      /// </summary>
      [Description("music")]
      public const string Music = "\uf001";

      /// <summary>
      /// fa-music-note unicode value ("\uf8cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/music-note
      /// </summary>
      [Description("music-note")]
      public const string MusicNote = "\uf8cf";

      /// <summary>
      /// fa-music-note-slash unicode value ("\uf8d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/music-note-slash
      /// </summary>
      [Description("music-note-slash")]
      public const string MusicNoteSlash = "\uf8d0";

      /// <summary>
      /// fa-music-slash unicode value ("\uf8d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/music-slash
      /// </summary>
      [Description("music-slash")]
      public const string MusicSlash = "\uf8d1";

      /// <summary>
      /// fa-mustache unicode value ("\ue5bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/mustache
      /// </summary>
      [Description("mustache")]
      public const string Mustache = "\ue5bc";

      /// <summary>
      /// fa-n unicode value ("\u004e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/n
      /// </summary>
      [Description("n")]
      public const string N = "\u004e";

      /// <summary>
      /// fa-naira-sign unicode value ("\ue1f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/naira-sign
      /// </summary>
      [Description("naira-sign")]
      public const string NairaSign = "\ue1f6";

      /// <summary>
      /// fa-napster unicode value ("\uf3d2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/napster
      /// </summary>
      [Description("napster")]
      public const string Napster = "\uf3d2";

      /// <summary>
      /// fa-narwhal unicode value ("\uf6fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/narwhal
      /// </summary>
      [Description("narwhal")]
      public const string Narwhal = "\uf6fe";

      /// <summary>
      /// fa-neos unicode value ("\uf612").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/neos
      /// </summary>
      [Description("neos")]
      public const string Neos = "\uf612";

      /// <summary>
      /// fa-nesting-dolls unicode value ("\ue3ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nesting-dolls
      /// </summary>
      [Description("nesting-dolls")]
      public const string NestingDolls = "\ue3ba";

      /// <summary>
      /// fa-network-wired unicode value ("\uf6ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/network-wired
      /// </summary>
      [Description("network-wired")]
      public const string NetworkWired = "\uf6ff";

      /// <summary>
      /// fa-neuter unicode value ("\uf22c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/neuter
      /// </summary>
      [Description("neuter")]
      public const string Neuter = "\uf22c";

      /// <summary>
      /// fa-newspaper unicode value ("\uf1ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/newspaper
      /// </summary>
      [Description("newspaper")]
      public const string Newspaper = "\uf1ea";

      /// <summary>
      /// fa-nfc unicode value ("\ue1f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc
      /// </summary>
      [Description("nfc")]
      public const string Nfc = "\ue1f7";

      /// <summary>
      /// fa-nfc-directional unicode value ("\ue530").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-directional
      /// </summary>
      [Description("nfc-directional")]
      public const string NfcDirectional = "\ue530";

      /// <summary>
      /// fa-nfc-lock unicode value ("\ue1f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-lock
      /// </summary>
      [Description("nfc-lock")]
      public const string NfcLock = "\ue1f8";

      /// <summary>
      /// fa-nfc-magnifying-glass unicode value ("\ue1f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-magnifying-glass
      /// </summary>
      [Description("nfc-magnifying-glass")]
      public const string NfcMagnifyingGlass = "\ue1f9";

      /// <summary>
      /// fa-nfc-pen unicode value ("\ue1fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-pen
      /// </summary>
      [Description("nfc-pen")]
      public const string NfcPen = "\ue1fa";

      /// <summary>
      /// fa-nfc-signal unicode value ("\ue1fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-signal
      /// </summary>
      [Description("nfc-signal")]
      public const string NfcSignal = "\ue1fb";

      /// <summary>
      /// fa-nfc-slash unicode value ("\ue1fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-slash
      /// </summary>
      [Description("nfc-slash")]
      public const string NfcSlash = "\ue1fc";

      /// <summary>
      /// fa-nfc-symbol unicode value ("\ue531").
      /// <para/>
      /// This icon supports the following styles: Brands, Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-symbol
      /// </summary>
      [Description("nfc-symbol")]
      public const string NfcSymbol = "\ue531";

      /// <summary>
      /// fa-nfc-trash unicode value ("\ue1fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nfc-trash
      /// </summary>
      [Description("nfc-trash")]
      public const string NfcTrash = "\ue1fd";

      /// <summary>
      /// fa-nimblr unicode value ("\uf5a8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nimblr
      /// </summary>
      [Description("nimblr")]
      public const string Nimblr = "\uf5a8";

      /// <summary>
      /// fa-node unicode value ("\uf419").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/node
      /// </summary>
      [Description("node")]
      public const string Node = "\uf419";

      /// <summary>
      /// fa-node-js unicode value ("\uf3d3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/node-js
      /// </summary>
      [Description("node-js")]
      public const string NodeJs = "\uf3d3";

      /// <summary>
      /// fa-nose unicode value ("\ue5bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nose
      /// </summary>
      [Description("nose")]
      public const string Nose = "\ue5bd";

      /// <summary>
      /// fa-not-equal unicode value ("\uf53e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/not-equal
      /// </summary>
      [Description("not-equal")]
      public const string NotEqual = "\uf53e";

      /// <summary>
      /// fa-notdef unicode value ("\ue1fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/notdef
      /// </summary>
      [Description("notdef")]
      public const string Notdef = "\ue1fe";

      /// <summary>
      /// fa-note unicode value ("\ue1ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/note
      /// </summary>
      [Description("note")]
      public const string Note = "\ue1ff";

      /// <summary>
      /// fa-note-medical unicode value ("\ue200").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/note-medical
      /// </summary>
      [Description("note-medical")]
      public const string NoteMedical = "\ue200";

      /// <summary>
      /// fa-note-sticky unicode value ("\uf249").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/note-sticky
      /// </summary>
      [Description("note-sticky")]
      public const string NoteSticky = "\uf249";

      /// <summary>
      /// fa-notebook unicode value ("\ue201").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/notebook
      /// </summary>
      [Description("notebook")]
      public const string Notebook = "\ue201";

      /// <summary>
      /// fa-notes unicode value ("\ue202").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/notes
      /// </summary>
      [Description("notes")]
      public const string Notes = "\ue202";

      /// <summary>
      /// fa-notes-medical unicode value ("\uf481").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/notes-medical
      /// </summary>
      [Description("notes-medical")]
      public const string NotesMedical = "\uf481";

      /// <summary>
      /// fa-npm unicode value ("\uf3d4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/npm
      /// </summary>
      [Description("npm")]
      public const string Npm = "\uf3d4";

      /// <summary>
      /// fa-ns8 unicode value ("\uf3d5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ns8
      /// </summary>
      [Description("ns8")]
      public const string Ns8 = "\uf3d5";

      /// <summary>
      /// fa-nutritionix unicode value ("\uf3d6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/nutritionix
      /// </summary>
      [Description("nutritionix")]
      public const string Nutritionix = "\uf3d6";

      /// <summary>
      /// fa-o unicode value ("\u004f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/o
      /// </summary>
      [Description("o")]
      public const string O = "\u004f";

      /// <summary>
      /// fa-object-exclude unicode value ("\ue49c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-exclude
      /// </summary>
      [Description("object-exclude")]
      public const string ObjectExclude = "\ue49c";

      /// <summary>
      /// fa-object-group unicode value ("\uf247").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-group
      /// </summary>
      [Description("object-group")]
      public const string ObjectGroup = "\uf247";

      /// <summary>
      /// fa-object-intersect unicode value ("\ue49d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-intersect
      /// </summary>
      [Description("object-intersect")]
      public const string ObjectIntersect = "\ue49d";

      /// <summary>
      /// fa-object-subtract unicode value ("\ue49e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-subtract
      /// </summary>
      [Description("object-subtract")]
      public const string ObjectSubtract = "\ue49e";

      /// <summary>
      /// fa-object-ungroup unicode value ("\uf248").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-ungroup
      /// </summary>
      [Description("object-ungroup")]
      public const string ObjectUngroup = "\uf248";

      /// <summary>
      /// fa-object-union unicode value ("\ue49f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/object-union
      /// </summary>
      [Description("object-union")]
      public const string ObjectUnion = "\ue49f";

      /// <summary>
      /// fa-objects-align-bottom unicode value ("\ue3bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-bottom
      /// </summary>
      [Description("objects-align-bottom")]
      public const string ObjectsAlignBottom = "\ue3bb";

      /// <summary>
      /// fa-objects-align-center-horizontal unicode value ("\ue3bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-center-horizontal
      /// </summary>
      [Description("objects-align-center-horizontal")]
      public const string ObjectsAlignCenterHorizontal = "\ue3bc";

      /// <summary>
      /// fa-objects-align-center-vertical unicode value ("\ue3bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-center-vertical
      /// </summary>
      [Description("objects-align-center-vertical")]
      public const string ObjectsAlignCenterVertical = "\ue3bd";

      /// <summary>
      /// fa-objects-align-left unicode value ("\ue3be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-left
      /// </summary>
      [Description("objects-align-left")]
      public const string ObjectsAlignLeft = "\ue3be";

      /// <summary>
      /// fa-objects-align-right unicode value ("\ue3bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-right
      /// </summary>
      [Description("objects-align-right")]
      public const string ObjectsAlignRight = "\ue3bf";

      /// <summary>
      /// fa-objects-align-top unicode value ("\ue3c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-align-top
      /// </summary>
      [Description("objects-align-top")]
      public const string ObjectsAlignTop = "\ue3c0";

      /// <summary>
      /// fa-objects-column unicode value ("\ue3c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/objects-column
      /// </summary>
      [Description("objects-column")]
      public const string ObjectsColumn = "\ue3c1";

      /// <summary>
      /// fa-octagon unicode value ("\uf306").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon
      /// </summary>
      [Description("octagon")]
      public const string Octagon = "\uf306";

      /// <summary>
      /// fa-octagon-check unicode value ("\ue426").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-check
      /// </summary>
      [Description("octagon-check")]
      public const string OctagonCheck = "\ue426";

      /// <summary>
      /// fa-octagon-divide unicode value ("\ue203").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-divide
      /// </summary>
      [Description("octagon-divide")]
      public const string OctagonDivide = "\ue203";

      /// <summary>
      /// fa-octagon-exclamation unicode value ("\ue204").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-exclamation
      /// </summary>
      [Description("octagon-exclamation")]
      public const string OctagonExclamation = "\ue204";

      /// <summary>
      /// fa-octagon-minus unicode value ("\uf308").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-minus
      /// </summary>
      [Description("octagon-minus")]
      public const string OctagonMinus = "\uf308";

      /// <summary>
      /// fa-octagon-plus unicode value ("\uf301").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-plus
      /// </summary>
      [Description("octagon-plus")]
      public const string OctagonPlus = "\uf301";

      /// <summary>
      /// fa-octagon-xmark unicode value ("\uf2f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octagon-xmark
      /// </summary>
      [Description("octagon-xmark")]
      public const string OctagonXmark = "\uf2f0";

      /// <summary>
      /// fa-octopus-deploy unicode value ("\ue082").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/octopus-deploy
      /// </summary>
      [Description("octopus-deploy")]
      public const string OctopusDeploy = "\ue082";

      /// <summary>
      /// fa-odnoklassniki unicode value ("\uf263").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/odnoklassniki
      /// </summary>
      [Description("odnoklassniki")]
      public const string Odnoklassniki = "\uf263";

      /// <summary>
      /// fa-odysee unicode value ("\ue5c6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.3.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/odysee
      /// </summary>
      [Description("odysee")]
      public const string Odysee = "\ue5c6";

      /// <summary>
      /// fa-oil-can unicode value ("\uf613").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/oil-can
      /// </summary>
      [Description("oil-can")]
      public const string OilCan = "\uf613";

      /// <summary>
      /// fa-oil-can-drip unicode value ("\ue205").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/oil-can-drip
      /// </summary>
      [Description("oil-can-drip")]
      public const string OilCanDrip = "\ue205";

      /// <summary>
      /// fa-oil-temperature unicode value ("\uf614").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/oil-temperature
      /// </summary>
      [Description("oil-temperature")]
      public const string OilTemperature = "\uf614";

      /// <summary>
      /// fa-oil-well unicode value ("\ue532").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/oil-well
      /// </summary>
      [Description("oil-well")]
      public const string OilWell = "\ue532";

      /// <summary>
      /// fa-old-republic unicode value ("\uf510").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/old-republic
      /// </summary>
      [Description("old-republic")]
      public const string OldRepublic = "\uf510";

      /// <summary>
      /// fa-olive unicode value ("\ue316").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/olive
      /// </summary>
      [Description("olive")]
      public const string Olive = "\ue316";

      /// <summary>
      /// fa-olive-branch unicode value ("\ue317").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/olive-branch
      /// </summary>
      [Description("olive-branch")]
      public const string OliveBranch = "\ue317";

      /// <summary>
      /// fa-om unicode value ("\uf679").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/om
      /// </summary>
      [Description("om")]
      public const string Om = "\uf679";

      /// <summary>
      /// fa-omega unicode value ("\uf67a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/omega
      /// </summary>
      [Description("omega")]
      public const string Omega = "\uf67a";

      /// <summary>
      /// fa-onion unicode value ("\ue427").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/onion
      /// </summary>
      [Description("onion")]
      public const string Onion = "\ue427";

      /// <summary>
      /// fa-opencart unicode value ("\uf23d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/opencart
      /// </summary>
      [Description("opencart")]
      public const string Opencart = "\uf23d";

      /// <summary>
      /// fa-openid unicode value ("\uf19b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/openid
      /// </summary>
      [Description("openid")]
      public const string Openid = "\uf19b";

      /// <summary>
      /// fa-opera unicode value ("\uf26a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/opera
      /// </summary>
      [Description("opera")]
      public const string Opera = "\uf26a";

      /// <summary>
      /// fa-optin-monster unicode value ("\uf23c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/optin-monster
      /// </summary>
      [Description("optin-monster")]
      public const string OptinMonster = "\uf23c";

      /// <summary>
      /// fa-option unicode value ("\ue318").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/option
      /// </summary>
      [Description("option")]
      public const string Option = "\ue318";

      /// <summary>
      /// fa-orcid unicode value ("\uf8d2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/orcid
      /// </summary>
      [Description("orcid")]
      public const string Orcid = "\uf8d2";

      /// <summary>
      /// fa-ornament unicode value ("\uf7b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ornament
      /// </summary>
      [Description("ornament")]
      public const string Ornament = "\uf7b8";

      /// <summary>
      /// fa-osi unicode value ("\uf41a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/osi
      /// </summary>
      [Description("osi")]
      public const string Osi = "\uf41a";

      /// <summary>
      /// fa-otter unicode value ("\uf700").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/otter
      /// </summary>
      [Description("otter")]
      public const string Otter = "\uf700";

      /// <summary>
      /// fa-outdent unicode value ("\uf03b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/outdent
      /// </summary>
      [Description("outdent")]
      public const string Outdent = "\uf03b";

      /// <summary>
      /// fa-outlet unicode value ("\ue01c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/outlet
      /// </summary>
      [Description("outlet")]
      public const string Outlet = "\ue01c";

      /// <summary>
      /// fa-oven unicode value ("\ue01d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/oven
      /// </summary>
      [Description("oven")]
      public const string Oven = "\ue01d";

      /// <summary>
      /// fa-overline unicode value ("\uf876").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/overline
      /// </summary>
      [Description("overline")]
      public const string Overline = "\uf876";

      /// <summary>
      /// fa-p unicode value ("\u0050").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/p
      /// </summary>
      [Description("p")]
      public const string P = "\u0050";

      /// <summary>
      /// fa-padlet unicode value ("\ue4a0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/padlet
      /// </summary>
      [Description("padlet")]
      public const string Padlet = "\ue4a0";

      /// <summary>
      /// fa-page unicode value ("\ue428").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/page
      /// </summary>
      [Description("page")]
      public const string Page = "\ue428";

      /// <summary>
      /// fa-page-caret-down unicode value ("\ue429").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/page-caret-down
      /// </summary>
      [Description("page-caret-down")]
      public const string PageCaretDown = "\ue429";

      /// <summary>
      /// fa-page-caret-up unicode value ("\ue42a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/page-caret-up
      /// </summary>
      [Description("page-caret-up")]
      public const string PageCaretUp = "\ue42a";

      /// <summary>
      /// fa-page4 unicode value ("\uf3d7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/page4
      /// </summary>
      [Description("page4")]
      public const string Page4 = "\uf3d7";

      /// <summary>
      /// fa-pagelines unicode value ("\uf18c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pagelines
      /// </summary>
      [Description("pagelines")]
      public const string Pagelines = "\uf18c";

      /// <summary>
      /// fa-pager unicode value ("\uf815").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pager
      /// </summary>
      [Description("pager")]
      public const string Pager = "\uf815";

      /// <summary>
      /// fa-paint-roller unicode value ("\uf5aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paint-roller
      /// </summary>
      [Description("paint-roller")]
      public const string PaintRoller = "\uf5aa";

      /// <summary>
      /// fa-paintbrush unicode value ("\uf1fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paintbrush
      /// </summary>
      [Description("paintbrush")]
      public const string Paintbrush = "\uf1fc";

      /// <summary>
      /// fa-paintbrush-fine unicode value ("\uf5a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paintbrush-fine
      /// </summary>
      [Description("paintbrush-fine")]
      public const string PaintbrushFine = "\uf5a9";

      /// <summary>
      /// fa-paintbrush-pencil unicode value ("\ue206").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paintbrush-pencil
      /// </summary>
      [Description("paintbrush-pencil")]
      public const string PaintbrushPencil = "\ue206";

      /// <summary>
      /// fa-palette unicode value ("\uf53f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/palette
      /// </summary>
      [Description("palette")]
      public const string Palette = "\uf53f";

      /// <summary>
      /// fa-palfed unicode value ("\uf3d8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/palfed
      /// </summary>
      [Description("palfed")]
      public const string Palfed = "\uf3d8";

      /// <summary>
      /// fa-pallet unicode value ("\uf482").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pallet
      /// </summary>
      [Description("pallet")]
      public const string Pallet = "\uf482";

      /// <summary>
      /// fa-pallet-box unicode value ("\ue208").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pallet-box
      /// </summary>
      [Description("pallet-box")]
      public const string PalletBox = "\ue208";

      /// <summary>
      /// fa-pallet-boxes unicode value ("\uf483").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pallet-boxes
      /// </summary>
      [Description("pallet-boxes")]
      public const string PalletBoxes = "\uf483";

      /// <summary>
      /// fa-pan-food unicode value ("\ue42b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pan-food
      /// </summary>
      [Description("pan-food")]
      public const string PanFood = "\ue42b";

      /// <summary>
      /// fa-pan-frying unicode value ("\ue42c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pan-frying
      /// </summary>
      [Description("pan-frying")]
      public const string PanFrying = "\ue42c";

      /// <summary>
      /// fa-pancakes unicode value ("\ue42d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pancakes
      /// </summary>
      [Description("pancakes")]
      public const string Pancakes = "\ue42d";

      /// <summary>
      /// fa-panel-ews unicode value ("\ue42e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/panel-ews
      /// </summary>
      [Description("panel-ews")]
      public const string PanelEws = "\ue42e";

      /// <summary>
      /// fa-panel-fire unicode value ("\ue42f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/panel-fire
      /// </summary>
      [Description("panel-fire")]
      public const string PanelFire = "\ue42f";

      /// <summary>
      /// fa-panorama unicode value ("\ue209").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/panorama
      /// </summary>
      [Description("panorama")]
      public const string Panorama = "\ue209";

      /// <summary>
      /// fa-paper-plane unicode value ("\uf1d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paper-plane
      /// </summary>
      [Description("paper-plane")]
      public const string PaperPlane = "\uf1d8";

      /// <summary>
      /// fa-paper-plane-top unicode value ("\ue20a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paper-plane-top
      /// </summary>
      [Description("paper-plane-top")]
      public const string PaperPlaneTop = "\ue20a";

      /// <summary>
      /// fa-paperclip unicode value ("\uf0c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paperclip
      /// </summary>
      [Description("paperclip")]
      public const string Paperclip = "\uf0c6";

      /// <summary>
      /// fa-paperclip-vertical unicode value ("\ue3c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paperclip-vertical
      /// </summary>
      [Description("paperclip-vertical")]
      public const string PaperclipVertical = "\ue3c2";

      /// <summary>
      /// fa-parachute-box unicode value ("\uf4cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/parachute-box
      /// </summary>
      [Description("parachute-box")]
      public const string ParachuteBox = "\uf4cd";

      /// <summary>
      /// fa-paragraph unicode value ("\uf1dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paragraph
      /// </summary>
      [Description("paragraph")]
      public const string Paragraph = "\uf1dd";

      /// <summary>
      /// fa-paragraph-left unicode value ("\uf878").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paragraph-left
      /// </summary>
      [Description("paragraph-left")]
      public const string ParagraphLeft = "\uf878";

      /// <summary>
      /// fa-party-bell unicode value ("\ue31a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/party-bell
      /// </summary>
      [Description("party-bell")]
      public const string PartyBell = "\ue31a";

      /// <summary>
      /// fa-party-horn unicode value ("\ue31b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/party-horn
      /// </summary>
      [Description("party-horn")]
      public const string PartyHorn = "\ue31b";

      /// <summary>
      /// fa-passport unicode value ("\uf5ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/passport
      /// </summary>
      [Description("passport")]
      public const string Passport = "\uf5ab";

      /// <summary>
      /// fa-paste unicode value ("\uf0ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paste
      /// </summary>
      [Description("paste")]
      public const string Paste = "\uf0ea";

      /// <summary>
      /// fa-patreon unicode value ("\uf3d9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/patreon
      /// </summary>
      [Description("patreon")]
      public const string Patreon = "\uf3d9";

      /// <summary>
      /// fa-pause unicode value ("\uf04c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pause
      /// </summary>
      [Description("pause")]
      public const string Pause = "\uf04c";

      /// <summary>
      /// fa-paw unicode value ("\uf1b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paw
      /// </summary>
      [Description("paw")]
      public const string Paw = "\uf1b0";

      /// <summary>
      /// fa-paw-claws unicode value ("\uf702").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paw-claws
      /// </summary>
      [Description("paw-claws")]
      public const string PawClaws = "\uf702";

      /// <summary>
      /// fa-paw-simple unicode value ("\uf701").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paw-simple
      /// </summary>
      [Description("paw-simple")]
      public const string PawSimple = "\uf701";

      /// <summary>
      /// fa-paypal unicode value ("\uf1ed").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/paypal
      /// </summary>
      [Description("paypal")]
      public const string Paypal = "\uf1ed";

      /// <summary>
      /// fa-peace unicode value ("\uf67c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peace
      /// </summary>
      [Description("peace")]
      public const string Peace = "\uf67c";

      /// <summary>
      /// fa-peach unicode value ("\ue20b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peach
      /// </summary>
      [Description("peach")]
      public const string Peach = "\ue20b";

      /// <summary>
      /// fa-peanut unicode value ("\ue430").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peanut
      /// </summary>
      [Description("peanut")]
      public const string Peanut = "\ue430";

      /// <summary>
      /// fa-peanuts unicode value ("\ue431").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peanuts
      /// </summary>
      [Description("peanuts")]
      public const string Peanuts = "\ue431";

      /// <summary>
      /// fa-peapod unicode value ("\ue31c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peapod
      /// </summary>
      [Description("peapod")]
      public const string Peapod = "\ue31c";

      /// <summary>
      /// fa-pear unicode value ("\ue20c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pear
      /// </summary>
      [Description("pear")]
      public const string Pear = "\ue20c";

      /// <summary>
      /// fa-pedestal unicode value ("\ue20d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pedestal
      /// </summary>
      [Description("pedestal")]
      public const string Pedestal = "\ue20d";

      /// <summary>
      /// fa-pegasus unicode value ("\uf703").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pegasus
      /// </summary>
      [Description("pegasus")]
      public const string Pegasus = "\uf703";

      /// <summary>
      /// fa-pen unicode value ("\uf304").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen
      /// </summary>
      [Description("pen")]
      public const string Pen = "\uf304";

      /// <summary>
      /// fa-pen-circle unicode value ("\ue20e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-circle
      /// </summary>
      [Description("pen-circle")]
      public const string PenCircle = "\ue20e";

      /// <summary>
      /// fa-pen-clip unicode value ("\uf305").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-clip
      /// </summary>
      [Description("pen-clip")]
      public const string PenClip = "\uf305";

      /// <summary>
      /// fa-pen-clip-slash unicode value ("\ue20f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-clip-slash
      /// </summary>
      [Description("pen-clip-slash")]
      public const string PenClipSlash = "\ue20f";

      /// <summary>
      /// fa-pen-fancy unicode value ("\uf5ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-fancy
      /// </summary>
      [Description("pen-fancy")]
      public const string PenFancy = "\uf5ac";

      /// <summary>
      /// fa-pen-fancy-slash unicode value ("\ue210").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-fancy-slash
      /// </summary>
      [Description("pen-fancy-slash")]
      public const string PenFancySlash = "\ue210";

      /// <summary>
      /// fa-pen-field unicode value ("\ue211").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-field
      /// </summary>
      [Description("pen-field")]
      public const string PenField = "\ue211";

      /// <summary>
      /// fa-pen-line unicode value ("\ue212").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-line
      /// </summary>
      [Description("pen-line")]
      public const string PenLine = "\ue212";

      /// <summary>
      /// fa-pen-nib unicode value ("\uf5ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-nib
      /// </summary>
      [Description("pen-nib")]
      public const string PenNib = "\uf5ad";

      /// <summary>
      /// fa-pen-nib-slash unicode value ("\ue4a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-nib-slash
      /// </summary>
      [Description("pen-nib-slash")]
      public const string PenNibSlash = "\ue4a1";

      /// <summary>
      /// fa-pen-paintbrush unicode value ("\uf618").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-paintbrush
      /// </summary>
      [Description("pen-paintbrush")]
      public const string PenPaintbrush = "\uf618";

      /// <summary>
      /// fa-pen-ruler unicode value ("\uf5ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-ruler
      /// </summary>
      [Description("pen-ruler")]
      public const string PenRuler = "\uf5ae";

      /// <summary>
      /// fa-pen-slash unicode value ("\ue213").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-slash
      /// </summary>
      [Description("pen-slash")]
      public const string PenSlash = "\ue213";

      /// <summary>
      /// fa-pen-swirl unicode value ("\ue214").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-swirl
      /// </summary>
      [Description("pen-swirl")]
      public const string PenSwirl = "\ue214";

      /// <summary>
      /// fa-pen-to-square unicode value ("\uf044").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pen-to-square
      /// </summary>
      [Description("pen-to-square")]
      public const string PenToSquare = "\uf044";

      /// <summary>
      /// fa-pencil unicode value ("\uf303").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pencil
      /// </summary>
      [Description("pencil")]
      public const string Pencil = "\uf303";

      /// <summary>
      /// fa-pencil-mechanical unicode value ("\ue5ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pencil-mechanical
      /// </summary>
      [Description("pencil-mechanical")]
      public const string PencilMechanical = "\ue5ca";

      /// <summary>
      /// fa-pencil-slash unicode value ("\ue215").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pencil-slash
      /// </summary>
      [Description("pencil-slash")]
      public const string PencilSlash = "\ue215";

      /// <summary>
      /// fa-people unicode value ("\ue216").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people
      /// </summary>
      [Description("people")]
      public const string People = "\ue216";

      /// <summary>
      /// fa-people-arrows unicode value ("\ue068").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-arrows
      /// </summary>
      [Description("people-arrows")]
      public const string PeopleArrows = "\ue068";

      /// <summary>
      /// fa-people-carry-box unicode value ("\uf4ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-carry-box
      /// </summary>
      [Description("people-carry-box")]
      public const string PeopleCarryBox = "\uf4ce";

      /// <summary>
      /// fa-people-dress unicode value ("\ue217").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-dress
      /// </summary>
      [Description("people-dress")]
      public const string PeopleDress = "\ue217";

      /// <summary>
      /// fa-people-dress-simple unicode value ("\ue218").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-dress-simple
      /// </summary>
      [Description("people-dress-simple")]
      public const string PeopleDressSimple = "\ue218";

      /// <summary>
      /// fa-people-group unicode value ("\ue533").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-group
      /// </summary>
      [Description("people-group")]
      public const string PeopleGroup = "\ue533";

      /// <summary>
      /// fa-people-line unicode value ("\ue534").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-line
      /// </summary>
      [Description("people-line")]
      public const string PeopleLine = "\ue534";

      /// <summary>
      /// fa-people-pants unicode value ("\ue219").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-pants
      /// </summary>
      [Description("people-pants")]
      public const string PeoplePants = "\ue219";

      /// <summary>
      /// fa-people-pants-simple unicode value ("\ue21a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-pants-simple
      /// </summary>
      [Description("people-pants-simple")]
      public const string PeoplePantsSimple = "\ue21a";

      /// <summary>
      /// fa-people-pulling unicode value ("\ue535").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-pulling
      /// </summary>
      [Description("people-pulling")]
      public const string PeoplePulling = "\ue535";

      /// <summary>
      /// fa-people-robbery unicode value ("\ue536").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-robbery
      /// </summary>
      [Description("people-robbery")]
      public const string PeopleRobbery = "\ue536";

      /// <summary>
      /// fa-people-roof unicode value ("\ue537").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-roof
      /// </summary>
      [Description("people-roof")]
      public const string PeopleRoof = "\ue537";

      /// <summary>
      /// fa-people-simple unicode value ("\ue21b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/people-simple
      /// </summary>
      [Description("people-simple")]
      public const string PeopleSimple = "\ue21b";

      /// <summary>
      /// fa-pepper unicode value ("\ue432").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pepper
      /// </summary>
      [Description("pepper")]
      public const string Pepper = "\ue432";

      /// <summary>
      /// fa-pepper-hot unicode value ("\uf816").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pepper-hot
      /// </summary>
      [Description("pepper-hot")]
      public const string PepperHot = "\uf816";

      /// <summary>
      /// fa-perbyte unicode value ("\ue083").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/perbyte
      /// </summary>
      [Description("perbyte")]
      public const string Perbyte = "\ue083";

      /// <summary>
      /// fa-percent unicode value ("\u0025").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/percent
      /// </summary>
      [Description("percent")]
      public const string Percent = "\u0025";

      /// <summary>
      /// fa-period unicode value ("\u002e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/period
      /// </summary>
      [Description("period")]
      public const string Period = "\u002e";

      /// <summary>
      /// fa-periscope unicode value ("\uf3da").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/periscope
      /// </summary>
      [Description("periscope")]
      public const string Periscope = "\uf3da";

      /// <summary>
      /// fa-person unicode value ("\uf183").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person
      /// </summary>
      [Description("person")]
      public const string Person = "\uf183";

      /// <summary>
      /// fa-person-arrow-down-to-line unicode value ("\ue538").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-arrow-down-to-line
      /// </summary>
      [Description("person-arrow-down-to-line")]
      public const string PersonArrowDownToLine = "\ue538";

      /// <summary>
      /// fa-person-arrow-up-from-line unicode value ("\ue539").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-arrow-up-from-line
      /// </summary>
      [Description("person-arrow-up-from-line")]
      public const string PersonArrowUpFromLine = "\ue539";

      /// <summary>
      /// fa-person-biking unicode value ("\uf84a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-biking
      /// </summary>
      [Description("person-biking")]
      public const string PersonBiking = "\uf84a";

      /// <summary>
      /// fa-person-biking-mountain unicode value ("\uf84b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-biking-mountain
      /// </summary>
      [Description("person-biking-mountain")]
      public const string PersonBikingMountain = "\uf84b";

      /// <summary>
      /// fa-person-booth unicode value ("\uf756").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-booth
      /// </summary>
      [Description("person-booth")]
      public const string PersonBooth = "\uf756";

      /// <summary>
      /// fa-person-breastfeeding unicode value ("\ue53a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-breastfeeding
      /// </summary>
      [Description("person-breastfeeding")]
      public const string PersonBreastfeeding = "\ue53a";

      /// <summary>
      /// fa-person-burst unicode value ("\ue53b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-burst
      /// </summary>
      [Description("person-burst")]
      public const string PersonBurst = "\ue53b";

      /// <summary>
      /// fa-person-cane unicode value ("\ue53c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-cane
      /// </summary>
      [Description("person-cane")]
      public const string PersonCane = "\ue53c";

      /// <summary>
      /// fa-person-carry-box unicode value ("\uf4cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-carry-box
      /// </summary>
      [Description("person-carry-box")]
      public const string PersonCarryBox = "\uf4cf";

      /// <summary>
      /// fa-person-chalkboard unicode value ("\ue53d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-chalkboard
      /// </summary>
      [Description("person-chalkboard")]
      public const string PersonChalkboard = "\ue53d";

      /// <summary>
      /// fa-person-circle-check unicode value ("\ue53e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-check
      /// </summary>
      [Description("person-circle-check")]
      public const string PersonCircleCheck = "\ue53e";

      /// <summary>
      /// fa-person-circle-exclamation unicode value ("\ue53f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-exclamation
      /// </summary>
      [Description("person-circle-exclamation")]
      public const string PersonCircleExclamation = "\ue53f";

      /// <summary>
      /// fa-person-circle-minus unicode value ("\ue540").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-minus
      /// </summary>
      [Description("person-circle-minus")]
      public const string PersonCircleMinus = "\ue540";

      /// <summary>
      /// fa-person-circle-plus unicode value ("\ue541").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-plus
      /// </summary>
      [Description("person-circle-plus")]
      public const string PersonCirclePlus = "\ue541";

      /// <summary>
      /// fa-person-circle-question unicode value ("\ue542").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-question
      /// </summary>
      [Description("person-circle-question")]
      public const string PersonCircleQuestion = "\ue542";

      /// <summary>
      /// fa-person-circle-xmark unicode value ("\ue543").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-circle-xmark
      /// </summary>
      [Description("person-circle-xmark")]
      public const string PersonCircleXmark = "\ue543";

      /// <summary>
      /// fa-person-digging unicode value ("\uf85e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-digging
      /// </summary>
      [Description("person-digging")]
      public const string PersonDigging = "\uf85e";

      /// <summary>
      /// fa-person-dolly unicode value ("\uf4d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dolly
      /// </summary>
      [Description("person-dolly")]
      public const string PersonDolly = "\uf4d0";

      /// <summary>
      /// fa-person-dolly-empty unicode value ("\uf4d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dolly-empty
      /// </summary>
      [Description("person-dolly-empty")]
      public const string PersonDollyEmpty = "\uf4d1";

      /// <summary>
      /// fa-person-dots-from-line unicode value ("\uf470").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dots-from-line
      /// </summary>
      [Description("person-dots-from-line")]
      public const string PersonDotsFromLine = "\uf470";

      /// <summary>
      /// fa-person-dress unicode value ("\uf182").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dress
      /// </summary>
      [Description("person-dress")]
      public const string PersonDress = "\uf182";

      /// <summary>
      /// fa-person-dress-burst unicode value ("\ue544").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dress-burst
      /// </summary>
      [Description("person-dress-burst")]
      public const string PersonDressBurst = "\ue544";

      /// <summary>
      /// fa-person-dress-simple unicode value ("\ue21c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-dress-simple
      /// </summary>
      [Description("person-dress-simple")]
      public const string PersonDressSimple = "\ue21c";

      /// <summary>
      /// fa-person-drowning unicode value ("\ue545").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-drowning
      /// </summary>
      [Description("person-drowning")]
      public const string PersonDrowning = "\ue545";

      /// <summary>
      /// fa-person-falling unicode value ("\ue546").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-falling
      /// </summary>
      [Description("person-falling")]
      public const string PersonFalling = "\ue546";

      /// <summary>
      /// fa-person-falling-burst unicode value ("\ue547").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-falling-burst
      /// </summary>
      [Description("person-falling-burst")]
      public const string PersonFallingBurst = "\ue547";

      /// <summary>
      /// fa-person-from-portal unicode value ("\ue023").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-from-portal
      /// </summary>
      [Description("person-from-portal")]
      public const string PersonFromPortal = "\ue023";

      /// <summary>
      /// fa-person-half-dress unicode value ("\ue548").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-half-dress
      /// </summary>
      [Description("person-half-dress")]
      public const string PersonHalfDress = "\ue548";

      /// <summary>
      /// fa-person-harassing unicode value ("\ue549").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-harassing
      /// </summary>
      [Description("person-harassing")]
      public const string PersonHarassing = "\ue549";

      /// <summary>
      /// fa-person-hiking unicode value ("\uf6ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-hiking
      /// </summary>
      [Description("person-hiking")]
      public const string PersonHiking = "\uf6ec";

      /// <summary>
      /// fa-person-military-pointing unicode value ("\ue54a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-military-pointing
      /// </summary>
      [Description("person-military-pointing")]
      public const string PersonMilitaryPointing = "\ue54a";

      /// <summary>
      /// fa-person-military-rifle unicode value ("\ue54b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-military-rifle
      /// </summary>
      [Description("person-military-rifle")]
      public const string PersonMilitaryRifle = "\ue54b";

      /// <summary>
      /// fa-person-military-to-person unicode value ("\ue54c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-military-to-person
      /// </summary>
      [Description("person-military-to-person")]
      public const string PersonMilitaryToPerson = "\ue54c";

      /// <summary>
      /// fa-person-pinball unicode value ("\ue21d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-pinball
      /// </summary>
      [Description("person-pinball")]
      public const string PersonPinball = "\ue21d";

      /// <summary>
      /// fa-person-praying unicode value ("\uf683").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-praying
      /// </summary>
      [Description("person-praying")]
      public const string PersonPraying = "\uf683";

      /// <summary>
      /// fa-person-pregnant unicode value ("\ue31e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-pregnant
      /// </summary>
      [Description("person-pregnant")]
      public const string PersonPregnant = "\ue31e";

      /// <summary>
      /// fa-person-rays unicode value ("\ue54d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-rays
      /// </summary>
      [Description("person-rays")]
      public const string PersonRays = "\ue54d";

      /// <summary>
      /// fa-person-rifle unicode value ("\ue54e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-rifle
      /// </summary>
      [Description("person-rifle")]
      public const string PersonRifle = "\ue54e";

      /// <summary>
      /// fa-person-running unicode value ("\uf70c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-running
      /// </summary>
      [Description("person-running")]
      public const string PersonRunning = "\uf70c";

      /// <summary>
      /// fa-person-running-fast unicode value ("\ue5ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-running-fast
      /// </summary>
      [Description("person-running-fast")]
      public const string PersonRunningFast = "\ue5ff";

      /// <summary>
      /// fa-person-seat unicode value ("\ue21e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-seat
      /// </summary>
      [Description("person-seat")]
      public const string PersonSeat = "\ue21e";

      /// <summary>
      /// fa-person-seat-reclined unicode value ("\ue21f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-seat-reclined
      /// </summary>
      [Description("person-seat-reclined")]
      public const string PersonSeatReclined = "\ue21f";

      /// <summary>
      /// fa-person-shelter unicode value ("\ue54f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-shelter
      /// </summary>
      [Description("person-shelter")]
      public const string PersonShelter = "\ue54f";

      /// <summary>
      /// fa-person-sign unicode value ("\uf757").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-sign
      /// </summary>
      [Description("person-sign")]
      public const string PersonSign = "\uf757";

      /// <summary>
      /// fa-person-simple unicode value ("\ue220").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-simple
      /// </summary>
      [Description("person-simple")]
      public const string PersonSimple = "\ue220";

      /// <summary>
      /// fa-person-skating unicode value ("\uf7c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-skating
      /// </summary>
      [Description("person-skating")]
      public const string PersonSkating = "\uf7c5";

      /// <summary>
      /// fa-person-ski-jumping unicode value ("\uf7c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-ski-jumping
      /// </summary>
      [Description("person-ski-jumping")]
      public const string PersonSkiJumping = "\uf7c7";

      /// <summary>
      /// fa-person-ski-lift unicode value ("\uf7c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-ski-lift
      /// </summary>
      [Description("person-ski-lift")]
      public const string PersonSkiLift = "\uf7c8";

      /// <summary>
      /// fa-person-skiing unicode value ("\uf7c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-skiing
      /// </summary>
      [Description("person-skiing")]
      public const string PersonSkiing = "\uf7c9";

      /// <summary>
      /// fa-person-skiing-nordic unicode value ("\uf7ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-skiing-nordic
      /// </summary>
      [Description("person-skiing-nordic")]
      public const string PersonSkiingNordic = "\uf7ca";

      /// <summary>
      /// fa-person-sledding unicode value ("\uf7cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-sledding
      /// </summary>
      [Description("person-sledding")]
      public const string PersonSledding = "\uf7cb";

      /// <summary>
      /// fa-person-snowboarding unicode value ("\uf7ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-snowboarding
      /// </summary>
      [Description("person-snowboarding")]
      public const string PersonSnowboarding = "\uf7ce";

      /// <summary>
      /// fa-person-snowmobiling unicode value ("\uf7d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-snowmobiling
      /// </summary>
      [Description("person-snowmobiling")]
      public const string PersonSnowmobiling = "\uf7d1";

      /// <summary>
      /// fa-person-swimming unicode value ("\uf5c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-swimming
      /// </summary>
      [Description("person-swimming")]
      public const string PersonSwimming = "\uf5c4";

      /// <summary>
      /// fa-person-through-window unicode value ("\ue5a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-through-window
      /// </summary>
      [Description("person-through-window")]
      public const string PersonThroughWindow = "\ue5a9";

      /// <summary>
      /// fa-person-to-door unicode value ("\ue433").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-to-door
      /// </summary>
      [Description("person-to-door")]
      public const string PersonToDoor = "\ue433";

      /// <summary>
      /// fa-person-to-portal unicode value ("\ue022").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-to-portal
      /// </summary>
      [Description("person-to-portal")]
      public const string PersonToPortal = "\ue022";

      /// <summary>
      /// fa-person-walking unicode value ("\uf554").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking
      /// </summary>
      [Description("person-walking")]
      public const string PersonWalking = "\uf554";

      /// <summary>
      /// fa-person-walking-arrow-loop-left unicode value ("\ue551").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking-arrow-loop-left
      /// </summary>
      [Description("person-walking-arrow-loop-left")]
      public const string PersonWalkingArrowLoopLeft = "\ue551";

      /// <summary>
      /// fa-person-walking-arrow-right unicode value ("\ue552").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking-arrow-right
      /// </summary>
      [Description("person-walking-arrow-right")]
      public const string PersonWalkingArrowRight = "\ue552";

      /// <summary>
      /// fa-person-walking-dashed-line-arrow-right unicode value ("\ue553").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking-dashed-line-arrow-right
      /// </summary>
      [Description("person-walking-dashed-line-arrow-right")]
      public const string PersonWalkingDashedLineArrowRight = "\ue553";

      /// <summary>
      /// fa-person-walking-luggage unicode value ("\ue554").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking-luggage
      /// </summary>
      [Description("person-walking-luggage")]
      public const string PersonWalkingLuggage = "\ue554";

      /// <summary>
      /// fa-person-walking-with-cane unicode value ("\uf29d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/person-walking-with-cane
      /// </summary>
      [Description("person-walking-with-cane")]
      public const string PersonWalkingWithCane = "\uf29d";

      /// <summary>
      /// fa-peseta-sign unicode value ("\ue221").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peseta-sign
      /// </summary>
      [Description("peseta-sign")]
      public const string PesetaSign = "\ue221";

      /// <summary>
      /// fa-peso-sign unicode value ("\ue222").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/peso-sign
      /// </summary>
      [Description("peso-sign")]
      public const string PesoSign = "\ue222";

      /// <summary>
      /// fa-phabricator unicode value ("\uf3db").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phabricator
      /// </summary>
      [Description("phabricator")]
      public const string Phabricator = "\uf3db";

      /// <summary>
      /// fa-phoenix-framework unicode value ("\uf3dc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/phoenix-framework
      /// </summary>
      [Description("phoenix-framework")]
      public const string PhoenixFramework = "\uf3dc";

      /// <summary>
      /// fa-phoenix-squadron unicode value ("\uf511").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phoenix-squadron
      /// </summary>
      [Description("phoenix-squadron")]
      public const string PhoenixSquadron = "\uf511";

      /// <summary>
      /// fa-phone unicode value ("\uf095").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone
      /// </summary>
      [Description("phone")]
      public const string Phone = "\uf095";

      /// <summary>
      /// fa-phone-arrow-down-left unicode value ("\ue223").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-arrow-down-left
      /// </summary>
      [Description("phone-arrow-down-left")]
      public const string PhoneArrowDownLeft = "\ue223";

      /// <summary>
      /// fa-phone-arrow-right unicode value ("\ue5be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-arrow-right
      /// </summary>
      [Description("phone-arrow-right")]
      public const string PhoneArrowRight = "\ue5be";

      /// <summary>
      /// fa-phone-arrow-up-right unicode value ("\ue224").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-arrow-up-right
      /// </summary>
      [Description("phone-arrow-up-right")]
      public const string PhoneArrowUpRight = "\ue224";

      /// <summary>
      /// fa-phone-flip unicode value ("\uf879").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-flip
      /// </summary>
      [Description("phone-flip")]
      public const string PhoneFlip = "\uf879";

      /// <summary>
      /// fa-phone-hangup unicode value ("\ue225").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-hangup
      /// </summary>
      [Description("phone-hangup")]
      public const string PhoneHangup = "\ue225";

      /// <summary>
      /// fa-phone-intercom unicode value ("\ue434").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-intercom
      /// </summary>
      [Description("phone-intercom")]
      public const string PhoneIntercom = "\ue434";

      /// <summary>
      /// fa-phone-missed unicode value ("\ue226").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-missed
      /// </summary>
      [Description("phone-missed")]
      public const string PhoneMissed = "\ue226";

      /// <summary>
      /// fa-phone-office unicode value ("\uf67d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-office
      /// </summary>
      [Description("phone-office")]
      public const string PhoneOffice = "\uf67d";

      /// <summary>
      /// fa-phone-plus unicode value ("\uf4d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-plus
      /// </summary>
      [Description("phone-plus")]
      public const string PhonePlus = "\uf4d2";

      /// <summary>
      /// fa-phone-rotary unicode value ("\uf8d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-rotary
      /// </summary>
      [Description("phone-rotary")]
      public const string PhoneRotary = "\uf8d3";

      /// <summary>
      /// fa-phone-slash unicode value ("\uf3dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-slash
      /// </summary>
      [Description("phone-slash")]
      public const string PhoneSlash = "\uf3dd";

      /// <summary>
      /// fa-phone-volume unicode value ("\uf2a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-volume
      /// </summary>
      [Description("phone-volume")]
      public const string PhoneVolume = "\uf2a0";

      /// <summary>
      /// fa-phone-xmark unicode value ("\ue227").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/phone-xmark
      /// </summary>
      [Description("phone-xmark")]
      public const string PhoneXmark = "\ue227";

      /// <summary>
      /// fa-photo-film unicode value ("\uf87c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/photo-film
      /// </summary>
      [Description("photo-film")]
      public const string PhotoFilm = "\uf87c";

      /// <summary>
      /// fa-photo-film-music unicode value ("\ue228").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/photo-film-music
      /// </summary>
      [Description("photo-film-music")]
      public const string PhotoFilmMusic = "\ue228";

      /// <summary>
      /// fa-php unicode value ("\uf457").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '5.0.5'.
      /// <para/>
      /// See https://fontawesome.com/icons/php
      /// </summary>
      [Description("php")]
      public const string Php = "\uf457";

      /// <summary>
      /// fa-pi unicode value ("\uf67e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pi
      /// </summary>
      [Description("pi")]
      public const string Pi = "\uf67e";

      /// <summary>
      /// fa-piano unicode value ("\uf8d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/piano
      /// </summary>
      [Description("piano")]
      public const string Piano = "\uf8d4";

      /// <summary>
      /// fa-piano-keyboard unicode value ("\uf8d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/piano-keyboard
      /// </summary>
      [Description("piano-keyboard")]
      public const string PianoKeyboard = "\uf8d5";

      /// <summary>
      /// fa-pickaxe unicode value ("\ue5bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pickaxe
      /// </summary>
      [Description("pickaxe")]
      public const string Pickaxe = "\ue5bf";

      /// <summary>
      /// fa-pickleball unicode value ("\ue435").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pickleball
      /// </summary>
      [Description("pickleball")]
      public const string Pickleball = "\ue435";

      /// <summary>
      /// fa-pie unicode value ("\uf705").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pie
      /// </summary>
      [Description("pie")]
      public const string Pie = "\uf705";

      /// <summary>
      /// fa-pied-piper unicode value ("\uf2ae").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pied-piper
      /// </summary>
      [Description("pied-piper")]
      public const string PiedPiper = "\uf2ae";

      /// <summary>
      /// fa-pied-piper-alt unicode value ("\uf1a8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pied-piper-alt
      /// </summary>
      [Description("pied-piper-alt")]
      public const string PiedPiperAlt = "\uf1a8";

      /// <summary>
      /// fa-pied-piper-hat unicode value ("\uf4e5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.10', Last Modified in '5.0.10'.
      /// <para/>
      /// See https://fontawesome.com/icons/pied-piper-hat
      /// </summary>
      [Description("pied-piper-hat")]
      public const string PiedPiperHat = "\uf4e5";

      /// <summary>
      /// fa-pied-piper-pp unicode value ("\uf1a7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pied-piper-pp
      /// </summary>
      [Description("pied-piper-pp")]
      public const string PiedPiperPp = "\uf1a7";

      /// <summary>
      /// fa-pig unicode value ("\uf706").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pig
      /// </summary>
      [Description("pig")]
      public const string Pig = "\uf706";

      /// <summary>
      /// fa-piggy-bank unicode value ("\uf4d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/piggy-bank
      /// </summary>
      [Description("piggy-bank")]
      public const string PiggyBank = "\uf4d3";

      /// <summary>
      /// fa-pills unicode value ("\uf484").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pills
      /// </summary>
      [Description("pills")]
      public const string Pills = "\uf484";

      /// <summary>
      /// fa-pinata unicode value ("\ue3c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pinata
      /// </summary>
      [Description("pinata")]
      public const string Pinata = "\ue3c3";

      /// <summary>
      /// fa-pinball unicode value ("\ue229").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pinball
      /// </summary>
      [Description("pinball")]
      public const string Pinball = "\ue229";

      /// <summary>
      /// fa-pineapple unicode value ("\ue31f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pineapple
      /// </summary>
      [Description("pineapple")]
      public const string Pineapple = "\ue31f";

      /// <summary>
      /// fa-pinterest unicode value ("\uf0d2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pinterest
      /// </summary>
      [Description("pinterest")]
      public const string Pinterest = "\uf0d2";

      /// <summary>
      /// fa-pinterest-p unicode value ("\uf231").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pinterest-p
      /// </summary>
      [Description("pinterest-p")]
      public const string PinterestP = "\uf231";

      /// <summary>
      /// fa-pipe unicode value ("\u007c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe
      /// </summary>
      [Description("pipe")]
      public const string Pipe = "\u007c";

      /// <summary>
      /// fa-pipe-circle-check unicode value ("\ue436").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe-circle-check
      /// </summary>
      [Description("pipe-circle-check")]
      public const string PipeCircleCheck = "\ue436";

      /// <summary>
      /// fa-pipe-collar unicode value ("\ue437").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe-collar
      /// </summary>
      [Description("pipe-collar")]
      public const string PipeCollar = "\ue437";

      /// <summary>
      /// fa-pipe-section unicode value ("\ue438").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe-section
      /// </summary>
      [Description("pipe-section")]
      public const string PipeSection = "\ue438";

      /// <summary>
      /// fa-pipe-smoking unicode value ("\ue3c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe-smoking
      /// </summary>
      [Description("pipe-smoking")]
      public const string PipeSmoking = "\ue3c4";

      /// <summary>
      /// fa-pipe-valve unicode value ("\ue439").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pipe-valve
      /// </summary>
      [Description("pipe-valve")]
      public const string PipeValve = "\ue439";

      /// <summary>
      /// fa-pix unicode value ("\ue43a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/pix
      /// </summary>
      [Description("pix")]
      public const string Pix = "\ue43a";

      /// <summary>
      /// fa-pizza unicode value ("\uf817").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pizza
      /// </summary>
      [Description("pizza")]
      public const string Pizza = "\uf817";

      /// <summary>
      /// fa-pizza-slice unicode value ("\uf818").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pizza-slice
      /// </summary>
      [Description("pizza-slice")]
      public const string PizzaSlice = "\uf818";

      /// <summary>
      /// fa-place-of-worship unicode value ("\uf67f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/place-of-worship
      /// </summary>
      [Description("place-of-worship")]
      public const string PlaceOfWorship = "\uf67f";

      /// <summary>
      /// fa-plane unicode value ("\uf072").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane
      /// </summary>
      [Description("plane")]
      public const string Plane = "\uf072";

      /// <summary>
      /// fa-plane-arrival unicode value ("\uf5af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-arrival
      /// </summary>
      [Description("plane-arrival")]
      public const string PlaneArrival = "\uf5af";

      /// <summary>
      /// fa-plane-circle-check unicode value ("\ue555").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-circle-check
      /// </summary>
      [Description("plane-circle-check")]
      public const string PlaneCircleCheck = "\ue555";

      /// <summary>
      /// fa-plane-circle-exclamation unicode value ("\ue556").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-circle-exclamation
      /// </summary>
      [Description("plane-circle-exclamation")]
      public const string PlaneCircleExclamation = "\ue556";

      /// <summary>
      /// fa-plane-circle-xmark unicode value ("\ue557").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-circle-xmark
      /// </summary>
      [Description("plane-circle-xmark")]
      public const string PlaneCircleXmark = "\ue557";

      /// <summary>
      /// fa-plane-departure unicode value ("\uf5b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-departure
      /// </summary>
      [Description("plane-departure")]
      public const string PlaneDeparture = "\uf5b0";

      /// <summary>
      /// fa-plane-engines unicode value ("\uf3de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-engines
      /// </summary>
      [Description("plane-engines")]
      public const string PlaneEngines = "\uf3de";

      /// <summary>
      /// fa-plane-lock unicode value ("\ue558").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-lock
      /// </summary>
      [Description("plane-lock")]
      public const string PlaneLock = "\ue558";

      /// <summary>
      /// fa-plane-prop unicode value ("\ue22b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-prop
      /// </summary>
      [Description("plane-prop")]
      public const string PlaneProp = "\ue22b";

      /// <summary>
      /// fa-plane-slash unicode value ("\ue069").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-slash
      /// </summary>
      [Description("plane-slash")]
      public const string PlaneSlash = "\ue069";

      /// <summary>
      /// fa-plane-tail unicode value ("\ue22c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-tail
      /// </summary>
      [Description("plane-tail")]
      public const string PlaneTail = "\ue22c";

      /// <summary>
      /// fa-plane-up unicode value ("\ue22d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-up
      /// </summary>
      [Description("plane-up")]
      public const string PlaneUp = "\ue22d";

      /// <summary>
      /// fa-plane-up-slash unicode value ("\ue22e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plane-up-slash
      /// </summary>
      [Description("plane-up-slash")]
      public const string PlaneUpSlash = "\ue22e";

      /// <summary>
      /// fa-planet-moon unicode value ("\ue01f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/planet-moon
      /// </summary>
      [Description("planet-moon")]
      public const string PlanetMoon = "\ue01f";

      /// <summary>
      /// fa-planet-ringed unicode value ("\ue020").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/planet-ringed
      /// </summary>
      [Description("planet-ringed")]
      public const string PlanetRinged = "\ue020";

      /// <summary>
      /// fa-plant-wilt unicode value ("\ue5aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plant-wilt
      /// </summary>
      [Description("plant-wilt")]
      public const string PlantWilt = "\ue5aa";

      /// <summary>
      /// fa-plate-utensils unicode value ("\ue43b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plate-utensils
      /// </summary>
      [Description("plate-utensils")]
      public const string PlateUtensils = "\ue43b";

      /// <summary>
      /// fa-plate-wheat unicode value ("\ue55a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plate-wheat
      /// </summary>
      [Description("plate-wheat")]
      public const string PlateWheat = "\ue55a";

      /// <summary>
      /// fa-play unicode value ("\uf04b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/play
      /// </summary>
      [Description("play")]
      public const string Play = "\uf04b";

      /// <summary>
      /// fa-play-pause unicode value ("\ue22f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/play-pause
      /// </summary>
      [Description("play-pause")]
      public const string PlayPause = "\ue22f";

      /// <summary>
      /// fa-playstation unicode value ("\uf3df").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/playstation
      /// </summary>
      [Description("playstation")]
      public const string Playstation = "\uf3df";

      /// <summary>
      /// fa-plug unicode value ("\uf1e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug
      /// </summary>
      [Description("plug")]
      public const string Plug = "\uf1e6";

      /// <summary>
      /// fa-plug-circle-bolt unicode value ("\ue55b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-bolt
      /// </summary>
      [Description("plug-circle-bolt")]
      public const string PlugCircleBolt = "\ue55b";

      /// <summary>
      /// fa-plug-circle-check unicode value ("\ue55c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-check
      /// </summary>
      [Description("plug-circle-check")]
      public const string PlugCircleCheck = "\ue55c";

      /// <summary>
      /// fa-plug-circle-exclamation unicode value ("\ue55d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-exclamation
      /// </summary>
      [Description("plug-circle-exclamation")]
      public const string PlugCircleExclamation = "\ue55d";

      /// <summary>
      /// fa-plug-circle-minus unicode value ("\ue55e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-minus
      /// </summary>
      [Description("plug-circle-minus")]
      public const string PlugCircleMinus = "\ue55e";

      /// <summary>
      /// fa-plug-circle-plus unicode value ("\ue55f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-plus
      /// </summary>
      [Description("plug-circle-plus")]
      public const string PlugCirclePlus = "\ue55f";

      /// <summary>
      /// fa-plug-circle-xmark unicode value ("\ue560").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plug-circle-xmark
      /// </summary>
      [Description("plug-circle-xmark")]
      public const string PlugCircleXmark = "\ue560";

      /// <summary>
      /// fa-plus unicode value ("\u002b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plus
      /// </summary>
      [Description("plus")]
      public const string Plus = "\u002b";

      /// <summary>
      /// fa-plus-large unicode value ("\ue59e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plus-large
      /// </summary>
      [Description("plus-large")]
      public const string PlusLarge = "\ue59e";

      /// <summary>
      /// fa-plus-minus unicode value ("\ue43c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/plus-minus
      /// </summary>
      [Description("plus-minus")]
      public const string PlusMinus = "\ue43c";

      /// <summary>
      /// fa-podcast unicode value ("\uf2ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/podcast
      /// </summary>
      [Description("podcast")]
      public const string Podcast = "\uf2ce";

      /// <summary>
      /// fa-podium unicode value ("\uf680").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/podium
      /// </summary>
      [Description("podium")]
      public const string Podium = "\uf680";

      /// <summary>
      /// fa-podium-star unicode value ("\uf758").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/podium-star
      /// </summary>
      [Description("podium-star")]
      public const string PodiumStar = "\uf758";

      /// <summary>
      /// fa-police-box unicode value ("\ue021").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/police-box
      /// </summary>
      [Description("police-box")]
      public const string PoliceBox = "\ue021";

      /// <summary>
      /// fa-poll-people unicode value ("\uf759").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/poll-people
      /// </summary>
      [Description("poll-people")]
      public const string PollPeople = "\uf759";

      /// <summary>
      /// fa-pompebled unicode value ("\ue43d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pompebled
      /// </summary>
      [Description("pompebled")]
      public const string Pompebled = "\ue43d";

      /// <summary>
      /// fa-poo unicode value ("\uf2fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/poo
      /// </summary>
      [Description("poo")]
      public const string Poo = "\uf2fe";

      /// <summary>
      /// fa-poo-storm unicode value ("\uf75a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/poo-storm
      /// </summary>
      [Description("poo-storm")]
      public const string PooStorm = "\uf75a";

      /// <summary>
      /// fa-pool-8-ball unicode value ("\ue3c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pool-8-ball
      /// </summary>
      [Description("pool-8-ball")]
      public const string Pool8Ball = "\ue3c5";

      /// <summary>
      /// fa-poop unicode value ("\uf619").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/poop
      /// </summary>
      [Description("poop")]
      public const string Poop = "\uf619";

      /// <summary>
      /// fa-popcorn unicode value ("\uf819").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/popcorn
      /// </summary>
      [Description("popcorn")]
      public const string Popcorn = "\uf819";

      /// <summary>
      /// fa-popsicle unicode value ("\ue43e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/popsicle
      /// </summary>
      [Description("popsicle")]
      public const string Popsicle = "\ue43e";

      /// <summary>
      /// fa-pot-food unicode value ("\ue43f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pot-food
      /// </summary>
      [Description("pot-food")]
      public const string PotFood = "\ue43f";

      /// <summary>
      /// fa-potato unicode value ("\ue440").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/potato
      /// </summary>
      [Description("potato")]
      public const string Potato = "\ue440";

      /// <summary>
      /// fa-power-off unicode value ("\uf011").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/power-off
      /// </summary>
      [Description("power-off")]
      public const string PowerOff = "\uf011";

      /// <summary>
      /// fa-prescription unicode value ("\uf5b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/prescription
      /// </summary>
      [Description("prescription")]
      public const string Prescription = "\uf5b1";

      /// <summary>
      /// fa-prescription-bottle unicode value ("\uf485").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/prescription-bottle
      /// </summary>
      [Description("prescription-bottle")]
      public const string PrescriptionBottle = "\uf485";

      /// <summary>
      /// fa-prescription-bottle-medical unicode value ("\uf486").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/prescription-bottle-medical
      /// </summary>
      [Description("prescription-bottle-medical")]
      public const string PrescriptionBottleMedical = "\uf486";

      /// <summary>
      /// fa-prescription-bottle-pill unicode value ("\ue5c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/prescription-bottle-pill
      /// </summary>
      [Description("prescription-bottle-pill")]
      public const string PrescriptionBottlePill = "\ue5c0";

      /// <summary>
      /// fa-presentation-screen unicode value ("\uf685").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/presentation-screen
      /// </summary>
      [Description("presentation-screen")]
      public const string PresentationScreen = "\uf685";

      /// <summary>
      /// fa-pretzel unicode value ("\ue441").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pretzel
      /// </summary>
      [Description("pretzel")]
      public const string Pretzel = "\ue441";

      /// <summary>
      /// fa-print unicode value ("\uf02f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/print
      /// </summary>
      [Description("print")]
      public const string Print = "\uf02f";

      /// <summary>
      /// fa-print-magnifying-glass unicode value ("\uf81a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/print-magnifying-glass
      /// </summary>
      [Description("print-magnifying-glass")]
      public const string PrintMagnifyingGlass = "\uf81a";

      /// <summary>
      /// fa-print-slash unicode value ("\uf686").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/print-slash
      /// </summary>
      [Description("print-slash")]
      public const string PrintSlash = "\uf686";

      /// <summary>
      /// fa-product-hunt unicode value ("\uf288").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/product-hunt
      /// </summary>
      [Description("product-hunt")]
      public const string ProductHunt = "\uf288";

      /// <summary>
      /// fa-projector unicode value ("\uf8d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/projector
      /// </summary>
      [Description("projector")]
      public const string Projector = "\uf8d6";

      /// <summary>
      /// fa-pump unicode value ("\ue442").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pump
      /// </summary>
      [Description("pump")]
      public const string Pump = "\ue442";

      /// <summary>
      /// fa-pump-medical unicode value ("\ue06a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pump-medical
      /// </summary>
      [Description("pump-medical")]
      public const string PumpMedical = "\ue06a";

      /// <summary>
      /// fa-pump-soap unicode value ("\ue06b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pump-soap
      /// </summary>
      [Description("pump-soap")]
      public const string PumpSoap = "\ue06b";

      /// <summary>
      /// fa-pumpkin unicode value ("\uf707").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pumpkin
      /// </summary>
      [Description("pumpkin")]
      public const string Pumpkin = "\uf707";

      /// <summary>
      /// fa-pushed unicode value ("\uf3e1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/pushed
      /// </summary>
      [Description("pushed")]
      public const string Pushed = "\uf3e1";

      /// <summary>
      /// fa-puzzle unicode value ("\ue443").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/puzzle
      /// </summary>
      [Description("puzzle")]
      public const string Puzzle = "\ue443";

      /// <summary>
      /// fa-puzzle-piece unicode value ("\uf12e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/puzzle-piece
      /// </summary>
      [Description("puzzle-piece")]
      public const string PuzzlePiece = "\uf12e";

      /// <summary>
      /// fa-puzzle-piece-simple unicode value ("\ue231").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/puzzle-piece-simple
      /// </summary>
      [Description("puzzle-piece-simple")]
      public const string PuzzlePieceSimple = "\ue231";

      /// <summary>
      /// fa-python unicode value ("\uf3e2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/python
      /// </summary>
      [Description("python")]
      public const string Python = "\uf3e2";

      /// <summary>
      /// fa-q unicode value ("\u0051").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/q
      /// </summary>
      [Description("q")]
      public const string Q = "\u0051";

      /// <summary>
      /// fa-qq unicode value ("\uf1d6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/qq
      /// </summary>
      [Description("qq")]
      public const string Qq = "\uf1d6";

      /// <summary>
      /// fa-qrcode unicode value ("\uf029").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/qrcode
      /// </summary>
      [Description("qrcode")]
      public const string Qrcode = "\uf029";

      /// <summary>
      /// fa-question unicode value ("\u003f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/question
      /// </summary>
      [Description("question")]
      public const string Question = "\u003f";

      /// <summary>
      /// fa-quinscape unicode value ("\uf459").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/quinscape
      /// </summary>
      [Description("quinscape")]
      public const string Quinscape = "\uf459";

      /// <summary>
      /// fa-quora unicode value ("\uf2c4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/quora
      /// </summary>
      [Description("quora")]
      public const string Quora = "\uf2c4";

      /// <summary>
      /// fa-quote-left unicode value ("\uf10d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/quote-left
      /// </summary>
      [Description("quote-left")]
      public const string QuoteLeft = "\uf10d";

      /// <summary>
      /// fa-quote-right unicode value ("\uf10e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/quote-right
      /// </summary>
      [Description("quote-right")]
      public const string QuoteRight = "\uf10e";

      /// <summary>
      /// fa-quotes unicode value ("\ue234").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/quotes
      /// </summary>
      [Description("quotes")]
      public const string Quotes = "\ue234";

      /// <summary>
      /// fa-r unicode value ("\u0052").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/r
      /// </summary>
      [Description("r")]
      public const string R = "\u0052";

      /// <summary>
      /// fa-r-project unicode value ("\uf4f7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/r-project
      /// </summary>
      [Description("r-project")]
      public const string RProject = "\uf4f7";

      /// <summary>
      /// fa-rabbit unicode value ("\uf708").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rabbit
      /// </summary>
      [Description("rabbit")]
      public const string Rabbit = "\uf708";

      /// <summary>
      /// fa-rabbit-running unicode value ("\uf709").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rabbit-running
      /// </summary>
      [Description("rabbit-running")]
      public const string RabbitRunning = "\uf709";

      /// <summary>
      /// fa-racquet unicode value ("\uf45a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/racquet
      /// </summary>
      [Description("racquet")]
      public const string Racquet = "\uf45a";

      /// <summary>
      /// fa-radar unicode value ("\ue024").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/radar
      /// </summary>
      [Description("radar")]
      public const string Radar = "\ue024";

      /// <summary>
      /// fa-radiation unicode value ("\uf7b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/radiation
      /// </summary>
      [Description("radiation")]
      public const string Radiation = "\uf7b9";

      /// <summary>
      /// fa-radio unicode value ("\uf8d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/radio
      /// </summary>
      [Description("radio")]
      public const string Radio = "\uf8d7";

      /// <summary>
      /// fa-radio-tuner unicode value ("\uf8d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/radio-tuner
      /// </summary>
      [Description("radio-tuner")]
      public const string RadioTuner = "\uf8d8";

      /// <summary>
      /// fa-rainbow unicode value ("\uf75b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rainbow
      /// </summary>
      [Description("rainbow")]
      public const string Rainbow = "\uf75b";

      /// <summary>
      /// fa-raindrops unicode value ("\uf75c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/raindrops
      /// </summary>
      [Description("raindrops")]
      public const string Raindrops = "\uf75c";

      /// <summary>
      /// fa-ram unicode value ("\uf70a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ram
      /// </summary>
      [Description("ram")]
      public const string Ram = "\uf70a";

      /// <summary>
      /// fa-ramp-loading unicode value ("\uf4d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ramp-loading
      /// </summary>
      [Description("ramp-loading")]
      public const string RampLoading = "\uf4d4";

      /// <summary>
      /// fa-ranking-star unicode value ("\ue561").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ranking-star
      /// </summary>
      [Description("ranking-star")]
      public const string RankingStar = "\ue561";

      /// <summary>
      /// fa-raspberry-pi unicode value ("\uf7bb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/raspberry-pi
      /// </summary>
      [Description("raspberry-pi")]
      public const string RaspberryPi = "\uf7bb";

      /// <summary>
      /// fa-ravelry unicode value ("\uf2d9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.15.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/ravelry
      /// </summary>
      [Description("ravelry")]
      public const string Ravelry = "\uf2d9";

      /// <summary>
      /// fa-raygun unicode value ("\ue025").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/raygun
      /// </summary>
      [Description("raygun")]
      public const string Raygun = "\ue025";

      /// <summary>
      /// fa-react unicode value ("\uf41b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/react
      /// </summary>
      [Description("react")]
      public const string React = "\uf41b";

      /// <summary>
      /// fa-reacteurope unicode value ("\uf75d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reacteurope
      /// </summary>
      [Description("reacteurope")]
      public const string Reacteurope = "\uf75d";

      /// <summary>
      /// fa-readme unicode value ("\uf4d5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '5.0.10'.
      /// <para/>
      /// See https://fontawesome.com/icons/readme
      /// </summary>
      [Description("readme")]
      public const string Readme = "\uf4d5";

      /// <summary>
      /// fa-rebel unicode value ("\uf1d0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rebel
      /// </summary>
      [Description("rebel")]
      public const string Rebel = "\uf1d0";

      /// <summary>
      /// fa-receipt unicode value ("\uf543").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/receipt
      /// </summary>
      [Description("receipt")]
      public const string Receipt = "\uf543";

      /// <summary>
      /// fa-record-vinyl unicode value ("\uf8d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/record-vinyl
      /// </summary>
      [Description("record-vinyl")]
      public const string RecordVinyl = "\uf8d9";

      /// <summary>
      /// fa-rectangle unicode value ("\uf2fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle
      /// </summary>
      [Description("rectangle")]
      public const string Rectangle = "\uf2fa";

      /// <summary>
      /// fa-rectangle-ad unicode value ("\uf641").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-ad
      /// </summary>
      [Description("rectangle-ad")]
      public const string RectangleAd = "\uf641";

      /// <summary>
      /// fa-rectangle-barcode unicode value ("\uf463").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-barcode
      /// </summary>
      [Description("rectangle-barcode")]
      public const string RectangleBarcode = "\uf463";

      /// <summary>
      /// fa-rectangle-code unicode value ("\ue322").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-code
      /// </summary>
      [Description("rectangle-code")]
      public const string RectangleCode = "\ue322";

      /// <summary>
      /// fa-rectangle-history unicode value ("\ue4a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-history
      /// </summary>
      [Description("rectangle-history")]
      public const string RectangleHistory = "\ue4a2";

      /// <summary>
      /// fa-rectangle-history-circle-plus unicode value ("\ue4a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-history-circle-plus
      /// </summary>
      [Description("rectangle-history-circle-plus")]
      public const string RectangleHistoryCirclePlus = "\ue4a3";

      /// <summary>
      /// fa-rectangle-history-circle-user unicode value ("\ue4a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-history-circle-user
      /// </summary>
      [Description("rectangle-history-circle-user")]
      public const string RectangleHistoryCircleUser = "\ue4a4";

      /// <summary>
      /// fa-rectangle-list unicode value ("\uf022").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-list
      /// </summary>
      [Description("rectangle-list")]
      public const string RectangleList = "\uf022";

      /// <summary>
      /// fa-rectangle-pro unicode value ("\ue235").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-pro
      /// </summary>
      [Description("rectangle-pro")]
      public const string RectanglePro = "\ue235";

      /// <summary>
      /// fa-rectangle-terminal unicode value ("\ue236").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-terminal
      /// </summary>
      [Description("rectangle-terminal")]
      public const string RectangleTerminal = "\ue236";

      /// <summary>
      /// fa-rectangle-vertical unicode value ("\uf2fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-vertical
      /// </summary>
      [Description("rectangle-vertical")]
      public const string RectangleVertical = "\uf2fb";

      /// <summary>
      /// fa-rectangle-vertical-history unicode value ("\ue237").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-vertical-history
      /// </summary>
      [Description("rectangle-vertical-history")]
      public const string RectangleVerticalHistory = "\ue237";

      /// <summary>
      /// fa-rectangle-wide unicode value ("\uf2fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-wide
      /// </summary>
      [Description("rectangle-wide")]
      public const string RectangleWide = "\uf2fc";

      /// <summary>
      /// fa-rectangle-xmark unicode value ("\uf410").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangle-xmark
      /// </summary>
      [Description("rectangle-xmark")]
      public const string RectangleXmark = "\uf410";

      /// <summary>
      /// fa-rectangles-mixed unicode value ("\ue323").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rectangles-mixed
      /// </summary>
      [Description("rectangles-mixed")]
      public const string RectanglesMixed = "\ue323";

      /// <summary>
      /// fa-recycle unicode value ("\uf1b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/recycle
      /// </summary>
      [Description("recycle")]
      public const string Recycle = "\uf1b8";

      /// <summary>
      /// fa-red-river unicode value ("\uf3e3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/red-river
      /// </summary>
      [Description("red-river")]
      public const string RedRiver = "\uf3e3";

      /// <summary>
      /// fa-reddit unicode value ("\uf1a1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reddit
      /// </summary>
      [Description("reddit")]
      public const string Reddit = "\uf1a1";

      /// <summary>
      /// fa-reddit-alien unicode value ("\uf281").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reddit-alien
      /// </summary>
      [Description("reddit-alien")]
      public const string RedditAlien = "\uf281";

      /// <summary>
      /// fa-redhat unicode value ("\uf7bc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/redhat
      /// </summary>
      [Description("redhat")]
      public const string Redhat = "\uf7bc";

      /// <summary>
      /// fa-reel unicode value ("\ue238").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reel
      /// </summary>
      [Description("reel")]
      public const string Reel = "\ue238";

      /// <summary>
      /// fa-refrigerator unicode value ("\ue026").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/refrigerator
      /// </summary>
      [Description("refrigerator")]
      public const string Refrigerator = "\ue026";

      /// <summary>
      /// fa-registered unicode value ("\uf25d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/registered
      /// </summary>
      [Description("registered")]
      public const string Registered = "\uf25d";

      /// <summary>
      /// fa-renren unicode value ("\uf18b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/renren
      /// </summary>
      [Description("renren")]
      public const string Renren = "\uf18b";

      /// <summary>
      /// fa-repeat unicode value ("\uf363").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/repeat
      /// </summary>
      [Description("repeat")]
      public const string Repeat = "\uf363";

      /// <summary>
      /// fa-repeat-1 unicode value ("\uf365").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/repeat-1
      /// </summary>
      [Description("repeat-1")]
      public const string Repeat1 = "\uf365";

      /// <summary>
      /// fa-reply unicode value ("\uf3e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reply
      /// </summary>
      [Description("reply")]
      public const string Reply = "\uf3e5";

      /// <summary>
      /// fa-reply-all unicode value ("\uf122").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reply-all
      /// </summary>
      [Description("reply-all")]
      public const string ReplyAll = "\uf122";

      /// <summary>
      /// fa-reply-clock unicode value ("\ue239").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/reply-clock
      /// </summary>
      [Description("reply-clock")]
      public const string ReplyClock = "\ue239";

      /// <summary>
      /// fa-replyd unicode value ("\uf3e6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/replyd
      /// </summary>
      [Description("replyd")]
      public const string Replyd = "\uf3e6";

      /// <summary>
      /// fa-republican unicode value ("\uf75e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/republican
      /// </summary>
      [Description("republican")]
      public const string Republican = "\uf75e";

      /// <summary>
      /// fa-researchgate unicode value ("\uf4f8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '5.0.11'.
      /// <para/>
      /// See https://fontawesome.com/icons/researchgate
      /// </summary>
      [Description("researchgate")]
      public const string Researchgate = "\uf4f8";

      /// <summary>
      /// fa-resolving unicode value ("\uf3e7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/resolving
      /// </summary>
      [Description("resolving")]
      public const string Resolving = "\uf3e7";

      /// <summary>
      /// fa-restroom unicode value ("\uf7bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/restroom
      /// </summary>
      [Description("restroom")]
      public const string Restroom = "\uf7bd";

      /// <summary>
      /// fa-restroom-simple unicode value ("\ue23a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/restroom-simple
      /// </summary>
      [Description("restroom-simple")]
      public const string RestroomSimple = "\ue23a";

      /// <summary>
      /// fa-retweet unicode value ("\uf079").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/retweet
      /// </summary>
      [Description("retweet")]
      public const string Retweet = "\uf079";

      /// <summary>
      /// fa-rev unicode value ("\uf5b2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rev
      /// </summary>
      [Description("rev")]
      public const string Rev = "\uf5b2";

      /// <summary>
      /// fa-rhombus unicode value ("\ue23b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rhombus
      /// </summary>
      [Description("rhombus")]
      public const string Rhombus = "\ue23b";

      /// <summary>
      /// fa-ribbon unicode value ("\uf4d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ribbon
      /// </summary>
      [Description("ribbon")]
      public const string Ribbon = "\uf4d6";

      /// <summary>
      /// fa-right unicode value ("\uf356").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right
      /// </summary>
      [Description("right")]
      public const string Right = "\uf356";

      /// <summary>
      /// fa-right-from-bracket unicode value ("\uf2f5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-from-bracket
      /// </summary>
      [Description("right-from-bracket")]
      public const string RightFromBracket = "\uf2f5";

      /// <summary>
      /// fa-right-from-line unicode value ("\uf347").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-from-line
      /// </summary>
      [Description("right-from-line")]
      public const string RightFromLine = "\uf347";

      /// <summary>
      /// fa-right-left unicode value ("\uf362").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-left
      /// </summary>
      [Description("right-left")]
      public const string RightLeft = "\uf362";

      /// <summary>
      /// fa-right-left-large unicode value ("\ue5e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-left-large
      /// </summary>
      [Description("right-left-large")]
      public const string RightLeftLarge = "\ue5e1";

      /// <summary>
      /// fa-right-long unicode value ("\uf30b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-long
      /// </summary>
      [Description("right-long")]
      public const string RightLong = "\uf30b";

      /// <summary>
      /// fa-right-long-to-line unicode value ("\ue444").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-long-to-line
      /// </summary>
      [Description("right-long-to-line")]
      public const string RightLongToLine = "\ue444";

      /// <summary>
      /// fa-right-to-bracket unicode value ("\uf2f6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-to-bracket
      /// </summary>
      [Description("right-to-bracket")]
      public const string RightToBracket = "\uf2f6";

      /// <summary>
      /// fa-right-to-line unicode value ("\uf34c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/right-to-line
      /// </summary>
      [Description("right-to-line")]
      public const string RightToLine = "\uf34c";

      /// <summary>
      /// fa-ring unicode value ("\uf70b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ring
      /// </summary>
      [Description("ring")]
      public const string Ring = "\uf70b";

      /// <summary>
      /// fa-ring-diamond unicode value ("\ue5ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ring-diamond
      /// </summary>
      [Description("ring-diamond")]
      public const string RingDiamond = "\ue5ab";

      /// <summary>
      /// fa-rings-wedding unicode value ("\uf81b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rings-wedding
      /// </summary>
      [Description("rings-wedding")]
      public const string RingsWedding = "\uf81b";

      /// <summary>
      /// fa-road unicode value ("\uf018").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road
      /// </summary>
      [Description("road")]
      public const string Road = "\uf018";

      /// <summary>
      /// fa-road-barrier unicode value ("\ue562").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-barrier
      /// </summary>
      [Description("road-barrier")]
      public const string RoadBarrier = "\ue562";

      /// <summary>
      /// fa-road-bridge unicode value ("\ue563").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-bridge
      /// </summary>
      [Description("road-bridge")]
      public const string RoadBridge = "\ue563";

      /// <summary>
      /// fa-road-circle-check unicode value ("\ue564").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-circle-check
      /// </summary>
      [Description("road-circle-check")]
      public const string RoadCircleCheck = "\ue564";

      /// <summary>
      /// fa-road-circle-exclamation unicode value ("\ue565").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-circle-exclamation
      /// </summary>
      [Description("road-circle-exclamation")]
      public const string RoadCircleExclamation = "\ue565";

      /// <summary>
      /// fa-road-circle-xmark unicode value ("\ue566").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-circle-xmark
      /// </summary>
      [Description("road-circle-xmark")]
      public const string RoadCircleXmark = "\ue566";

      /// <summary>
      /// fa-road-lock unicode value ("\ue567").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-lock
      /// </summary>
      [Description("road-lock")]
      public const string RoadLock = "\ue567";

      /// <summary>
      /// fa-road-spikes unicode value ("\ue568").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/road-spikes
      /// </summary>
      [Description("road-spikes")]
      public const string RoadSpikes = "\ue568";

      /// <summary>
      /// fa-robot unicode value ("\uf544").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/robot
      /// </summary>
      [Description("robot")]
      public const string Robot = "\uf544";

      /// <summary>
      /// fa-robot-astromech unicode value ("\ue2d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/robot-astromech
      /// </summary>
      [Description("robot-astromech")]
      public const string RobotAstromech = "\ue2d2";

      /// <summary>
      /// fa-rocket unicode value ("\uf135").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rocket
      /// </summary>
      [Description("rocket")]
      public const string Rocket = "\uf135";

      /// <summary>
      /// fa-rocket-launch unicode value ("\ue027").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rocket-launch
      /// </summary>
      [Description("rocket-launch")]
      public const string RocketLaunch = "\ue027";

      /// <summary>
      /// fa-rocketchat unicode value ("\uf3e8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rocketchat
      /// </summary>
      [Description("rocketchat")]
      public const string Rocketchat = "\uf3e8";

      /// <summary>
      /// fa-rockrms unicode value ("\uf3e9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rockrms
      /// </summary>
      [Description("rockrms")]
      public const string Rockrms = "\uf3e9";

      /// <summary>
      /// fa-roller-coaster unicode value ("\ue324").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/roller-coaster
      /// </summary>
      [Description("roller-coaster")]
      public const string RollerCoaster = "\ue324";

      /// <summary>
      /// fa-rotate unicode value ("\uf2f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rotate
      /// </summary>
      [Description("rotate")]
      public const string Rotate = "\uf2f1";

      /// <summary>
      /// fa-rotate-exclamation unicode value ("\ue23c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rotate-exclamation
      /// </summary>
      [Description("rotate-exclamation")]
      public const string RotateExclamation = "\ue23c";

      /// <summary>
      /// fa-rotate-left unicode value ("\uf2ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rotate-left
      /// </summary>
      [Description("rotate-left")]
      public const string RotateLeft = "\uf2ea";

      /// <summary>
      /// fa-rotate-right unicode value ("\uf2f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rotate-right
      /// </summary>
      [Description("rotate-right")]
      public const string RotateRight = "\uf2f9";

      /// <summary>
      /// fa-route unicode value ("\uf4d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/route
      /// </summary>
      [Description("route")]
      public const string Route = "\uf4d7";

      /// <summary>
      /// fa-route-highway unicode value ("\uf61a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/route-highway
      /// </summary>
      [Description("route-highway")]
      public const string RouteHighway = "\uf61a";

      /// <summary>
      /// fa-route-interstate unicode value ("\uf61b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/route-interstate
      /// </summary>
      [Description("route-interstate")]
      public const string RouteInterstate = "\uf61b";

      /// <summary>
      /// fa-router unicode value ("\uf8da").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/router
      /// </summary>
      [Description("router")]
      public const string Router = "\uf8da";

      /// <summary>
      /// fa-rss unicode value ("\uf09e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rss
      /// </summary>
      [Description("rss")]
      public const string Rss = "\uf09e";

      /// <summary>
      /// fa-ruble-sign unicode value ("\uf158").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruble-sign
      /// </summary>
      [Description("ruble-sign")]
      public const string RubleSign = "\uf158";

      /// <summary>
      /// fa-rug unicode value ("\ue569").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rug
      /// </summary>
      [Description("rug")]
      public const string Rug = "\ue569";

      /// <summary>
      /// fa-rugby-ball unicode value ("\ue3c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rugby-ball
      /// </summary>
      [Description("rugby-ball")]
      public const string RugbyBall = "\ue3c6";

      /// <summary>
      /// fa-ruler unicode value ("\uf545").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruler
      /// </summary>
      [Description("ruler")]
      public const string Ruler = "\uf545";

      /// <summary>
      /// fa-ruler-combined unicode value ("\uf546").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruler-combined
      /// </summary>
      [Description("ruler-combined")]
      public const string RulerCombined = "\uf546";

      /// <summary>
      /// fa-ruler-horizontal unicode value ("\uf547").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruler-horizontal
      /// </summary>
      [Description("ruler-horizontal")]
      public const string RulerHorizontal = "\uf547";

      /// <summary>
      /// fa-ruler-triangle unicode value ("\uf61c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruler-triangle
      /// </summary>
      [Description("ruler-triangle")]
      public const string RulerTriangle = "\uf61c";

      /// <summary>
      /// fa-ruler-vertical unicode value ("\uf548").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ruler-vertical
      /// </summary>
      [Description("ruler-vertical")]
      public const string RulerVertical = "\uf548";

      /// <summary>
      /// fa-rupee-sign unicode value ("\uf156").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rupee-sign
      /// </summary>
      [Description("rupee-sign")]
      public const string RupeeSign = "\uf156";

      /// <summary>
      /// fa-rupiah-sign unicode value ("\ue23d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rupiah-sign
      /// </summary>
      [Description("rupiah-sign")]
      public const string RupiahSign = "\ue23d";

      /// <summary>
      /// fa-rust unicode value ("\ue07a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rust
      /// </summary>
      [Description("rust")]
      public const string Rust = "\ue07a";

      /// <summary>
      /// fa-rv unicode value ("\uf7be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/rv
      /// </summary>
      [Description("rv")]
      public const string Rv = "\uf7be";

      /// <summary>
      /// fa-s unicode value ("\u0053").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/s
      /// </summary>
      [Description("s")]
      public const string S = "\u0053";

      /// <summary>
      /// fa-sack unicode value ("\uf81c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sack
      /// </summary>
      [Description("sack")]
      public const string Sack = "\uf81c";

      /// <summary>
      /// fa-sack-dollar unicode value ("\uf81d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sack-dollar
      /// </summary>
      [Description("sack-dollar")]
      public const string SackDollar = "\uf81d";

      /// <summary>
      /// fa-sack-xmark unicode value ("\ue56a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sack-xmark
      /// </summary>
      [Description("sack-xmark")]
      public const string SackXmark = "\ue56a";

      /// <summary>
      /// fa-safari unicode value ("\uf267").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/safari
      /// </summary>
      [Description("safari")]
      public const string Safari = "\uf267";

      /// <summary>
      /// fa-sailboat unicode value ("\ue445").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sailboat
      /// </summary>
      [Description("sailboat")]
      public const string Sailboat = "\ue445";

      /// <summary>
      /// fa-salad unicode value ("\uf81e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/salad
      /// </summary>
      [Description("salad")]
      public const string Salad = "\uf81e";

      /// <summary>
      /// fa-salesforce unicode value ("\uf83b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/salesforce
      /// </summary>
      [Description("salesforce")]
      public const string Salesforce = "\uf83b";

      /// <summary>
      /// fa-salt-shaker unicode value ("\ue446").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/salt-shaker
      /// </summary>
      [Description("salt-shaker")]
      public const string SaltShaker = "\ue446";

      /// <summary>
      /// fa-sandwich unicode value ("\uf81f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sandwich
      /// </summary>
      [Description("sandwich")]
      public const string Sandwich = "\uf81f";

      /// <summary>
      /// fa-sass unicode value ("\uf41e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sass
      /// </summary>
      [Description("sass")]
      public const string Sass = "\uf41e";

      /// <summary>
      /// fa-satellite unicode value ("\uf7bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/satellite
      /// </summary>
      [Description("satellite")]
      public const string Satellite = "\uf7bf";

      /// <summary>
      /// fa-satellite-dish unicode value ("\uf7c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/satellite-dish
      /// </summary>
      [Description("satellite-dish")]
      public const string SatelliteDish = "\uf7c0";

      /// <summary>
      /// fa-sausage unicode value ("\uf820").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sausage
      /// </summary>
      [Description("sausage")]
      public const string Sausage = "\uf820";

      /// <summary>
      /// fa-saxophone unicode value ("\uf8dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/saxophone
      /// </summary>
      [Description("saxophone")]
      public const string Saxophone = "\uf8dc";

      /// <summary>
      /// fa-saxophone-fire unicode value ("\uf8db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/saxophone-fire
      /// </summary>
      [Description("saxophone-fire")]
      public const string SaxophoneFire = "\uf8db";

      /// <summary>
      /// fa-scale-balanced unicode value ("\uf24e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scale-balanced
      /// </summary>
      [Description("scale-balanced")]
      public const string ScaleBalanced = "\uf24e";

      /// <summary>
      /// fa-scale-unbalanced unicode value ("\uf515").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scale-unbalanced
      /// </summary>
      [Description("scale-unbalanced")]
      public const string ScaleUnbalanced = "\uf515";

      /// <summary>
      /// fa-scale-unbalanced-flip unicode value ("\uf516").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scale-unbalanced-flip
      /// </summary>
      [Description("scale-unbalanced-flip")]
      public const string ScaleUnbalancedFlip = "\uf516";

      /// <summary>
      /// fa-scalpel unicode value ("\uf61d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scalpel
      /// </summary>
      [Description("scalpel")]
      public const string Scalpel = "\uf61d";

      /// <summary>
      /// fa-scalpel-line-dashed unicode value ("\uf61e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scalpel-line-dashed
      /// </summary>
      [Description("scalpel-line-dashed")]
      public const string ScalpelLineDashed = "\uf61e";

      /// <summary>
      /// fa-scanner-gun unicode value ("\uf488").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scanner-gun
      /// </summary>
      [Description("scanner-gun")]
      public const string ScannerGun = "\uf488";

      /// <summary>
      /// fa-scanner-image unicode value ("\uf8f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scanner-image
      /// </summary>
      [Description("scanner-image")]
      public const string ScannerImage = "\uf8f3";

      /// <summary>
      /// fa-scanner-keyboard unicode value ("\uf489").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scanner-keyboard
      /// </summary>
      [Description("scanner-keyboard")]
      public const string ScannerKeyboard = "\uf489";

      /// <summary>
      /// fa-scanner-touchscreen unicode value ("\uf48a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scanner-touchscreen
      /// </summary>
      [Description("scanner-touchscreen")]
      public const string ScannerTouchscreen = "\uf48a";

      /// <summary>
      /// fa-scarecrow unicode value ("\uf70d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scarecrow
      /// </summary>
      [Description("scarecrow")]
      public const string Scarecrow = "\uf70d";

      /// <summary>
      /// fa-scarf unicode value ("\uf7c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scarf
      /// </summary>
      [Description("scarf")]
      public const string Scarf = "\uf7c1";

      /// <summary>
      /// fa-schlix unicode value ("\uf3ea").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/schlix
      /// </summary>
      [Description("schlix")]
      public const string Schlix = "\uf3ea";

      /// <summary>
      /// fa-school unicode value ("\uf549").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school
      /// </summary>
      [Description("school")]
      public const string School = "\uf549";

      /// <summary>
      /// fa-school-circle-check unicode value ("\ue56b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school-circle-check
      /// </summary>
      [Description("school-circle-check")]
      public const string SchoolCircleCheck = "\ue56b";

      /// <summary>
      /// fa-school-circle-exclamation unicode value ("\ue56c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school-circle-exclamation
      /// </summary>
      [Description("school-circle-exclamation")]
      public const string SchoolCircleExclamation = "\ue56c";

      /// <summary>
      /// fa-school-circle-xmark unicode value ("\ue56d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school-circle-xmark
      /// </summary>
      [Description("school-circle-xmark")]
      public const string SchoolCircleXmark = "\ue56d";

      /// <summary>
      /// fa-school-flag unicode value ("\ue56e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school-flag
      /// </summary>
      [Description("school-flag")]
      public const string SchoolFlag = "\ue56e";

      /// <summary>
      /// fa-school-lock unicode value ("\ue56f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/school-lock
      /// </summary>
      [Description("school-lock")]
      public const string SchoolLock = "\ue56f";

      /// <summary>
      /// fa-scissors unicode value ("\uf0c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scissors
      /// </summary>
      [Description("scissors")]
      public const string Scissors = "\uf0c4";

      /// <summary>
      /// fa-screen-users unicode value ("\uf63d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/screen-users
      /// </summary>
      [Description("screen-users")]
      public const string ScreenUsers = "\uf63d";

      /// <summary>
      /// fa-screencast unicode value ("\ue23e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/screencast
      /// </summary>
      [Description("screencast")]
      public const string Screencast = "\ue23e";

      /// <summary>
      /// fa-screenpal unicode value ("\ue570").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/screenpal
      /// </summary>
      [Description("screenpal")]
      public const string Screenpal = "\ue570";

      /// <summary>
      /// fa-screwdriver unicode value ("\uf54a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/screwdriver
      /// </summary>
      [Description("screwdriver")]
      public const string Screwdriver = "\uf54a";

      /// <summary>
      /// fa-screwdriver-wrench unicode value ("\uf7d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/screwdriver-wrench
      /// </summary>
      [Description("screwdriver-wrench")]
      public const string ScrewdriverWrench = "\uf7d9";

      /// <summary>
      /// fa-scribble unicode value ("\ue23f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scribble
      /// </summary>
      [Description("scribble")]
      public const string Scribble = "\ue23f";

      /// <summary>
      /// fa-scribd unicode value ("\uf28a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scribd
      /// </summary>
      [Description("scribd")]
      public const string Scribd = "\uf28a";

      /// <summary>
      /// fa-scroll unicode value ("\uf70e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scroll
      /// </summary>
      [Description("scroll")]
      public const string Scroll = "\uf70e";

      /// <summary>
      /// fa-scroll-old unicode value ("\uf70f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scroll-old
      /// </summary>
      [Description("scroll-old")]
      public const string ScrollOld = "\uf70f";

      /// <summary>
      /// fa-scroll-torah unicode value ("\uf6a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scroll-torah
      /// </summary>
      [Description("scroll-torah")]
      public const string ScrollTorah = "\uf6a0";

      /// <summary>
      /// fa-scrubber unicode value ("\uf2f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scrubber
      /// </summary>
      [Description("scrubber")]
      public const string Scrubber = "\uf2f8";

      /// <summary>
      /// fa-scythe unicode value ("\uf710").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/scythe
      /// </summary>
      [Description("scythe")]
      public const string Scythe = "\uf710";

      /// <summary>
      /// fa-sd-card unicode value ("\uf7c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sd-card
      /// </summary>
      [Description("sd-card")]
      public const string SdCard = "\uf7c2";

      /// <summary>
      /// fa-sd-cards unicode value ("\ue240").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sd-cards
      /// </summary>
      [Description("sd-cards")]
      public const string SdCards = "\ue240";

      /// <summary>
      /// fa-seal unicode value ("\ue241").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/seal
      /// </summary>
      [Description("seal")]
      public const string Seal = "\ue241";

      /// <summary>
      /// fa-seal-exclamation unicode value ("\ue242").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/seal-exclamation
      /// </summary>
      [Description("seal-exclamation")]
      public const string SealExclamation = "\ue242";

      /// <summary>
      /// fa-seal-question unicode value ("\ue243").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/seal-question
      /// </summary>
      [Description("seal-question")]
      public const string SealQuestion = "\ue243";

      /// <summary>
      /// fa-searchengin unicode value ("\uf3eb").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/searchengin
      /// </summary>
      [Description("searchengin")]
      public const string Searchengin = "\uf3eb";

      /// <summary>
      /// fa-seat-airline unicode value ("\ue244").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/seat-airline
      /// </summary>
      [Description("seat-airline")]
      public const string SeatAirline = "\ue244";

      /// <summary>
      /// fa-section unicode value ("\ue447").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/section
      /// </summary>
      [Description("section")]
      public const string Section = "\ue447";

      /// <summary>
      /// fa-seedling unicode value ("\uf4d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/seedling
      /// </summary>
      [Description("seedling")]
      public const string Seedling = "\uf4d8";

      /// <summary>
      /// fa-sellcast unicode value ("\uf2da").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sellcast
      /// </summary>
      [Description("sellcast")]
      public const string Sellcast = "\uf2da";

      /// <summary>
      /// fa-sellsy unicode value ("\uf213").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sellsy
      /// </summary>
      [Description("sellsy")]
      public const string Sellsy = "\uf213";

      /// <summary>
      /// fa-semicolon unicode value ("\u003b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/semicolon
      /// </summary>
      [Description("semicolon")]
      public const string Semicolon = "\u003b";

      /// <summary>
      /// fa-send-back unicode value ("\uf87e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/send-back
      /// </summary>
      [Description("send-back")]
      public const string SendBack = "\uf87e";

      /// <summary>
      /// fa-send-backward unicode value ("\uf87f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/send-backward
      /// </summary>
      [Description("send-backward")]
      public const string SendBackward = "\uf87f";

      /// <summary>
      /// fa-sensor unicode value ("\ue028").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sensor
      /// </summary>
      [Description("sensor")]
      public const string Sensor = "\ue028";

      /// <summary>
      /// fa-sensor-cloud unicode value ("\ue02c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sensor-cloud
      /// </summary>
      [Description("sensor-cloud")]
      public const string SensorCloud = "\ue02c";

      /// <summary>
      /// fa-sensor-fire unicode value ("\ue02a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sensor-fire
      /// </summary>
      [Description("sensor-fire")]
      public const string SensorFire = "\ue02a";

      /// <summary>
      /// fa-sensor-on unicode value ("\ue02b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sensor-on
      /// </summary>
      [Description("sensor-on")]
      public const string SensorOn = "\ue02b";

      /// <summary>
      /// fa-sensor-triangle-exclamation unicode value ("\ue029").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sensor-triangle-exclamation
      /// </summary>
      [Description("sensor-triangle-exclamation")]
      public const string SensorTriangleExclamation = "\ue029";

      /// <summary>
      /// fa-server unicode value ("\uf233").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/server
      /// </summary>
      [Description("server")]
      public const string Server = "\uf233";

      /// <summary>
      /// fa-servicestack unicode value ("\uf3ec").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/servicestack
      /// </summary>
      [Description("servicestack")]
      public const string Servicestack = "\uf3ec";

      /// <summary>
      /// fa-shapes unicode value ("\uf61f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shapes
      /// </summary>
      [Description("shapes")]
      public const string Shapes = "\uf61f";

      /// <summary>
      /// fa-share unicode value ("\uf064").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/share
      /// </summary>
      [Description("share")]
      public const string Share = "\uf064";

      /// <summary>
      /// fa-share-all unicode value ("\uf367").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/share-all
      /// </summary>
      [Description("share-all")]
      public const string ShareAll = "\uf367";

      /// <summary>
      /// fa-share-from-square unicode value ("\uf14d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/share-from-square
      /// </summary>
      [Description("share-from-square")]
      public const string ShareFromSquare = "\uf14d";

      /// <summary>
      /// fa-share-nodes unicode value ("\uf1e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/share-nodes
      /// </summary>
      [Description("share-nodes")]
      public const string ShareNodes = "\uf1e0";

      /// <summary>
      /// fa-sheep unicode value ("\uf711").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sheep
      /// </summary>
      [Description("sheep")]
      public const string Sheep = "\uf711";

      /// <summary>
      /// fa-sheet-plastic unicode value ("\ue571").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sheet-plastic
      /// </summary>
      [Description("sheet-plastic")]
      public const string SheetPlastic = "\ue571";

      /// <summary>
      /// fa-shekel-sign unicode value ("\uf20b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shekel-sign
      /// </summary>
      [Description("shekel-sign")]
      public const string ShekelSign = "\uf20b";

      /// <summary>
      /// fa-shelves unicode value ("\uf480").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shelves
      /// </summary>
      [Description("shelves")]
      public const string Shelves = "\uf480";

      /// <summary>
      /// fa-shelves-empty unicode value ("\ue246").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shelves-empty
      /// </summary>
      [Description("shelves-empty")]
      public const string ShelvesEmpty = "\ue246";

      /// <summary>
      /// fa-shield unicode value ("\uf132").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield
      /// </summary>
      [Description("shield")]
      public const string Shield = "\uf132";

      /// <summary>
      /// fa-shield-cat unicode value ("\ue572").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-cat
      /// </summary>
      [Description("shield-cat")]
      public const string ShieldCat = "\ue572";

      /// <summary>
      /// fa-shield-check unicode value ("\uf2f7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-check
      /// </summary>
      [Description("shield-check")]
      public const string ShieldCheck = "\uf2f7";

      /// <summary>
      /// fa-shield-cross unicode value ("\uf712").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-cross
      /// </summary>
      [Description("shield-cross")]
      public const string ShieldCross = "\uf712";

      /// <summary>
      /// fa-shield-dog unicode value ("\ue573").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-dog
      /// </summary>
      [Description("shield-dog")]
      public const string ShieldDog = "\ue573";

      /// <summary>
      /// fa-shield-exclamation unicode value ("\ue247").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-exclamation
      /// </summary>
      [Description("shield-exclamation")]
      public const string ShieldExclamation = "\ue247";

      /// <summary>
      /// fa-shield-halved unicode value ("\uf3ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-halved
      /// </summary>
      [Description("shield-halved")]
      public const string ShieldHalved = "\uf3ed";

      /// <summary>
      /// fa-shield-heart unicode value ("\ue574").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-heart
      /// </summary>
      [Description("shield-heart")]
      public const string ShieldHeart = "\ue574";

      /// <summary>
      /// fa-shield-keyhole unicode value ("\ue248").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-keyhole
      /// </summary>
      [Description("shield-keyhole")]
      public const string ShieldKeyhole = "\ue248";

      /// <summary>
      /// fa-shield-minus unicode value ("\ue249").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-minus
      /// </summary>
      [Description("shield-minus")]
      public const string ShieldMinus = "\ue249";

      /// <summary>
      /// fa-shield-plus unicode value ("\ue24a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-plus
      /// </summary>
      [Description("shield-plus")]
      public const string ShieldPlus = "\ue24a";

      /// <summary>
      /// fa-shield-quartered unicode value ("\ue575").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-quartered
      /// </summary>
      [Description("shield-quartered")]
      public const string ShieldQuartered = "\ue575";

      /// <summary>
      /// fa-shield-slash unicode value ("\ue24b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-slash
      /// </summary>
      [Description("shield-slash")]
      public const string ShieldSlash = "\ue24b";

      /// <summary>
      /// fa-shield-virus unicode value ("\ue06c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-virus
      /// </summary>
      [Description("shield-virus")]
      public const string ShieldVirus = "\ue06c";

      /// <summary>
      /// fa-shield-xmark unicode value ("\ue24c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shield-xmark
      /// </summary>
      [Description("shield-xmark")]
      public const string ShieldXmark = "\ue24c";

      /// <summary>
      /// fa-ship unicode value ("\uf21a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ship
      /// </summary>
      [Description("ship")]
      public const string Ship = "\uf21a";

      /// <summary>
      /// fa-shirt unicode value ("\uf553").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shirt
      /// </summary>
      [Description("shirt")]
      public const string Shirt = "\uf553";

      /// <summary>
      /// fa-shirt-long-sleeve unicode value ("\ue3c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shirt-long-sleeve
      /// </summary>
      [Description("shirt-long-sleeve")]
      public const string ShirtLongSleeve = "\ue3c7";

      /// <summary>
      /// fa-shirt-running unicode value ("\ue3c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shirt-running
      /// </summary>
      [Description("shirt-running")]
      public const string ShirtRunning = "\ue3c8";

      /// <summary>
      /// fa-shirt-tank-top unicode value ("\ue3c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shirt-tank-top
      /// </summary>
      [Description("shirt-tank-top")]
      public const string ShirtTankTop = "\ue3c9";

      /// <summary>
      /// fa-shirtsinbulk unicode value ("\uf214").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shirtsinbulk
      /// </summary>
      [Description("shirtsinbulk")]
      public const string Shirtsinbulk = "\uf214";

      /// <summary>
      /// fa-shish-kebab unicode value ("\uf821").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shish-kebab
      /// </summary>
      [Description("shish-kebab")]
      public const string ShishKebab = "\uf821";

      /// <summary>
      /// fa-shoe-prints unicode value ("\uf54b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shoe-prints
      /// </summary>
      [Description("shoe-prints")]
      public const string ShoePrints = "\uf54b";

      /// <summary>
      /// fa-shop unicode value ("\uf54f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shop
      /// </summary>
      [Description("shop")]
      public const string Shop = "\uf54f";

      /// <summary>
      /// fa-shop-lock unicode value ("\ue4a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shop-lock
      /// </summary>
      [Description("shop-lock")]
      public const string ShopLock = "\ue4a5";

      /// <summary>
      /// fa-shop-slash unicode value ("\ue070").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shop-slash
      /// </summary>
      [Description("shop-slash")]
      public const string ShopSlash = "\ue070";

      /// <summary>
      /// fa-shopify unicode value ("\ue057").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shopify
      /// </summary>
      [Description("shopify")]
      public const string Shopify = "\ue057";

      /// <summary>
      /// fa-shopware unicode value ("\uf5b5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shopware
      /// </summary>
      [Description("shopware")]
      public const string Shopware = "\uf5b5";

      /// <summary>
      /// fa-shovel unicode value ("\uf713").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shovel
      /// </summary>
      [Description("shovel")]
      public const string Shovel = "\uf713";

      /// <summary>
      /// fa-shovel-snow unicode value ("\uf7c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shovel-snow
      /// </summary>
      [Description("shovel-snow")]
      public const string ShovelSnow = "\uf7c3";

      /// <summary>
      /// fa-shower unicode value ("\uf2cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shower
      /// </summary>
      [Description("shower")]
      public const string Shower = "\uf2cc";

      /// <summary>
      /// fa-shower-down unicode value ("\ue24d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shower-down
      /// </summary>
      [Description("shower-down")]
      public const string ShowerDown = "\ue24d";

      /// <summary>
      /// fa-shredder unicode value ("\uf68a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shredder
      /// </summary>
      [Description("shredder")]
      public const string Shredder = "\uf68a";

      /// <summary>
      /// fa-shrimp unicode value ("\ue448").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shrimp
      /// </summary>
      [Description("shrimp")]
      public const string Shrimp = "\ue448";

      /// <summary>
      /// fa-shuffle unicode value ("\uf074").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shuffle
      /// </summary>
      [Description("shuffle")]
      public const string Shuffle = "\uf074";

      /// <summary>
      /// fa-shutters unicode value ("\ue449").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shutters
      /// </summary>
      [Description("shutters")]
      public const string Shutters = "\ue449";

      /// <summary>
      /// fa-shuttle-space unicode value ("\uf197").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shuttle-space
      /// </summary>
      [Description("shuttle-space")]
      public const string ShuttleSpace = "\uf197";

      /// <summary>
      /// fa-shuttlecock unicode value ("\uf45b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/shuttlecock
      /// </summary>
      [Description("shuttlecock")]
      public const string Shuttlecock = "\uf45b";

      /// <summary>
      /// fa-sickle unicode value ("\uf822").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sickle
      /// </summary>
      [Description("sickle")]
      public const string Sickle = "\uf822";

      /// <summary>
      /// fa-sidebar unicode value ("\ue24e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sidebar
      /// </summary>
      [Description("sidebar")]
      public const string Sidebar = "\ue24e";

      /// <summary>
      /// fa-sidebar-flip unicode value ("\ue24f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sidebar-flip
      /// </summary>
      [Description("sidebar-flip")]
      public const string SidebarFlip = "\ue24f";

      /// <summary>
      /// fa-sigma unicode value ("\uf68b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sigma
      /// </summary>
      [Description("sigma")]
      public const string Sigma = "\uf68b";

      /// <summary>
      /// fa-sign-hanging unicode value ("\uf4d9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sign-hanging
      /// </summary>
      [Description("sign-hanging")]
      public const string SignHanging = "\uf4d9";

      /// <summary>
      /// fa-signal unicode value ("\uf012").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal
      /// </summary>
      [Description("signal")]
      public const string Signal = "\uf012";

      /// <summary>
      /// fa-signal-bars unicode value ("\uf690").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-bars
      /// </summary>
      [Description("signal-bars")]
      public const string SignalBars = "\uf690";

      /// <summary>
      /// fa-signal-bars-fair unicode value ("\uf692").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-bars-fair
      /// </summary>
      [Description("signal-bars-fair")]
      public const string SignalBarsFair = "\uf692";

      /// <summary>
      /// fa-signal-bars-good unicode value ("\uf693").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-bars-good
      /// </summary>
      [Description("signal-bars-good")]
      public const string SignalBarsGood = "\uf693";

      /// <summary>
      /// fa-signal-bars-slash unicode value ("\uf694").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-bars-slash
      /// </summary>
      [Description("signal-bars-slash")]
      public const string SignalBarsSlash = "\uf694";

      /// <summary>
      /// fa-signal-bars-weak unicode value ("\uf691").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-bars-weak
      /// </summary>
      [Description("signal-bars-weak")]
      public const string SignalBarsWeak = "\uf691";

      /// <summary>
      /// fa-signal-fair unicode value ("\uf68d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-fair
      /// </summary>
      [Description("signal-fair")]
      public const string SignalFair = "\uf68d";

      /// <summary>
      /// fa-signal-good unicode value ("\uf68e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-good
      /// </summary>
      [Description("signal-good")]
      public const string SignalGood = "\uf68e";

      /// <summary>
      /// fa-signal-slash unicode value ("\uf695").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-slash
      /// </summary>
      [Description("signal-slash")]
      public const string SignalSlash = "\uf695";

      /// <summary>
      /// fa-signal-stream unicode value ("\uf8dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-stream
      /// </summary>
      [Description("signal-stream")]
      public const string SignalStream = "\uf8dd";

      /// <summary>
      /// fa-signal-stream-slash unicode value ("\ue250").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-stream-slash
      /// </summary>
      [Description("signal-stream-slash")]
      public const string SignalStreamSlash = "\ue250";

      /// <summary>
      /// fa-signal-strong unicode value ("\uf68f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-strong
      /// </summary>
      [Description("signal-strong")]
      public const string SignalStrong = "\uf68f";

      /// <summary>
      /// fa-signal-weak unicode value ("\uf68c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signal-weak
      /// </summary>
      [Description("signal-weak")]
      public const string SignalWeak = "\uf68c";

      /// <summary>
      /// fa-signature unicode value ("\uf5b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signature
      /// </summary>
      [Description("signature")]
      public const string Signature = "\uf5b7";

      /// <summary>
      /// fa-signature-lock unicode value ("\ue3ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signature-lock
      /// </summary>
      [Description("signature-lock")]
      public const string SignatureLock = "\ue3ca";

      /// <summary>
      /// fa-signature-slash unicode value ("\ue3cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signature-slash
      /// </summary>
      [Description("signature-slash")]
      public const string SignatureSlash = "\ue3cb";

      /// <summary>
      /// fa-signs-post unicode value ("\uf277").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/signs-post
      /// </summary>
      [Description("signs-post")]
      public const string SignsPost = "\uf277";

      /// <summary>
      /// fa-sim-card unicode value ("\uf7c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sim-card
      /// </summary>
      [Description("sim-card")]
      public const string SimCard = "\uf7c4";

      /// <summary>
      /// fa-sim-cards unicode value ("\ue251").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sim-cards
      /// </summary>
      [Description("sim-cards")]
      public const string SimCards = "\ue251";

      /// <summary>
      /// fa-simplybuilt unicode value ("\uf215").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/simplybuilt
      /// </summary>
      [Description("simplybuilt")]
      public const string Simplybuilt = "\uf215";

      /// <summary>
      /// fa-sink unicode value ("\ue06d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sink
      /// </summary>
      [Description("sink")]
      public const string Sink = "\ue06d";

      /// <summary>
      /// fa-siren unicode value ("\ue02d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/siren
      /// </summary>
      [Description("siren")]
      public const string Siren = "\ue02d";

      /// <summary>
      /// fa-siren-on unicode value ("\ue02e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/siren-on
      /// </summary>
      [Description("siren-on")]
      public const string SirenOn = "\ue02e";

      /// <summary>
      /// fa-sistrix unicode value ("\uf3ee").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sistrix
      /// </summary>
      [Description("sistrix")]
      public const string Sistrix = "\uf3ee";

      /// <summary>
      /// fa-sitemap unicode value ("\uf0e8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sitemap
      /// </summary>
      [Description("sitemap")]
      public const string Sitemap = "\uf0e8";

      /// <summary>
      /// fa-sith unicode value ("\uf512").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/sith
      /// </summary>
      [Description("sith")]
      public const string Sith = "\uf512";

      /// <summary>
      /// fa-sitrox unicode value ("\ue44a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.0.0-beta2'.
      /// <para/>
      /// See https://fontawesome.com/icons/sitrox
      /// </summary>
      [Description("sitrox")]
      public const string Sitrox = "\ue44a";

      /// <summary>
      /// fa-skeleton unicode value ("\uf620").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skeleton
      /// </summary>
      [Description("skeleton")]
      public const string Skeleton = "\uf620";

      /// <summary>
      /// fa-skeleton-ribs unicode value ("\ue5cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skeleton-ribs
      /// </summary>
      [Description("skeleton-ribs")]
      public const string SkeletonRibs = "\ue5cb";

      /// <summary>
      /// fa-sketch unicode value ("\uf7c6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sketch
      /// </summary>
      [Description("sketch")]
      public const string Sketch = "\uf7c6";

      /// <summary>
      /// fa-ski-boot unicode value ("\ue3cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ski-boot
      /// </summary>
      [Description("ski-boot")]
      public const string SkiBoot = "\ue3cc";

      /// <summary>
      /// fa-ski-boot-ski unicode value ("\ue3cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ski-boot-ski
      /// </summary>
      [Description("ski-boot-ski")]
      public const string SkiBootSki = "\ue3cd";

      /// <summary>
      /// fa-skull unicode value ("\uf54c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skull
      /// </summary>
      [Description("skull")]
      public const string Skull = "\uf54c";

      /// <summary>
      /// fa-skull-cow unicode value ("\uf8de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skull-cow
      /// </summary>
      [Description("skull-cow")]
      public const string SkullCow = "\uf8de";

      /// <summary>
      /// fa-skull-crossbones unicode value ("\uf714").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skull-crossbones
      /// </summary>
      [Description("skull-crossbones")]
      public const string SkullCrossbones = "\uf714";

      /// <summary>
      /// fa-skyatlas unicode value ("\uf216").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/skyatlas
      /// </summary>
      [Description("skyatlas")]
      public const string Skyatlas = "\uf216";

      /// <summary>
      /// fa-skype unicode value ("\uf17e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/skype
      /// </summary>
      [Description("skype")]
      public const string Skype = "\uf17e";

      /// <summary>
      /// fa-slack unicode value ("\uf198").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slack
      /// </summary>
      [Description("slack")]
      public const string Slack = "\uf198";

      /// <summary>
      /// fa-slash unicode value ("\uf715").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slash
      /// </summary>
      [Description("slash")]
      public const string Slash = "\uf715";

      /// <summary>
      /// fa-slash-back unicode value ("\u005c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slash-back
      /// </summary>
      [Description("slash-back")]
      public const string SlashBack = "\u005c";

      /// <summary>
      /// fa-slash-forward unicode value ("\u002f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slash-forward
      /// </summary>
      [Description("slash-forward")]
      public const string SlashForward = "\u002f";

      /// <summary>
      /// fa-sleigh unicode value ("\uf7cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sleigh
      /// </summary>
      [Description("sleigh")]
      public const string Sleigh = "\uf7cc";

      /// <summary>
      /// fa-slider unicode value ("\ue252").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slider
      /// </summary>
      [Description("slider")]
      public const string Slider = "\ue252";

      /// <summary>
      /// fa-sliders unicode value ("\uf1de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sliders
      /// </summary>
      [Description("sliders")]
      public const string Sliders = "\uf1de";

      /// <summary>
      /// fa-sliders-simple unicode value ("\ue253").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sliders-simple
      /// </summary>
      [Description("sliders-simple")]
      public const string SlidersSimple = "\ue253";

      /// <summary>
      /// fa-sliders-up unicode value ("\uf3f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sliders-up
      /// </summary>
      [Description("sliders-up")]
      public const string SlidersUp = "\uf3f1";

      /// <summary>
      /// fa-slideshare unicode value ("\uf1e7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slideshare
      /// </summary>
      [Description("slideshare")]
      public const string Slideshare = "\uf1e7";

      /// <summary>
      /// fa-slot-machine unicode value ("\ue3ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/slot-machine
      /// </summary>
      [Description("slot-machine")]
      public const string SlotMachine = "\ue3ce";

      /// <summary>
      /// fa-smog unicode value ("\uf75f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/smog
      /// </summary>
      [Description("smog")]
      public const string Smog = "\uf75f";

      /// <summary>
      /// fa-smoke unicode value ("\uf760").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/smoke
      /// </summary>
      [Description("smoke")]
      public const string Smoke = "\uf760";

      /// <summary>
      /// fa-smoking unicode value ("\uf48d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/smoking
      /// </summary>
      [Description("smoking")]
      public const string Smoking = "\uf48d";

      /// <summary>
      /// fa-snake unicode value ("\uf716").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snake
      /// </summary>
      [Description("snake")]
      public const string Snake = "\uf716";

      /// <summary>
      /// fa-snapchat unicode value ("\uf2ab").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/snapchat
      /// </summary>
      [Description("snapchat")]
      public const string Snapchat = "\uf2ab";

      /// <summary>
      /// fa-snooze unicode value ("\uf880").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snooze
      /// </summary>
      [Description("snooze")]
      public const string Snooze = "\uf880";

      /// <summary>
      /// fa-snow-blowing unicode value ("\uf761").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snow-blowing
      /// </summary>
      [Description("snow-blowing")]
      public const string SnowBlowing = "\uf761";

      /// <summary>
      /// fa-snowflake unicode value ("\uf2dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowflake
      /// </summary>
      [Description("snowflake")]
      public const string Snowflake = "\uf2dc";

      /// <summary>
      /// fa-snowflake-droplets unicode value ("\ue5c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowflake-droplets
      /// </summary>
      [Description("snowflake-droplets")]
      public const string SnowflakeDroplets = "\ue5c1";

      /// <summary>
      /// fa-snowflakes unicode value ("\uf7cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowflakes
      /// </summary>
      [Description("snowflakes")]
      public const string Snowflakes = "\uf7cf";

      /// <summary>
      /// fa-snowman unicode value ("\uf7d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowman
      /// </summary>
      [Description("snowman")]
      public const string Snowman = "\uf7d0";

      /// <summary>
      /// fa-snowman-head unicode value ("\uf79b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowman-head
      /// </summary>
      [Description("snowman-head")]
      public const string SnowmanHead = "\uf79b";

      /// <summary>
      /// fa-snowplow unicode value ("\uf7d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/snowplow
      /// </summary>
      [Description("snowplow")]
      public const string Snowplow = "\uf7d2";

      /// <summary>
      /// fa-soap unicode value ("\ue06e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/soap
      /// </summary>
      [Description("soap")]
      public const string Soap = "\ue06e";

      /// <summary>
      /// fa-socks unicode value ("\uf696").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/socks
      /// </summary>
      [Description("socks")]
      public const string Socks = "\uf696";

      /// <summary>
      /// fa-soft-serve unicode value ("\ue400").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/soft-serve
      /// </summary>
      [Description("soft-serve")]
      public const string SoftServe = "\ue400";

      /// <summary>
      /// fa-solar-panel unicode value ("\uf5ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/solar-panel
      /// </summary>
      [Description("solar-panel")]
      public const string SolarPanel = "\uf5ba";

      /// <summary>
      /// fa-solar-system unicode value ("\ue02f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/solar-system
      /// </summary>
      [Description("solar-system")]
      public const string SolarSystem = "\ue02f";

      /// <summary>
      /// fa-sort unicode value ("\uf0dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sort
      /// </summary>
      [Description("sort")]
      public const string Sort = "\uf0dc";

      /// <summary>
      /// fa-sort-down unicode value ("\uf0dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sort-down
      /// </summary>
      [Description("sort-down")]
      public const string SortDown = "\uf0dd";

      /// <summary>
      /// fa-sort-up unicode value ("\uf0de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sort-up
      /// </summary>
      [Description("sort-up")]
      public const string SortUp = "\uf0de";

      /// <summary>
      /// fa-soundcloud unicode value ("\uf1be").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/soundcloud
      /// </summary>
      [Description("soundcloud")]
      public const string Soundcloud = "\uf1be";

      /// <summary>
      /// fa-sourcetree unicode value ("\uf7d3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sourcetree
      /// </summary>
      [Description("sourcetree")]
      public const string Sourcetree = "\uf7d3";

      /// <summary>
      /// fa-spa unicode value ("\uf5bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spa
      /// </summary>
      [Description("spa")]
      public const string Spa = "\uf5bb";

      /// <summary>
      /// fa-space-awesome unicode value ("\ue5ac").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/space-awesome
      /// </summary>
      [Description("space-awesome")]
      public const string SpaceAwesome = "\ue5ac";

      /// <summary>
      /// fa-space-station-moon unicode value ("\ue033").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/space-station-moon
      /// </summary>
      [Description("space-station-moon")]
      public const string SpaceStationMoon = "\ue033";

      /// <summary>
      /// fa-space-station-moon-construction unicode value ("\ue034").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/space-station-moon-construction
      /// </summary>
      [Description("space-station-moon-construction")]
      public const string SpaceStationMoonConstruction = "\ue034";

      /// <summary>
      /// fa-spade unicode value ("\uf2f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spade
      /// </summary>
      [Description("spade")]
      public const string Spade = "\uf2f4";

      /// <summary>
      /// fa-spaghetti-monster-flying unicode value ("\uf67b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spaghetti-monster-flying
      /// </summary>
      [Description("spaghetti-monster-flying")]
      public const string SpaghettiMonsterFlying = "\uf67b";

      /// <summary>
      /// fa-sparkle unicode value ("\ue5d6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sparkle
      /// </summary>
      [Description("sparkle")]
      public const string Sparkle = "\ue5d6";

      /// <summary>
      /// fa-sparkles unicode value ("\uf890").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sparkles
      /// </summary>
      [Description("sparkles")]
      public const string Sparkles = "\uf890";

      /// <summary>
      /// fa-speakap unicode value ("\uf3f3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/speakap
      /// </summary>
      [Description("speakap")]
      public const string Speakap = "\uf3f3";

      /// <summary>
      /// fa-speaker unicode value ("\uf8df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/speaker
      /// </summary>
      [Description("speaker")]
      public const string Speaker = "\uf8df";

      /// <summary>
      /// fa-speaker-deck unicode value ("\uf83c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/speaker-deck
      /// </summary>
      [Description("speaker-deck")]
      public const string SpeakerDeck = "\uf83c";

      /// <summary>
      /// fa-speakers unicode value ("\uf8e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/speakers
      /// </summary>
      [Description("speakers")]
      public const string Speakers = "\uf8e0";

      /// <summary>
      /// fa-spell-check unicode value ("\uf891").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spell-check
      /// </summary>
      [Description("spell-check")]
      public const string SpellCheck = "\uf891";

      /// <summary>
      /// fa-spider unicode value ("\uf717").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spider
      /// </summary>
      [Description("spider")]
      public const string Spider = "\uf717";

      /// <summary>
      /// fa-spider-black-widow unicode value ("\uf718").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spider-black-widow
      /// </summary>
      [Description("spider-black-widow")]
      public const string SpiderBlackWidow = "\uf718";

      /// <summary>
      /// fa-spider-web unicode value ("\uf719").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spider-web
      /// </summary>
      [Description("spider-web")]
      public const string SpiderWeb = "\uf719";

      /// <summary>
      /// fa-spinner unicode value ("\uf110").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spinner
      /// </summary>
      [Description("spinner")]
      public const string Spinner = "\uf110";

      /// <summary>
      /// fa-spinner-third unicode value ("\uf3f4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spinner-third
      /// </summary>
      [Description("spinner-third")]
      public const string SpinnerThird = "\uf3f4";

      /// <summary>
      /// fa-split unicode value ("\ue254").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/split
      /// </summary>
      [Description("split")]
      public const string Split = "\ue254";

      /// <summary>
      /// fa-splotch unicode value ("\uf5bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/splotch
      /// </summary>
      [Description("splotch")]
      public const string Splotch = "\uf5bc";

      /// <summary>
      /// fa-spoon unicode value ("\uf2e5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spoon
      /// </summary>
      [Description("spoon")]
      public const string Spoon = "\uf2e5";

      /// <summary>
      /// fa-sportsball unicode value ("\ue44b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sportsball
      /// </summary>
      [Description("sportsball")]
      public const string Sportsball = "\ue44b";

      /// <summary>
      /// fa-spotify unicode value ("\uf1bc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spotify
      /// </summary>
      [Description("spotify")]
      public const string Spotify = "\uf1bc";

      /// <summary>
      /// fa-spray-can unicode value ("\uf5bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spray-can
      /// </summary>
      [Description("spray-can")]
      public const string SprayCan = "\uf5bd";

      /// <summary>
      /// fa-spray-can-sparkles unicode value ("\uf5d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/spray-can-sparkles
      /// </summary>
      [Description("spray-can-sparkles")]
      public const string SprayCanSparkles = "\uf5d0";

      /// <summary>
      /// fa-sprinkler unicode value ("\ue035").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sprinkler
      /// </summary>
      [Description("sprinkler")]
      public const string Sprinkler = "\ue035";

      /// <summary>
      /// fa-sprinkler-ceiling unicode value ("\ue44c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sprinkler-ceiling
      /// </summary>
      [Description("sprinkler-ceiling")]
      public const string SprinklerCeiling = "\ue44c";

      /// <summary>
      /// fa-square unicode value ("\uf0c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square
      /// </summary>
      [Description("square")]
      public const string Square = "\uf0c8";

      /// <summary>
      /// fa-square-0 unicode value ("\ue255").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-0
      /// </summary>
      [Description("square-0")]
      public const string Square0 = "\ue255";

      /// <summary>
      /// fa-square-1 unicode value ("\ue256").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-1
      /// </summary>
      [Description("square-1")]
      public const string Square1 = "\ue256";

      /// <summary>
      /// fa-square-2 unicode value ("\ue257").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-2
      /// </summary>
      [Description("square-2")]
      public const string Square2 = "\ue257";

      /// <summary>
      /// fa-square-3 unicode value ("\ue258").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-3
      /// </summary>
      [Description("square-3")]
      public const string Square3 = "\ue258";

      /// <summary>
      /// fa-square-4 unicode value ("\ue259").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-4
      /// </summary>
      [Description("square-4")]
      public const string Square4 = "\ue259";

      /// <summary>
      /// fa-square-5 unicode value ("\ue25a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-5
      /// </summary>
      [Description("square-5")]
      public const string Square5 = "\ue25a";

      /// <summary>
      /// fa-square-6 unicode value ("\ue25b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-6
      /// </summary>
      [Description("square-6")]
      public const string Square6 = "\ue25b";

      /// <summary>
      /// fa-square-7 unicode value ("\ue25c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-7
      /// </summary>
      [Description("square-7")]
      public const string Square7 = "\ue25c";

      /// <summary>
      /// fa-square-8 unicode value ("\ue25d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-8
      /// </summary>
      [Description("square-8")]
      public const string Square8 = "\ue25d";

      /// <summary>
      /// fa-square-9 unicode value ("\ue25e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-9
      /// </summary>
      [Description("square-9")]
      public const string Square9 = "\ue25e";

      /// <summary>
      /// fa-square-a unicode value ("\ue25f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-a
      /// </summary>
      [Description("square-a")]
      public const string SquareA = "\ue25f";

      /// <summary>
      /// fa-square-a-lock unicode value ("\ue44d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-a-lock
      /// </summary>
      [Description("square-a-lock")]
      public const string SquareALock = "\ue44d";

      /// <summary>
      /// fa-square-ampersand unicode value ("\ue260").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-ampersand
      /// </summary>
      [Description("square-ampersand")]
      public const string SquareAmpersand = "\ue260";

      /// <summary>
      /// fa-square-arrow-down unicode value ("\uf339").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-down
      /// </summary>
      [Description("square-arrow-down")]
      public const string SquareArrowDown = "\uf339";

      /// <summary>
      /// fa-square-arrow-down-left unicode value ("\ue261").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-down-left
      /// </summary>
      [Description("square-arrow-down-left")]
      public const string SquareArrowDownLeft = "\ue261";

      /// <summary>
      /// fa-square-arrow-down-right unicode value ("\ue262").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-down-right
      /// </summary>
      [Description("square-arrow-down-right")]
      public const string SquareArrowDownRight = "\ue262";

      /// <summary>
      /// fa-square-arrow-left unicode value ("\uf33a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-left
      /// </summary>
      [Description("square-arrow-left")]
      public const string SquareArrowLeft = "\uf33a";

      /// <summary>
      /// fa-square-arrow-right unicode value ("\uf33b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-right
      /// </summary>
      [Description("square-arrow-right")]
      public const string SquareArrowRight = "\uf33b";

      /// <summary>
      /// fa-square-arrow-up unicode value ("\uf33c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-up
      /// </summary>
      [Description("square-arrow-up")]
      public const string SquareArrowUp = "\uf33c";

      /// <summary>
      /// fa-square-arrow-up-left unicode value ("\ue263").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-up-left
      /// </summary>
      [Description("square-arrow-up-left")]
      public const string SquareArrowUpLeft = "\ue263";

      /// <summary>
      /// fa-square-arrow-up-right unicode value ("\uf14c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-arrow-up-right
      /// </summary>
      [Description("square-arrow-up-right")]
      public const string SquareArrowUpRight = "\uf14c";

      /// <summary>
      /// fa-square-b unicode value ("\ue264").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-b
      /// </summary>
      [Description("square-b")]
      public const string SquareB = "\ue264";

      /// <summary>
      /// fa-square-behance unicode value ("\uf1b5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-behance
      /// </summary>
      [Description("square-behance")]
      public const string SquareBehance = "\uf1b5";

      /// <summary>
      /// fa-square-bolt unicode value ("\ue265").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-bolt
      /// </summary>
      [Description("square-bolt")]
      public const string SquareBolt = "\ue265";

      /// <summary>
      /// fa-square-c unicode value ("\ue266").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-c
      /// </summary>
      [Description("square-c")]
      public const string SquareC = "\ue266";

      /// <summary>
      /// fa-square-caret-down unicode value ("\uf150").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-caret-down
      /// </summary>
      [Description("square-caret-down")]
      public const string SquareCaretDown = "\uf150";

      /// <summary>
      /// fa-square-caret-left unicode value ("\uf191").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-caret-left
      /// </summary>
      [Description("square-caret-left")]
      public const string SquareCaretLeft = "\uf191";

      /// <summary>
      /// fa-square-caret-right unicode value ("\uf152").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-caret-right
      /// </summary>
      [Description("square-caret-right")]
      public const string SquareCaretRight = "\uf152";

      /// <summary>
      /// fa-square-caret-up unicode value ("\uf151").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-caret-up
      /// </summary>
      [Description("square-caret-up")]
      public const string SquareCaretUp = "\uf151";

      /// <summary>
      /// fa-square-check unicode value ("\uf14a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-check
      /// </summary>
      [Description("square-check")]
      public const string SquareCheck = "\uf14a";

      /// <summary>
      /// fa-square-chevron-down unicode value ("\uf329").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-chevron-down
      /// </summary>
      [Description("square-chevron-down")]
      public const string SquareChevronDown = "\uf329";

      /// <summary>
      /// fa-square-chevron-left unicode value ("\uf32a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-chevron-left
      /// </summary>
      [Description("square-chevron-left")]
      public const string SquareChevronLeft = "\uf32a";

      /// <summary>
      /// fa-square-chevron-right unicode value ("\uf32b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-chevron-right
      /// </summary>
      [Description("square-chevron-right")]
      public const string SquareChevronRight = "\uf32b";

      /// <summary>
      /// fa-square-chevron-up unicode value ("\uf32c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-chevron-up
      /// </summary>
      [Description("square-chevron-up")]
      public const string SquareChevronUp = "\uf32c";

      /// <summary>
      /// fa-square-code unicode value ("\ue267").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-code
      /// </summary>
      [Description("square-code")]
      public const string SquareCode = "\ue267";

      /// <summary>
      /// fa-square-d unicode value ("\ue268").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-d
      /// </summary>
      [Description("square-d")]
      public const string SquareD = "\ue268";

      /// <summary>
      /// fa-square-dashed unicode value ("\ue269").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-dashed
      /// </summary>
      [Description("square-dashed")]
      public const string SquareDashed = "\ue269";

      /// <summary>
      /// fa-square-dashed-circle-plus unicode value ("\ue5c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-dashed-circle-plus
      /// </summary>
      [Description("square-dashed-circle-plus")]
      public const string SquareDashedCirclePlus = "\ue5c2";

      /// <summary>
      /// fa-square-divide unicode value ("\ue26a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-divide
      /// </summary>
      [Description("square-divide")]
      public const string SquareDivide = "\ue26a";

      /// <summary>
      /// fa-square-dollar unicode value ("\uf2e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-dollar
      /// </summary>
      [Description("square-dollar")]
      public const string SquareDollar = "\uf2e9";

      /// <summary>
      /// fa-square-down unicode value ("\uf350").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-down
      /// </summary>
      [Description("square-down")]
      public const string SquareDown = "\uf350";

      /// <summary>
      /// fa-square-down-left unicode value ("\ue26b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-down-left
      /// </summary>
      [Description("square-down-left")]
      public const string SquareDownLeft = "\ue26b";

      /// <summary>
      /// fa-square-down-right unicode value ("\ue26c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-down-right
      /// </summary>
      [Description("square-down-right")]
      public const string SquareDownRight = "\ue26c";

      /// <summary>
      /// fa-square-dribbble unicode value ("\uf397").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-dribbble
      /// </summary>
      [Description("square-dribbble")]
      public const string SquareDribbble = "\uf397";

      /// <summary>
      /// fa-square-e unicode value ("\ue26d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-e
      /// </summary>
      [Description("square-e")]
      public const string SquareE = "\ue26d";

      /// <summary>
      /// fa-square-ellipsis unicode value ("\ue26e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-ellipsis
      /// </summary>
      [Description("square-ellipsis")]
      public const string SquareEllipsis = "\ue26e";

      /// <summary>
      /// fa-square-ellipsis-vertical unicode value ("\ue26f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-ellipsis-vertical
      /// </summary>
      [Description("square-ellipsis-vertical")]
      public const string SquareEllipsisVertical = "\ue26f";

      /// <summary>
      /// fa-square-envelope unicode value ("\uf199").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-envelope
      /// </summary>
      [Description("square-envelope")]
      public const string SquareEnvelope = "\uf199";

      /// <summary>
      /// fa-square-exclamation unicode value ("\uf321").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-exclamation
      /// </summary>
      [Description("square-exclamation")]
      public const string SquareExclamation = "\uf321";

      /// <summary>
      /// fa-square-f unicode value ("\ue270").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-f
      /// </summary>
      [Description("square-f")]
      public const string SquareF = "\ue270";

      /// <summary>
      /// fa-square-facebook unicode value ("\uf082").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-facebook
      /// </summary>
      [Description("square-facebook")]
      public const string SquareFacebook = "\uf082";

      /// <summary>
      /// fa-square-font-awesome unicode value ("\ue5ad").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-font-awesome
      /// </summary>
      [Description("square-font-awesome")]
      public const string SquareFontAwesome = "\ue5ad";

      /// <summary>
      /// fa-square-font-awesome-stroke unicode value ("\uf35c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-font-awesome-stroke
      /// </summary>
      [Description("square-font-awesome-stroke")]
      public const string SquareFontAwesomeStroke = "\uf35c";

      /// <summary>
      /// fa-square-fragile unicode value ("\uf49b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-fragile
      /// </summary>
      [Description("square-fragile")]
      public const string SquareFragile = "\uf49b";

      /// <summary>
      /// fa-square-full unicode value ("\uf45c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-full
      /// </summary>
      [Description("square-full")]
      public const string SquareFull = "\uf45c";

      /// <summary>
      /// fa-square-g unicode value ("\ue271").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-g
      /// </summary>
      [Description("square-g")]
      public const string SquareG = "\ue271";

      /// <summary>
      /// fa-square-git unicode value ("\uf1d2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-git
      /// </summary>
      [Description("square-git")]
      public const string SquareGit = "\uf1d2";

      /// <summary>
      /// fa-square-github unicode value ("\uf092").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-github
      /// </summary>
      [Description("square-github")]
      public const string SquareGithub = "\uf092";

      /// <summary>
      /// fa-square-gitlab unicode value ("\ue5ae").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-gitlab
      /// </summary>
      [Description("square-gitlab")]
      public const string SquareGitlab = "\ue5ae";

      /// <summary>
      /// fa-square-google-plus unicode value ("\uf0d4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-google-plus
      /// </summary>
      [Description("square-google-plus")]
      public const string SquareGooglePlus = "\uf0d4";

      /// <summary>
      /// fa-square-h unicode value ("\uf0fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-h
      /// </summary>
      [Description("square-h")]
      public const string SquareH = "\uf0fd";

      /// <summary>
      /// fa-square-hacker-news unicode value ("\uf3af").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-hacker-news
      /// </summary>
      [Description("square-hacker-news")]
      public const string SquareHackerNews = "\uf3af";

      /// <summary>
      /// fa-square-heart unicode value ("\uf4c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-heart
      /// </summary>
      [Description("square-heart")]
      public const string SquareHeart = "\uf4c8";

      /// <summary>
      /// fa-square-i unicode value ("\ue272").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-i
      /// </summary>
      [Description("square-i")]
      public const string SquareI = "\ue272";

      /// <summary>
      /// fa-square-info unicode value ("\uf30f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-info
      /// </summary>
      [Description("square-info")]
      public const string SquareInfo = "\uf30f";

      /// <summary>
      /// fa-square-instagram unicode value ("\ue055").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-instagram
      /// </summary>
      [Description("square-instagram")]
      public const string SquareInstagram = "\ue055";

      /// <summary>
      /// fa-square-j unicode value ("\ue273").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-j
      /// </summary>
      [Description("square-j")]
      public const string SquareJ = "\ue273";

      /// <summary>
      /// fa-square-js unicode value ("\uf3b9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-js
      /// </summary>
      [Description("square-js")]
      public const string SquareJs = "\uf3b9";

      /// <summary>
      /// fa-square-k unicode value ("\ue274").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-k
      /// </summary>
      [Description("square-k")]
      public const string SquareK = "\ue274";

      /// <summary>
      /// fa-square-kanban unicode value ("\ue488").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-kanban
      /// </summary>
      [Description("square-kanban")]
      public const string SquareKanban = "\ue488";

      /// <summary>
      /// fa-square-l unicode value ("\ue275").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-l
      /// </summary>
      [Description("square-l")]
      public const string SquareL = "\ue275";

      /// <summary>
      /// fa-square-lastfm unicode value ("\uf203").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-lastfm
      /// </summary>
      [Description("square-lastfm")]
      public const string SquareLastfm = "\uf203";

      /// <summary>
      /// fa-square-left unicode value ("\uf351").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-left
      /// </summary>
      [Description("square-left")]
      public const string SquareLeft = "\uf351";

      /// <summary>
      /// fa-square-list unicode value ("\ue489").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-list
      /// </summary>
      [Description("square-list")]
      public const string SquareList = "\ue489";

      /// <summary>
      /// fa-square-m unicode value ("\ue276").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-m
      /// </summary>
      [Description("square-m")]
      public const string SquareM = "\ue276";

      /// <summary>
      /// fa-square-minus unicode value ("\uf146").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-minus
      /// </summary>
      [Description("square-minus")]
      public const string SquareMinus = "\uf146";

      /// <summary>
      /// fa-square-n unicode value ("\ue277").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-n
      /// </summary>
      [Description("square-n")]
      public const string SquareN = "\ue277";

      /// <summary>
      /// fa-square-nfi unicode value ("\ue576").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-nfi
      /// </summary>
      [Description("square-nfi")]
      public const string SquareNfi = "\ue576";

      /// <summary>
      /// fa-square-o unicode value ("\ue278").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-o
      /// </summary>
      [Description("square-o")]
      public const string SquareO = "\ue278";

      /// <summary>
      /// fa-square-odnoklassniki unicode value ("\uf264").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-odnoklassniki
      /// </summary>
      [Description("square-odnoklassniki")]
      public const string SquareOdnoklassniki = "\uf264";

      /// <summary>
      /// fa-square-p unicode value ("\ue279").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-p
      /// </summary>
      [Description("square-p")]
      public const string SquareP = "\ue279";

      /// <summary>
      /// fa-square-parking unicode value ("\uf540").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-parking
      /// </summary>
      [Description("square-parking")]
      public const string SquareParking = "\uf540";

      /// <summary>
      /// fa-square-parking-slash unicode value ("\uf617").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-parking-slash
      /// </summary>
      [Description("square-parking-slash")]
      public const string SquareParkingSlash = "\uf617";

      /// <summary>
      /// fa-square-pen unicode value ("\uf14b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-pen
      /// </summary>
      [Description("square-pen")]
      public const string SquarePen = "\uf14b";

      /// <summary>
      /// fa-square-person-confined unicode value ("\ue577").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-person-confined
      /// </summary>
      [Description("square-person-confined")]
      public const string SquarePersonConfined = "\ue577";

      /// <summary>
      /// fa-square-phone unicode value ("\uf098").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-phone
      /// </summary>
      [Description("square-phone")]
      public const string SquarePhone = "\uf098";

      /// <summary>
      /// fa-square-phone-flip unicode value ("\uf87b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-phone-flip
      /// </summary>
      [Description("square-phone-flip")]
      public const string SquarePhoneFlip = "\uf87b";

      /// <summary>
      /// fa-square-phone-hangup unicode value ("\ue27a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-phone-hangup
      /// </summary>
      [Description("square-phone-hangup")]
      public const string SquarePhoneHangup = "\ue27a";

      /// <summary>
      /// fa-square-pied-piper unicode value ("\ue01e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-pied-piper
      /// </summary>
      [Description("square-pied-piper")]
      public const string SquarePiedPiper = "\ue01e";

      /// <summary>
      /// fa-square-pinterest unicode value ("\uf0d3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-pinterest
      /// </summary>
      [Description("square-pinterest")]
      public const string SquarePinterest = "\uf0d3";

      /// <summary>
      /// fa-square-plus unicode value ("\uf0fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-plus
      /// </summary>
      [Description("square-plus")]
      public const string SquarePlus = "\uf0fe";

      /// <summary>
      /// fa-square-poll-horizontal unicode value ("\uf682").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-poll-horizontal
      /// </summary>
      [Description("square-poll-horizontal")]
      public const string SquarePollHorizontal = "\uf682";

      /// <summary>
      /// fa-square-poll-vertical unicode value ("\uf681").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-poll-vertical
      /// </summary>
      [Description("square-poll-vertical")]
      public const string SquarePollVertical = "\uf681";

      /// <summary>
      /// fa-square-q unicode value ("\ue27b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-q
      /// </summary>
      [Description("square-q")]
      public const string SquareQ = "\ue27b";

      /// <summary>
      /// fa-square-quarters unicode value ("\ue44e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-quarters
      /// </summary>
      [Description("square-quarters")]
      public const string SquareQuarters = "\ue44e";

      /// <summary>
      /// fa-square-question unicode value ("\uf2fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-question
      /// </summary>
      [Description("square-question")]
      public const string SquareQuestion = "\uf2fd";

      /// <summary>
      /// fa-square-quote unicode value ("\ue329").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-quote
      /// </summary>
      [Description("square-quote")]
      public const string SquareQuote = "\ue329";

      /// <summary>
      /// fa-square-r unicode value ("\ue27c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-r
      /// </summary>
      [Description("square-r")]
      public const string SquareR = "\ue27c";

      /// <summary>
      /// fa-square-reddit unicode value ("\uf1a2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-reddit
      /// </summary>
      [Description("square-reddit")]
      public const string SquareReddit = "\uf1a2";

      /// <summary>
      /// fa-square-right unicode value ("\uf352").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-right
      /// </summary>
      [Description("square-right")]
      public const string SquareRight = "\uf352";

      /// <summary>
      /// fa-square-ring unicode value ("\ue44f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-ring
      /// </summary>
      [Description("square-ring")]
      public const string SquareRing = "\ue44f";

      /// <summary>
      /// fa-square-root unicode value ("\uf697").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-root
      /// </summary>
      [Description("square-root")]
      public const string SquareRoot = "\uf697";

      /// <summary>
      /// fa-square-root-variable unicode value ("\uf698").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-root-variable
      /// </summary>
      [Description("square-root-variable")]
      public const string SquareRootVariable = "\uf698";

      /// <summary>
      /// fa-square-rss unicode value ("\uf143").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-rss
      /// </summary>
      [Description("square-rss")]
      public const string SquareRss = "\uf143";

      /// <summary>
      /// fa-square-s unicode value ("\ue27d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-s
      /// </summary>
      [Description("square-s")]
      public const string SquareS = "\ue27d";

      /// <summary>
      /// fa-square-share-nodes unicode value ("\uf1e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-share-nodes
      /// </summary>
      [Description("square-share-nodes")]
      public const string SquareShareNodes = "\uf1e1";

      /// <summary>
      /// fa-square-sliders unicode value ("\uf3f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-sliders
      /// </summary>
      [Description("square-sliders")]
      public const string SquareSliders = "\uf3f0";

      /// <summary>
      /// fa-square-sliders-vertical unicode value ("\uf3f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-sliders-vertical
      /// </summary>
      [Description("square-sliders-vertical")]
      public const string SquareSlidersVertical = "\uf3f2";

      /// <summary>
      /// fa-square-small unicode value ("\ue27e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-small
      /// </summary>
      [Description("square-small")]
      public const string SquareSmall = "\ue27e";

      /// <summary>
      /// fa-square-snapchat unicode value ("\uf2ad").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-snapchat
      /// </summary>
      [Description("square-snapchat")]
      public const string SquareSnapchat = "\uf2ad";

      /// <summary>
      /// fa-square-star unicode value ("\ue27f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-star
      /// </summary>
      [Description("square-star")]
      public const string SquareStar = "\ue27f";

      /// <summary>
      /// fa-square-steam unicode value ("\uf1b7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-steam
      /// </summary>
      [Description("square-steam")]
      public const string SquareSteam = "\uf1b7";

      /// <summary>
      /// fa-square-t unicode value ("\ue280").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-t
      /// </summary>
      [Description("square-t")]
      public const string SquareT = "\ue280";

      /// <summary>
      /// fa-square-terminal unicode value ("\ue32a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-terminal
      /// </summary>
      [Description("square-terminal")]
      public const string SquareTerminal = "\ue32a";

      /// <summary>
      /// fa-square-this-way-up unicode value ("\uf49f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-this-way-up
      /// </summary>
      [Description("square-this-way-up")]
      public const string SquareThisWayUp = "\uf49f";

      /// <summary>
      /// fa-square-tumblr unicode value ("\uf174").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-tumblr
      /// </summary>
      [Description("square-tumblr")]
      public const string SquareTumblr = "\uf174";

      /// <summary>
      /// fa-square-twitter unicode value ("\uf081").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-twitter
      /// </summary>
      [Description("square-twitter")]
      public const string SquareTwitter = "\uf081";

      /// <summary>
      /// fa-square-u unicode value ("\ue281").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-u
      /// </summary>
      [Description("square-u")]
      public const string SquareU = "\ue281";

      /// <summary>
      /// fa-square-up unicode value ("\uf353").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-up
      /// </summary>
      [Description("square-up")]
      public const string SquareUp = "\uf353";

      /// <summary>
      /// fa-square-up-left unicode value ("\ue282").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-up-left
      /// </summary>
      [Description("square-up-left")]
      public const string SquareUpLeft = "\ue282";

      /// <summary>
      /// fa-square-up-right unicode value ("\uf360").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-up-right
      /// </summary>
      [Description("square-up-right")]
      public const string SquareUpRight = "\uf360";

      /// <summary>
      /// fa-square-user unicode value ("\ue283").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-user
      /// </summary>
      [Description("square-user")]
      public const string SquareUser = "\ue283";

      /// <summary>
      /// fa-square-v unicode value ("\ue284").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-v
      /// </summary>
      [Description("square-v")]
      public const string SquareV = "\ue284";

      /// <summary>
      /// fa-square-viadeo unicode value ("\uf2aa").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-viadeo
      /// </summary>
      [Description("square-viadeo")]
      public const string SquareViadeo = "\uf2aa";

      /// <summary>
      /// fa-square-vimeo unicode value ("\uf194").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-vimeo
      /// </summary>
      [Description("square-vimeo")]
      public const string SquareVimeo = "\uf194";

      /// <summary>
      /// fa-square-virus unicode value ("\ue578").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-virus
      /// </summary>
      [Description("square-virus")]
      public const string SquareVirus = "\ue578";

      /// <summary>
      /// fa-square-w unicode value ("\ue285").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-w
      /// </summary>
      [Description("square-w")]
      public const string SquareW = "\ue285";

      /// <summary>
      /// fa-square-whatsapp unicode value ("\uf40c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-whatsapp
      /// </summary>
      [Description("square-whatsapp")]
      public const string SquareWhatsapp = "\uf40c";

      /// <summary>
      /// fa-square-x unicode value ("\ue286").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-x
      /// </summary>
      [Description("square-x")]
      public const string SquareX = "\ue286";

      /// <summary>
      /// fa-square-xing unicode value ("\uf169").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-xing
      /// </summary>
      [Description("square-xing")]
      public const string SquareXing = "\uf169";

      /// <summary>
      /// fa-square-xmark unicode value ("\uf2d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-xmark
      /// </summary>
      [Description("square-xmark")]
      public const string SquareXmark = "\uf2d3";

      /// <summary>
      /// fa-square-y unicode value ("\ue287").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-y
      /// </summary>
      [Description("square-y")]
      public const string SquareY = "\ue287";

      /// <summary>
      /// fa-square-youtube unicode value ("\uf431").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.3', Last Modified in '6.1.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-youtube
      /// </summary>
      [Description("square-youtube")]
      public const string SquareYoutube = "\uf431";

      /// <summary>
      /// fa-square-z unicode value ("\ue288").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/square-z
      /// </summary>
      [Description("square-z")]
      public const string SquareZ = "\ue288";

      /// <summary>
      /// fa-squarespace unicode value ("\uf5be").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/squarespace
      /// </summary>
      [Description("squarespace")]
      public const string Squarespace = "\uf5be";

      /// <summary>
      /// fa-squid unicode value ("\ue450").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/squid
      /// </summary>
      [Description("squid")]
      public const string Squid = "\ue450";

      /// <summary>
      /// fa-squirrel unicode value ("\uf71a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/squirrel
      /// </summary>
      [Description("squirrel")]
      public const string Squirrel = "\uf71a";

      /// <summary>
      /// fa-stack-exchange unicode value ("\uf18d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/stack-exchange
      /// </summary>
      [Description("stack-exchange")]
      public const string StackExchange = "\uf18d";

      /// <summary>
      /// fa-stack-overflow unicode value ("\uf16c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stack-overflow
      /// </summary>
      [Description("stack-overflow")]
      public const string StackOverflow = "\uf16c";

      /// <summary>
      /// fa-stackpath unicode value ("\uf842").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.2', Last Modified in '5.8.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/stackpath
      /// </summary>
      [Description("stackpath")]
      public const string Stackpath = "\uf842";

      /// <summary>
      /// fa-staff unicode value ("\uf71b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/staff
      /// </summary>
      [Description("staff")]
      public const string Staff = "\uf71b";

      /// <summary>
      /// fa-staff-snake unicode value ("\ue579").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/staff-snake
      /// </summary>
      [Description("staff-snake")]
      public const string StaffSnake = "\ue579";

      /// <summary>
      /// fa-stairs unicode value ("\ue289").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stairs
      /// </summary>
      [Description("stairs")]
      public const string Stairs = "\ue289";

      /// <summary>
      /// fa-stamp unicode value ("\uf5bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stamp
      /// </summary>
      [Description("stamp")]
      public const string Stamp = "\uf5bf";

      /// <summary>
      /// fa-standard-definition unicode value ("\ue28a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/standard-definition
      /// </summary>
      [Description("standard-definition")]
      public const string StandardDefinition = "\ue28a";

      /// <summary>
      /// fa-stapler unicode value ("\ue5af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stapler
      /// </summary>
      [Description("stapler")]
      public const string Stapler = "\ue5af";

      /// <summary>
      /// fa-star unicode value ("\uf005").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star
      /// </summary>
      [Description("star")]
      public const string Star = "\uf005";

      /// <summary>
      /// fa-star-and-crescent unicode value ("\uf699").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-and-crescent
      /// </summary>
      [Description("star-and-crescent")]
      public const string StarAndCrescent = "\uf699";

      /// <summary>
      /// fa-star-christmas unicode value ("\uf7d4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-christmas
      /// </summary>
      [Description("star-christmas")]
      public const string StarChristmas = "\uf7d4";

      /// <summary>
      /// fa-star-exclamation unicode value ("\uf2f3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-exclamation
      /// </summary>
      [Description("star-exclamation")]
      public const string StarExclamation = "\uf2f3";

      /// <summary>
      /// fa-star-half unicode value ("\uf089").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-half
      /// </summary>
      [Description("star-half")]
      public const string StarHalf = "\uf089";

      /// <summary>
      /// fa-star-half-stroke unicode value ("\uf5c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-half-stroke
      /// </summary>
      [Description("star-half-stroke")]
      public const string StarHalfStroke = "\uf5c0";

      /// <summary>
      /// fa-star-of-david unicode value ("\uf69a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-of-david
      /// </summary>
      [Description("star-of-david")]
      public const string StarOfDavid = "\uf69a";

      /// <summary>
      /// fa-star-of-life unicode value ("\uf621").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-of-life
      /// </summary>
      [Description("star-of-life")]
      public const string StarOfLife = "\uf621";

      /// <summary>
      /// fa-star-sharp unicode value ("\ue28b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-sharp
      /// </summary>
      [Description("star-sharp")]
      public const string StarSharp = "\ue28b";

      /// <summary>
      /// fa-star-sharp-half unicode value ("\ue28c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-sharp-half
      /// </summary>
      [Description("star-sharp-half")]
      public const string StarSharpHalf = "\ue28c";

      /// <summary>
      /// fa-star-sharp-half-stroke unicode value ("\ue28d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-sharp-half-stroke
      /// </summary>
      [Description("star-sharp-half-stroke")]
      public const string StarSharpHalfStroke = "\ue28d";

      /// <summary>
      /// fa-star-shooting unicode value ("\ue036").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/star-shooting
      /// </summary>
      [Description("star-shooting")]
      public const string StarShooting = "\ue036";

      /// <summary>
      /// fa-starfighter unicode value ("\ue037").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/starfighter
      /// </summary>
      [Description("starfighter")]
      public const string Starfighter = "\ue037";

      /// <summary>
      /// fa-starfighter-twin-ion-engine unicode value ("\ue038").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/starfighter-twin-ion-engine
      /// </summary>
      [Description("starfighter-twin-ion-engine")]
      public const string StarfighterTwinIonEngine = "\ue038";

      /// <summary>
      /// fa-starfighter-twin-ion-engine-advanced unicode value ("\ue28e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/starfighter-twin-ion-engine-advanced
      /// </summary>
      [Description("starfighter-twin-ion-engine-advanced")]
      public const string StarfighterTwinIonEngineAdvanced = "\ue28e";

      /// <summary>
      /// fa-stars unicode value ("\uf762").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stars
      /// </summary>
      [Description("stars")]
      public const string Stars = "\uf762";

      /// <summary>
      /// fa-starship unicode value ("\ue039").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/starship
      /// </summary>
      [Description("starship")]
      public const string Starship = "\ue039";

      /// <summary>
      /// fa-starship-freighter unicode value ("\ue03a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/starship-freighter
      /// </summary>
      [Description("starship-freighter")]
      public const string StarshipFreighter = "\ue03a";

      /// <summary>
      /// fa-staylinked unicode value ("\uf3f5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/staylinked
      /// </summary>
      [Description("staylinked")]
      public const string Staylinked = "\uf3f5";

      /// <summary>
      /// fa-steak unicode value ("\uf824").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/steak
      /// </summary>
      [Description("steak")]
      public const string Steak = "\uf824";

      /// <summary>
      /// fa-steam unicode value ("\uf1b6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/steam
      /// </summary>
      [Description("steam")]
      public const string Steam = "\uf1b6";

      /// <summary>
      /// fa-steam-symbol unicode value ("\uf3f6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/steam-symbol
      /// </summary>
      [Description("steam-symbol")]
      public const string SteamSymbol = "\uf3f6";

      /// <summary>
      /// fa-steering-wheel unicode value ("\uf622").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/steering-wheel
      /// </summary>
      [Description("steering-wheel")]
      public const string SteeringWheel = "\uf622";

      /// <summary>
      /// fa-sterling-sign unicode value ("\uf154").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sterling-sign
      /// </summary>
      [Description("sterling-sign")]
      public const string SterlingSign = "\uf154";

      /// <summary>
      /// fa-stethoscope unicode value ("\uf0f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stethoscope
      /// </summary>
      [Description("stethoscope")]
      public const string Stethoscope = "\uf0f1";

      /// <summary>
      /// fa-sticker-mule unicode value ("\uf3f7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.7.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sticker-mule
      /// </summary>
      [Description("sticker-mule")]
      public const string StickerMule = "\uf3f7";

      /// <summary>
      /// fa-stocking unicode value ("\uf7d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stocking
      /// </summary>
      [Description("stocking")]
      public const string Stocking = "\uf7d5";

      /// <summary>
      /// fa-stomach unicode value ("\uf623").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stomach
      /// </summary>
      [Description("stomach")]
      public const string Stomach = "\uf623";

      /// <summary>
      /// fa-stop unicode value ("\uf04d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stop
      /// </summary>
      [Description("stop")]
      public const string Stop = "\uf04d";

      /// <summary>
      /// fa-stopwatch unicode value ("\uf2f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stopwatch
      /// </summary>
      [Description("stopwatch")]
      public const string Stopwatch = "\uf2f2";

      /// <summary>
      /// fa-stopwatch-20 unicode value ("\ue06f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stopwatch-20
      /// </summary>
      [Description("stopwatch-20")]
      public const string Stopwatch20 = "\ue06f";

      /// <summary>
      /// fa-store unicode value ("\uf54e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/store
      /// </summary>
      [Description("store")]
      public const string Store = "\uf54e";

      /// <summary>
      /// fa-store-lock unicode value ("\ue4a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/store-lock
      /// </summary>
      [Description("store-lock")]
      public const string StoreLock = "\ue4a6";

      /// <summary>
      /// fa-store-slash unicode value ("\ue071").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/store-slash
      /// </summary>
      [Description("store-slash")]
      public const string StoreSlash = "\ue071";

      /// <summary>
      /// fa-strava unicode value ("\uf428").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/strava
      /// </summary>
      [Description("strava")]
      public const string Strava = "\uf428";

      /// <summary>
      /// fa-strawberry unicode value ("\ue32b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/strawberry
      /// </summary>
      [Description("strawberry")]
      public const string Strawberry = "\ue32b";

      /// <summary>
      /// fa-street-view unicode value ("\uf21d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/street-view
      /// </summary>
      [Description("street-view")]
      public const string StreetView = "\uf21d";

      /// <summary>
      /// fa-stretcher unicode value ("\uf825").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stretcher
      /// </summary>
      [Description("stretcher")]
      public const string Stretcher = "\uf825";

      /// <summary>
      /// fa-strikethrough unicode value ("\uf0cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/strikethrough
      /// </summary>
      [Description("strikethrough")]
      public const string Strikethrough = "\uf0cc";

      /// <summary>
      /// fa-stripe unicode value ("\uf429").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/stripe
      /// </summary>
      [Description("stripe")]
      public const string Stripe = "\uf429";

      /// <summary>
      /// fa-stripe-s unicode value ("\uf42a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.1', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stripe-s
      /// </summary>
      [Description("stripe-s")]
      public const string StripeS = "\uf42a";

      /// <summary>
      /// fa-stroopwafel unicode value ("\uf551").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stroopwafel
      /// </summary>
      [Description("stroopwafel")]
      public const string Stroopwafel = "\uf551";

      /// <summary>
      /// fa-stubber unicode value ("\ue5c7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.3.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stubber
      /// </summary>
      [Description("stubber")]
      public const string Stubber = "\ue5c7";

      /// <summary>
      /// fa-studiovinari unicode value ("\uf3f8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/studiovinari
      /// </summary>
      [Description("studiovinari")]
      public const string Studiovinari = "\uf3f8";

      /// <summary>
      /// fa-stumbleupon unicode value ("\uf1a4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stumbleupon
      /// </summary>
      [Description("stumbleupon")]
      public const string Stumbleupon = "\uf1a4";

      /// <summary>
      /// fa-stumbleupon-circle unicode value ("\uf1a3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/stumbleupon-circle
      /// </summary>
      [Description("stumbleupon-circle")]
      public const string StumbleuponCircle = "\uf1a3";

      /// <summary>
      /// fa-subscript unicode value ("\uf12c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/subscript
      /// </summary>
      [Description("subscript")]
      public const string Subscript = "\uf12c";

      /// <summary>
      /// fa-suitcase unicode value ("\uf0f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/suitcase
      /// </summary>
      [Description("suitcase")]
      public const string Suitcase = "\uf0f2";

      /// <summary>
      /// fa-suitcase-medical unicode value ("\uf0fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/suitcase-medical
      /// </summary>
      [Description("suitcase-medical")]
      public const string SuitcaseMedical = "\uf0fa";

      /// <summary>
      /// fa-suitcase-rolling unicode value ("\uf5c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/suitcase-rolling
      /// </summary>
      [Description("suitcase-rolling")]
      public const string SuitcaseRolling = "\uf5c1";

      /// <summary>
      /// fa-sun unicode value ("\uf185").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun
      /// </summary>
      [Description("sun")]
      public const string Sun = "\uf185";

      /// <summary>
      /// fa-sun-bright unicode value ("\ue28f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun-bright
      /// </summary>
      [Description("sun-bright")]
      public const string SunBright = "\ue28f";

      /// <summary>
      /// fa-sun-cloud unicode value ("\uf763").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun-cloud
      /// </summary>
      [Description("sun-cloud")]
      public const string SunCloud = "\uf763";

      /// <summary>
      /// fa-sun-dust unicode value ("\uf764").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun-dust
      /// </summary>
      [Description("sun-dust")]
      public const string SunDust = "\uf764";

      /// <summary>
      /// fa-sun-haze unicode value ("\uf765").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun-haze
      /// </summary>
      [Description("sun-haze")]
      public const string SunHaze = "\uf765";

      /// <summary>
      /// fa-sun-plant-wilt unicode value ("\ue57a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sun-plant-wilt
      /// </summary>
      [Description("sun-plant-wilt")]
      public const string SunPlantWilt = "\ue57a";

      /// <summary>
      /// fa-sunglasses unicode value ("\uf892").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sunglasses
      /// </summary>
      [Description("sunglasses")]
      public const string Sunglasses = "\uf892";

      /// <summary>
      /// fa-sunrise unicode value ("\uf766").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sunrise
      /// </summary>
      [Description("sunrise")]
      public const string Sunrise = "\uf766";

      /// <summary>
      /// fa-sunset unicode value ("\uf767").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sunset
      /// </summary>
      [Description("sunset")]
      public const string Sunset = "\uf767";

      /// <summary>
      /// fa-superpowers unicode value ("\uf2dd").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/superpowers
      /// </summary>
      [Description("superpowers")]
      public const string Superpowers = "\uf2dd";

      /// <summary>
      /// fa-superscript unicode value ("\uf12b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/superscript
      /// </summary>
      [Description("superscript")]
      public const string Superscript = "\uf12b";

      /// <summary>
      /// fa-supple unicode value ("\uf3f9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/supple
      /// </summary>
      [Description("supple")]
      public const string Supple = "\uf3f9";

      /// <summary>
      /// fa-suse unicode value ("\uf7d6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/suse
      /// </summary>
      [Description("suse")]
      public const string Suse = "\uf7d6";

      /// <summary>
      /// fa-sushi unicode value ("\ue48a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sushi
      /// </summary>
      [Description("sushi")]
      public const string Sushi = "\ue48a";

      /// <summary>
      /// fa-sushi-roll unicode value ("\ue48b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta3', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sushi-roll
      /// </summary>
      [Description("sushi-roll")]
      public const string SushiRoll = "\ue48b";

      /// <summary>
      /// fa-swatchbook unicode value ("\uf5c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/swatchbook
      /// </summary>
      [Description("swatchbook")]
      public const string Swatchbook = "\uf5c3";

      /// <summary>
      /// fa-swift unicode value ("\uf8e1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/swift
      /// </summary>
      [Description("swift")]
      public const string Swift = "\uf8e1";

      /// <summary>
      /// fa-sword unicode value ("\uf71c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sword
      /// </summary>
      [Description("sword")]
      public const string Sword = "\uf71c";

      /// <summary>
      /// fa-sword-laser unicode value ("\ue03b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sword-laser
      /// </summary>
      [Description("sword-laser")]
      public const string SwordLaser = "\ue03b";

      /// <summary>
      /// fa-sword-laser-alt unicode value ("\ue03c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/sword-laser-alt
      /// </summary>
      [Description("sword-laser-alt")]
      public const string SwordLaserAlt = "\ue03c";

      /// <summary>
      /// fa-swords unicode value ("\uf71d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/swords
      /// </summary>
      [Description("swords")]
      public const string Swords = "\uf71d";

      /// <summary>
      /// fa-swords-laser unicode value ("\ue03d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/swords-laser
      /// </summary>
      [Description("swords-laser")]
      public const string SwordsLaser = "\ue03d";

      /// <summary>
      /// fa-symbols unicode value ("\uf86e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/symbols
      /// </summary>
      [Description("symbols")]
      public const string Symbols = "\uf86e";

      /// <summary>
      /// fa-symfony unicode value ("\uf83d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/symfony
      /// </summary>
      [Description("symfony")]
      public const string Symfony = "\uf83d";

      /// <summary>
      /// fa-synagogue unicode value ("\uf69b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/synagogue
      /// </summary>
      [Description("synagogue")]
      public const string Synagogue = "\uf69b";

      /// <summary>
      /// fa-syringe unicode value ("\uf48e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/syringe
      /// </summary>
      [Description("syringe")]
      public const string Syringe = "\uf48e";

      /// <summary>
      /// fa-t unicode value ("\u0054").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/t
      /// </summary>
      [Description("t")]
      public const string T = "\u0054";

      /// <summary>
      /// fa-table unicode value ("\uf0ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table
      /// </summary>
      [Description("table")]
      public const string Table = "\uf0ce";

      /// <summary>
      /// fa-table-cells unicode value ("\uf00a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-cells
      /// </summary>
      [Description("table-cells")]
      public const string TableCells = "\uf00a";

      /// <summary>
      /// fa-table-cells-large unicode value ("\uf009").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-cells-large
      /// </summary>
      [Description("table-cells-large")]
      public const string TableCellsLarge = "\uf009";

      /// <summary>
      /// fa-table-columns unicode value ("\uf0db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-columns
      /// </summary>
      [Description("table-columns")]
      public const string TableColumns = "\uf0db";

      /// <summary>
      /// fa-table-layout unicode value ("\ue290").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-layout
      /// </summary>
      [Description("table-layout")]
      public const string TableLayout = "\ue290";

      /// <summary>
      /// fa-table-list unicode value ("\uf00b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-list
      /// </summary>
      [Description("table-list")]
      public const string TableList = "\uf00b";

      /// <summary>
      /// fa-table-picnic unicode value ("\ue32d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-picnic
      /// </summary>
      [Description("table-picnic")]
      public const string TablePicnic = "\ue32d";

      /// <summary>
      /// fa-table-pivot unicode value ("\ue291").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-pivot
      /// </summary>
      [Description("table-pivot")]
      public const string TablePivot = "\ue291";

      /// <summary>
      /// fa-table-rows unicode value ("\ue292").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-rows
      /// </summary>
      [Description("table-rows")]
      public const string TableRows = "\ue292";

      /// <summary>
      /// fa-table-tennis-paddle-ball unicode value ("\uf45d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-tennis-paddle-ball
      /// </summary>
      [Description("table-tennis-paddle-ball")]
      public const string TableTennisPaddleBall = "\uf45d";

      /// <summary>
      /// fa-table-tree unicode value ("\ue293").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/table-tree
      /// </summary>
      [Description("table-tree")]
      public const string TableTree = "\ue293";

      /// <summary>
      /// fa-tablet unicode value ("\uf3fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablet
      /// </summary>
      [Description("tablet")]
      public const string Tablet = "\uf3fb";

      /// <summary>
      /// fa-tablet-button unicode value ("\uf10a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablet-button
      /// </summary>
      [Description("tablet-button")]
      public const string TabletButton = "\uf10a";

      /// <summary>
      /// fa-tablet-rugged unicode value ("\uf48f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablet-rugged
      /// </summary>
      [Description("tablet-rugged")]
      public const string TabletRugged = "\uf48f";

      /// <summary>
      /// fa-tablet-screen unicode value ("\uf3fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablet-screen
      /// </summary>
      [Description("tablet-screen")]
      public const string TabletScreen = "\uf3fc";

      /// <summary>
      /// fa-tablet-screen-button unicode value ("\uf3fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablet-screen-button
      /// </summary>
      [Description("tablet-screen-button")]
      public const string TabletScreenButton = "\uf3fa";

      /// <summary>
      /// fa-tablets unicode value ("\uf490").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tablets
      /// </summary>
      [Description("tablets")]
      public const string Tablets = "\uf490";

      /// <summary>
      /// fa-tachograph-digital unicode value ("\uf566").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tachograph-digital
      /// </summary>
      [Description("tachograph-digital")]
      public const string TachographDigital = "\uf566";

      /// <summary>
      /// fa-taco unicode value ("\uf826").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/taco
      /// </summary>
      [Description("taco")]
      public const string Taco = "\uf826";

      /// <summary>
      /// fa-tag unicode value ("\uf02b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tag
      /// </summary>
      [Description("tag")]
      public const string Tag = "\uf02b";

      /// <summary>
      /// fa-tags unicode value ("\uf02c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tags
      /// </summary>
      [Description("tags")]
      public const string Tags = "\uf02c";

      /// <summary>
      /// fa-tally unicode value ("\uf69c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tally
      /// </summary>
      [Description("tally")]
      public const string Tally = "\uf69c";

      /// <summary>
      /// fa-tally-1 unicode value ("\ue294").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tally-1
      /// </summary>
      [Description("tally-1")]
      public const string Tally1 = "\ue294";

      /// <summary>
      /// fa-tally-2 unicode value ("\ue295").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tally-2
      /// </summary>
      [Description("tally-2")]
      public const string Tally2 = "\ue295";

      /// <summary>
      /// fa-tally-3 unicode value ("\ue296").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tally-3
      /// </summary>
      [Description("tally-3")]
      public const string Tally3 = "\ue296";

      /// <summary>
      /// fa-tally-4 unicode value ("\ue297").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tally-4
      /// </summary>
      [Description("tally-4")]
      public const string Tally4 = "\ue297";

      /// <summary>
      /// fa-tamale unicode value ("\ue451").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tamale
      /// </summary>
      [Description("tamale")]
      public const string Tamale = "\ue451";

      /// <summary>
      /// fa-tank-water unicode value ("\ue452").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tank-water
      /// </summary>
      [Description("tank-water")]
      public const string TankWater = "\ue452";

      /// <summary>
      /// fa-tape unicode value ("\uf4db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tape
      /// </summary>
      [Description("tape")]
      public const string Tape = "\uf4db";

      /// <summary>
      /// fa-tarp unicode value ("\ue57b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tarp
      /// </summary>
      [Description("tarp")]
      public const string Tarp = "\ue57b";

      /// <summary>
      /// fa-tarp-droplet unicode value ("\ue57c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tarp-droplet
      /// </summary>
      [Description("tarp-droplet")]
      public const string TarpDroplet = "\ue57c";

      /// <summary>
      /// fa-taxi unicode value ("\uf1ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/taxi
      /// </summary>
      [Description("taxi")]
      public const string Taxi = "\uf1ba";

      /// <summary>
      /// fa-taxi-bus unicode value ("\ue298").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/taxi-bus
      /// </summary>
      [Description("taxi-bus")]
      public const string TaxiBus = "\ue298";

      /// <summary>
      /// fa-teamspeak unicode value ("\uf4f9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/teamspeak
      /// </summary>
      [Description("teamspeak")]
      public const string Teamspeak = "\uf4f9";

      /// <summary>
      /// fa-teddy-bear unicode value ("\ue3cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/teddy-bear
      /// </summary>
      [Description("teddy-bear")]
      public const string TeddyBear = "\ue3cf";

      /// <summary>
      /// fa-teeth unicode value ("\uf62e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/teeth
      /// </summary>
      [Description("teeth")]
      public const string Teeth = "\uf62e";

      /// <summary>
      /// fa-teeth-open unicode value ("\uf62f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/teeth-open
      /// </summary>
      [Description("teeth-open")]
      public const string TeethOpen = "\uf62f";

      /// <summary>
      /// fa-telegram unicode value ("\uf2c6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/telegram
      /// </summary>
      [Description("telegram")]
      public const string Telegram = "\uf2c6";

      /// <summary>
      /// fa-telescope unicode value ("\ue03e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/telescope
      /// </summary>
      [Description("telescope")]
      public const string Telescope = "\ue03e";

      /// <summary>
      /// fa-temperature-arrow-down unicode value ("\ue03f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-arrow-down
      /// </summary>
      [Description("temperature-arrow-down")]
      public const string TemperatureArrowDown = "\ue03f";

      /// <summary>
      /// fa-temperature-arrow-up unicode value ("\ue040").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-arrow-up
      /// </summary>
      [Description("temperature-arrow-up")]
      public const string TemperatureArrowUp = "\ue040";

      /// <summary>
      /// fa-temperature-empty unicode value ("\uf2cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-empty
      /// </summary>
      [Description("temperature-empty")]
      public const string TemperatureEmpty = "\uf2cb";

      /// <summary>
      /// fa-temperature-full unicode value ("\uf2c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-full
      /// </summary>
      [Description("temperature-full")]
      public const string TemperatureFull = "\uf2c7";

      /// <summary>
      /// fa-temperature-half unicode value ("\uf2c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-half
      /// </summary>
      [Description("temperature-half")]
      public const string TemperatureHalf = "\uf2c9";

      /// <summary>
      /// fa-temperature-high unicode value ("\uf769").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-high
      /// </summary>
      [Description("temperature-high")]
      public const string TemperatureHigh = "\uf769";

      /// <summary>
      /// fa-temperature-list unicode value ("\ue299").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-list
      /// </summary>
      [Description("temperature-list")]
      public const string TemperatureList = "\ue299";

      /// <summary>
      /// fa-temperature-low unicode value ("\uf76b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-low
      /// </summary>
      [Description("temperature-low")]
      public const string TemperatureLow = "\uf76b";

      /// <summary>
      /// fa-temperature-quarter unicode value ("\uf2ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-quarter
      /// </summary>
      [Description("temperature-quarter")]
      public const string TemperatureQuarter = "\uf2ca";

      /// <summary>
      /// fa-temperature-snow unicode value ("\uf768").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-snow
      /// </summary>
      [Description("temperature-snow")]
      public const string TemperatureSnow = "\uf768";

      /// <summary>
      /// fa-temperature-sun unicode value ("\uf76a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-sun
      /// </summary>
      [Description("temperature-sun")]
      public const string TemperatureSun = "\uf76a";

      /// <summary>
      /// fa-temperature-three-quarters unicode value ("\uf2c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/temperature-three-quarters
      /// </summary>
      [Description("temperature-three-quarters")]
      public const string TemperatureThreeQuarters = "\uf2c8";

      /// <summary>
      /// fa-tencent-weibo unicode value ("\uf1d5").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tencent-weibo
      /// </summary>
      [Description("tencent-weibo")]
      public const string TencentWeibo = "\uf1d5";

      /// <summary>
      /// fa-tenge-sign unicode value ("\uf7d7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tenge-sign
      /// </summary>
      [Description("tenge-sign")]
      public const string TengeSign = "\uf7d7";

      /// <summary>
      /// fa-tennis-ball unicode value ("\uf45e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tennis-ball
      /// </summary>
      [Description("tennis-ball")]
      public const string TennisBall = "\uf45e";

      /// <summary>
      /// fa-tent unicode value ("\ue57d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tent
      /// </summary>
      [Description("tent")]
      public const string Tent = "\ue57d";

      /// <summary>
      /// fa-tent-arrow-down-to-line unicode value ("\ue57e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tent-arrow-down-to-line
      /// </summary>
      [Description("tent-arrow-down-to-line")]
      public const string TentArrowDownToLine = "\ue57e";

      /// <summary>
      /// fa-tent-arrow-left-right unicode value ("\ue57f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tent-arrow-left-right
      /// </summary>
      [Description("tent-arrow-left-right")]
      public const string TentArrowLeftRight = "\ue57f";

      /// <summary>
      /// fa-tent-arrow-turn-left unicode value ("\ue580").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tent-arrow-turn-left
      /// </summary>
      [Description("tent-arrow-turn-left")]
      public const string TentArrowTurnLeft = "\ue580";

      /// <summary>
      /// fa-tent-arrows-down unicode value ("\ue581").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tent-arrows-down
      /// </summary>
      [Description("tent-arrows-down")]
      public const string TentArrowsDown = "\ue581";

      /// <summary>
      /// fa-tents unicode value ("\ue582").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tents
      /// </summary>
      [Description("tents")]
      public const string Tents = "\ue582";

      /// <summary>
      /// fa-terminal unicode value ("\uf120").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/terminal
      /// </summary>
      [Description("terminal")]
      public const string Terminal = "\uf120";

      /// <summary>
      /// fa-text unicode value ("\uf893").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/text
      /// </summary>
      [Description("text")]
      public const string Text = "\uf893";

      /// <summary>
      /// fa-text-height unicode value ("\uf034").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/text-height
      /// </summary>
      [Description("text-height")]
      public const string TextHeight = "\uf034";

      /// <summary>
      /// fa-text-size unicode value ("\uf894").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/text-size
      /// </summary>
      [Description("text-size")]
      public const string TextSize = "\uf894";

      /// <summary>
      /// fa-text-slash unicode value ("\uf87d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/text-slash
      /// </summary>
      [Description("text-slash")]
      public const string TextSlash = "\uf87d";

      /// <summary>
      /// fa-text-width unicode value ("\uf035").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/text-width
      /// </summary>
      [Description("text-width")]
      public const string TextWidth = "\uf035";

      /// <summary>
      /// fa-the-red-yeti unicode value ("\uf69d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/the-red-yeti
      /// </summary>
      [Description("the-red-yeti")]
      public const string TheRedYeti = "\uf69d";

      /// <summary>
      /// fa-themeco unicode value ("\uf5c6").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/themeco
      /// </summary>
      [Description("themeco")]
      public const string Themeco = "\uf5c6";

      /// <summary>
      /// fa-themeisle unicode value ("\uf2b2").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/themeisle
      /// </summary>
      [Description("themeisle")]
      public const string Themeisle = "\uf2b2";

      /// <summary>
      /// fa-thermometer unicode value ("\uf491").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/thermometer
      /// </summary>
      [Description("thermometer")]
      public const string Thermometer = "\uf491";

      /// <summary>
      /// fa-theta unicode value ("\uf69e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/theta
      /// </summary>
      [Description("theta")]
      public const string Theta = "\uf69e";

      /// <summary>
      /// fa-think-peaks unicode value ("\uf731").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.2', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/think-peaks
      /// </summary>
      [Description("think-peaks")]
      public const string ThinkPeaks = "\uf731";

      /// <summary>
      /// fa-thought-bubble unicode value ("\ue32e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/thought-bubble
      /// </summary>
      [Description("thought-bubble")]
      public const string ThoughtBubble = "\ue32e";

      /// <summary>
      /// fa-thumbs-down unicode value ("\uf165").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/thumbs-down
      /// </summary>
      [Description("thumbs-down")]
      public const string ThumbsDown = "\uf165";

      /// <summary>
      /// fa-thumbs-up unicode value ("\uf164").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/thumbs-up
      /// </summary>
      [Description("thumbs-up")]
      public const string ThumbsUp = "\uf164";

      /// <summary>
      /// fa-thumbtack unicode value ("\uf08d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/thumbtack
      /// </summary>
      [Description("thumbtack")]
      public const string Thumbtack = "\uf08d";

      /// <summary>
      /// fa-tick unicode value ("\ue32f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tick
      /// </summary>
      [Description("tick")]
      public const string Tick = "\ue32f";

      /// <summary>
      /// fa-ticket unicode value ("\uf145").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ticket
      /// </summary>
      [Description("ticket")]
      public const string Ticket = "\uf145";

      /// <summary>
      /// fa-ticket-airline unicode value ("\ue29a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ticket-airline
      /// </summary>
      [Description("ticket-airline")]
      public const string TicketAirline = "\ue29a";

      /// <summary>
      /// fa-ticket-simple unicode value ("\uf3ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ticket-simple
      /// </summary>
      [Description("ticket-simple")]
      public const string TicketSimple = "\uf3ff";

      /// <summary>
      /// fa-tickets-airline unicode value ("\ue29b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tickets-airline
      /// </summary>
      [Description("tickets-airline")]
      public const string TicketsAirline = "\ue29b";

      /// <summary>
      /// fa-tiktok unicode value ("\ue07b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tiktok
      /// </summary>
      [Description("tiktok")]
      public const string Tiktok = "\ue07b";

      /// <summary>
      /// fa-tilde unicode value ("\u007e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tilde
      /// </summary>
      [Description("tilde")]
      public const string Tilde = "\u007e";

      /// <summary>
      /// fa-timeline unicode value ("\ue29c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/timeline
      /// </summary>
      [Description("timeline")]
      public const string Timeline = "\ue29c";

      /// <summary>
      /// fa-timeline-arrow unicode value ("\ue29d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/timeline-arrow
      /// </summary>
      [Description("timeline-arrow")]
      public const string TimelineArrow = "\ue29d";

      /// <summary>
      /// fa-timer unicode value ("\ue29e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/timer
      /// </summary>
      [Description("timer")]
      public const string Timer = "\ue29e";

      /// <summary>
      /// fa-tire unicode value ("\uf631").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tire
      /// </summary>
      [Description("tire")]
      public const string Tire = "\uf631";

      /// <summary>
      /// fa-tire-flat unicode value ("\uf632").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tire-flat
      /// </summary>
      [Description("tire-flat")]
      public const string TireFlat = "\uf632";

      /// <summary>
      /// fa-tire-pressure-warning unicode value ("\uf633").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tire-pressure-warning
      /// </summary>
      [Description("tire-pressure-warning")]
      public const string TirePressureWarning = "\uf633";

      /// <summary>
      /// fa-tire-rugged unicode value ("\uf634").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tire-rugged
      /// </summary>
      [Description("tire-rugged")]
      public const string TireRugged = "\uf634";

      /// <summary>
      /// fa-toggle-large-off unicode value ("\ue5b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toggle-large-off
      /// </summary>
      [Description("toggle-large-off")]
      public const string ToggleLargeOff = "\ue5b0";

      /// <summary>
      /// fa-toggle-large-on unicode value ("\ue5b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toggle-large-on
      /// </summary>
      [Description("toggle-large-on")]
      public const string ToggleLargeOn = "\ue5b1";

      /// <summary>
      /// fa-toggle-off unicode value ("\uf204").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toggle-off
      /// </summary>
      [Description("toggle-off")]
      public const string ToggleOff = "\uf204";

      /// <summary>
      /// fa-toggle-on unicode value ("\uf205").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toggle-on
      /// </summary>
      [Description("toggle-on")]
      public const string ToggleOn = "\uf205";

      /// <summary>
      /// fa-toilet unicode value ("\uf7d8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet
      /// </summary>
      [Description("toilet")]
      public const string Toilet = "\uf7d8";

      /// <summary>
      /// fa-toilet-paper unicode value ("\uf71e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper
      /// </summary>
      [Description("toilet-paper")]
      public const string ToiletPaper = "\uf71e";

      /// <summary>
      /// fa-toilet-paper-blank unicode value ("\uf71f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-blank
      /// </summary>
      [Description("toilet-paper-blank")]
      public const string ToiletPaperBlank = "\uf71f";

      /// <summary>
      /// fa-toilet-paper-blank-under unicode value ("\ue29f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-blank-under
      /// </summary>
      [Description("toilet-paper-blank-under")]
      public const string ToiletPaperBlankUnder = "\ue29f";

      /// <summary>
      /// fa-toilet-paper-check unicode value ("\ue5b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-check
      /// </summary>
      [Description("toilet-paper-check")]
      public const string ToiletPaperCheck = "\ue5b2";

      /// <summary>
      /// fa-toilet-paper-slash unicode value ("\ue072").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-slash
      /// </summary>
      [Description("toilet-paper-slash")]
      public const string ToiletPaperSlash = "\ue072";

      /// <summary>
      /// fa-toilet-paper-under unicode value ("\ue2a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-under
      /// </summary>
      [Description("toilet-paper-under")]
      public const string ToiletPaperUnder = "\ue2a0";

      /// <summary>
      /// fa-toilet-paper-under-slash unicode value ("\ue2a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-under-slash
      /// </summary>
      [Description("toilet-paper-under-slash")]
      public const string ToiletPaperUnderSlash = "\ue2a1";

      /// <summary>
      /// fa-toilet-paper-xmark unicode value ("\ue5b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-paper-xmark
      /// </summary>
      [Description("toilet-paper-xmark")]
      public const string ToiletPaperXmark = "\ue5b3";

      /// <summary>
      /// fa-toilet-portable unicode value ("\ue583").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilet-portable
      /// </summary>
      [Description("toilet-portable")]
      public const string ToiletPortable = "\ue583";

      /// <summary>
      /// fa-toilets-portable unicode value ("\ue584").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toilets-portable
      /// </summary>
      [Description("toilets-portable")]
      public const string ToiletsPortable = "\ue584";

      /// <summary>
      /// fa-tomato unicode value ("\ue330").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tomato
      /// </summary>
      [Description("tomato")]
      public const string Tomato = "\ue330";

      /// <summary>
      /// fa-tombstone unicode value ("\uf720").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tombstone
      /// </summary>
      [Description("tombstone")]
      public const string Tombstone = "\uf720";

      /// <summary>
      /// fa-tombstone-blank unicode value ("\uf721").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tombstone-blank
      /// </summary>
      [Description("tombstone-blank")]
      public const string TombstoneBlank = "\uf721";

      /// <summary>
      /// fa-toolbox unicode value ("\uf552").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toolbox
      /// </summary>
      [Description("toolbox")]
      public const string Toolbox = "\uf552";

      /// <summary>
      /// fa-tooth unicode value ("\uf5c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tooth
      /// </summary>
      [Description("tooth")]
      public const string Tooth = "\uf5c9";

      /// <summary>
      /// fa-toothbrush unicode value ("\uf635").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/toothbrush
      /// </summary>
      [Description("toothbrush")]
      public const string Toothbrush = "\uf635";

      /// <summary>
      /// fa-torii-gate unicode value ("\uf6a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/torii-gate
      /// </summary>
      [Description("torii-gate")]
      public const string ToriiGate = "\uf6a1";

      /// <summary>
      /// fa-tornado unicode value ("\uf76f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tornado
      /// </summary>
      [Description("tornado")]
      public const string Tornado = "\uf76f";

      /// <summary>
      /// fa-tower-broadcast unicode value ("\uf519").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tower-broadcast
      /// </summary>
      [Description("tower-broadcast")]
      public const string TowerBroadcast = "\uf519";

      /// <summary>
      /// fa-tower-cell unicode value ("\ue585").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tower-cell
      /// </summary>
      [Description("tower-cell")]
      public const string TowerCell = "\ue585";

      /// <summary>
      /// fa-tower-control unicode value ("\ue2a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tower-control
      /// </summary>
      [Description("tower-control")]
      public const string TowerControl = "\ue2a2";

      /// <summary>
      /// fa-tower-observation unicode value ("\ue586").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tower-observation
      /// </summary>
      [Description("tower-observation")]
      public const string TowerObservation = "\ue586";

      /// <summary>
      /// fa-tractor unicode value ("\uf722").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tractor
      /// </summary>
      [Description("tractor")]
      public const string Tractor = "\uf722";

      /// <summary>
      /// fa-trade-federation unicode value ("\uf513").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.0.12'.
      /// <para/>
      /// See https://fontawesome.com/icons/trade-federation
      /// </summary>
      [Description("trade-federation")]
      public const string TradeFederation = "\uf513";

      /// <summary>
      /// fa-trademark unicode value ("\uf25c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trademark
      /// </summary>
      [Description("trademark")]
      public const string Trademark = "\uf25c";

      /// <summary>
      /// fa-traffic-cone unicode value ("\uf636").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/traffic-cone
      /// </summary>
      [Description("traffic-cone")]
      public const string TrafficCone = "\uf636";

      /// <summary>
      /// fa-traffic-light unicode value ("\uf637").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/traffic-light
      /// </summary>
      [Description("traffic-light")]
      public const string TrafficLight = "\uf637";

      /// <summary>
      /// fa-traffic-light-go unicode value ("\uf638").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/traffic-light-go
      /// </summary>
      [Description("traffic-light-go")]
      public const string TrafficLightGo = "\uf638";

      /// <summary>
      /// fa-traffic-light-slow unicode value ("\uf639").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/traffic-light-slow
      /// </summary>
      [Description("traffic-light-slow")]
      public const string TrafficLightSlow = "\uf639";

      /// <summary>
      /// fa-traffic-light-stop unicode value ("\uf63a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/traffic-light-stop
      /// </summary>
      [Description("traffic-light-stop")]
      public const string TrafficLightStop = "\uf63a";

      /// <summary>
      /// fa-trailer unicode value ("\ue041").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trailer
      /// </summary>
      [Description("trailer")]
      public const string Trailer = "\ue041";

      /// <summary>
      /// fa-train unicode value ("\uf238").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train
      /// </summary>
      [Description("train")]
      public const string Train = "\uf238";

      /// <summary>
      /// fa-train-subway unicode value ("\uf239").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train-subway
      /// </summary>
      [Description("train-subway")]
      public const string TrainSubway = "\uf239";

      /// <summary>
      /// fa-train-subway-tunnel unicode value ("\ue2a3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train-subway-tunnel
      /// </summary>
      [Description("train-subway-tunnel")]
      public const string TrainSubwayTunnel = "\ue2a3";

      /// <summary>
      /// fa-train-track unicode value ("\ue453").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train-track
      /// </summary>
      [Description("train-track")]
      public const string TrainTrack = "\ue453";

      /// <summary>
      /// fa-train-tram unicode value ("\ue5b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train-tram
      /// </summary>
      [Description("train-tram")]
      public const string TrainTram = "\ue5b4";

      /// <summary>
      /// fa-train-tunnel unicode value ("\ue454").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/train-tunnel
      /// </summary>
      [Description("train-tunnel")]
      public const string TrainTunnel = "\ue454";

      /// <summary>
      /// fa-transformer-bolt unicode value ("\ue2a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transformer-bolt
      /// </summary>
      [Description("transformer-bolt")]
      public const string TransformerBolt = "\ue2a4";

      /// <summary>
      /// fa-transgender unicode value ("\uf225").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transgender
      /// </summary>
      [Description("transgender")]
      public const string Transgender = "\uf225";

      /// <summary>
      /// fa-transporter unicode value ("\ue042").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter
      /// </summary>
      [Description("transporter")]
      public const string Transporter = "\ue042";

      /// <summary>
      /// fa-transporter-1 unicode value ("\ue043").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-1
      /// </summary>
      [Description("transporter-1")]
      public const string Transporter1 = "\ue043";

      /// <summary>
      /// fa-transporter-2 unicode value ("\ue044").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-2
      /// </summary>
      [Description("transporter-2")]
      public const string Transporter2 = "\ue044";

      /// <summary>
      /// fa-transporter-3 unicode value ("\ue045").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-3
      /// </summary>
      [Description("transporter-3")]
      public const string Transporter3 = "\ue045";

      /// <summary>
      /// fa-transporter-4 unicode value ("\ue2a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-4
      /// </summary>
      [Description("transporter-4")]
      public const string Transporter4 = "\ue2a5";

      /// <summary>
      /// fa-transporter-5 unicode value ("\ue2a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-5
      /// </summary>
      [Description("transporter-5")]
      public const string Transporter5 = "\ue2a6";

      /// <summary>
      /// fa-transporter-6 unicode value ("\ue2a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-6
      /// </summary>
      [Description("transporter-6")]
      public const string Transporter6 = "\ue2a7";

      /// <summary>
      /// fa-transporter-7 unicode value ("\ue2a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-7
      /// </summary>
      [Description("transporter-7")]
      public const string Transporter7 = "\ue2a8";

      /// <summary>
      /// fa-transporter-empty unicode value ("\ue046").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/transporter-empty
      /// </summary>
      [Description("transporter-empty")]
      public const string TransporterEmpty = "\ue046";

      /// <summary>
      /// fa-trash unicode value ("\uf1f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash
      /// </summary>
      [Description("trash")]
      public const string Trash = "\uf1f8";

      /// <summary>
      /// fa-trash-arrow-up unicode value ("\uf829").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-arrow-up
      /// </summary>
      [Description("trash-arrow-up")]
      public const string TrashArrowUp = "\uf829";

      /// <summary>
      /// fa-trash-can unicode value ("\uf2ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can
      /// </summary>
      [Description("trash-can")]
      public const string TrashCan = "\uf2ed";

      /// <summary>
      /// fa-trash-can-arrow-up unicode value ("\uf82a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-arrow-up
      /// </summary>
      [Description("trash-can-arrow-up")]
      public const string TrashCanArrowUp = "\uf82a";

      /// <summary>
      /// fa-trash-can-check unicode value ("\ue2a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-check
      /// </summary>
      [Description("trash-can-check")]
      public const string TrashCanCheck = "\ue2a9";

      /// <summary>
      /// fa-trash-can-clock unicode value ("\ue2aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-clock
      /// </summary>
      [Description("trash-can-clock")]
      public const string TrashCanClock = "\ue2aa";

      /// <summary>
      /// fa-trash-can-list unicode value ("\ue2ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-list
      /// </summary>
      [Description("trash-can-list")]
      public const string TrashCanList = "\ue2ab";

      /// <summary>
      /// fa-trash-can-plus unicode value ("\ue2ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-plus
      /// </summary>
      [Description("trash-can-plus")]
      public const string TrashCanPlus = "\ue2ac";

      /// <summary>
      /// fa-trash-can-slash unicode value ("\ue2ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-slash
      /// </summary>
      [Description("trash-can-slash")]
      public const string TrashCanSlash = "\ue2ad";

      /// <summary>
      /// fa-trash-can-undo unicode value ("\uf896").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-undo
      /// </summary>
      [Description("trash-can-undo")]
      public const string TrashCanUndo = "\uf896";

      /// <summary>
      /// fa-trash-can-xmark unicode value ("\ue2ae").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-can-xmark
      /// </summary>
      [Description("trash-can-xmark")]
      public const string TrashCanXmark = "\ue2ae";

      /// <summary>
      /// fa-trash-check unicode value ("\ue2af").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-check
      /// </summary>
      [Description("trash-check")]
      public const string TrashCheck = "\ue2af";

      /// <summary>
      /// fa-trash-clock unicode value ("\ue2b0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-clock
      /// </summary>
      [Description("trash-clock")]
      public const string TrashClock = "\ue2b0";

      /// <summary>
      /// fa-trash-list unicode value ("\ue2b1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-list
      /// </summary>
      [Description("trash-list")]
      public const string TrashList = "\ue2b1";

      /// <summary>
      /// fa-trash-plus unicode value ("\ue2b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-plus
      /// </summary>
      [Description("trash-plus")]
      public const string TrashPlus = "\ue2b2";

      /// <summary>
      /// fa-trash-slash unicode value ("\ue2b3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-slash
      /// </summary>
      [Description("trash-slash")]
      public const string TrashSlash = "\ue2b3";

      /// <summary>
      /// fa-trash-undo unicode value ("\uf895").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-undo
      /// </summary>
      [Description("trash-undo")]
      public const string TrashUndo = "\uf895";

      /// <summary>
      /// fa-trash-xmark unicode value ("\ue2b4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trash-xmark
      /// </summary>
      [Description("trash-xmark")]
      public const string TrashXmark = "\ue2b4";

      /// <summary>
      /// fa-treasure-chest unicode value ("\uf723").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/treasure-chest
      /// </summary>
      [Description("treasure-chest")]
      public const string TreasureChest = "\uf723";

      /// <summary>
      /// fa-tree unicode value ("\uf1bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree
      /// </summary>
      [Description("tree")]
      public const string Tree = "\uf1bb";

      /// <summary>
      /// fa-tree-christmas unicode value ("\uf7db").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-christmas
      /// </summary>
      [Description("tree-christmas")]
      public const string TreeChristmas = "\uf7db";

      /// <summary>
      /// fa-tree-city unicode value ("\ue587").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-city
      /// </summary>
      [Description("tree-city")]
      public const string TreeCity = "\ue587";

      /// <summary>
      /// fa-tree-deciduous unicode value ("\uf400").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-deciduous
      /// </summary>
      [Description("tree-deciduous")]
      public const string TreeDeciduous = "\uf400";

      /// <summary>
      /// fa-tree-decorated unicode value ("\uf7dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-decorated
      /// </summary>
      [Description("tree-decorated")]
      public const string TreeDecorated = "\uf7dc";

      /// <summary>
      /// fa-tree-large unicode value ("\uf7dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-large
      /// </summary>
      [Description("tree-large")]
      public const string TreeLarge = "\uf7dd";

      /// <summary>
      /// fa-tree-palm unicode value ("\uf82b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tree-palm
      /// </summary>
      [Description("tree-palm")]
      public const string TreePalm = "\uf82b";

      /// <summary>
      /// fa-trees unicode value ("\uf724").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trees
      /// </summary>
      [Description("trees")]
      public const string Trees = "\uf724";

      /// <summary>
      /// fa-trello unicode value ("\uf181").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trello
      /// </summary>
      [Description("trello")]
      public const string Trello = "\uf181";

      /// <summary>
      /// fa-triangle unicode value ("\uf2ec").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/triangle
      /// </summary>
      [Description("triangle")]
      public const string Triangle = "\uf2ec";

      /// <summary>
      /// fa-triangle-exclamation unicode value ("\uf071").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/triangle-exclamation
      /// </summary>
      [Description("triangle-exclamation")]
      public const string TriangleExclamation = "\uf071";

      /// <summary>
      /// fa-triangle-instrument unicode value ("\uf8e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/triangle-instrument
      /// </summary>
      [Description("triangle-instrument")]
      public const string TriangleInstrument = "\uf8e2";

      /// <summary>
      /// fa-triangle-person-digging unicode value ("\uf85d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/triangle-person-digging
      /// </summary>
      [Description("triangle-person-digging")]
      public const string TrianglePersonDigging = "\uf85d";

      /// <summary>
      /// fa-tricycle unicode value ("\ue5c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tricycle
      /// </summary>
      [Description("tricycle")]
      public const string Tricycle = "\ue5c3";

      /// <summary>
      /// fa-tricycle-adult unicode value ("\ue5c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tricycle-adult
      /// </summary>
      [Description("tricycle-adult")]
      public const string TricycleAdult = "\ue5c4";

      /// <summary>
      /// fa-trillium unicode value ("\ue588").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trillium
      /// </summary>
      [Description("trillium")]
      public const string Trillium = "\ue588";

      /// <summary>
      /// fa-trophy unicode value ("\uf091").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trophy
      /// </summary>
      [Description("trophy")]
      public const string Trophy = "\uf091";

      /// <summary>
      /// fa-trophy-star unicode value ("\uf2eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trophy-star
      /// </summary>
      [Description("trophy-star")]
      public const string TrophyStar = "\uf2eb";

      /// <summary>
      /// fa-trowel unicode value ("\ue589").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trowel
      /// </summary>
      [Description("trowel")]
      public const string Trowel = "\ue589";

      /// <summary>
      /// fa-trowel-bricks unicode value ("\ue58a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trowel-bricks
      /// </summary>
      [Description("trowel-bricks")]
      public const string TrowelBricks = "\ue58a";

      /// <summary>
      /// fa-truck unicode value ("\uf0d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck
      /// </summary>
      [Description("truck")]
      public const string Truck = "\uf0d1";

      /// <summary>
      /// fa-truck-arrow-right unicode value ("\ue58b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-arrow-right
      /// </summary>
      [Description("truck-arrow-right")]
      public const string TruckArrowRight = "\ue58b";

      /// <summary>
      /// fa-truck-bolt unicode value ("\ue3d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-bolt
      /// </summary>
      [Description("truck-bolt")]
      public const string TruckBolt = "\ue3d0";

      /// <summary>
      /// fa-truck-clock unicode value ("\uf48c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-clock
      /// </summary>
      [Description("truck-clock")]
      public const string TruckClock = "\uf48c";

      /// <summary>
      /// fa-truck-container unicode value ("\uf4dc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-container
      /// </summary>
      [Description("truck-container")]
      public const string TruckContainer = "\uf4dc";

      /// <summary>
      /// fa-truck-container-empty unicode value ("\ue2b5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-container-empty
      /// </summary>
      [Description("truck-container-empty")]
      public const string TruckContainerEmpty = "\ue2b5";

      /// <summary>
      /// fa-truck-droplet unicode value ("\ue58c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-droplet
      /// </summary>
      [Description("truck-droplet")]
      public const string TruckDroplet = "\ue58c";

      /// <summary>
      /// fa-truck-fast unicode value ("\uf48b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-fast
      /// </summary>
      [Description("truck-fast")]
      public const string TruckFast = "\uf48b";

      /// <summary>
      /// fa-truck-field unicode value ("\ue58d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-field
      /// </summary>
      [Description("truck-field")]
      public const string TruckField = "\ue58d";

      /// <summary>
      /// fa-truck-field-un unicode value ("\ue58e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-field-un
      /// </summary>
      [Description("truck-field-un")]
      public const string TruckFieldUn = "\ue58e";

      /// <summary>
      /// fa-truck-flatbed unicode value ("\ue2b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-flatbed
      /// </summary>
      [Description("truck-flatbed")]
      public const string TruckFlatbed = "\ue2b6";

      /// <summary>
      /// fa-truck-front unicode value ("\ue2b7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-front
      /// </summary>
      [Description("truck-front")]
      public const string TruckFront = "\ue2b7";

      /// <summary>
      /// fa-truck-medical unicode value ("\uf0f9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-medical
      /// </summary>
      [Description("truck-medical")]
      public const string TruckMedical = "\uf0f9";

      /// <summary>
      /// fa-truck-monster unicode value ("\uf63b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-monster
      /// </summary>
      [Description("truck-monster")]
      public const string TruckMonster = "\uf63b";

      /// <summary>
      /// fa-truck-moving unicode value ("\uf4df").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-moving
      /// </summary>
      [Description("truck-moving")]
      public const string TruckMoving = "\uf4df";

      /// <summary>
      /// fa-truck-pickup unicode value ("\uf63c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-pickup
      /// </summary>
      [Description("truck-pickup")]
      public const string TruckPickup = "\uf63c";

      /// <summary>
      /// fa-truck-plane unicode value ("\ue58f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-plane
      /// </summary>
      [Description("truck-plane")]
      public const string TruckPlane = "\ue58f";

      /// <summary>
      /// fa-truck-plow unicode value ("\uf7de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-plow
      /// </summary>
      [Description("truck-plow")]
      public const string TruckPlow = "\uf7de";

      /// <summary>
      /// fa-truck-ramp unicode value ("\uf4e0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-ramp
      /// </summary>
      [Description("truck-ramp")]
      public const string TruckRamp = "\uf4e0";

      /// <summary>
      /// fa-truck-ramp-box unicode value ("\uf4de").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-ramp-box
      /// </summary>
      [Description("truck-ramp-box")]
      public const string TruckRampBox = "\uf4de";

      /// <summary>
      /// fa-truck-ramp-couch unicode value ("\uf4dd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-ramp-couch
      /// </summary>
      [Description("truck-ramp-couch")]
      public const string TruckRampCouch = "\uf4dd";

      /// <summary>
      /// fa-truck-tow unicode value ("\ue2b8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/truck-tow
      /// </summary>
      [Description("truck-tow")]
      public const string TruckTow = "\ue2b8";

      /// <summary>
      /// fa-trumpet unicode value ("\uf8e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/trumpet
      /// </summary>
      [Description("trumpet")]
      public const string Trumpet = "\uf8e3";

      /// <summary>
      /// fa-tty unicode value ("\uf1e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tty
      /// </summary>
      [Description("tty")]
      public const string Tty = "\uf1e4";

      /// <summary>
      /// fa-tty-answer unicode value ("\ue2b9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tty-answer
      /// </summary>
      [Description("tty-answer")]
      public const string TtyAnswer = "\ue2b9";

      /// <summary>
      /// fa-tugrik-sign unicode value ("\ue2ba").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tugrik-sign
      /// </summary>
      [Description("tugrik-sign")]
      public const string TugrikSign = "\ue2ba";

      /// <summary>
      /// fa-tumblr unicode value ("\uf173").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tumblr
      /// </summary>
      [Description("tumblr")]
      public const string Tumblr = "\uf173";

      /// <summary>
      /// fa-turkey unicode value ("\uf725").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turkey
      /// </summary>
      [Description("turkey")]
      public const string Turkey = "\uf725";

      /// <summary>
      /// fa-turkish-lira-sign unicode value ("\ue2bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turkish-lira-sign
      /// </summary>
      [Description("turkish-lira-sign")]
      public const string TurkishLiraSign = "\ue2bb";

      /// <summary>
      /// fa-turn-down unicode value ("\uf3be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turn-down
      /// </summary>
      [Description("turn-down")]
      public const string TurnDown = "\uf3be";

      /// <summary>
      /// fa-turn-down-left unicode value ("\ue331").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turn-down-left
      /// </summary>
      [Description("turn-down-left")]
      public const string TurnDownLeft = "\ue331";

      /// <summary>
      /// fa-turn-down-right unicode value ("\ue455").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turn-down-right
      /// </summary>
      [Description("turn-down-right")]
      public const string TurnDownRight = "\ue455";

      /// <summary>
      /// fa-turn-up unicode value ("\uf3bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turn-up
      /// </summary>
      [Description("turn-up")]
      public const string TurnUp = "\uf3bf";

      /// <summary>
      /// fa-turntable unicode value ("\uf8e4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turntable
      /// </summary>
      [Description("turntable")]
      public const string Turntable = "\uf8e4";

      /// <summary>
      /// fa-turtle unicode value ("\uf726").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/turtle
      /// </summary>
      [Description("turtle")]
      public const string Turtle = "\uf726";

      /// <summary>
      /// fa-tv unicode value ("\uf26c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tv
      /// </summary>
      [Description("tv")]
      public const string Tv = "\uf26c";

      /// <summary>
      /// fa-tv-music unicode value ("\uf8e6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tv-music
      /// </summary>
      [Description("tv-music")]
      public const string TvMusic = "\uf8e6";

      /// <summary>
      /// fa-tv-retro unicode value ("\uf401").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/tv-retro
      /// </summary>
      [Description("tv-retro")]
      public const string TvRetro = "\uf401";

      /// <summary>
      /// fa-twitch unicode value ("\uf1e8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.12.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/twitch
      /// </summary>
      [Description("twitch")]
      public const string Twitch = "\uf1e8";

      /// <summary>
      /// fa-twitter unicode value ("\uf099").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/twitter
      /// </summary>
      [Description("twitter")]
      public const string Twitter = "\uf099";

      /// <summary>
      /// fa-typewriter unicode value ("\uf8e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/typewriter
      /// </summary>
      [Description("typewriter")]
      public const string Typewriter = "\uf8e7";

      /// <summary>
      /// fa-typo3 unicode value ("\uf42b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.1', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/typo3
      /// </summary>
      [Description("typo3")]
      public const string Typo3 = "\uf42b";

      /// <summary>
      /// fa-u unicode value ("\u0055").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/u
      /// </summary>
      [Description("u")]
      public const string U = "\u0055";

      /// <summary>
      /// fa-uber unicode value ("\uf402").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/uber
      /// </summary>
      [Description("uber")]
      public const string Uber = "\uf402";

      /// <summary>
      /// fa-ubuntu unicode value ("\uf7df").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ubuntu
      /// </summary>
      [Description("ubuntu")]
      public const string Ubuntu = "\uf7df";

      /// <summary>
      /// fa-ufo unicode value ("\ue047").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ufo
      /// </summary>
      [Description("ufo")]
      public const string Ufo = "\ue047";

      /// <summary>
      /// fa-ufo-beam unicode value ("\ue048").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ufo-beam
      /// </summary>
      [Description("ufo-beam")]
      public const string UfoBeam = "\ue048";

      /// <summary>
      /// fa-uikit unicode value ("\uf403").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/uikit
      /// </summary>
      [Description("uikit")]
      public const string Uikit = "\uf403";

      /// <summary>
      /// fa-umbraco unicode value ("\uf8e8").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '5.11.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/umbraco
      /// </summary>
      [Description("umbraco")]
      public const string Umbraco = "\uf8e8";

      /// <summary>
      /// fa-umbrella unicode value ("\uf0e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/umbrella
      /// </summary>
      [Description("umbrella")]
      public const string Umbrella = "\uf0e9";

      /// <summary>
      /// fa-umbrella-beach unicode value ("\uf5ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/umbrella-beach
      /// </summary>
      [Description("umbrella-beach")]
      public const string UmbrellaBeach = "\uf5ca";

      /// <summary>
      /// fa-umbrella-simple unicode value ("\ue2bc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/umbrella-simple
      /// </summary>
      [Description("umbrella-simple")]
      public const string UmbrellaSimple = "\ue2bc";

      /// <summary>
      /// fa-uncharted unicode value ("\ue084").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/uncharted
      /// </summary>
      [Description("uncharted")]
      public const string Uncharted = "\ue084";

      /// <summary>
      /// fa-underline unicode value ("\uf0cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/underline
      /// </summary>
      [Description("underline")]
      public const string Underline = "\uf0cd";

      /// <summary>
      /// fa-unicorn unicode value ("\uf727").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/unicorn
      /// </summary>
      [Description("unicorn")]
      public const string Unicorn = "\uf727";

      /// <summary>
      /// fa-uniform-martial-arts unicode value ("\ue3d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/uniform-martial-arts
      /// </summary>
      [Description("uniform-martial-arts")]
      public const string UniformMartialArts = "\ue3d1";

      /// <summary>
      /// fa-union unicode value ("\uf6a2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/union
      /// </summary>
      [Description("union")]
      public const string Union = "\uf6a2";

      /// <summary>
      /// fa-uniregistry unicode value ("\uf404").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/uniregistry
      /// </summary>
      [Description("uniregistry")]
      public const string Uniregistry = "\uf404";

      /// <summary>
      /// fa-unity unicode value ("\ue049").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.0.0-beta3'.
      /// <para/>
      /// See https://fontawesome.com/icons/unity
      /// </summary>
      [Description("unity")]
      public const string Unity = "\ue049";

      /// <summary>
      /// fa-universal-access unicode value ("\uf29a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/universal-access
      /// </summary>
      [Description("universal-access")]
      public const string UniversalAccess = "\uf29a";

      /// <summary>
      /// fa-unlock unicode value ("\uf09c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/unlock
      /// </summary>
      [Description("unlock")]
      public const string Unlock = "\uf09c";

      /// <summary>
      /// fa-unlock-keyhole unicode value ("\uf13e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/unlock-keyhole
      /// </summary>
      [Description("unlock-keyhole")]
      public const string UnlockKeyhole = "\uf13e";

      /// <summary>
      /// fa-unsplash unicode value ("\ue07c").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.13.1', Last Modified in '5.14.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/unsplash
      /// </summary>
      [Description("unsplash")]
      public const string Unsplash = "\ue07c";

      /// <summary>
      /// fa-untappd unicode value ("\uf405").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/untappd
      /// </summary>
      [Description("untappd")]
      public const string Untappd = "\uf405";

      /// <summary>
      /// fa-up unicode value ("\uf357").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up
      /// </summary>
      [Description("up")]
      public const string Up = "\uf357";

      /// <summary>
      /// fa-up-down unicode value ("\uf338").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-down
      /// </summary>
      [Description("up-down")]
      public const string UpDown = "\uf338";

      /// <summary>
      /// fa-up-down-left-right unicode value ("\uf0b2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-down-left-right
      /// </summary>
      [Description("up-down-left-right")]
      public const string UpDownLeftRight = "\uf0b2";

      /// <summary>
      /// fa-up-from-bracket unicode value ("\ue590").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-from-bracket
      /// </summary>
      [Description("up-from-bracket")]
      public const string UpFromBracket = "\ue590";

      /// <summary>
      /// fa-up-from-dotted-line unicode value ("\ue456").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-from-dotted-line
      /// </summary>
      [Description("up-from-dotted-line")]
      public const string UpFromDottedLine = "\ue456";

      /// <summary>
      /// fa-up-from-line unicode value ("\uf346").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-from-line
      /// </summary>
      [Description("up-from-line")]
      public const string UpFromLine = "\uf346";

      /// <summary>
      /// fa-up-left unicode value ("\ue2bd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-left
      /// </summary>
      [Description("up-left")]
      public const string UpLeft = "\ue2bd";

      /// <summary>
      /// fa-up-long unicode value ("\uf30c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-long
      /// </summary>
      [Description("up-long")]
      public const string UpLong = "\uf30c";

      /// <summary>
      /// fa-up-right unicode value ("\ue2be").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-right
      /// </summary>
      [Description("up-right")]
      public const string UpRight = "\ue2be";

      /// <summary>
      /// fa-up-right-and-down-left-from-center unicode value ("\uf424").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-right-and-down-left-from-center
      /// </summary>
      [Description("up-right-and-down-left-from-center")]
      public const string UpRightAndDownLeftFromCenter = "\uf424";

      /// <summary>
      /// fa-up-right-from-square unicode value ("\uf35d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-right-from-square
      /// </summary>
      [Description("up-right-from-square")]
      public const string UpRightFromSquare = "\uf35d";

      /// <summary>
      /// fa-up-to-dotted-line unicode value ("\ue457").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-to-dotted-line
      /// </summary>
      [Description("up-to-dotted-line")]
      public const string UpToDottedLine = "\ue457";

      /// <summary>
      /// fa-up-to-line unicode value ("\uf34d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/up-to-line
      /// </summary>
      [Description("up-to-line")]
      public const string UpToLine = "\uf34d";

      /// <summary>
      /// fa-upload unicode value ("\uf093").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/upload
      /// </summary>
      [Description("upload")]
      public const string Upload = "\uf093";

      /// <summary>
      /// fa-ups unicode value ("\uf7e0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ups
      /// </summary>
      [Description("ups")]
      public const string Ups = "\uf7e0";

      /// <summary>
      /// fa-usb unicode value ("\uf287").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.5.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/usb
      /// </summary>
      [Description("usb")]
      public const string Usb = "\uf287";

      /// <summary>
      /// fa-usb-drive unicode value ("\uf8e9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/usb-drive
      /// </summary>
      [Description("usb-drive")]
      public const string UsbDrive = "\uf8e9";

      /// <summary>
      /// fa-user unicode value ("\uf007").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user
      /// </summary>
      [Description("user")]
      public const string User = "\uf007";

      /// <summary>
      /// fa-user-alien unicode value ("\ue04a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-alien
      /// </summary>
      [Description("user-alien")]
      public const string UserAlien = "\ue04a";

      /// <summary>
      /// fa-user-astronaut unicode value ("\uf4fb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-astronaut
      /// </summary>
      [Description("user-astronaut")]
      public const string UserAstronaut = "\uf4fb";

      /// <summary>
      /// fa-user-bounty-hunter unicode value ("\ue2bf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-bounty-hunter
      /// </summary>
      [Description("user-bounty-hunter")]
      public const string UserBountyHunter = "\ue2bf";

      /// <summary>
      /// fa-user-check unicode value ("\uf4fc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-check
      /// </summary>
      [Description("user-check")]
      public const string UserCheck = "\uf4fc";

      /// <summary>
      /// fa-user-chef unicode value ("\ue3d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-chef
      /// </summary>
      [Description("user-chef")]
      public const string UserChef = "\ue3d2";

      /// <summary>
      /// fa-user-clock unicode value ("\uf4fd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-clock
      /// </summary>
      [Description("user-clock")]
      public const string UserClock = "\uf4fd";

      /// <summary>
      /// fa-user-cowboy unicode value ("\uf8ea").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-cowboy
      /// </summary>
      [Description("user-cowboy")]
      public const string UserCowboy = "\uf8ea";

      /// <summary>
      /// fa-user-crown unicode value ("\uf6a4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-crown
      /// </summary>
      [Description("user-crown")]
      public const string UserCrown = "\uf6a4";

      /// <summary>
      /// fa-user-doctor unicode value ("\uf0f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-doctor
      /// </summary>
      [Description("user-doctor")]
      public const string UserDoctor = "\uf0f0";

      /// <summary>
      /// fa-user-doctor-hair unicode value ("\ue458").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-doctor-hair
      /// </summary>
      [Description("user-doctor-hair")]
      public const string UserDoctorHair = "\ue458";

      /// <summary>
      /// fa-user-doctor-hair-long unicode value ("\ue459").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-doctor-hair-long
      /// </summary>
      [Description("user-doctor-hair-long")]
      public const string UserDoctorHairLong = "\ue459";

      /// <summary>
      /// fa-user-doctor-message unicode value ("\uf82e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-doctor-message
      /// </summary>
      [Description("user-doctor-message")]
      public const string UserDoctorMessage = "\uf82e";

      /// <summary>
      /// fa-user-gear unicode value ("\uf4fe").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-gear
      /// </summary>
      [Description("user-gear")]
      public const string UserGear = "\uf4fe";

      /// <summary>
      /// fa-user-graduate unicode value ("\uf501").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-graduate
      /// </summary>
      [Description("user-graduate")]
      public const string UserGraduate = "\uf501";

      /// <summary>
      /// fa-user-group unicode value ("\uf500").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-group
      /// </summary>
      [Description("user-group")]
      public const string UserGroup = "\uf500";

      /// <summary>
      /// fa-user-group-crown unicode value ("\uf6a5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-group-crown
      /// </summary>
      [Description("user-group-crown")]
      public const string UserGroupCrown = "\uf6a5";

      /// <summary>
      /// fa-user-group-simple unicode value ("\ue603").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-group-simple
      /// </summary>
      [Description("user-group-simple")]
      public const string UserGroupSimple = "\ue603";

      /// <summary>
      /// fa-user-hair unicode value ("\ue45a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-hair
      /// </summary>
      [Description("user-hair")]
      public const string UserHair = "\ue45a";

      /// <summary>
      /// fa-user-hair-buns unicode value ("\ue3d3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-hair-buns
      /// </summary>
      [Description("user-hair-buns")]
      public const string UserHairBuns = "\ue3d3";

      /// <summary>
      /// fa-user-hair-long unicode value ("\ue45b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-hair-long
      /// </summary>
      [Description("user-hair-long")]
      public const string UserHairLong = "\ue45b";

      /// <summary>
      /// fa-user-hair-mullet unicode value ("\ue45c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-hair-mullet
      /// </summary>
      [Description("user-hair-mullet")]
      public const string UserHairMullet = "\ue45c";

      /// <summary>
      /// fa-user-headset unicode value ("\uf82d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-headset
      /// </summary>
      [Description("user-headset")]
      public const string UserHeadset = "\uf82d";

      /// <summary>
      /// fa-user-helmet-safety unicode value ("\uf82c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-helmet-safety
      /// </summary>
      [Description("user-helmet-safety")]
      public const string UserHelmetSafety = "\uf82c";

      /// <summary>
      /// fa-user-injured unicode value ("\uf728").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-injured
      /// </summary>
      [Description("user-injured")]
      public const string UserInjured = "\uf728";

      /// <summary>
      /// fa-user-large unicode value ("\uf406").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-large
      /// </summary>
      [Description("user-large")]
      public const string UserLarge = "\uf406";

      /// <summary>
      /// fa-user-large-slash unicode value ("\uf4fa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-large-slash
      /// </summary>
      [Description("user-large-slash")]
      public const string UserLargeSlash = "\uf4fa";

      /// <summary>
      /// fa-user-lock unicode value ("\uf502").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-lock
      /// </summary>
      [Description("user-lock")]
      public const string UserLock = "\uf502";

      /// <summary>
      /// fa-user-magnifying-glass unicode value ("\ue5c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.2.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-magnifying-glass
      /// </summary>
      [Description("user-magnifying-glass")]
      public const string UserMagnifyingGlass = "\ue5c5";

      /// <summary>
      /// fa-user-minus unicode value ("\uf503").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-minus
      /// </summary>
      [Description("user-minus")]
      public const string UserMinus = "\uf503";

      /// <summary>
      /// fa-user-music unicode value ("\uf8eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-music
      /// </summary>
      [Description("user-music")]
      public const string UserMusic = "\uf8eb";

      /// <summary>
      /// fa-user-ninja unicode value ("\uf504").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-ninja
      /// </summary>
      [Description("user-ninja")]
      public const string UserNinja = "\uf504";

      /// <summary>
      /// fa-user-nurse unicode value ("\uf82f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-nurse
      /// </summary>
      [Description("user-nurse")]
      public const string UserNurse = "\uf82f";

      /// <summary>
      /// fa-user-nurse-hair unicode value ("\ue45d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-nurse-hair
      /// </summary>
      [Description("user-nurse-hair")]
      public const string UserNurseHair = "\ue45d";

      /// <summary>
      /// fa-user-nurse-hair-long unicode value ("\ue45e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-nurse-hair-long
      /// </summary>
      [Description("user-nurse-hair-long")]
      public const string UserNurseHairLong = "\ue45e";

      /// <summary>
      /// fa-user-pen unicode value ("\uf4ff").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-pen
      /// </summary>
      [Description("user-pen")]
      public const string UserPen = "\uf4ff";

      /// <summary>
      /// fa-user-pilot unicode value ("\ue2c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-pilot
      /// </summary>
      [Description("user-pilot")]
      public const string UserPilot = "\ue2c0";

      /// <summary>
      /// fa-user-pilot-tie unicode value ("\ue2c1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-pilot-tie
      /// </summary>
      [Description("user-pilot-tie")]
      public const string UserPilotTie = "\ue2c1";

      /// <summary>
      /// fa-user-plus unicode value ("\uf234").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-plus
      /// </summary>
      [Description("user-plus")]
      public const string UserPlus = "\uf234";

      /// <summary>
      /// fa-user-police unicode value ("\ue333").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-police
      /// </summary>
      [Description("user-police")]
      public const string UserPolice = "\ue333";

      /// <summary>
      /// fa-user-police-tie unicode value ("\ue334").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-police-tie
      /// </summary>
      [Description("user-police-tie")]
      public const string UserPoliceTie = "\ue334";

      /// <summary>
      /// fa-user-robot unicode value ("\ue04b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-robot
      /// </summary>
      [Description("user-robot")]
      public const string UserRobot = "\ue04b";

      /// <summary>
      /// fa-user-robot-xmarks unicode value ("\ue4a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-robot-xmarks
      /// </summary>
      [Description("user-robot-xmarks")]
      public const string UserRobotXmarks = "\ue4a7";

      /// <summary>
      /// fa-user-secret unicode value ("\uf21b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-secret
      /// </summary>
      [Description("user-secret")]
      public const string UserSecret = "\uf21b";

      /// <summary>
      /// fa-user-shakespeare unicode value ("\ue2c2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-shakespeare
      /// </summary>
      [Description("user-shakespeare")]
      public const string UserShakespeare = "\ue2c2";

      /// <summary>
      /// fa-user-shield unicode value ("\uf505").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-shield
      /// </summary>
      [Description("user-shield")]
      public const string UserShield = "\uf505";

      /// <summary>
      /// fa-user-slash unicode value ("\uf506").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-slash
      /// </summary>
      [Description("user-slash")]
      public const string UserSlash = "\uf506";

      /// <summary>
      /// fa-user-tag unicode value ("\uf507").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-tag
      /// </summary>
      [Description("user-tag")]
      public const string UserTag = "\uf507";

      /// <summary>
      /// fa-user-tie unicode value ("\uf508").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-tie
      /// </summary>
      [Description("user-tie")]
      public const string UserTie = "\uf508";

      /// <summary>
      /// fa-user-tie-hair unicode value ("\ue45f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-tie-hair
      /// </summary>
      [Description("user-tie-hair")]
      public const string UserTieHair = "\ue45f";

      /// <summary>
      /// fa-user-tie-hair-long unicode value ("\ue460").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-tie-hair-long
      /// </summary>
      [Description("user-tie-hair-long")]
      public const string UserTieHairLong = "\ue460";

      /// <summary>
      /// fa-user-unlock unicode value ("\ue058").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-unlock
      /// </summary>
      [Description("user-unlock")]
      public const string UserUnlock = "\ue058";

      /// <summary>
      /// fa-user-visor unicode value ("\ue04c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-visor
      /// </summary>
      [Description("user-visor")]
      public const string UserVisor = "\ue04c";

      /// <summary>
      /// fa-user-vneck unicode value ("\ue461").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-vneck
      /// </summary>
      [Description("user-vneck")]
      public const string UserVneck = "\ue461";

      /// <summary>
      /// fa-user-vneck-hair unicode value ("\ue462").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-vneck-hair
      /// </summary>
      [Description("user-vneck-hair")]
      public const string UserVneckHair = "\ue462";

      /// <summary>
      /// fa-user-vneck-hair-long unicode value ("\ue463").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-vneck-hair-long
      /// </summary>
      [Description("user-vneck-hair-long")]
      public const string UserVneckHairLong = "\ue463";

      /// <summary>
      /// fa-user-xmark unicode value ("\uf235").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/user-xmark
      /// </summary>
      [Description("user-xmark")]
      public const string UserXmark = "\uf235";

      /// <summary>
      /// fa-users unicode value ("\uf0c0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users
      /// </summary>
      [Description("users")]
      public const string Users = "\uf0c0";

      /// <summary>
      /// fa-users-between-lines unicode value ("\ue591").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-between-lines
      /// </summary>
      [Description("users-between-lines")]
      public const string UsersBetweenLines = "\ue591";

      /// <summary>
      /// fa-users-gear unicode value ("\uf509").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.11', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-gear
      /// </summary>
      [Description("users-gear")]
      public const string UsersGear = "\uf509";

      /// <summary>
      /// fa-users-line unicode value ("\ue592").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-line
      /// </summary>
      [Description("users-line")]
      public const string UsersLine = "\ue592";

      /// <summary>
      /// fa-users-medical unicode value ("\uf830").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-medical
      /// </summary>
      [Description("users-medical")]
      public const string UsersMedical = "\uf830";

      /// <summary>
      /// fa-users-rays unicode value ("\ue593").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-rays
      /// </summary>
      [Description("users-rays")]
      public const string UsersRays = "\ue593";

      /// <summary>
      /// fa-users-rectangle unicode value ("\ue594").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-rectangle
      /// </summary>
      [Description("users-rectangle")]
      public const string UsersRectangle = "\ue594";

      /// <summary>
      /// fa-users-slash unicode value ("\ue073").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-slash
      /// </summary>
      [Description("users-slash")]
      public const string UsersSlash = "\ue073";

      /// <summary>
      /// fa-users-viewfinder unicode value ("\ue595").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/users-viewfinder
      /// </summary>
      [Description("users-viewfinder")]
      public const string UsersViewfinder = "\ue595";

      /// <summary>
      /// fa-usps unicode value ("\uf7e1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/usps
      /// </summary>
      [Description("usps")]
      public const string Usps = "\uf7e1";

      /// <summary>
      /// fa-ussunnah unicode value ("\uf407").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/ussunnah
      /// </summary>
      [Description("ussunnah")]
      public const string Ussunnah = "\uf407";

      /// <summary>
      /// fa-utensils unicode value ("\uf2e7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/utensils
      /// </summary>
      [Description("utensils")]
      public const string Utensils = "\uf2e7";

      /// <summary>
      /// fa-utensils-slash unicode value ("\ue464").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/utensils-slash
      /// </summary>
      [Description("utensils-slash")]
      public const string UtensilsSlash = "\ue464";

      /// <summary>
      /// fa-utility-pole unicode value ("\ue2c3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/utility-pole
      /// </summary>
      [Description("utility-pole")]
      public const string UtilityPole = "\ue2c3";

      /// <summary>
      /// fa-utility-pole-double unicode value ("\ue2c4").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/utility-pole-double
      /// </summary>
      [Description("utility-pole-double")]
      public const string UtilityPoleDouble = "\ue2c4";

      /// <summary>
      /// fa-v unicode value ("\u0056").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/v
      /// </summary>
      [Description("v")]
      public const string V = "\u0056";

      /// <summary>
      /// fa-vaadin unicode value ("\uf408").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vaadin
      /// </summary>
      [Description("vaadin")]
      public const string Vaadin = "\uf408";

      /// <summary>
      /// fa-vacuum unicode value ("\ue04d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vacuum
      /// </summary>
      [Description("vacuum")]
      public const string Vacuum = "\ue04d";

      /// <summary>
      /// fa-vacuum-robot unicode value ("\ue04e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vacuum-robot
      /// </summary>
      [Description("vacuum-robot")]
      public const string VacuumRobot = "\ue04e";

      /// <summary>
      /// fa-value-absolute unicode value ("\uf6a6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/value-absolute
      /// </summary>
      [Description("value-absolute")]
      public const string ValueAbsolute = "\uf6a6";

      /// <summary>
      /// fa-van-shuttle unicode value ("\uf5b6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/van-shuttle
      /// </summary>
      [Description("van-shuttle")]
      public const string VanShuttle = "\uf5b6";

      /// <summary>
      /// fa-vault unicode value ("\ue2c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vault
      /// </summary>
      [Description("vault")]
      public const string Vault = "\ue2c5";

      /// <summary>
      /// fa-vector-circle unicode value ("\ue2c6").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vector-circle
      /// </summary>
      [Description("vector-circle")]
      public const string VectorCircle = "\ue2c6";

      /// <summary>
      /// fa-vector-polygon unicode value ("\ue2c7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vector-polygon
      /// </summary>
      [Description("vector-polygon")]
      public const string VectorPolygon = "\ue2c7";

      /// <summary>
      /// fa-vector-square unicode value ("\uf5cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vector-square
      /// </summary>
      [Description("vector-square")]
      public const string VectorSquare = "\uf5cb";

      /// <summary>
      /// fa-vent-damper unicode value ("\ue465").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vent-damper
      /// </summary>
      [Description("vent-damper")]
      public const string VentDamper = "\ue465";

      /// <summary>
      /// fa-venus unicode value ("\uf221").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/venus
      /// </summary>
      [Description("venus")]
      public const string Venus = "\uf221";

      /// <summary>
      /// fa-venus-double unicode value ("\uf226").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/venus-double
      /// </summary>
      [Description("venus-double")]
      public const string VenusDouble = "\uf226";

      /// <summary>
      /// fa-venus-mars unicode value ("\uf228").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/venus-mars
      /// </summary>
      [Description("venus-mars")]
      public const string VenusMars = "\uf228";

      /// <summary>
      /// fa-vest unicode value ("\ue085").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vest
      /// </summary>
      [Description("vest")]
      public const string Vest = "\ue085";

      /// <summary>
      /// fa-vest-patches unicode value ("\ue086").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vest-patches
      /// </summary>
      [Description("vest-patches")]
      public const string VestPatches = "\ue086";

      /// <summary>
      /// fa-viacoin unicode value ("\uf237").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/viacoin
      /// </summary>
      [Description("viacoin")]
      public const string Viacoin = "\uf237";

      /// <summary>
      /// fa-viadeo unicode value ("\uf2a9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/viadeo
      /// </summary>
      [Description("viadeo")]
      public const string Viadeo = "\uf2a9";

      /// <summary>
      /// fa-vial unicode value ("\uf492").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vial
      /// </summary>
      [Description("vial")]
      public const string Vial = "\uf492";

      /// <summary>
      /// fa-vial-circle-check unicode value ("\ue596").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vial-circle-check
      /// </summary>
      [Description("vial-circle-check")]
      public const string VialCircleCheck = "\ue596";

      /// <summary>
      /// fa-vial-virus unicode value ("\ue597").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vial-virus
      /// </summary>
      [Description("vial-virus")]
      public const string VialVirus = "\ue597";

      /// <summary>
      /// fa-vials unicode value ("\uf493").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vials
      /// </summary>
      [Description("vials")]
      public const string Vials = "\uf493";

      /// <summary>
      /// fa-viber unicode value ("\uf409").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/viber
      /// </summary>
      [Description("viber")]
      public const string Viber = "\uf409";

      /// <summary>
      /// fa-video unicode value ("\uf03d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/video
      /// </summary>
      [Description("video")]
      public const string Video = "\uf03d";

      /// <summary>
      /// fa-video-arrow-down-left unicode value ("\ue2c8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/video-arrow-down-left
      /// </summary>
      [Description("video-arrow-down-left")]
      public const string VideoArrowDownLeft = "\ue2c8";

      /// <summary>
      /// fa-video-arrow-up-right unicode value ("\ue2c9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/video-arrow-up-right
      /// </summary>
      [Description("video-arrow-up-right")]
      public const string VideoArrowUpRight = "\ue2c9";

      /// <summary>
      /// fa-video-plus unicode value ("\uf4e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/video-plus
      /// </summary>
      [Description("video-plus")]
      public const string VideoPlus = "\uf4e1";

      /// <summary>
      /// fa-video-slash unicode value ("\uf4e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/video-slash
      /// </summary>
      [Description("video-slash")]
      public const string VideoSlash = "\uf4e2";

      /// <summary>
      /// fa-vihara unicode value ("\uf6a7").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vihara
      /// </summary>
      [Description("vihara")]
      public const string Vihara = "\uf6a7";

      /// <summary>
      /// fa-vimeo unicode value ("\uf40a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vimeo
      /// </summary>
      [Description("vimeo")]
      public const string Vimeo = "\uf40a";

      /// <summary>
      /// fa-vimeo-v unicode value ("\uf27d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vimeo-v
      /// </summary>
      [Description("vimeo-v")]
      public const string VimeoV = "\uf27d";

      /// <summary>
      /// fa-vine unicode value ("\uf1ca").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vine
      /// </summary>
      [Description("vine")]
      public const string Vine = "\uf1ca";

      /// <summary>
      /// fa-violin unicode value ("\uf8ed").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/violin
      /// </summary>
      [Description("violin")]
      public const string Violin = "\uf8ed";

      /// <summary>
      /// fa-virus unicode value ("\ue074").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/virus
      /// </summary>
      [Description("virus")]
      public const string Virus = "\ue074";

      /// <summary>
      /// fa-virus-covid unicode value ("\ue4a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/virus-covid
      /// </summary>
      [Description("virus-covid")]
      public const string VirusCovid = "\ue4a8";

      /// <summary>
      /// fa-virus-covid-slash unicode value ("\ue4a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/virus-covid-slash
      /// </summary>
      [Description("virus-covid-slash")]
      public const string VirusCovidSlash = "\ue4a9";

      /// <summary>
      /// fa-virus-slash unicode value ("\ue075").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/virus-slash
      /// </summary>
      [Description("virus-slash")]
      public const string VirusSlash = "\ue075";

      /// <summary>
      /// fa-viruses unicode value ("\ue076").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.13.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/viruses
      /// </summary>
      [Description("viruses")]
      public const string Viruses = "\ue076";

      /// <summary>
      /// fa-vk unicode value ("\uf189").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.0.0-beta3'.
      /// <para/>
      /// See https://fontawesome.com/icons/vk
      /// </summary>
      [Description("vk")]
      public const string Vk = "\uf189";

      /// <summary>
      /// fa-vnv unicode value ("\uf40b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vnv
      /// </summary>
      [Description("vnv")]
      public const string Vnv = "\uf40b";

      /// <summary>
      /// fa-voicemail unicode value ("\uf897").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/voicemail
      /// </summary>
      [Description("voicemail")]
      public const string Voicemail = "\uf897";

      /// <summary>
      /// fa-volcano unicode value ("\uf770").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volcano
      /// </summary>
      [Description("volcano")]
      public const string Volcano = "\uf770";

      /// <summary>
      /// fa-volleyball unicode value ("\uf45f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volleyball
      /// </summary>
      [Description("volleyball")]
      public const string Volleyball = "\uf45f";

      /// <summary>
      /// fa-volume unicode value ("\uf6a8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume
      /// </summary>
      [Description("volume")]
      public const string Volume = "\uf6a8";

      /// <summary>
      /// fa-volume-high unicode value ("\uf028").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume-high
      /// </summary>
      [Description("volume-high")]
      public const string VolumeHigh = "\uf028";

      /// <summary>
      /// fa-volume-low unicode value ("\uf027").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume-low
      /// </summary>
      [Description("volume-low")]
      public const string VolumeLow = "\uf027";

      /// <summary>
      /// fa-volume-off unicode value ("\uf026").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume-off
      /// </summary>
      [Description("volume-off")]
      public const string VolumeOff = "\uf026";

      /// <summary>
      /// fa-volume-slash unicode value ("\uf2e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume-slash
      /// </summary>
      [Description("volume-slash")]
      public const string VolumeSlash = "\uf2e2";

      /// <summary>
      /// fa-volume-xmark unicode value ("\uf6a9").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/volume-xmark
      /// </summary>
      [Description("volume-xmark")]
      public const string VolumeXmark = "\uf6a9";

      /// <summary>
      /// fa-vr-cardboard unicode value ("\uf729").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vr-cardboard
      /// </summary>
      [Description("vr-cardboard")]
      public const string VrCardboard = "\uf729";

      /// <summary>
      /// fa-vuejs unicode value ("\uf41f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/vuejs
      /// </summary>
      [Description("vuejs")]
      public const string Vuejs = "\uf41f";

      /// <summary>
      /// fa-w unicode value ("\u0057").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/w
      /// </summary>
      [Description("w")]
      public const string W = "\u0057";

      /// <summary>
      /// fa-waffle unicode value ("\ue466").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta2', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/waffle
      /// </summary>
      [Description("waffle")]
      public const string Waffle = "\ue466";

      /// <summary>
      /// fa-wagon-covered unicode value ("\uf8ee").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wagon-covered
      /// </summary>
      [Description("wagon-covered")]
      public const string WagonCovered = "\uf8ee";

      /// <summary>
      /// fa-walker unicode value ("\uf831").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/walker
      /// </summary>
      [Description("walker")]
      public const string Walker = "\uf831";

      /// <summary>
      /// fa-walkie-talkie unicode value ("\uf8ef").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/walkie-talkie
      /// </summary>
      [Description("walkie-talkie")]
      public const string WalkieTalkie = "\uf8ef";

      /// <summary>
      /// fa-wallet unicode value ("\uf555").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.13', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wallet
      /// </summary>
      [Description("wallet")]
      public const string Wallet = "\uf555";

      /// <summary>
      /// fa-wand unicode value ("\uf72a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wand
      /// </summary>
      [Description("wand")]
      public const string Wand = "\uf72a";

      /// <summary>
      /// fa-wand-magic unicode value ("\uf0d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wand-magic
      /// </summary>
      [Description("wand-magic")]
      public const string WandMagic = "\uf0d0";

      /// <summary>
      /// fa-wand-magic-sparkles unicode value ("\ue2ca").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wand-magic-sparkles
      /// </summary>
      [Description("wand-magic-sparkles")]
      public const string WandMagicSparkles = "\ue2ca";

      /// <summary>
      /// fa-wand-sparkles unicode value ("\uf72b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wand-sparkles
      /// </summary>
      [Description("wand-sparkles")]
      public const string WandSparkles = "\uf72b";

      /// <summary>
      /// fa-warehouse unicode value ("\uf494").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/warehouse
      /// </summary>
      [Description("warehouse")]
      public const string Warehouse = "\uf494";

      /// <summary>
      /// fa-warehouse-full unicode value ("\uf495").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/warehouse-full
      /// </summary>
      [Description("warehouse-full")]
      public const string WarehouseFull = "\uf495";

      /// <summary>
      /// fa-washing-machine unicode value ("\uf898").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/washing-machine
      /// </summary>
      [Description("washing-machine")]
      public const string WashingMachine = "\uf898";

      /// <summary>
      /// fa-watch unicode value ("\uf2e1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watch
      /// </summary>
      [Description("watch")]
      public const string Watch = "\uf2e1";

      /// <summary>
      /// fa-watch-apple unicode value ("\ue2cb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watch-apple
      /// </summary>
      [Description("watch-apple")]
      public const string WatchApple = "\ue2cb";

      /// <summary>
      /// fa-watch-calculator unicode value ("\uf8f0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watch-calculator
      /// </summary>
      [Description("watch-calculator")]
      public const string WatchCalculator = "\uf8f0";

      /// <summary>
      /// fa-watch-fitness unicode value ("\uf63e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watch-fitness
      /// </summary>
      [Description("watch-fitness")]
      public const string WatchFitness = "\uf63e";

      /// <summary>
      /// fa-watch-smart unicode value ("\ue2cc").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watch-smart
      /// </summary>
      [Description("watch-smart")]
      public const string WatchSmart = "\ue2cc";

      /// <summary>
      /// fa-watchman-monitoring unicode value ("\ue087").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watchman-monitoring
      /// </summary>
      [Description("watchman-monitoring")]
      public const string WatchmanMonitoring = "\ue087";

      /// <summary>
      /// fa-water unicode value ("\uf773").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/water
      /// </summary>
      [Description("water")]
      public const string Water = "\uf773";

      /// <summary>
      /// fa-water-arrow-down unicode value ("\uf774").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/water-arrow-down
      /// </summary>
      [Description("water-arrow-down")]
      public const string WaterArrowDown = "\uf774";

      /// <summary>
      /// fa-water-arrow-up unicode value ("\uf775").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/water-arrow-up
      /// </summary>
      [Description("water-arrow-up")]
      public const string WaterArrowUp = "\uf775";

      /// <summary>
      /// fa-water-ladder unicode value ("\uf5c5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/water-ladder
      /// </summary>
      [Description("water-ladder")]
      public const string WaterLadder = "\uf5c5";

      /// <summary>
      /// fa-watermelon-slice unicode value ("\ue337").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/watermelon-slice
      /// </summary>
      [Description("watermelon-slice")]
      public const string WatermelonSlice = "\ue337";

      /// <summary>
      /// fa-wave-pulse unicode value ("\uf5f8").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wave-pulse
      /// </summary>
      [Description("wave-pulse")]
      public const string WavePulse = "\uf5f8";

      /// <summary>
      /// fa-wave-sine unicode value ("\uf899").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wave-sine
      /// </summary>
      [Description("wave-sine")]
      public const string WaveSine = "\uf899";

      /// <summary>
      /// fa-wave-square unicode value ("\uf83e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wave-square
      /// </summary>
      [Description("wave-square")]
      public const string WaveSquare = "\uf83e";

      /// <summary>
      /// fa-wave-triangle unicode value ("\uf89a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wave-triangle
      /// </summary>
      [Description("wave-triangle")]
      public const string WaveTriangle = "\uf89a";

      /// <summary>
      /// fa-waveform unicode value ("\uf8f1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/waveform
      /// </summary>
      [Description("waveform")]
      public const string Waveform = "\uf8f1";

      /// <summary>
      /// fa-waveform-lines unicode value ("\uf8f2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.11.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/waveform-lines
      /// </summary>
      [Description("waveform-lines")]
      public const string WaveformLines = "\uf8f2";

      /// <summary>
      /// fa-waze unicode value ("\uf83f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/waze
      /// </summary>
      [Description("waze")]
      public const string Waze = "\uf83f";

      /// <summary>
      /// fa-webhook unicode value ("\ue5d5").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/webhook
      /// </summary>
      [Description("webhook")]
      public const string Webhook = "\ue5d5";

      /// <summary>
      /// fa-weebly unicode value ("\uf5cc").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/weebly
      /// </summary>
      [Description("weebly")]
      public const string Weebly = "\uf5cc";

      /// <summary>
      /// fa-weibo unicode value ("\uf18a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/weibo
      /// </summary>
      [Description("weibo")]
      public const string Weibo = "\uf18a";

      /// <summary>
      /// fa-weight-hanging unicode value ("\uf5cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/weight-hanging
      /// </summary>
      [Description("weight-hanging")]
      public const string WeightHanging = "\uf5cd";

      /// <summary>
      /// fa-weight-scale unicode value ("\uf496").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/weight-scale
      /// </summary>
      [Description("weight-scale")]
      public const string WeightScale = "\uf496";

      /// <summary>
      /// fa-weixin unicode value ("\uf1d7").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/weixin
      /// </summary>
      [Description("weixin")]
      public const string Weixin = "\uf1d7";

      /// <summary>
      /// fa-whale unicode value ("\uf72c").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whale
      /// </summary>
      [Description("whale")]
      public const string Whale = "\uf72c";

      /// <summary>
      /// fa-whatsapp unicode value ("\uf232").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.3.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whatsapp
      /// </summary>
      [Description("whatsapp")]
      public const string Whatsapp = "\uf232";

      /// <summary>
      /// fa-wheat unicode value ("\uf72d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheat
      /// </summary>
      [Description("wheat")]
      public const string Wheat = "\uf72d";

      /// <summary>
      /// fa-wheat-awn unicode value ("\ue2cd").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheat-awn
      /// </summary>
      [Description("wheat-awn")]
      public const string WheatAwn = "\ue2cd";

      /// <summary>
      /// fa-wheat-awn-circle-exclamation unicode value ("\ue598").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheat-awn-circle-exclamation
      /// </summary>
      [Description("wheat-awn-circle-exclamation")]
      public const string WheatAwnCircleExclamation = "\ue598";

      /// <summary>
      /// fa-wheat-awn-slash unicode value ("\ue338").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheat-awn-slash
      /// </summary>
      [Description("wheat-awn-slash")]
      public const string WheatAwnSlash = "\ue338";

      /// <summary>
      /// fa-wheat-slash unicode value ("\ue339").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheat-slash
      /// </summary>
      [Description("wheat-slash")]
      public const string WheatSlash = "\ue339";

      /// <summary>
      /// fa-wheelchair unicode value ("\uf193").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheelchair
      /// </summary>
      [Description("wheelchair")]
      public const string Wheelchair = "\uf193";

      /// <summary>
      /// fa-wheelchair-move unicode value ("\ue2ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wheelchair-move
      /// </summary>
      [Description("wheelchair-move")]
      public const string WheelchairMove = "\ue2ce";

      /// <summary>
      /// fa-whiskey-glass unicode value ("\uf7a0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whiskey-glass
      /// </summary>
      [Description("whiskey-glass")]
      public const string WhiskeyGlass = "\uf7a0";

      /// <summary>
      /// fa-whiskey-glass-ice unicode value ("\uf7a1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whiskey-glass-ice
      /// </summary>
      [Description("whiskey-glass-ice")]
      public const string WhiskeyGlassIce = "\uf7a1";

      /// <summary>
      /// fa-whistle unicode value ("\uf460").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.5', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whistle
      /// </summary>
      [Description("whistle")]
      public const string Whistle = "\uf460";

      /// <summary>
      /// fa-whmcs unicode value ("\uf40d").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/whmcs
      /// </summary>
      [Description("whmcs")]
      public const string Whmcs = "\uf40d";

      /// <summary>
      /// fa-wifi unicode value ("\uf1eb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wifi
      /// </summary>
      [Description("wifi")]
      public const string Wifi = "\uf1eb";

      /// <summary>
      /// fa-wifi-exclamation unicode value ("\ue2cf").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wifi-exclamation
      /// </summary>
      [Description("wifi-exclamation")]
      public const string WifiExclamation = "\ue2cf";

      /// <summary>
      /// fa-wifi-fair unicode value ("\uf6ab").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wifi-fair
      /// </summary>
      [Description("wifi-fair")]
      public const string WifiFair = "\uf6ab";

      /// <summary>
      /// fa-wifi-slash unicode value ("\uf6ac").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wifi-slash
      /// </summary>
      [Description("wifi-slash")]
      public const string WifiSlash = "\uf6ac";

      /// <summary>
      /// fa-wifi-weak unicode value ("\uf6aa").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wifi-weak
      /// </summary>
      [Description("wifi-weak")]
      public const string WifiWeak = "\uf6aa";

      /// <summary>
      /// fa-wikipedia-w unicode value ("\uf266").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wikipedia-w
      /// </summary>
      [Description("wikipedia-w")]
      public const string WikipediaW = "\uf266";

      /// <summary>
      /// fa-wind unicode value ("\uf72e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wind
      /// </summary>
      [Description("wind")]
      public const string Wind = "\uf72e";

      /// <summary>
      /// fa-wind-turbine unicode value ("\uf89b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.9.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wind-turbine
      /// </summary>
      [Description("wind-turbine")]
      public const string WindTurbine = "\uf89b";

      /// <summary>
      /// fa-wind-warning unicode value ("\uf776").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wind-warning
      /// </summary>
      [Description("wind-warning")]
      public const string WindWarning = "\uf776";

      /// <summary>
      /// fa-window unicode value ("\uf40e").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window
      /// </summary>
      [Description("window")]
      public const string Window = "\uf40e";

      /// <summary>
      /// fa-window-flip unicode value ("\uf40f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-flip
      /// </summary>
      [Description("window-flip")]
      public const string WindowFlip = "\uf40f";

      /// <summary>
      /// fa-window-frame unicode value ("\ue04f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-frame
      /// </summary>
      [Description("window-frame")]
      public const string WindowFrame = "\ue04f";

      /// <summary>
      /// fa-window-frame-open unicode value ("\ue050").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.12.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-frame-open
      /// </summary>
      [Description("window-frame-open")]
      public const string WindowFrameOpen = "\ue050";

      /// <summary>
      /// fa-window-maximize unicode value ("\uf2d0").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-maximize
      /// </summary>
      [Description("window-maximize")]
      public const string WindowMaximize = "\uf2d0";

      /// <summary>
      /// fa-window-minimize unicode value ("\uf2d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-minimize
      /// </summary>
      [Description("window-minimize")]
      public const string WindowMinimize = "\uf2d1";

      /// <summary>
      /// fa-window-restore unicode value ("\uf2d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular, Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/window-restore
      /// </summary>
      [Description("window-restore")]
      public const string WindowRestore = "\uf2d2";

      /// <summary>
      /// fa-windows unicode value ("\uf17a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/windows
      /// </summary>
      [Description("windows")]
      public const string Windows = "\uf17a";

      /// <summary>
      /// fa-windsock unicode value ("\uf777").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/windsock
      /// </summary>
      [Description("windsock")]
      public const string Windsock = "\uf777";

      /// <summary>
      /// fa-wine-bottle unicode value ("\uf72f").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wine-bottle
      /// </summary>
      [Description("wine-bottle")]
      public const string WineBottle = "\uf72f";

      /// <summary>
      /// fa-wine-glass unicode value ("\uf4e3").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wine-glass
      /// </summary>
      [Description("wine-glass")]
      public const string WineGlass = "\uf4e3";

      /// <summary>
      /// fa-wine-glass-crack unicode value ("\uf4bb").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.9', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wine-glass-crack
      /// </summary>
      [Description("wine-glass-crack")]
      public const string WineGlassCrack = "\uf4bb";

      /// <summary>
      /// fa-wine-glass-empty unicode value ("\uf5ce").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wine-glass-empty
      /// </summary>
      [Description("wine-glass-empty")]
      public const string WineGlassEmpty = "\uf5ce";

      /// <summary>
      /// fa-wirsindhandwerk unicode value ("\ue2d0").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/wirsindhandwerk
      /// </summary>
      [Description("wirsindhandwerk")]
      public const string Wirsindhandwerk = "\ue2d0";

      /// <summary>
      /// fa-wix unicode value ("\uf5cf").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.1.0', Last Modified in '5.1.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wix
      /// </summary>
      [Description("wix")]
      public const string Wix = "\uf5cf";

      /// <summary>
      /// fa-wizards-of-the-coast unicode value ("\uf730").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.0', Last Modified in '5.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wizards-of-the-coast
      /// </summary>
      [Description("wizards-of-the-coast")]
      public const string WizardsOfTheCoast = "\uf730";

      /// <summary>
      /// fa-wodu unicode value ("\ue088").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.15.0', Last Modified in '5.15.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wodu
      /// </summary>
      [Description("wodu")]
      public const string Wodu = "\ue088";

      /// <summary>
      /// fa-wolf-pack-battalion unicode value ("\uf514").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.12', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wolf-pack-battalion
      /// </summary>
      [Description("wolf-pack-battalion")]
      public const string WolfPackBattalion = "\uf514";

      /// <summary>
      /// fa-won-sign unicode value ("\uf159").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/won-sign
      /// </summary>
      [Description("won-sign")]
      public const string WonSign = "\uf159";

      /// <summary>
      /// fa-wordpress unicode value ("\uf19a").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wordpress
      /// </summary>
      [Description("wordpress")]
      public const string Wordpress = "\uf19a";

      /// <summary>
      /// fa-wordpress-simple unicode value ("\uf411").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wordpress-simple
      /// </summary>
      [Description("wordpress-simple")]
      public const string WordpressSimple = "\uf411";

      /// <summary>
      /// fa-worm unicode value ("\ue599").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/worm
      /// </summary>
      [Description("worm")]
      public const string Worm = "\ue599";

      /// <summary>
      /// fa-wpbeginner unicode value ("\uf297").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wpbeginner
      /// </summary>
      [Description("wpbeginner")]
      public const string Wpbeginner = "\uf297";

      /// <summary>
      /// fa-wpexplorer unicode value ("\uf2de").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.7.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wpexplorer
      /// </summary>
      [Description("wpexplorer")]
      public const string Wpexplorer = "\uf2de";

      /// <summary>
      /// fa-wpforms unicode value ("\uf298").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wpforms
      /// </summary>
      [Description("wpforms")]
      public const string Wpforms = "\uf298";

      /// <summary>
      /// fa-wpressr unicode value ("\uf3e4").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.4.2', Last Modified in '5.4.2'.
      /// <para/>
      /// See https://fontawesome.com/icons/wpressr
      /// </summary>
      [Description("wpressr")]
      public const string Wpressr = "\uf3e4";

      /// <summary>
      /// fa-wreath unicode value ("\uf7e2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wreath
      /// </summary>
      [Description("wreath")]
      public const string Wreath = "\uf7e2";

      /// <summary>
      /// fa-wreath-laurel unicode value ("\ue5d2").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wreath-laurel
      /// </summary>
      [Description("wreath-laurel")]
      public const string WreathLaurel = "\ue5d2";

      /// <summary>
      /// fa-wrench unicode value ("\uf0ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '2.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wrench
      /// </summary>
      [Description("wrench")]
      public const string Wrench = "\uf0ad";

      /// <summary>
      /// fa-wrench-simple unicode value ("\ue2d1").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/wrench-simple
      /// </summary>
      [Description("wrench-simple")]
      public const string WrenchSimple = "\ue2d1";

      /// <summary>
      /// fa-x unicode value ("\u0058").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/x
      /// </summary>
      [Description("x")]
      public const string X = "\u0058";

      /// <summary>
      /// fa-x-ray unicode value ("\uf497").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.0.7', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/x-ray
      /// </summary>
      [Description("x-ray")]
      public const string XRay = "\uf497";

      /// <summary>
      /// fa-xbox unicode value ("\uf412").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xbox
      /// </summary>
      [Description("xbox")]
      public const string Xbox = "\uf412";

      /// <summary>
      /// fa-xing unicode value ("\uf168").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xing
      /// </summary>
      [Description("xing")]
      public const string Xing = "\uf168";

      /// <summary>
      /// fa-xmark unicode value ("\uf00d").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '1.0.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xmark
      /// </summary>
      [Description("xmark")]
      public const string Xmark = "\uf00d";

      /// <summary>
      /// fa-xmark-large unicode value ("\ue59b").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xmark-large
      /// </summary>
      [Description("xmark-large")]
      public const string XmarkLarge = "\ue59b";

      /// <summary>
      /// fa-xmark-to-slot unicode value ("\uf771").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid (Pro), Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.5.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xmark-to-slot
      /// </summary>
      [Description("xmark-to-slot")]
      public const string XmarkToSlot = "\uf771";

      /// <summary>
      /// fa-xmarks-lines unicode value ("\ue59a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.1.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/xmarks-lines
      /// </summary>
      [Description("xmarks-lines")]
      public const string XmarksLines = "\ue59a";

      /// <summary>
      /// fa-y unicode value ("\u0059").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/y
      /// </summary>
      [Description("y")]
      public const string Y = "\u0059";

      /// <summary>
      /// fa-y-combinator unicode value ("\uf23b").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.4.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/y-combinator
      /// </summary>
      [Description("y-combinator")]
      public const string YCombinator = "\uf23b";

      /// <summary>
      /// fa-yahoo unicode value ("\uf19e").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.1.0', Last Modified in '5.13.1'.
      /// <para/>
      /// See https://fontawesome.com/icons/yahoo
      /// </summary>
      [Description("yahoo")]
      public const string Yahoo = "\uf19e";

      /// <summary>
      /// fa-yammer unicode value ("\uf840").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.8.0', Last Modified in '6.0.0-beta1'.
      /// <para/>
      /// See https://fontawesome.com/icons/yammer
      /// </summary>
      [Description("yammer")]
      public const string Yammer = "\uf840";

      /// <summary>
      /// fa-yandex unicode value ("\uf413").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yandex
      /// </summary>
      [Description("yandex")]
      public const string Yandex = "\uf413";

      /// <summary>
      /// fa-yandex-international unicode value ("\uf414").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.0.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yandex-international
      /// </summary>
      [Description("yandex-international")]
      public const string YandexInternational = "\uf414";

      /// <summary>
      /// fa-yarn unicode value ("\uf7e3").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.6.0', Last Modified in '5.6.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yarn
      /// </summary>
      [Description("yarn")]
      public const string Yarn = "\uf7e3";

      /// <summary>
      /// fa-yelp unicode value ("\uf1e9").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.2.0', Last Modified in '5.8.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yelp
      /// </summary>
      [Description("yelp")]
      public const string Yelp = "\uf1e9";

      /// <summary>
      /// fa-yen-sign unicode value ("\uf157").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yen-sign
      /// </summary>
      [Description("yen-sign")]
      public const string YenSign = "\uf157";

      /// <summary>
      /// fa-yin-yang unicode value ("\uf6ad").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '5.3.0', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/yin-yang
      /// </summary>
      [Description("yin-yang")]
      public const string YinYang = "\uf6ad";

      /// <summary>
      /// fa-yoast unicode value ("\uf2b1").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '4.6.0', Last Modified in '5.0.3'.
      /// <para/>
      /// See https://fontawesome.com/icons/yoast
      /// </summary>
      [Description("yoast")]
      public const string Yoast = "\uf2b1";

      /// <summary>
      /// fa-youtube unicode value ("\uf167").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '3.2.0', Last Modified in '5.0.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/youtube
      /// </summary>
      [Description("youtube")]
      public const string Youtube = "\uf167";

      /// <summary>
      /// fa-z unicode value ("\u005a").
      /// <para/>
      /// This icon supports the following styles: Light (Pro), Regular (Pro), Solid, Duotone (Pro), Thin (Pro)
      /// <para/>
      /// Introduced in '6.0.0-beta1', Last Modified in '6.4.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/z
      /// </summary>
      [Description("z")]
      public const string Z = "\u005a";

      /// <summary>
      /// fa-zhihu unicode value ("\uf63f").
      /// <para/>
      /// This icon supports the following styles: Brands
      /// <para/>
      /// Introduced in '5.2.0', Last Modified in '5.2.0'.
      /// <para/>
      /// See https://fontawesome.com/icons/zhihu
      /// </summary>
      [Description("zhihu")]
      public const string Zhihu = "\uf63f";

   }

}